function [y, t]=analytic_impulse(Fs, fc, td, tau, delay, A1, A2, rt)
% % analytic_impulse: Impulsive noise with known peak level, frequency, duration
% % 
% % Syntax:
% %
% % [y, t]=analytic_impulse(Fs, fc, td, tau, delay, A1, A2, rt);
% %
% % *********************************************************************
% % 
% % Description
% % 
% % This program produces an impulsive test signal 
% % 
% % The signal is an exponentially decaying sin wave with background 
% % gaussian noise.  
% % 
% % *********************************************************************
% % 
% % Input Variables
% % 
% % Fs=50000;           % Hz sampling rate of the impulsive noise
% %                     % default is Fs=50000;
% % 
% % fc=1000;            % Hz ringing frequency of sin wave
% %                     % default is fc=1000; 
% % 
% % td=1;               % seconds time duration of the test signal
% %                     % default is td=5; 
% % 
% % tau=0.1;            % seconds decay constant for modelling the 
% %                     % decay of the ringing noise.  
% %                     % default is tau=2;
% % 
% % delay=1;            % seconds time delay between impulsive noise 
% %                     % and test signal recording.
% %                     % default is delay=1; 
% % 
% % A1=1;               % Pa amplitude of the background gaussian noise
% %                     % default is A1=1;
% % 
% % A2=100;             % Pa initial ampitude of the of the sin wave
% %                     % default is A2=100;
% %   
% % rt=0.0005;          % seconds time to reach peak value from back ground
% %                     % noise level. (variation of rise time)
% %                     % default is rt=0.0005;  
% %   
% % *********************************************************************
%
%
% Example='1';
%
%
% % Each of the inputs should be a constant 
% 
% tau=0.01;         % seconds decay constant for modelling the ringing
% td=1;             % seconds time duration of the test signal
% A2=20;            % Pa initial ampitude of the of the sin wave
% A1=1;             % Pa amplitude of the background gaussian noise
% Fs=100000;        % Hz sampling rate of the impulsive noise
% fc=1000;          % Hz ringing frequency of sin wave
% delay=0.1;        % seconds time delay between impulsive noise 
%                   % and test signal recording
% rt=0.0005;        % seconds time to reach peak value from back ground
%                   % noise level
% 
% [y, t]=analytic_impulse(Fs, fc, td, tau, delay, A1, A2, rt);
% 
% 
% 
% Example='2';
% 
% 
% tau=0.0001;           % seconds decay constant for modelling the ringing
% td=0.2;               % seconds time duration of the test signal
% A2=20;                % Pa initial ampitude of the of the sin wave
% A1=1;                 % Pa amplitude of the background gaussian noise
% Fs=800000;            % Hz sampling rate of the impulsive noise
% fc=40000;             % Hz ringing frequency of sin wave
% delay=0.05;           % seconds time delay between impulsive noise 
%                       % and test signal recording
% rt=0.00005;           % seconds time to reach peak value from back ground
%                       % noise level (variation of rise time)
% 
% % [y, t]=analytic_impulse(Fs, fc, td, tau, delay, A1, A2, rt);
% 
% 
% 
% 
% 
% 
% % *********************************************************************
% % 
% % Output variables
% % 
% % y the time record in Pa
% % 
% % t the time array in seconds
% % 
% % n is the number of data points in the rise time.  
% % 
% % *********************************************************************
% % 
% % Program Written by Edward L. Zechmann
% %    
% %     Date  17 December  2007
% % 
% % modified  18 December   2007    Added Comments
% % 
% % modified  21 August     2008    Updated Comments
% % 
% % modified  15 September	2008    Updated Comments
% % 
% % modified   3 December   2008    Updated Comments
% % 
% % modified   9 July       2010    
% % 
% % *********************************************************************
% % 
% % Please Feel Free to Modify This Program
% %    
% % See Also: Freidlander, Impulsive_Noise_Meter   
% %    

if nargin < 1 || isempty(Fs) || ~isnumeric(Fs)
    Fs=50000;
end

if nargin < 2 || isempty(fc) || ~isnumeric(fc)
    fc=1000;
end

if nargin < 3 || isempty(td) || ~isnumeric(td)
    td=5; 
end

if nargin < 4 || isempty(tau) || ~isnumeric(tau)
    tau=2;
end

if nargin < 5 || isempty(delay) || ~isnumeric(delay)
    delay=1;
end

if nargin < 6 || isempty(A1) || ~isnumeric(A1)
    A1=1;
end

if nargin < 7 || isempty(A2) || ~isnumeric(A2)
    A2=100;
end

if nargin < 8 || isempty(rt) || ~isnumeric(rt)
    rt=0.0005;  % variation of rise time
end



Fs=Fs(1);
fc=fc(1);
td=td(1);
tau=tau(1);
delay=delay(1);
A1=A1(1);
A2=A2(1);

num_samples=ceil(Fs*td);

% Create the background noise
y=A1.*randn(num_samples, 1);

% rt is a variation of the rise time
if rt > td
    rt=td/100;
end

% Create an exponential rise
num_rise_pts=ceil(Fs*rt);
if num_rise_pts >  num_samples   
    num_rise_pts=1;
end

delay_index=ceil(Fs*delay);
if delay_index >  num_samples    
    delay_index=ceil(num_samples/10); 
end


y_rise=A2.*geospace(0, 1, num_rise_pts, 1).^3;

decay_curve=zeros(num_samples, 1);

decay_curve((delay_index-num_rise_pts+1):(delay_index))=y_rise;

sig_pts=(delay_index:num_samples);

t=1/Fs*(sig_pts-sig_pts(1));

decay_curve(sig_pts)=A2.*exp(-1./(tau).*t);


time_shift=(-delay_index)/Fs;

t=1/Fs*(-1+(1:num_samples))+time_shift;


y=y+decay_curve.*cos(2*pi*fc.*t');

t=t-time_shift;

