function [alpha]=t_alpha(t, nu)
% % t_alpha:  cumulative probability function of the t-distribution, 
% %
% % Syntax='';
% %
% % [alpha]=t_alpha(t, nu);
% %
% % ********************************************************************
% %
% % Description
% %
% % This program calculates the student-t confidence level given
% % the statistic t, with nu degrees of freedom.  t can be a vector.  
% % t can handle values of -Inf, 0, and Inf. nu should be a constant.  
% %
% % In more technical language this program is the cumulative probability 
% % density function for the t-distribution.  
% %
% % For t less than 0, the alpha is less than 0.5.
% %
% % t=1 is the default measured value of the statistic.
% %
% % nu is the number of degrees of freedom.  The default value of nu is Inf.
% %    By default the normal distribution approximation is used.
% %
% % For values of nu in the range 1 <= nu <= 170,  analytical solutions
% % are used and the accuracy can be to machine precision.
% %
% % For values of nu in the range 171 <= nu <= 320, the generalized
% % hypergeometric funciton solution is used.  Some of the numbers are quite
% % large and rounding errors and other computation problems are a
% % possiblility.  The tolerance is hard coded to 10.^(-15).  Calculations
% % can have 8 digits or more of accuracy; however,
% % compuational problems can arise.
% %
% % For values of nu in the range 321 <= nu <= Inf, the normal distribution
% % approximation is used, because the hypergeometric function crashes due
% % to computational problems (large numers and small step sizes).
% %
% % ********************************************************************
% %
% % Input Variables
% %
% % t is the measured statistic value -Inf < t < Inf.
% %
% % nu is the number of degree of freedom 1 <= nu < Inf.
% %
% % ********************************************************************
% %
% % Output Variables
% %
% % alpha   % student-t level of significance, cumulative probability
% %
% % ********************************************************************
% %
% Example='';
%
% t=1;         % statistic
% nu=Inf;      % number of degrees of freedom
%
% [alpha]=t_alpha(t, nu);
%
% t=1;
% nu=10;
% [alpha2]=t_alpha(t, nu);
%
% 
% % ********************************************************************
% % 
% % 
% % Subprograms
% % 
% % List of Dependent Subprograms for 
% % t_alpha
% % 
% % FEX ID# is the File ID on the Matlab Central File Exchange
% % 
% % 
% % Program Name   Author   FEX ID#
% % 1) genHyper		Ben Barrowes		6218	
% %
% % ********************************************************************
% %
% % This program was written by Edward L. Zechmann
% %
% %     date 20 January     2008
% %
% % modified 30 January     2008    made regimes integrate from t0=0.
% %                                 increased speed of program
% %
% % modified  8 March       2008    Implemented the analytical solution to the
% %                                 cumulative probability function.
% %                                 Maple was used to derive the solutions.
% %
% % modified  9 March       2008    Added additional analytical solutions
% %                                 Updated comments.
% % 
% % modified 11 September   2008    Updated Comments
% % 
% % modified  6 October     2009    Updated comments
% % 
% %
% % ********************************************************************
% %
% % Feel free to modify this code.
% %

if nargin < 1 || isempty(t) || ~isnumeric(t)
    t=1;
end

if nargin < 2 || isempty(nu) || logical(nu < 1)  || ~isnumeric(nu)
    nu=Inf;
end

nu=round(nu(1));

% Check special cases of t 
% set the values of alpha for the special cases
n1=length(t);

alpha=zeros(size(t));
ix=ismember(-Inf, t );

set_t=setdiff(1:n1, ix);

if ~isempty(ix)
    alpha(ix)=0;
end

ix=ismember(Inf,t);
set_t=setdiff(set_t, ix);

if ~isempty(ix)
    alpha(ix)=1;
end

ix=ismember(0,t);
set_t=setdiff(set_t, ix);

if ~isempty(ix)
    alpha(ix)=0.5;
end


t=t(set_t);

% make sure t is not empty before checking special casaes of nu

if ~isempty(t)
    % Check special cases of nu
    % nu < 171 analytical solutions
    % 171 <= nu <= 320 Generalized Hypergeometric solution
    % nu >= 321 normal distribution approximation
    switch nu
        case 1;
            alpha(set_t)=1./2+1./pi.*atan(t);

        case 2;
            alpha(set_t)=1./2+1./2.*t./(2+t.^2).^(1./2);

        case 3;
            alpha(set_t)=1./2+1./pi.*atan(1./3.*t.*3.^(1./2))+1./pi.*t.*3.^(1./2)./(3+t.^2);

        case 4;
            alpha(set_t)=1./2+1./2.*t./(4+t.^2).^(1./2).*(1+1./2./(1+1./4.*t.^2));

        case 5;
            alpha(set_t)=1./2+1./pi.*atan(1./5.*t.*5.^(1./2))+1./pi.*t.*5.^(1./2)./(5+t.^2).*(1+2./3./(1+1./5.*t.^2));

        case 6;
            alpha(set_t)=1./2+1./2.*t./(6+t.^2).^(1./2).*(1+1./2./(1+1./6.*t.^2)+3./8./(1+1./6.*t.^2).^2);

        case 7;
            alpha(set_t)=1./2+1./pi.*atan(1./7.*t.*7.^(1./2))+1./pi.*t.*7.^(1./2)./(7+t.^2).*(1+2./3./(1+1./7.*t.^2)+8./15./(1+1./7.*t.^2).^2);

        case 8;
            alpha(set_t)=1./2+1./2.*t./(8+t.^2).^(1./2).*(1+1./2./(1+1./8.*t.^2)+3./8./(1+1./8.*t.^2).^2+5./16./(1+1./8.*t.^2).^3);

        case 9;
            alpha(set_t)=1./2+1./pi.*atan(1./9.*t.*9.^(1./2))+1./pi.*t.*9.^(1./2)./(9+t.^2).*(1+2./3./(1+1./9.*t.^2)+8./15./(1+1./9.*t.^2).^2+16./35./(1+1./9.*t.^2).^3);

        case 10;
            alpha(set_t)=1./2+1./2.*t./(10+t.^2).^(1./2).*(1+1./2./(1+1./10.*t.^2)+3./8./(1+1./10.*t.^2).^2+5./16./(1+1./10.*t.^2).^3+35./128./(1+1./10.*t.^2).^4);

        case 11;
            alpha(set_t)=1./2+1./pi.*atan(1./11.*t.*11.^(1./2))+1./pi.*t.*11.^(1./2)./(11+t.^2).*(1+2./3./(1+1./11.*t.^2)+8./15./(1+1./11.*t.^2).^2+16./35./(1+1./11.*t.^2).^3+128./315./(1+1./11.*t.^2).^4);

        case 12;
            alpha(set_t)=1./2+1./2.*t./(12+t.^2).^(1./2).*(1+1./2./(1+1./12.*t.^2)+3./8./(1+1./12.*t.^2).^2+5./16./(1+1./12.*t.^2).^3+35./128./(1+1./12.*t.^2).^4+63./256./(1+1./12.*t.^2).^5);

        case 13;
            alpha(set_t)=1./2+1./pi.*atan(1./13.*t.*13.^(1./2))+1./pi.*t.*13.^(1./2)./(13+t.^2).*(1+2./3./(1+1./13.*t.^2)+8./15./(1+1./13.*t.^2).^2+16./35./(1+1./13.*t.^2).^3+128./315./(1+1./13.*t.^2).^4+256./693./(1+1./13.*t.^2).^5);

        case 14;
            alpha(set_t)=1./2+1./2.*t./(14+t.^2).^(1./2).*(1+1./2./(1+1./14.*t.^2)+3./8./(1+1./14.*t.^2).^2+5./16./(1+1./14.*t.^2).^3+35./128./(1+1./14.*t.^2).^4+63./256./(1+1./14.*t.^2).^5+231./1024./(1+1./14.*t.^2).^6);

        case 15;
            alpha(set_t)=1./2+1./pi.*atan(1./15.*t.*15.^(1./2))+1./pi.*t.*15.^(1./2)./(15+t.^2).*(1+2./3./(1+1./15.*t.^2)+8./15./(1+1./15.*t.^2).^2+16./35./(1+1./15.*t.^2).^3+128./315./(1+1./15.*t.^2).^4+256./693./(1+1./15.*t.^2).^5+1024./3003./(1+1./15.*t.^2).^6);

        case 16;
            alpha(set_t)=1./2+1./2.*t./(16+t.^2).^(1./2).*(1+1./2./(1+1./16.*t.^2)+3./8./(1+1./16.*t.^2).^2+5./16./(1+1./16.*t.^2).^3+35./128./(1+1./16.*t.^2).^4+63./256./(1+1./16.*t.^2).^5+231./1024./(1+1./16.*t.^2).^6+429./2048./(1+1./16.*t.^2).^7);

        case 17;
            alpha(set_t)=1./2+1./pi.*atan(1./17.*t.*17.^(1./2))+1./pi.*t.*17.^(1./2)./(17+t.^2).*(1+2./3./(1+1./17.*t.^2)+8./15./(1+1./17.*t.^2).^2+16./35./(1+1./17.*t.^2).^3+128./315./(1+1./17.*t.^2).^4+256./693./(1+1./17.*t.^2).^5+1024./3003./(1+1./17.*t.^2).^6+2048./6435./(1+1./17.*t.^2).^7);

        case 18;
            alpha(set_t)=1./2+1./2.*t./(18+t.^2).^(1./2).*(1+1./2./(1+1./18.*t.^2)+3./8./(1+1./18.*t.^2).^2+5./16./(1+1./18.*t.^2).^3+35./128./(1+1./18.*t.^2).^4+63./256./(1+1./18.*t.^2).^5+231./1024./(1+1./18.*t.^2).^6+429./2048./(1+1./18.*t.^2).^7+6435./32768./(1+1./18.*t.^2).^8);

        case 19;
            alpha(set_t)=1./2+1./pi.*atan(1./19.*t.*19.^(1./2))+1./pi.*t.*19.^(1./2)./(19+t.^2).*(1+2./3./(1+1./19.*t.^2)+8./15./(1+1./19.*t.^2).^2+16./35./(1+1./19.*t.^2).^3+128./315./(1+1./19.*t.^2).^4+256./693./(1+1./19.*t.^2).^5+1024./3003./(1+1./19.*t.^2).^6+2048./6435./(1+1./19.*t.^2).^7+32768./109395./(1+1./19.*t.^2).^8);

        case 20;
            alpha(set_t)=1./2+1./2.*t./(20+t.^2).^(1./2).*(1+1./2./(1+1./20.*t.^2)+3./8./(1+1./20.*t.^2).^2+5./16./(1+1./20.*t.^2).^3+35./128./(1+1./20.*t.^2).^4+63./256./(1+1./20.*t.^2).^5+231./1024./(1+1./20.*t.^2).^6+429./2048./(1+1./20.*t.^2).^7+6435./32768./(1+1./20.*t.^2).^8+12155./65536./(1+1./20.*t.^2).^9);

        case 21;
            alpha(set_t)=1./2+1./pi.*atan(1./21.*t.*21.^(1./2))+1./pi.*t.*21.^(1./2)./(21+t.^2).*(1+2./3./(1+1./21.*t.^2)+8./15./(1+1./21.*t.^2).^2+16./35./(1+1./21.*t.^2).^3+128./315./(1+1./21.*t.^2).^4+256./693./(1+1./21.*t.^2).^5+1024./3003./(1+1./21.*t.^2).^6+2048./6435./(1+1./21.*t.^2).^7+32768./109395./(1+1./21.*t.^2).^8+65536./230945./(1+1./21.*t.^2).^9);

        case 22;
            alpha(set_t)=1./2+1./2.*t./(22+t.^2).^(1./2).*(1+1./2./(1+1./22.*t.^2)+3./8./(1+1./22.*t.^2).^2+5./16./(1+1./22.*t.^2).^3+35./128./(1+1./22.*t.^2).^4+63./256./(1+1./22.*t.^2).^5+231./1024./(1+1./22.*t.^2).^6+429./2048./(1+1./22.*t.^2).^7+6435./32768./(1+1./22.*t.^2).^8+12155./65536./(1+1./22.*t.^2).^9+46189./262144./(1+1./22.*t.^2).^10);

        case 23;
            alpha(set_t)=1./2+1./pi.*atan(1./23.*t.*23.^(1./2))+1./pi.*t.*23.^(1./2)./(23+t.^2).*(1+2./3./(1+1./23.*t.^2)+8./15./(1+1./23.*t.^2).^2+16./35./(1+1./23.*t.^2).^3+128./315./(1+1./23.*t.^2).^4+256./693./(1+1./23.*t.^2).^5+1024./3003./(1+1./23.*t.^2).^6+2048./6435./(1+1./23.*t.^2).^7+32768./109395./(1+1./23.*t.^2).^8+65536./230945./(1+1./23.*t.^2).^9+262144./969969./(1+1./23.*t.^2).^10);

        case 24;
            alpha(set_t)=1./2+1./2.*t./(24+t.^2).^(1./2).*(1+1./2./(1+1./24.*t.^2)+3./8./(1+1./24.*t.^2).^2+5./16./(1+1./24.*t.^2).^3+35./128./(1+1./24.*t.^2).^4+63./256./(1+1./24.*t.^2).^5+231./1024./(1+1./24.*t.^2).^6+429./2048./(1+1./24.*t.^2).^7+6435./32768./(1+1./24.*t.^2).^8+12155./65536./(1+1./24.*t.^2).^9+46189./262144./(1+1./24.*t.^2).^10+88179./524288./(1+1./24.*t.^2).^11);

        case 25;
            alpha(set_t)=1./2+1./pi.*atan(1./25.*t.*25.^(1./2))+1./pi.*t.*25.^(1./2)./(25+t.^2).*(1+2./3./(1+1./25.*t.^2)+8./15./(1+1./25.*t.^2).^2+16./35./(1+1./25.*t.^2).^3+128./315./(1+1./25.*t.^2).^4+256./693./(1+1./25.*t.^2).^5+1024./3003./(1+1./25.*t.^2).^6+2048./6435./(1+1./25.*t.^2).^7+32768./109395./(1+1./25.*t.^2).^8+65536./230945./(1+1./25.*t.^2).^9+262144./969969./(1+1./25.*t.^2).^10+524288./2028117./(1+1./25.*t.^2).^11);

        case 26;
            alpha(set_t)=1./2+1./2.*t./(26+t.^2).^(1./2).*(1+1./2./(1+1./26.*t.^2)+3./8./(1+1./26.*t.^2).^2+5./16./(1+1./26.*t.^2).^3+35./128./(1+1./26.*t.^2).^4+63./256./(1+1./26.*t.^2).^5+231./1024./(1+1./26.*t.^2).^6+429./2048./(1+1./26.*t.^2).^7+6435./32768./(1+1./26.*t.^2).^8+12155./65536./(1+1./26.*t.^2).^9+46189./262144./(1+1./26.*t.^2).^10+88179./524288./(1+1./26.*t.^2).^11+676039./4194304./(1+1./26.*t.^2).^12);

        case 27;
            alpha(set_t)=1./2+1./pi.*atan(1./27.*t.*27.^(1./2))+1./pi.*t.*27.^(1./2)./(27+t.^2).*(1+2./3./(1+1./27.*t.^2)+8./15./(1+1./27.*t.^2).^2+16./35./(1+1./27.*t.^2).^3+128./315./(1+1./27.*t.^2).^4+256./693./(1+1./27.*t.^2).^5+1024./3003./(1+1./27.*t.^2).^6+2048./6435./(1+1./27.*t.^2).^7+32768./109395./(1+1./27.*t.^2).^8+65536./230945./(1+1./27.*t.^2).^9+262144./969969./(1+1./27.*t.^2).^10+524288./2028117./(1+1./27.*t.^2).^11+4194304./16900975./(1+1./27.*t.^2).^12);

        case 28;
            alpha(set_t)=1./2+1./2.*t./(28+t.^2).^(1./2).*(1+1./2./(1+1./28.*t.^2)+3./8./(1+1./28.*t.^2).^2+5./16./(1+1./28.*t.^2).^3+35./128./(1+1./28.*t.^2).^4+63./256./(1+1./28.*t.^2).^5+231./1024./(1+1./28.*t.^2).^6+429./2048./(1+1./28.*t.^2).^7+6435./32768./(1+1./28.*t.^2).^8+12155./65536./(1+1./28.*t.^2).^9+46189./262144./(1+1./28.*t.^2).^10+88179./524288./(1+1./28.*t.^2).^11+676039./4194304./(1+1./28.*t.^2).^12+1300075./8388608./(1+1./28.*t.^2).^13);

        case 29;
            alpha(set_t)=1./2+1./pi.*atan(1./29.*t.*29.^(1./2))+1./pi.*t.*29.^(1./2)./(29+t.^2).*(1+2./3./(1+1./29.*t.^2)+8./15./(1+1./29.*t.^2).^2+16./35./(1+1./29.*t.^2).^3+128./315./(1+1./29.*t.^2).^4+256./693./(1+1./29.*t.^2).^5+1024./3003./(1+1./29.*t.^2).^6+2048./6435./(1+1./29.*t.^2).^7+32768./109395./(1+1./29.*t.^2).^8+65536./230945./(1+1./29.*t.^2).^9+262144./969969./(1+1./29.*t.^2).^10+524288./2028117./(1+1./29.*t.^2).^11+4194304./16900975./(1+1./29.*t.^2).^12+8388608./35102025./(1+1./29.*t.^2).^13);

        case 30;
            alpha(set_t)=1./2+1./2.*t./(30+t.^2).^(1./2).*(1+1./2./(1+1./30.*t.^2)+3./8./(1+1./30.*t.^2).^2+5./16./(1+1./30.*t.^2).^3+35./128./(1+1./30.*t.^2).^4+63./256./(1+1./30.*t.^2).^5+231./1024./(1+1./30.*t.^2).^6+429./2048./(1+1./30.*t.^2).^7+6435./32768./(1+1./30.*t.^2).^8+12155./65536./(1+1./30.*t.^2).^9+46189./262144./(1+1./30.*t.^2).^10+88179./524288./(1+1./30.*t.^2).^11+676039./4194304./(1+1./30.*t.^2).^12+1300075./8388608./(1+1./30.*t.^2).^13+5014575./33554432./(1+1./30.*t.^2).^14);

        case 31;
            alpha(set_t)=1./2+1./pi.*atan(1./31.*t.*31.^(1./2))+1./pi.*t.*31.^(1./2)./(31+t.^2).*(1+2./3./(1+1./31.*t.^2)+8./15./(1+1./31.*t.^2).^2+16./35./(1+1./31.*t.^2).^3+128./315./(1+1./31.*t.^2).^4+256./693./(1+1./31.*t.^2).^5+1024./3003./(1+1./31.*t.^2).^6+2048./6435./(1+1./31.*t.^2).^7+32768./109395./(1+1./31.*t.^2).^8+65536./230945./(1+1./31.*t.^2).^9+262144./969969./(1+1./31.*t.^2).^10+524288./2028117./(1+1./31.*t.^2).^11+4194304./16900975./(1+1./31.*t.^2).^12+8388608./35102025./(1+1./31.*t.^2).^13+33554432./145422675./(1+1./31.*t.^2).^14);

        case 32;
            alpha(set_t)=1./2+1./2.*t./(32+t.^2).^(1./2).*(1+1./2./(1+1./32.*t.^2)+3./8./(1+1./32.*t.^2).^2+5./16./(1+1./32.*t.^2).^3+35./128./(1+1./32.*t.^2).^4+63./256./(1+1./32.*t.^2).^5+231./1024./(1+1./32.*t.^2).^6+429./2048./(1+1./32.*t.^2).^7+6435./32768./(1+1./32.*t.^2).^8+12155./65536./(1+1./32.*t.^2).^9+46189./262144./(1+1./32.*t.^2).^10+88179./524288./(1+1./32.*t.^2).^11+676039./4194304./(1+1./32.*t.^2).^12+1300075./8388608./(1+1./32.*t.^2).^13+5014575./33554432./(1+1./32.*t.^2).^14+9694845./67108864./(1+1./32.*t.^2).^15);

        case 33;
            alpha(set_t)=1./2+1./pi.*atan(1./33.*t.*33.^(1./2))+1./pi.*t.*33.^(1./2)./(33+t.^2).*(1+2./3./(1+1./33.*t.^2)+8./15./(1+1./33.*t.^2).^2+16./35./(1+1./33.*t.^2).^3+128./315./(1+1./33.*t.^2).^4+256./693./(1+1./33.*t.^2).^5+1024./3003./(1+1./33.*t.^2).^6+2048./6435./(1+1./33.*t.^2).^7+32768./109395./(1+1./33.*t.^2).^8+65536./230945./(1+1./33.*t.^2).^9+262144./969969./(1+1./33.*t.^2).^10+524288./2028117./(1+1./33.*t.^2).^11+4194304./16900975./(1+1./33.*t.^2).^12+8388608./35102025./(1+1./33.*t.^2).^13+33554432./145422675./(1+1./33.*t.^2).^14+67108864./300540195./(1+1./33.*t.^2).^15);

        case 34;
            alpha(set_t)=1./2+1./2.*t./(34+t.^2).^(1./2).*(1+1./2./(1+1./34.*t.^2)+3./8./(1+1./34.*t.^2).^2+5./16./(1+1./34.*t.^2).^3+35./128./(1+1./34.*t.^2).^4+63./256./(1+1./34.*t.^2).^5+231./1024./(1+1./34.*t.^2).^6+429./2048./(1+1./34.*t.^2).^7+6435./32768./(1+1./34.*t.^2).^8+12155./65536./(1+1./34.*t.^2).^9+46189./262144./(1+1./34.*t.^2).^10+88179./524288./(1+1./34.*t.^2).^11+676039./4194304./(1+1./34.*t.^2).^12+1300075./8388608./(1+1./34.*t.^2).^13+5014575./33554432./(1+1./34.*t.^2).^14+9694845./67108864./(1+1./34.*t.^2).^15+300540195./2147483648./(1+1./34.*t.^2).^16);

        case 35;
            alpha(set_t)=1./2+1./pi.*atan(1./35.*t.*35.^(1./2))+1./pi.*t.*35.^(1./2)./(35+t.^2).*(1+2./3./(1+1./35.*t.^2)+8./15./(1+1./35.*t.^2).^2+16./35./(1+1./35.*t.^2).^3+128./315./(1+1./35.*t.^2).^4+256./693./(1+1./35.*t.^2).^5+1024./3003./(1+1./35.*t.^2).^6+2048./6435./(1+1./35.*t.^2).^7+32768./109395./(1+1./35.*t.^2).^8+65536./230945./(1+1./35.*t.^2).^9+262144./969969./(1+1./35.*t.^2).^10+524288./2028117./(1+1./35.*t.^2).^11+4194304./16900975./(1+1./35.*t.^2).^12+8388608./35102025./(1+1./35.*t.^2).^13+33554432./145422675./(1+1./35.*t.^2).^14+67108864./300540195./(1+1./35.*t.^2).^15+2147483648./9917826435./(1+1./35.*t.^2).^16);

        case 36;
            alpha(set_t)=1./2+1./2.*t./(36+t.^2).^(1./2).*(1+1./2./(1+1./36.*t.^2)+3./8./(1+1./36.*t.^2).^2+5./16./(1+1./36.*t.^2).^3+35./128./(1+1./36.*t.^2).^4+63./256./(1+1./36.*t.^2).^5+231./1024./(1+1./36.*t.^2).^6+429./2048./(1+1./36.*t.^2).^7+6435./32768./(1+1./36.*t.^2).^8+12155./65536./(1+1./36.*t.^2).^9+46189./262144./(1+1./36.*t.^2).^10+88179./524288./(1+1./36.*t.^2).^11+676039./4194304./(1+1./36.*t.^2).^12+1300075./8388608./(1+1./36.*t.^2).^13+5014575./33554432./(1+1./36.*t.^2).^14+9694845./67108864./(1+1./36.*t.^2).^15+300540195./2147483648./(1+1./36.*t.^2).^16+583401555./4294967296./(1+1./36.*t.^2).^17);

        case 37;
            alpha(set_t)=1./2+1./pi.*atan(1./37.*t.*37.^(1./2))+1./pi.*t.*37.^(1./2)./(37+t.^2).*(1+2./3./(1+1./37.*t.^2)+8./15./(1+1./37.*t.^2).^2+16./35./(1+1./37.*t.^2).^3+128./315./(1+1./37.*t.^2).^4+256./693./(1+1./37.*t.^2).^5+1024./3003./(1+1./37.*t.^2).^6+2048./6435./(1+1./37.*t.^2).^7+32768./109395./(1+1./37.*t.^2).^8+65536./230945./(1+1./37.*t.^2).^9+262144./969969./(1+1./37.*t.^2).^10+524288./2028117./(1+1./37.*t.^2).^11+4194304./16900975./(1+1./37.*t.^2).^12+8388608./35102025./(1+1./37.*t.^2).^13+33554432./145422675./(1+1./37.*t.^2).^14+67108864./300540195./(1+1./37.*t.^2).^15+2147483648./9917826435./(1+1./37.*t.^2).^16+4294967296./20419054425./(1+1./37.*t.^2).^17);

        case 38;
            alpha(set_t)=1./2+1./2.*t./(38+t.^2).^(1./2).*(1+1./2./(1+1./38.*t.^2)+3./8./(1+1./38.*t.^2).^2+5./16./(1+1./38.*t.^2).^3+35./128./(1+1./38.*t.^2).^4+63./256./(1+1./38.*t.^2).^5+231./1024./(1+1./38.*t.^2).^6+429./2048./(1+1./38.*t.^2).^7+6435./32768./(1+1./38.*t.^2).^8+12155./65536./(1+1./38.*t.^2).^9+46189./262144./(1+1./38.*t.^2).^10+88179./524288./(1+1./38.*t.^2).^11+676039./4194304./(1+1./38.*t.^2).^12+1300075./8388608./(1+1./38.*t.^2).^13+5014575./33554432./(1+1./38.*t.^2).^14+9694845./67108864./(1+1./38.*t.^2).^15+300540195./2147483648./(1+1./38.*t.^2).^16+583401555./4294967296./(1+1./38.*t.^2).^17+2268783825./17179869184./(1+1./38.*t.^2).^18);

        case 39;
            alpha(set_t)=1./2+1./pi.*atan(1./39.*t.*39.^(1./2))+1./pi.*t.*39.^(1./2)./(39+t.^2).*(1+2./3./(1+1./39.*t.^2)+8./15./(1+1./39.*t.^2).^2+16./35./(1+1./39.*t.^2).^3+128./315./(1+1./39.*t.^2).^4+256./693./(1+1./39.*t.^2).^5+1024./3003./(1+1./39.*t.^2).^6+2048./6435./(1+1./39.*t.^2).^7+32768./109395./(1+1./39.*t.^2).^8+65536./230945./(1+1./39.*t.^2).^9+262144./969969./(1+1./39.*t.^2).^10+524288./2028117./(1+1./39.*t.^2).^11+4194304./16900975./(1+1./39.*t.^2).^12+8388608./35102025./(1+1./39.*t.^2).^13+33554432./145422675./(1+1./39.*t.^2).^14+67108864./300540195./(1+1./39.*t.^2).^15+2147483648./9917826435./(1+1./39.*t.^2).^16+4294967296./20419054425./(1+1./39.*t.^2).^17+17179869184./83945001525./(1+1./39.*t.^2).^18);

        case 40;
            alpha(set_t)=1./2+1./2.*t./(40+t.^2).^(1./2).*(1+1./2./(1+1./40.*t.^2)+3./8./(1+1./40.*t.^2).^2+5./16./(1+1./40.*t.^2).^3+35./128./(1+1./40.*t.^2).^4+63./256./(1+1./40.*t.^2).^5+231./1024./(1+1./40.*t.^2).^6+429./2048./(1+1./40.*t.^2).^7+6435./32768./(1+1./40.*t.^2).^8+12155./65536./(1+1./40.*t.^2).^9+46189./262144./(1+1./40.*t.^2).^10+88179./524288./(1+1./40.*t.^2).^11+676039./4194304./(1+1./40.*t.^2).^12+1300075./8388608./(1+1./40.*t.^2).^13+5014575./33554432./(1+1./40.*t.^2).^14+9694845./67108864./(1+1./40.*t.^2).^15+300540195./2147483648./(1+1./40.*t.^2).^16+583401555./4294967296./(1+1./40.*t.^2).^17+2268783825./17179869184./(1+1./40.*t.^2).^18+4418157975./34359738368./(1+1./40.*t.^2).^19);

        case 41;
            alpha(set_t)=1./2+1./pi.*atan(1./41.*t.*41.^(1./2))+1./pi.*t.*41.^(1./2)./(41+t.^2).*(1+2./3./(1+1./41.*t.^2)+8./15./(1+1./41.*t.^2).^2+16./35./(1+1./41.*t.^2).^3+128./315./(1+1./41.*t.^2).^4+256./693./(1+1./41.*t.^2).^5+1024./3003./(1+1./41.*t.^2).^6+2048./6435./(1+1./41.*t.^2).^7+32768./109395./(1+1./41.*t.^2).^8+65536./230945./(1+1./41.*t.^2).^9+262144./969969./(1+1./41.*t.^2).^10+524288./2028117./(1+1./41.*t.^2).^11+4194304./16900975./(1+1./41.*t.^2).^12+8388608./35102025./(1+1./41.*t.^2).^13+33554432./145422675./(1+1./41.*t.^2).^14+67108864./300540195./(1+1./41.*t.^2).^15+2147483648./9917826435./(1+1./41.*t.^2).^16+4294967296./20419054425./(1+1./41.*t.^2).^17+17179869184./83945001525./(1+1./41.*t.^2).^18+34359738368./172308161025./(1+1./41.*t.^2).^19);

        case 42;
            alpha(set_t)=1./2+1./2.*t./(42+t.^2).^(1./2).*(1+1./2./(1+1./42.*t.^2)+3./8./(1+1./42.*t.^2).^2+5./16./(1+1./42.*t.^2).^3+35./128./(1+1./42.*t.^2).^4+63./256./(1+1./42.*t.^2).^5+231./1024./(1+1./42.*t.^2).^6+429./2048./(1+1./42.*t.^2).^7+6435./32768./(1+1./42.*t.^2).^8+12155./65536./(1+1./42.*t.^2).^9+46189./262144./(1+1./42.*t.^2).^10+88179./524288./(1+1./42.*t.^2).^11+676039./4194304./(1+1./42.*t.^2).^12+1300075./8388608./(1+1./42.*t.^2).^13+5014575./33554432./(1+1./42.*t.^2).^14+9694845./67108864./(1+1./42.*t.^2).^15+300540195./2147483648./(1+1./42.*t.^2).^16+583401555./4294967296./(1+1./42.*t.^2).^17+2268783825./17179869184./(1+1./42.*t.^2).^18+4418157975./34359738368./(1+1./42.*t.^2).^19+34461632205./274877906944./(1+1./42.*t.^2).^20);

        case 43;
            alpha(set_t)=1./2+1./pi.*atan(1./43.*t.*43.^(1./2))+1./pi.*t.*43.^(1./2)./(43+t.^2).*(1+2./3./(1+1./43.*t.^2)+8./15./(1+1./43.*t.^2).^2+16./35./(1+1./43.*t.^2).^3+128./315./(1+1./43.*t.^2).^4+256./693./(1+1./43.*t.^2).^5+1024./3003./(1+1./43.*t.^2).^6+2048./6435./(1+1./43.*t.^2).^7+32768./109395./(1+1./43.*t.^2).^8+65536./230945./(1+1./43.*t.^2).^9+262144./969969./(1+1./43.*t.^2).^10+524288./2028117./(1+1./43.*t.^2).^11+4194304./16900975./(1+1./43.*t.^2).^12+8388608./35102025./(1+1./43.*t.^2).^13+33554432./145422675./(1+1./43.*t.^2).^14+67108864./300540195./(1+1./43.*t.^2).^15+2147483648./9917826435./(1+1./43.*t.^2).^16+4294967296./20419054425./(1+1./43.*t.^2).^17+17179869184./83945001525./(1+1./43.*t.^2).^18+34359738368./172308161025./(1+1./43.*t.^2).^19+274877906944./1412926920405./(1+1./43.*t.^2).^20);

        case 44;
            alpha(set_t)=1./2+1./2.*t./(44+t.^2).^(1./2).*(1+1./2./(1+1./44.*t.^2)+3./8./(1+1./44.*t.^2).^2+5./16./(1+1./44.*t.^2).^3+35./128./(1+1./44.*t.^2).^4+63./256./(1+1./44.*t.^2).^5+231./1024./(1+1./44.*t.^2).^6+429./2048./(1+1./44.*t.^2).^7+6435./32768./(1+1./44.*t.^2).^8+12155./65536./(1+1./44.*t.^2).^9+46189./262144./(1+1./44.*t.^2).^10+88179./524288./(1+1./44.*t.^2).^11+676039./4194304./(1+1./44.*t.^2).^12+1300075./8388608./(1+1./44.*t.^2).^13+5014575./33554432./(1+1./44.*t.^2).^14+9694845./67108864./(1+1./44.*t.^2).^15+300540195./2147483648./(1+1./44.*t.^2).^16+583401555./4294967296./(1+1./44.*t.^2).^17+2268783825./17179869184./(1+1./44.*t.^2).^18+4418157975./34359738368./(1+1./44.*t.^2).^19+34461632205./274877906944./(1+1./44.*t.^2).^20+67282234305./549755813888./(1+1./44.*t.^2).^21);

        case 45;
            alpha(set_t)=1./2+1./pi.*atan(1./45.*t.*45.^(1./2))+1./pi.*t.*45.^(1./2)./(45+t.^2).*(1+2./3./(1+1./45.*t.^2)+8./15./(1+1./45.*t.^2).^2+16./35./(1+1./45.*t.^2).^3+128./315./(1+1./45.*t.^2).^4+256./693./(1+1./45.*t.^2).^5+1024./3003./(1+1./45.*t.^2).^6+2048./6435./(1+1./45.*t.^2).^7+32768./109395./(1+1./45.*t.^2).^8+65536./230945./(1+1./45.*t.^2).^9+262144./969969./(1+1./45.*t.^2).^10+524288./2028117./(1+1./45.*t.^2).^11+4194304./16900975./(1+1./45.*t.^2).^12+8388608./35102025./(1+1./45.*t.^2).^13+33554432./145422675./(1+1./45.*t.^2).^14+67108864./300540195./(1+1./45.*t.^2).^15+2147483648./9917826435./(1+1./45.*t.^2).^16+4294967296./20419054425./(1+1./45.*t.^2).^17+17179869184./83945001525./(1+1./45.*t.^2).^18+34359738368./172308161025./(1+1./45.*t.^2).^19+274877906944./1412926920405./(1+1./45.*t.^2).^20+549755813888./2893136075115./(1+1./45.*t.^2).^21);

        case 46;
            alpha(set_t)=1./2+1./2.*t./(46+t.^2).^(1./2).*(1+1./2./(1+1./46.*t.^2)+3./8./(1+1./46.*t.^2).^2+5./16./(1+1./46.*t.^2).^3+35./128./(1+1./46.*t.^2).^4+63./256./(1+1./46.*t.^2).^5+231./1024./(1+1./46.*t.^2).^6+429./2048./(1+1./46.*t.^2).^7+6435./32768./(1+1./46.*t.^2).^8+12155./65536./(1+1./46.*t.^2).^9+46189./262144./(1+1./46.*t.^2).^10+88179./524288./(1+1./46.*t.^2).^11+676039./4194304./(1+1./46.*t.^2).^12+1300075./8388608./(1+1./46.*t.^2).^13+5014575./33554432./(1+1./46.*t.^2).^14+9694845./67108864./(1+1./46.*t.^2).^15+300540195./2147483648./(1+1./46.*t.^2).^16+583401555./4294967296./(1+1./46.*t.^2).^17+2268783825./17179869184./(1+1./46.*t.^2).^18+4418157975./34359738368./(1+1./46.*t.^2).^19+34461632205./274877906944./(1+1./46.*t.^2).^20+67282234305./549755813888./(1+1./46.*t.^2).^21+263012370465./2199023255552./(1+1./46.*t.^2).^22);

        case 47;
            alpha(set_t)=1./2+1./pi.*atan(1./47.*t.*47.^(1./2))+1./pi.*t.*47.^(1./2)./(47+t.^2).*(1+2./3./(1+1./47.*t.^2)+8./15./(1+1./47.*t.^2).^2+16./35./(1+1./47.*t.^2).^3+128./315./(1+1./47.*t.^2).^4+256./693./(1+1./47.*t.^2).^5+1024./3003./(1+1./47.*t.^2).^6+2048./6435./(1+1./47.*t.^2).^7+32768./109395./(1+1./47.*t.^2).^8+65536./230945./(1+1./47.*t.^2).^9+262144./969969./(1+1./47.*t.^2).^10+524288./2028117./(1+1./47.*t.^2).^11+4194304./16900975./(1+1./47.*t.^2).^12+8388608./35102025./(1+1./47.*t.^2).^13+33554432./145422675./(1+1./47.*t.^2).^14+67108864./300540195./(1+1./47.*t.^2).^15+2147483648./9917826435./(1+1./47.*t.^2).^16+4294967296./20419054425./(1+1./47.*t.^2).^17+17179869184./83945001525./(1+1./47.*t.^2).^18+34359738368./172308161025./(1+1./47.*t.^2).^19+274877906944./1412926920405./(1+1./47.*t.^2).^20+549755813888./2893136075115./(1+1./47.*t.^2).^21+2199023255552./11835556670925./(1+1./47.*t.^2).^22);

        case 48;
            alpha(set_t)=1./2+1./2.*t./(48+t.^2).^(1./2).*(1+300540195./2147483648./(1+1./48.*t.^2).^16+583401555./4294967296./(1+1./48.*t.^2).^17+2268783825./17179869184./(1+1./48.*t.^2).^18+4418157975./34359738368./(1+1./48.*t.^2).^19+34461632205./274877906944./(1+1./48.*t.^2).^20+67282234305./549755813888./(1+1./48.*t.^2).^21+263012370465./2199023255552./(1+1./48.*t.^2).^22+514589420475./4398046511104./(1+1./48.*t.^2).^23+1./2./(1+1./48.*t.^2)+3./8./(1+1./48.*t.^2).^2+5./16./(1+1./48.*t.^2).^3+35./128./(1+1./48.*t.^2).^4+63./256./(1+1./48.*t.^2).^5+231./1024./(1+1./48.*t.^2).^6+429./2048./(1+1./48.*t.^2).^7+12155./65536./(1+1./48.*t.^2).^9+6435./32768./(1+1./48.*t.^2).^8+46189./262144./(1+1./48.*t.^2).^10+88179./524288./(1+1./48.*t.^2).^11+1300075./8388608./(1+1./48.*t.^2).^13+676039./4194304./(1+1./48.*t.^2).^12+5014575./33554432./(1+1./48.*t.^2).^14+9694845./67108864./(1+1./48.*t.^2).^15);

        case 49;
            alpha(set_t)=1./2+1./pi.*atan(1./49.*t.*49.^(1./2))+1./pi.*t.*49.^(1./2)./(49+t.^2).*(1+274877906944./1412926920405./(1+1./49.*t.^2).^20+549755813888./2893136075115./(1+1./49.*t.^2).^21+262144./969969./(1+1./49.*t.^2).^10+524288./2028117./(1+1./49.*t.^2).^11+8./15./(1+1./49.*t.^2).^2+16./35./(1+1./49.*t.^2).^3+2199023255552./11835556670925./(1+1./49.*t.^2).^22+4194304./16900975./(1+1./49.*t.^2).^12+8388608./35102025./(1+1./49.*t.^2).^13+4398046511104./24185702762325./(1+1./49.*t.^2).^23+33554432./145422675./(1+1./49.*t.^2).^14+128./315./(1+1./49.*t.^2).^4+67108864./300540195./(1+1./49.*t.^2).^15+2147483648./9917826435./(1+1./49.*t.^2).^16+4294967296./20419054425./(1+1./49.*t.^2).^17+256./693./(1+1./49.*t.^2).^5+1024./3003./(1+1./49.*t.^2).^6+2048./6435./(1+1./49.*t.^2).^7+17179869184./83945001525./(1+1./49.*t.^2).^18+34359738368./172308161025./(1+1./49.*t.^2).^19+32768./109395./(1+1./49.*t.^2).^8+65536./230945./(1+1./49.*t.^2).^9+2./3./(1+1./49.*t.^2));

        case 50;
            alpha(set_t)=1./2+1./2.*t./(50+t.^2).^(1./2).*(1+1./2./(1+1./50.*t.^2)+3./8./(1+1./50.*t.^2).^2+5./16./(1+1./50.*t.^2).^3+63./256./(1+1./50.*t.^2).^5+35./128./(1+1./50.*t.^2).^4+231./1024./(1+1./50.*t.^2).^6+6435./32768./(1+1./50.*t.^2).^8+429./2048./(1+1./50.*t.^2).^7+46189./262144./(1+1./50.*t.^2).^10+12155./65536./(1+1./50.*t.^2).^9+88179./524288./(1+1./50.*t.^2).^11+676039./4194304./(1+1./50.*t.^2).^12+1300075./8388608./(1+1./50.*t.^2).^13+9694845./67108864./(1+1./50.*t.^2).^15+5014575./33554432./(1+1./50.*t.^2).^14+300540195./2147483648./(1+1./50.*t.^2).^16+2268783825./17179869184./(1+1./50.*t.^2).^18+583401555./4294967296./(1+1./50.*t.^2).^17+4418157975./34359738368./(1+1./50.*t.^2).^19+34461632205./274877906944./(1+1./50.*t.^2).^20+67282234305./549755813888./(1+1./50.*t.^2).^21+263012370465./2199023255552./(1+1./50.*t.^2).^22+514589420475./4398046511104./(1+1./50.*t.^2).^23+8061900920775./70368744177664./(1+1./50.*t.^2).^24);

        case 51;
            alpha(set_t)=1./2+1./pi.*atan(1./51.*t.*51.^(1./2))+1./pi.*t.*51.^(1./2)./(51+t.^2).*(1+2./3./(1+1./51.*t.^2)+8./15./(1+1./51.*t.^2).^2+16./35./(1+1./51.*t.^2).^3+128./315./(1+1./51.*t.^2).^4+256./693./(1+1./51.*t.^2).^5+1024./3003./(1+1./51.*t.^2).^6+2048./6435./(1+1./51.*t.^2).^7+32768./109395./(1+1./51.*t.^2).^8+65536./230945./(1+1./51.*t.^2).^9+262144./969969./(1+1./51.*t.^2).^10+524288./2028117./(1+1./51.*t.^2).^11+4194304./16900975./(1+1./51.*t.^2).^12+8388608./35102025./(1+1./51.*t.^2).^13+33554432./145422675./(1+1./51.*t.^2).^14+67108864./300540195./(1+1./51.*t.^2).^15+2147483648./9917826435./(1+1./51.*t.^2).^16+4294967296./20419054425./(1+1./51.*t.^2).^17+17179869184./83945001525./(1+1./51.*t.^2).^18+34359738368./172308161025./(1+1./51.*t.^2).^19+274877906944./1412926920405./(1+1./51.*t.^2).^20+549755813888./2893136075115./(1+1./51.*t.^2).^21+2199023255552./11835556670925./(1+1./51.*t.^2).^22+4398046511104./24185702762325./(1+1./51.*t.^2).^23+70368744177664./395033145117975./(1+1./51.*t.^2).^24);

        case 52;
            alpha(set_t)=1./2+1./2.*t./(52+t.^2).^(1./2).*(1+514589420475./4398046511104./(1+1./52.*t.^2).^23+8061900920775./70368744177664./(1+1./52.*t.^2).^24+15801325804719./140737488355328./(1+1./52.*t.^2).^25+4418157975./34359738368./(1+1./52.*t.^2).^19+1./2./(1+1./52.*t.^2)+3./8./(1+1./52.*t.^2).^2+5./16./(1+1./52.*t.^2).^3+35./128./(1+1./52.*t.^2).^4+63./256./(1+1./52.*t.^2).^5+231./1024./(1+1./52.*t.^2).^6+429./2048./(1+1./52.*t.^2).^7+34461632205./274877906944./(1+1./52.*t.^2).^20+6435./32768./(1+1./52.*t.^2).^8+12155./65536./(1+1./52.*t.^2).^9+46189./262144./(1+1./52.*t.^2).^10+88179./524288./(1+1./52.*t.^2).^11+1300075./8388608./(1+1./52.*t.^2).^13+676039./4194304./(1+1./52.*t.^2).^12+5014575./33554432./(1+1./52.*t.^2).^14+67282234305./549755813888./(1+1./52.*t.^2).^21+263012370465./2199023255552./(1+1./52.*t.^2).^22+9694845./67108864./(1+1./52.*t.^2).^15+300540195./2147483648./(1+1./52.*t.^2).^16+583401555./4294967296./(1+1./52.*t.^2).^17+2268783825./17179869184./(1+1./52.*t.^2).^18);

        case 53;
            alpha(set_t)=1./2+1./pi.*atan(1./53.*t.*53.^(1./2))+1./pi.*t.*53.^(1./2)./(53+t.^2).*(1+17179869184./83945001525./(1+1./53.*t.^2).^18+70368744177664./395033145117975./(1+1./53.*t.^2).^24+140737488355328./805867616040669./(1+1./53.*t.^2).^25+4294967296./20419054425./(1+1./53.*t.^2).^17+34359738368./172308161025./(1+1./53.*t.^2).^19+274877906944./1412926920405./(1+1./53.*t.^2).^20+2./3./(1+1./53.*t.^2)+8./15./(1+1./53.*t.^2).^2+16./35./(1+1./53.*t.^2).^3+128./315./(1+1./53.*t.^2).^4+256./693./(1+1./53.*t.^2).^5+1024./3003./(1+1./53.*t.^2).^6+549755813888./2893136075115./(1+1./53.*t.^2).^21+65536./230945./(1+1./53.*t.^2).^9+2048./6435./(1+1./53.*t.^2).^7+32768./109395./(1+1./53.*t.^2).^8+262144./969969./(1+1./53.*t.^2).^10+524288./2028117./(1+1./53.*t.^2).^11+4194304./16900975./(1+1./53.*t.^2).^12+2199023255552./11835556670925./(1+1./53.*t.^2).^22+4398046511104./24185702762325./(1+1./53.*t.^2).^23+33554432./145422675./(1+1./53.*t.^2).^14+8388608./35102025./(1+1./53.*t.^2).^13+2147483648./9917826435./(1+1./53.*t.^2).^16+67108864./300540195./(1+1./53.*t.^2).^15);

        case 54;
            alpha(set_t)=1./2+1./2.*t./(54+t.^2).^(1./2).*(1+3./8./(1+1./54.*t.^2).^2+1300075./8388608./(1+1./54.*t.^2).^13+5014575./33554432./(1+1./54.*t.^2).^14+8061900920775./70368744177664./(1+1./54.*t.^2).^24+15801325804719./140737488355328./(1+1./54.*t.^2).^25+5./16./(1+1./54.*t.^2).^3+61989816618513./562949953421312./(1+1./54.*t.^2).^26+9694845./67108864./(1+1./54.*t.^2).^15+2268783825./17179869184./(1+1./54.*t.^2).^18+300540195./2147483648./(1+1./54.*t.^2).^16+583401555./4294967296./(1+1./54.*t.^2).^17+4418157975./34359738368./(1+1./54.*t.^2).^19+35./128./(1+1./54.*t.^2).^4+63./256./(1+1./54.*t.^2).^5+34461632205./274877906944./(1+1./54.*t.^2).^20+231./1024./(1+1./54.*t.^2).^6+429./2048./(1+1./54.*t.^2).^7+6435./32768./(1+1./54.*t.^2).^8+67282234305./549755813888./(1+1./54.*t.^2).^21+263012370465./2199023255552./(1+1./54.*t.^2).^22+12155./65536./(1+1./54.*t.^2).^9+46189./262144./(1+1./54.*t.^2).^10+88179./524288./(1+1./54.*t.^2).^11+514589420475./4398046511104./(1+1./54.*t.^2).^23+676039./4194304./(1+1./54.*t.^2).^12+1./2./(1+1./54.*t.^2));

        case 55;
            alpha(set_t)=1./2+1./pi.*atan(1./55.*t.*55.^(1./2))+1./pi.*t.*55.^(1./2)./(55+t.^2).*(1+1024./3003./(1+1./55.*t.^2).^6+140737488355328./805867616040669./(1+1./55.*t.^2).^25+562949953421312./3285460280781189./(1+1./55.*t.^2).^26+2048./6435./(1+1./55.*t.^2).^7+32768./109395./(1+1./55.*t.^2).^8+65536./230945./(1+1./55.*t.^2).^9+262144./969969./(1+1./55.*t.^2).^10+524288./2028117./(1+1./55.*t.^2).^11+4194304./16900975./(1+1./55.*t.^2).^12+8388608./35102025./(1+1./55.*t.^2).^13+33554432./145422675./(1+1./55.*t.^2).^14+67108864./300540195./(1+1./55.*t.^2).^15+2147483648./9917826435./(1+1./55.*t.^2).^16+4294967296./20419054425./(1+1./55.*t.^2).^17+17179869184./83945001525./(1+1./55.*t.^2).^18+34359738368./172308161025./(1+1./55.*t.^2).^19+2./3./(1+1./55.*t.^2)+274877906944./1412926920405./(1+1./55.*t.^2).^20+549755813888./2893136075115./(1+1./55.*t.^2).^21+8./15./(1+1./55.*t.^2).^2+16./35./(1+1./55.*t.^2).^3+2199023255552./11835556670925./(1+1./55.*t.^2).^22+4398046511104./24185702762325./(1+1./55.*t.^2).^23+70368744177664./395033145117975./(1+1./55.*t.^2).^24+128./315./(1+1./55.*t.^2).^4+256./693./(1+1./55.*t.^2).^5);

        case 56;
            alpha(set_t)=1./2+1./2.*t./(56+t.^2).^(1./2).*(1+583401555./4294967296./(1+1./56.*t.^2).^17+2268783825./17179869184./(1+1./56.*t.^2).^18+4418157975./34359738368./(1+1./56.*t.^2).^19+3./8./(1+1./56.*t.^2).^2+5./16./(1+1./56.*t.^2).^3+67282234305./549755813888./(1+1./56.*t.^2).^21+34461632205./274877906944./(1+1./56.*t.^2).^20+35./128./(1+1./56.*t.^2).^4+263012370465./2199023255552./(1+1./56.*t.^2).^22+514589420475./4398046511104./(1+1./56.*t.^2).^23+63./256./(1+1./56.*t.^2).^5+231./1024./(1+1./56.*t.^2).^6+8061900920775./70368744177664./(1+1./56.*t.^2).^24+15801325804719./140737488355328./(1+1./56.*t.^2).^25+61989816618513./562949953421312./(1+1./56.*t.^2).^26+429./2048./(1+1./56.*t.^2).^7+6435./32768./(1+1./56.*t.^2).^8+121683714103007./1125899906842624./(1+1./56.*t.^2).^27+12155./65536./(1+1./56.*t.^2).^9+46189./262144./(1+1./56.*t.^2).^10+88179./524288./(1+1./56.*t.^2).^11+676039./4194304./(1+1./56.*t.^2).^12+1300075./8388608./(1+1./56.*t.^2).^13+5014575./33554432./(1+1./56.*t.^2).^14+9694845./67108864./(1+1./56.*t.^2).^15+1./2./(1+1./56.*t.^2)+300540195./2147483648./(1+1./56.*t.^2).^16);

        case 57;
            alpha(set_t)=1./2+1./pi.*atan(1./57.*t.*57.^(1./2))+1./pi.*t.*57.^(1./2)./(57+t.^2).*(1+128./315./(1+1./57.*t.^2).^4+256./693./(1+1./57.*t.^2).^5+562949953421312./3285460280781189./(1+1./57.*t.^2).^26+1125899906842624./6692604275665385./(1+1./57.*t.^2).^27+1024./3003./(1+1./57.*t.^2).^6+2048./6435./(1+1./57.*t.^2).^7+34359738368./172308161025./(1+1./57.*t.^2).^19+274877906944./1412926920405./(1+1./57.*t.^2).^20+549755813888./2893136075115./(1+1./57.*t.^2).^21+32768./109395./(1+1./57.*t.^2).^8+65536./230945./(1+1./57.*t.^2).^9+2199023255552./11835556670925./(1+1./57.*t.^2).^22+4398046511104./24185702762325./(1+1./57.*t.^2).^23+70368744177664./395033145117975./(1+1./57.*t.^2).^24+140737488355328./805867616040669./(1+1./57.*t.^2).^25+262144./969969./(1+1./57.*t.^2).^10+524288./2028117./(1+1./57.*t.^2).^11+4194304./16900975./(1+1./57.*t.^2).^12+8388608./35102025./(1+1./57.*t.^2).^13+33554432./145422675./(1+1./57.*t.^2).^14+4294967296./20419054425./(1+1./57.*t.^2).^17+17179869184./83945001525./(1+1./57.*t.^2).^18+2./3./(1+1./57.*t.^2)+67108864./300540195./(1+1./57.*t.^2).^15+2147483648./9917826435./(1+1./57.*t.^2).^16+8./15./(1+1./57.*t.^2).^2+16./35./(1+1./57.*t.^2).^3);

        case 58;
            alpha(set_t)=1./2+1./2.*t./(58+t.^2).^(1./2).*(1+1./2./(1+1./58.*t.^2)+67282234305./549755813888./(1+1./58.*t.^2).^21+263012370465./2199023255552./(1+1./58.*t.^2).^22+34461632205./274877906944./(1+1./58.*t.^2).^20+514589420475./4398046511104./(1+1./58.*t.^2).^23+8061900920775./70368744177664./(1+1./58.*t.^2).^24+15801325804719./140737488355328./(1+1./58.*t.^2).^25+61989816618513./562949953421312./(1+1./58.*t.^2).^26+121683714103007./1125899906842624./(1+1./58.*t.^2).^27+956086325095055./9007199254740992./(1+1./58.*t.^2).^28+3./8./(1+1./58.*t.^2).^2+5./16./(1+1./58.*t.^2).^3+35./128./(1+1./58.*t.^2).^4+63./256./(1+1./58.*t.^2).^5+231./1024./(1+1./58.*t.^2).^6+429./2048./(1+1./58.*t.^2).^7+6435./32768./(1+1./58.*t.^2).^8+12155./65536./(1+1./58.*t.^2).^9+46189./262144./(1+1./58.*t.^2).^10+88179./524288./(1+1./58.*t.^2).^11+676039./4194304./(1+1./58.*t.^2).^12+1300075./8388608./(1+1./58.*t.^2).^13+5014575./33554432./(1+1./58.*t.^2).^14+9694845./67108864./(1+1./58.*t.^2).^15+300540195./2147483648./(1+1./58.*t.^2).^16+583401555./4294967296./(1+1./58.*t.^2).^17+2268783825./17179869184./(1+1./58.*t.^2).^18+4418157975./34359738368./(1+1./58.*t.^2).^19);

        case 59;
            alpha(set_t)=1./2+1./pi.*atan(1./59.*t.*59.^(1./2))+1./pi.*t.*59.^(1./2)./(59+t.^2).*(1+65536./230945./(1+1./59.*t.^2).^9+262144./969969./(1+1./59.*t.^2).^10+549755813888./2893136075115./(1+1./59.*t.^2).^21+2199023255552./11835556670925./(1+1./59.*t.^2).^22+4398046511104./24185702762325./(1+1./59.*t.^2).^23+70368744177664./395033145117975./(1+1./59.*t.^2).^24+524288./2028117./(1+1./59.*t.^2).^11+4194304./16900975./(1+1./59.*t.^2).^12+2./3./(1+1./59.*t.^2)+8./15./(1+1./59.*t.^2).^2+140737488355328./805867616040669./(1+1./59.*t.^2).^25+562949953421312./3285460280781189./(1+1./59.*t.^2).^26+8388608./35102025./(1+1./59.*t.^2).^13+33554432./145422675./(1+1./59.*t.^2).^14+67108864./300540195./(1+1./59.*t.^2).^15+2147483648./9917826435./(1+1./59.*t.^2).^16+1125899906842624./6692604275665385./(1+1./59.*t.^2).^27+9007199254740992./54496920530418135./(1+1./59.*t.^2).^28+16./35./(1+1./59.*t.^2).^3+128./315./(1+1./59.*t.^2).^4+4294967296./20419054425./(1+1./59.*t.^2).^17+17179869184./83945001525./(1+1./59.*t.^2).^18+34359738368./172308161025./(1+1./59.*t.^2).^19+274877906944./1412926920405./(1+1./59.*t.^2).^20+256./693./(1+1./59.*t.^2).^5+1024./3003./(1+1./59.*t.^2).^6+2048./6435./(1+1./59.*t.^2).^7+32768./109395./(1+1./59.*t.^2).^8);

        case 60;
            alpha(set_t)=1./2+1./2.*t./(60+t.^2).^(1./2).*(1+34461632205./274877906944./(1+1./60.*t.^2).^20+67282234305./549755813888./(1+1./60.*t.^2).^21+263012370465./2199023255552./(1+1./60.*t.^2).^22+514589420475./4398046511104./(1+1./60.*t.^2).^23+8061900920775./70368744177664./(1+1./60.*t.^2).^24+9694845./67108864./(1+1./60.*t.^2).^15+300540195./2147483648./(1+1./60.*t.^2).^16+15801325804719./140737488355328./(1+1./60.*t.^2).^25+61989816618513./562949953421312./(1+1./60.*t.^2).^26+121683714103007./1125899906842624./(1+1./60.*t.^2).^27+956086325095055./9007199254740992./(1+1./60.*t.^2).^28+1879204156221315./18014398509481984./(1+1./60.*t.^2).^29+1./2./(1+1./60.*t.^2)+3./8./(1+1./60.*t.^2).^2+5./16./(1+1./60.*t.^2).^3+35./128./(1+1./60.*t.^2).^4+63./256./(1+1./60.*t.^2).^5+231./1024./(1+1./60.*t.^2).^6+429./2048./(1+1./60.*t.^2).^7+6435./32768./(1+1./60.*t.^2).^8+12155./65536./(1+1./60.*t.^2).^9+46189./262144./(1+1./60.*t.^2).^10+583401555./4294967296./(1+1./60.*t.^2).^17+2268783825./17179869184./(1+1./60.*t.^2).^18+4418157975./34359738368./(1+1./60.*t.^2).^19+88179./524288./(1+1./60.*t.^2).^11+676039./4194304./(1+1./60.*t.^2).^12+1300075./8388608./(1+1./60.*t.^2).^13+5014575./33554432./(1+1./60.*t.^2).^14);

        case 61;
            alpha(set_t)=1./2+1./pi.*atan(1./61.*t.*61.^(1./2))+1./pi.*t.*61.^(1./2)./(61+t.^2).*(1+18014398509481984./110873045217057585./(1+1./61.*t.^2).^29+549755813888./2893136075115./(1+1./61.*t.^2).^21+2199023255552./11835556670925./(1+1./61.*t.^2).^22+4398046511104./24185702762325./(1+1./61.*t.^2).^23+70368744177664./395033145117975./(1+1./61.*t.^2).^24+256./693./(1+1./61.*t.^2).^5+1024./3003./(1+1./61.*t.^2).^6+2048./6435./(1+1./61.*t.^2).^7+32768./109395./(1+1./61.*t.^2).^8+65536./230945./(1+1./61.*t.^2).^9+262144./969969./(1+1./61.*t.^2).^10+524288./2028117./(1+1./61.*t.^2).^11+4194304./16900975./(1+1./61.*t.^2).^12+8388608./35102025./(1+1./61.*t.^2).^13+140737488355328./805867616040669./(1+1./61.*t.^2).^25+562949953421312./3285460280781189./(1+1./61.*t.^2).^26+2./3./(1+1./61.*t.^2)+8./15./(1+1./61.*t.^2).^2+33554432./145422675./(1+1./61.*t.^2).^14+67108864./300540195./(1+1./61.*t.^2).^15+2147483648./9917826435./(1+1./61.*t.^2).^16+4294967296./20419054425./(1+1./61.*t.^2).^17+17179869184./83945001525./(1+1./61.*t.^2).^18+34359738368./172308161025./(1+1./61.*t.^2).^19+274877906944./1412926920405./(1+1./61.*t.^2).^20+1125899906842624./6692604275665385./(1+1./61.*t.^2).^27+9007199254740992./54496920530418135./(1+1./61.*t.^2).^28+16./35./(1+1./61.*t.^2).^3+128./315./(1+1./61.*t.^2).^4);

        case 62;
            alpha(set_t)=1./2+1./2.*t./(62+t.^2).^(1./2).*(1+63./256./(1+1./62.*t.^2).^5+231./1024./(1+1./62.*t.^2).^6+429./2048./(1+1./62.*t.^2).^7+6435./32768./(1+1./62.*t.^2).^8+88179./524288./(1+1./62.*t.^2).^11+676039./4194304./(1+1./62.*t.^2).^12+12155./65536./(1+1./62.*t.^2).^9+46189./262144./(1+1./62.*t.^2).^10+1300075./8388608./(1+1./62.*t.^2).^13+5014575./33554432./(1+1./62.*t.^2).^14+9694845./67108864./(1+1./62.*t.^2).^15+300540195./2147483648./(1+1./62.*t.^2).^16+583401555./4294967296./(1+1./62.*t.^2).^17+34461632205./274877906944./(1+1./62.*t.^2).^20+67282234305./549755813888./(1+1./62.*t.^2).^21+2268783825./17179869184./(1+1./62.*t.^2).^18+4418157975./34359738368./(1+1./62.*t.^2).^19+263012370465./2199023255552./(1+1./62.*t.^2).^22+514589420475./4398046511104./(1+1./62.*t.^2).^23+8061900920775./70368744177664./(1+1./62.*t.^2).^24+15801325804719./140737488355328./(1+1./62.*t.^2).^25+956086325095055./9007199254740992./(1+1./62.*t.^2).^28+1879204156221315./18014398509481984./(1+1./62.*t.^2).^29+61989816618513./562949953421312./(1+1./62.*t.^2).^26+121683714103007./1125899906842624./(1+1./62.*t.^2).^27+7391536347803839./72057594037927936./(1+1./62.*t.^2).^30+1./2./(1+1./62.*t.^2)+3./8./(1+1./62.*t.^2).^2+5./16./(1+1./62.*t.^2).^3+35./128./(1+1./62.*t.^2).^4);

        case 63;
            alpha(set_t)=1./2+1./pi.*atan(1./63.*t.*63.^(1./2))+1./pi.*t.*63.^(1./2)./(63+t.^2).*(1+70368744177664./395033145117975./(1+1./63.*t.^2).^24+140737488355328./805867616040669./(1+1./63.*t.^2).^25+2./3./(1+1./63.*t.^2)+8./15./(1+1./63.*t.^2).^2+16./35./(1+1./63.*t.^2).^3+128./315./(1+1./63.*t.^2).^4+562949953421312./3285460280781189./(1+1./63.*t.^2).^26+1125899906842624./6692604275665385./(1+1./63.*t.^2).^27+256./693./(1+1./63.*t.^2).^5+1024./3003./(1+1./63.*t.^2).^6+2048./6435./(1+1./63.*t.^2).^7+32768./109395./(1+1./63.*t.^2).^8+65536./230945./(1+1./63.*t.^2).^9+262144./969969./(1+1./63.*t.^2).^10+524288./2028117./(1+1./63.*t.^2).^11+33554432./145422675./(1+1./63.*t.^2).^14+67108864./300540195./(1+1./63.*t.^2).^15+4194304./16900975./(1+1./63.*t.^2).^12+8388608./35102025./(1+1./63.*t.^2).^13+2147483648./9917826435./(1+1./63.*t.^2).^16+4294967296./20419054425./(1+1./63.*t.^2).^17+17179869184./83945001525./(1+1./63.*t.^2).^18+9007199254740992./54496920530418135./(1+1./63.*t.^2).^28+18014398509481984./110873045217057585./(1+1./63.*t.^2).^29+34359738368./172308161025./(1+1./63.*t.^2).^19+274877906944./1412926920405./(1+1./63.*t.^2).^20+549755813888./2893136075115./(1+1./63.*t.^2).^21+2199023255552./11835556670925./(1+1./63.*t.^2).^22+4398046511104./24185702762325./(1+1./63.*t.^2).^23+72057594037927936./450883717216034179./(1+1./63.*t.^2).^30);

        case 64;
            alpha(set_t)=1./2+1./2.*t./(64+t.^2).^(1./2).*(1+1./2./(1+1./64.*t.^2)+3./8./(1+1./64.*t.^2).^2+5./16./(1+1./64.*t.^2).^3+35./128./(1+1./64.*t.^2).^4+15801325804719./140737488355328./(1+1./64.*t.^2).^25+63./256./(1+1./64.*t.^2).^5+231./1024./(1+1./64.*t.^2).^6+429./2048./(1+1./64.*t.^2).^7+6435./32768./(1+1./64.*t.^2).^8+12155./65536./(1+1./64.*t.^2).^9+46189./262144./(1+1./64.*t.^2).^10+88179./524288./(1+1./64.*t.^2).^11+676039./4194304./(1+1./64.*t.^2).^12+1879204156221315./18014398509481984./(1+1./64.*t.^2).^29+7391536347803839./72057594037927936./(1+1./64.*t.^2).^30+14544636039226909./144115188075855872./(1+1./64.*t.^2).^31+61989816618513./562949953421312./(1+1./64.*t.^2).^26+121683714103007./1125899906842624./(1+1./64.*t.^2).^27+956086325095055./9007199254740992./(1+1./64.*t.^2).^28+1300075./8388608./(1+1./64.*t.^2).^13+5014575./33554432./(1+1./64.*t.^2).^14+9694845./67108864./(1+1./64.*t.^2).^15+300540195./2147483648./(1+1./64.*t.^2).^16+583401555./4294967296./(1+1./64.*t.^2).^17+2268783825./17179869184./(1+1./64.*t.^2).^18+4418157975./34359738368./(1+1./64.*t.^2).^19+34461632205./274877906944./(1+1./64.*t.^2).^20+67282234305./549755813888./(1+1./64.*t.^2).^21+263012370465./2199023255552./(1+1./64.*t.^2).^22+514589420475./4398046511104./(1+1./64.*t.^2).^23+8061900920775./70368744177664./(1+1./64.*t.^2).^24);

        case 65;
            alpha(set_t)=1./2+1./pi.*atan(1./65.*t.*65.^(1./2))+1./pi.*t.*65.^(1./2)./(65+t.^2).*(1+67108864./300540195./(1+1./65.*t.^2).^15+2147483648./9917826435./(1+1./65.*t.^2).^16+4294967296./20419054425./(1+1./65.*t.^2).^17+17179869184./83945001525./(1+1./65.*t.^2).^18+549755813888./2893136075115./(1+1./65.*t.^2).^21+2199023255552./11835556670925./(1+1./65.*t.^2).^22+4398046511104./24185702762325./(1+1./65.*t.^2).^23+34359738368./172308161025./(1+1./65.*t.^2).^19+274877906944./1412926920405./(1+1./65.*t.^2).^20+70368744177664./395033145117975./(1+1./65.*t.^2).^24+140737488355328./805867616040669./(1+1./65.*t.^2).^25+562949953421312./3285460280781189./(1+1./65.*t.^2).^26+1125899906842624./6692604275665385./(1+1./65.*t.^2).^27+9007199254740992./54496920530418135./(1+1./65.*t.^2).^28+18014398509481984./110873045217057585./(1+1./65.*t.^2).^29+72057594037927936./450883717216034179./(1+1./65.*t.^2).^30+2048./6435./(1+1./65.*t.^2).^7+32768./109395./(1+1./65.*t.^2).^8+65536./230945./(1+1./65.*t.^2).^9+262144./969969./(1+1./65.*t.^2).^10+524288./2028117./(1+1./65.*t.^2).^11+4194304./16900975./(1+1./65.*t.^2).^12+144115188075855872./916312070471295267./(1+1./65.*t.^2).^31+8388608./35102025./(1+1./65.*t.^2).^13+33554432./145422675./(1+1./65.*t.^2).^14+2./3./(1+1./65.*t.^2)+8./15./(1+1./65.*t.^2).^2+16./35./(1+1./65.*t.^2).^3+256./693./(1+1./65.*t.^2).^5+1024./3003./(1+1./65.*t.^2).^6+128./315./(1+1./65.*t.^2).^4);

        case 66;
            alpha(set_t)=1./2+1./2.*t./(66+t.^2).^(1./2).*(1+34461632205./274877906944./(1+1./66.*t.^2).^20+67282234305./549755813888./(1+1./66.*t.^2).^21+2268783825./17179869184./(1+1./66.*t.^2).^18+4418157975./34359738368./(1+1./66.*t.^2).^19+429./2048./(1+1./66.*t.^2).^7+6435./32768./(1+1./66.*t.^2).^8+263012370465./2199023255552./(1+1./66.*t.^2).^22+514589420475./4398046511104./(1+1./66.*t.^2).^23+8061900920775./70368744177664./(1+1./66.*t.^2).^24+15801325804719./140737488355328./(1+1./66.*t.^2).^25+61989816618513./562949953421312./(1+1./66.*t.^2).^26+5014575./33554432./(1+1./66.*t.^2).^14+9694845./67108864./(1+1./66.*t.^2).^15+12155./65536./(1+1./66.*t.^2).^9+46189./262144./(1+1./66.*t.^2).^10+88179./524288./(1+1./66.*t.^2).^11+676039./4194304./(1+1./66.*t.^2).^12+1300075./8388608./(1+1./66.*t.^2).^13+300540195./2147483648./(1+1./66.*t.^2).^16+583401555./4294967296./(1+1./66.*t.^2).^17+121683714103007./1125899906842624./(1+1./66.*t.^2).^27+956086325095055./9007199254740992./(1+1./66.*t.^2).^28+1./2./(1+1./66.*t.^2)+1879204156221315./18014398509481984./(1+1./66.*t.^2).^29+7391536347803839./72057594037927936./(1+1./66.*t.^2).^30+14544636039226909./144115188075855872./(1+1./66.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./66.*t.^2).^32+3./8./(1+1./66.*t.^2).^2+5./16./(1+1./66.*t.^2).^3+35./128./(1+1./66.*t.^2).^4+63./256./(1+1./66.*t.^2).^5+231./1024./(1+1./66.*t.^2).^6);

        case 67;
            alpha(set_t)=1./2+1./pi.*atan(1./67.*t.*67.^(1./2))+1./pi.*t.*67.^(1./2)./(67+t.^2).*(1+256./693./(1+1./67.*t.^2).^5+1024./3003./(1+1./67.*t.^2).^6+4294967296./20419054425./(1+1./67.*t.^2).^17+17179869184./83945001525./(1+1./67.*t.^2).^18+34359738368./172308161025./(1+1./67.*t.^2).^19+2048./6435./(1+1./67.*t.^2).^7+32768./109395./(1+1./67.*t.^2).^8+65536./230945./(1+1./67.*t.^2).^9+274877906944./1412926920405./(1+1./67.*t.^2).^20+549755813888./2893136075115./(1+1./67.*t.^2).^21+262144./969969./(1+1./67.*t.^2).^10+524288./2028117./(1+1./67.*t.^2).^11+4194304./16900975./(1+1./67.*t.^2).^12+8388608./35102025./(1+1./67.*t.^2).^13+33554432./145422675./(1+1./67.*t.^2).^14+140737488355328./805867616040669./(1+1./67.*t.^2).^25+562949953421312./3285460280781189./(1+1./67.*t.^2).^26+2199023255552./11835556670925./(1+1./67.*t.^2).^22+4398046511104./24185702762325./(1+1./67.*t.^2).^23+70368744177664./395033145117975./(1+1./67.*t.^2).^24+1125899906842624./6692604275665385./(1+1./67.*t.^2).^27+9007199254740992./54496920530418135./(1+1./67.*t.^2).^28+18014398509481984./110873045217057585./(1+1./67.*t.^2).^29+2./3./(1+1./67.*t.^2)+8./15./(1+1./67.*t.^2).^2+16./35./(1+1./67.*t.^2).^3+128./315./(1+1./67.*t.^2).^4+72057594037927936./450883717216034179./(1+1./67.*t.^2).^30+144115188075855872./916312070471295267./(1+1./67.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./67.*t.^2).^32+67108864./300540195./(1+1./67.*t.^2).^15+2147483648./9917826435./(1+1./67.*t.^2).^16);

        case 68;
            alpha(set_t)=1./2+1./2.*t./(68+t.^2).^(1./2).*(1+1./2./(1+1./68.*t.^2)+3./8./(1+1./68.*t.^2).^2+263012370465./2199023255552./(1+1./68.*t.^2).^22+514589420475./4398046511104./(1+1./68.*t.^2).^23+5./16./(1+1./68.*t.^2).^3+35./128./(1+1./68.*t.^2).^4+63./256./(1+1./68.*t.^2).^5+956086325095055./9007199254740992./(1+1./68.*t.^2).^28+1879204156221315./18014398509481984./(1+1./68.*t.^2).^29+7391536347803839./72057594037927936./(1+1./68.*t.^2).^30+1804857108504066435./18446744073709551616./(1+1./68.*t.^2).^33+231./1024./(1+1./68.*t.^2).^6+429./2048./(1+1./68.*t.^2).^7+6435./32768./(1+1./68.*t.^2).^8+12155./65536./(1+1./68.*t.^2).^9+46189./262144./(1+1./68.*t.^2).^10+88179./524288./(1+1./68.*t.^2).^11+676039./4194304./(1+1./68.*t.^2).^12+1300075./8388608./(1+1./68.*t.^2).^13+8061900920775./70368744177664./(1+1./68.*t.^2).^24+15801325804719./140737488355328./(1+1./68.*t.^2).^25+61989816618513./562949953421312./(1+1./68.*t.^2).^26+121683714103007./1125899906842624./(1+1./68.*t.^2).^27+5014575./33554432./(1+1./68.*t.^2).^14+9694845./67108864./(1+1./68.*t.^2).^15+300540195./2147483648./(1+1./68.*t.^2).^16+583401555./4294967296./(1+1./68.*t.^2).^17+2268783825./17179869184./(1+1./68.*t.^2).^18+14544636039226909./144115188075855872./(1+1./68.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./68.*t.^2).^32+4418157975./34359738368./(1+1./68.*t.^2).^19+34461632205./274877906944./(1+1./68.*t.^2).^20+67282234305./549755813888./(1+1./68.*t.^2).^21);

        case 69;
            alpha(set_t)=1./2+1./pi.*atan(1./69.*t.*69.^(1./2))+1./pi.*t.*69.^(1./2)./(69+t.^2).*(1+32768./109395./(1+1./69.*t.^2).^8+65536./230945./(1+1./69.*t.^2).^9+262144./969969./(1+1./69.*t.^2).^10+524288./2028117./(1+1./69.*t.^2).^11+4194304./16900975./(1+1./69.*t.^2).^12+8388608./35102025./(1+1./69.*t.^2).^13+9223372036854775808./59560284580634192355./(1+1./69.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./69.*t.^2).^33+33554432./145422675./(1+1./69.*t.^2).^14+67108864./300540195./(1+1./69.*t.^2).^15+2147483648./9917826435./(1+1./69.*t.^2).^16+4294967296./20419054425./(1+1./69.*t.^2).^17+17179869184./83945001525./(1+1./69.*t.^2).^18+34359738368./172308161025./(1+1./69.*t.^2).^19+274877906944./1412926920405./(1+1./69.*t.^2).^20+549755813888./2893136075115./(1+1./69.*t.^2).^21+2199023255552./11835556670925./(1+1./69.*t.^2).^22+2./3./(1+1./69.*t.^2)+8./15./(1+1./69.*t.^2).^2+16./35./(1+1./69.*t.^2).^3+4398046511104./24185702762325./(1+1./69.*t.^2).^23+70368744177664./395033145117975./(1+1./69.*t.^2).^24+140737488355328./805867616040669./(1+1./69.*t.^2).^25+128./315./(1+1./69.*t.^2).^4+256./693./(1+1./69.*t.^2).^5+1125899906842624./6692604275665385./(1+1./69.*t.^2).^27+9007199254740992./54496920530418135./(1+1./69.*t.^2).^28+562949953421312./3285460280781189./(1+1./69.*t.^2).^26+1024./3003./(1+1./69.*t.^2).^6+2048./6435./(1+1./69.*t.^2).^7+18014398509481984./110873045217057585./(1+1./69.*t.^2).^29+72057594037927936./450883717216034179./(1+1./69.*t.^2).^30+144115188075855872./916312070471295267./(1+1./69.*t.^2).^31);

        case 70;
            alpha(set_t)=1./2+1./2.*t./(70+t.^2).^(1./2).*(1+12155./65536./(1+1./70.*t.^2).^9+46189./262144./(1+1./70.*t.^2).^10+5014575./33554432./(1+1./70.*t.^2).^14+9694845./67108864./(1+1./70.*t.^2).^15+300540195./2147483648./(1+1./70.*t.^2).^16+583401555./4294967296./(1+1./70.*t.^2).^17+514589420475./4398046511104./(1+1./70.*t.^2).^23+8061900920775./70368744177664./(1+1./70.*t.^2).^24+15801325804719./140737488355328./(1+1./70.*t.^2).^25+61989816618513./562949953421312./(1+1./70.*t.^2).^26+121683714103007./1125899906842624./(1+1./70.*t.^2).^27+1./2./(1+1./70.*t.^2)+14544636039226909./144115188075855872./(1+1./70.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./70.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./70.*t.^2).^33+3./8./(1+1./70.*t.^2).^2+5./16./(1+1./70.*t.^2).^3+7113260368810144185./73786976294838206464./(1+1./70.*t.^2).^34+2268783825./17179869184./(1+1./70.*t.^2).^18+4418157975./34359738368./(1+1./70.*t.^2).^19+34461632205./274877906944./(1+1./70.*t.^2).^20+67282234305./549755813888./(1+1./70.*t.^2).^21+263012370465./2199023255552./(1+1./70.*t.^2).^22+35./128./(1+1./70.*t.^2).^4+63./256./(1+1./70.*t.^2).^5+956086325095055./9007199254740992./(1+1./70.*t.^2).^28+1879204156221315./18014398509481984./(1+1./70.*t.^2).^29+7391536347803839./72057594037927936./(1+1./70.*t.^2).^30+88179./524288./(1+1./70.*t.^2).^11+676039./4194304./(1+1./70.*t.^2).^12+1300075./8388608./(1+1./70.*t.^2).^13+429./2048./(1+1./70.*t.^2).^7+6435./32768./(1+1./70.*t.^2).^8+231./1024./(1+1./70.*t.^2).^6);

        case 71;
            alpha(set_t)=1./2+1./pi.*atan(1./71.*t.*71.^(1./2))+1./pi.*t.*71.^(1./2)./(71+t.^2).*(1+67108864./300540195./(1+1./71.*t.^2).^15+2147483648./9917826435./(1+1./71.*t.^2).^16+8388608./35102025./(1+1./71.*t.^2).^13+33554432./145422675./(1+1./71.*t.^2).^14+4294967296./20419054425./(1+1./71.*t.^2).^17+17179869184./83945001525./(1+1./71.*t.^2).^18+34359738368./172308161025./(1+1./71.*t.^2).^19+18446744073709551616./120925426269772451145./(1+1./71.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./71.*t.^2).^34+2./3./(1+1./71.*t.^2)+8./15./(1+1./71.*t.^2).^2+16./35./(1+1./71.*t.^2).^3+128./315./(1+1./71.*t.^2).^4+256./693./(1+1./71.*t.^2).^5+1024./3003./(1+1./71.*t.^2).^6+2048./6435./(1+1./71.*t.^2).^7+274877906944./1412926920405./(1+1./71.*t.^2).^20+549755813888./2893136075115./(1+1./71.*t.^2).^21+2199023255552./11835556670925./(1+1./71.*t.^2).^22+32768./109395./(1+1./71.*t.^2).^8+65536./230945./(1+1./71.*t.^2).^9+262144./969969./(1+1./71.*t.^2).^10+562949953421312./3285460280781189./(1+1./71.*t.^2).^26+1125899906842624./6692604275665385./(1+1./71.*t.^2).^27+9007199254740992./54496920530418135./(1+1./71.*t.^2).^28+18014398509481984./110873045217057585./(1+1./71.*t.^2).^29+524288./2028117./(1+1./71.*t.^2).^11+4194304./16900975./(1+1./71.*t.^2).^12+72057594037927936./450883717216034179./(1+1./71.*t.^2).^30+144115188075855872./916312070471295267./(1+1./71.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./71.*t.^2).^32+4398046511104./24185702762325./(1+1./71.*t.^2).^23+70368744177664./395033145117975./(1+1./71.*t.^2).^24+140737488355328./805867616040669./(1+1./71.*t.^2).^25);

        case 72;
            alpha(set_t)=1./2+1./2.*t./(72+t.^2).^(1./2).*(1+121683714103007./1125899906842624./(1+1./72.*t.^2).^27+956086325095055./9007199254740992./(1+1./72.*t.^2).^28+1879204156221315./18014398509481984./(1+1./72.*t.^2).^29+15801325804719./140737488355328./(1+1./72.*t.^2).^25+61989816618513./562949953421312./(1+1./72.*t.^2).^26+1./2./(1+1./72.*t.^2)+3./8./(1+1./72.*t.^2).^2+5./16./(1+1./72.*t.^2).^3+7391536347803839./72057594037927936./(1+1./72.*t.^2).^30+14544636039226909./144115188075855872./(1+1./72.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./72.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./72.*t.^2).^33+35./128./(1+1./72.*t.^2).^4+63./256./(1+1./72.*t.^2).^5+231./1024./(1+1./72.*t.^2).^6+429./2048./(1+1./72.*t.^2).^7+6435./32768./(1+1./72.*t.^2).^8+12155./65536./(1+1./72.*t.^2).^9+7113260368810144185./73786976294838206464./(1+1./72.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./72.*t.^2).^35+46189./262144./(1+1./72.*t.^2).^10+88179./524288./(1+1./72.*t.^2).^11+676039./4194304./(1+1./72.*t.^2).^12+263012370465./2199023255552./(1+1./72.*t.^2).^22+514589420475./4398046511104./(1+1./72.*t.^2).^23+8061900920775./70368744177664./(1+1./72.*t.^2).^24+300540195./2147483648./(1+1./72.*t.^2).^16+583401555./4294967296./(1+1./72.*t.^2).^17+1300075./8388608./(1+1./72.*t.^2).^13+5014575./33554432./(1+1./72.*t.^2).^14+9694845./67108864./(1+1./72.*t.^2).^15+2268783825./17179869184./(1+1./72.*t.^2).^18+4418157975./34359738368./(1+1./72.*t.^2).^19+34461632205./274877906944./(1+1./72.*t.^2).^20+67282234305./549755813888./(1+1./72.*t.^2).^21);

        case 73;
            alpha(set_t)=1./2+1./pi.*atan(1./73.*t.*73.^(1./2))+1./pi.*t.*73.^(1./2)./(73+t.^2).*(1+2./3./(1+1./73.*t.^2)+128./315./(1+1./73.*t.^2).^4+256./693./(1+1./73.*t.^2).^5+1024./3003./(1+1./73.*t.^2).^6+8./15./(1+1./73.*t.^2).^2+16./35./(1+1./73.*t.^2).^3+2048./6435./(1+1./73.*t.^2).^7+32768./109395./(1+1./73.*t.^2).^8+65536./230945./(1+1./73.*t.^2).^9+262144./969969./(1+1./73.*t.^2).^10+524288./2028117./(1+1./73.*t.^2).^11+4194304./16900975./(1+1./73.*t.^2).^12+73786976294838206464./490814965447899948765./(1+1./73.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./73.*t.^2).^35+8388608./35102025./(1+1./73.*t.^2).^13+33554432./145422675./(1+1./73.*t.^2).^14+67108864./300540195./(1+1./73.*t.^2).^15+2147483648./9917826435./(1+1./73.*t.^2).^16+4294967296./20419054425./(1+1./73.*t.^2).^17+17179869184./83945001525./(1+1./73.*t.^2).^18+34359738368./172308161025./(1+1./73.*t.^2).^19+274877906944./1412926920405./(1+1./73.*t.^2).^20+549755813888./2893136075115./(1+1./73.*t.^2).^21+2199023255552./11835556670925./(1+1./73.*t.^2).^22+4398046511104./24185702762325./(1+1./73.*t.^2).^23+70368744177664./395033145117975./(1+1./73.*t.^2).^24+140737488355328./805867616040669./(1+1./73.*t.^2).^25+562949953421312./3285460280781189./(1+1./73.*t.^2).^26+1125899906842624./6692604275665385./(1+1./73.*t.^2).^27+9007199254740992./54496920530418135./(1+1./73.*t.^2).^28+18014398509481984./110873045217057585./(1+1./73.*t.^2).^29+72057594037927936./450883717216034179./(1+1./73.*t.^2).^30+144115188075855872./916312070471295267./(1+1./73.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./73.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./73.*t.^2).^33);

        case 74;
            alpha(set_t)=1./2+1./2.*t./(74+t.^2).^(1./2).*(1+429./2048./(1+1./74.*t.^2).^7+6435./32768./(1+1./74.*t.^2).^8+12155./65536./(1+1./74.*t.^2).^9+46189./262144./(1+1./74.*t.^2).^10+88179./524288./(1+1./74.*t.^2).^11+676039./4194304./(1+1./74.*t.^2).^12+514589420475./4398046511104./(1+1./74.*t.^2).^23+8061900920775./70368744177664./(1+1./74.*t.^2).^24+15801325804719./140737488355328./(1+1./74.*t.^2).^25+1300075./8388608./(1+1./74.*t.^2).^13+5014575./33554432./(1+1./74.*t.^2).^14+9694845./67108864./(1+1./74.*t.^2).^15+300540195./2147483648./(1+1./74.*t.^2).^16+110628135069209194801./1180591620717411303424./(1+1./74.*t.^2).^36+1./2./(1+1./74.*t.^2)+3./8./(1+1./74.*t.^2).^2+7391536347803839./72057594037927936./(1+1./74.*t.^2).^30+14544636039226909./144115188075855872./(1+1./74.*t.^2).^31+61989816618513./562949953421312./(1+1./74.*t.^2).^26+121683714103007./1125899906842624./(1+1./74.*t.^2).^27+956086325095055./9007199254740992./(1+1./74.*t.^2).^28+1879204156221315./18014398509481984./(1+1./74.*t.^2).^29+916312070471295267./9223372036854775808./(1+1./74.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./74.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./74.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./74.*t.^2).^35+34461632205./274877906944./(1+1./74.*t.^2).^20+67282234305./549755813888./(1+1./74.*t.^2).^21+263012370465./2199023255552./(1+1./74.*t.^2).^22+5./16./(1+1./74.*t.^2).^3+583401555./4294967296./(1+1./74.*t.^2).^17+2268783825./17179869184./(1+1./74.*t.^2).^18+4418157975./34359738368./(1+1./74.*t.^2).^19+35./128./(1+1./74.*t.^2).^4+63./256./(1+1./74.*t.^2).^5+231./1024./(1+1./74.*t.^2).^6);

        case 75;
            alpha(set_t)=1./2+1./pi.*atan(1./75.*t.*75.^(1./2))+1./pi.*t.*75.^(1./2)./(75+t.^2).*(1+549755813888./2893136075115./(1+1./75.*t.^2).^21+2199023255552./11835556670925./(1+1./75.*t.^2).^22+4398046511104./24185702762325./(1+1./75.*t.^2).^23+70368744177664./395033145117975./(1+1./75.*t.^2).^24+32768./109395./(1+1./75.*t.^2).^8+65536./230945./(1+1./75.*t.^2).^9+262144./969969./(1+1./75.*t.^2).^10+524288./2028117./(1+1./75.*t.^2).^11+2147483648./9917826435./(1+1./75.*t.^2).^16+4294967296./20419054425./(1+1./75.*t.^2).^17+17179869184./83945001525./(1+1./75.*t.^2).^18+34359738368./172308161025./(1+1./75.*t.^2).^19+274877906944./1412926920405./(1+1./75.*t.^2).^20+2./3./(1+1./75.*t.^2)+8./15./(1+1./75.*t.^2).^2+16./35./(1+1./75.*t.^2).^3+128./315./(1+1./75.*t.^2).^4+72057594037927936./450883717216034179./(1+1./75.*t.^2).^30+144115188075855872./916312070471295267./(1+1./75.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./75.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./75.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./75.*t.^2).^34+256./693./(1+1./75.*t.^2).^5+1024./3003./(1+1./75.*t.^2).^6+2048./6435./(1+1./75.*t.^2).^7+4194304./16900975./(1+1./75.*t.^2).^12+8388608./35102025./(1+1./75.*t.^2).^13+33554432./145422675./(1+1./75.*t.^2).^14+67108864./300540195./(1+1./75.*t.^2).^15+147573952589676412928./995653215622882753209./(1+1./75.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./75.*t.^2).^36+140737488355328./805867616040669./(1+1./75.*t.^2).^25+562949953421312./3285460280781189./(1+1./75.*t.^2).^26+1125899906842624./6692604275665385./(1+1./75.*t.^2).^27+9007199254740992./54496920530418135./(1+1./75.*t.^2).^28+18014398509481984./110873045217057585./(1+1./75.*t.^2).^29);

        case 76;
            alpha(set_t)=1./2+1./2.*t./(76+t.^2).^(1./2).*(1+514589420475./4398046511104./(1+1./76.*t.^2).^23+8061900920775./70368744177664./(1+1./76.*t.^2).^24+15801325804719./140737488355328./(1+1./76.*t.^2).^25+1./2./(1+1./76.*t.^2)+3./8./(1+1./76.*t.^2).^2+5./16./(1+1./76.*t.^2).^3+14544636039226909./144115188075855872./(1+1./76.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./76.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./76.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./76.*t.^2).^34+35./128./(1+1./76.*t.^2).^4+63./256./(1+1./76.*t.^2).^5+231./1024./(1+1./76.*t.^2).^6+429./2048./(1+1./76.*t.^2).^7+6435./32768./(1+1./76.*t.^2).^8+61989816618513./562949953421312./(1+1./76.*t.^2).^26+121683714103007./1125899906842624./(1+1./76.*t.^2).^27+956086325095055./9007199254740992./(1+1./76.*t.^2).^28+1879204156221315./18014398509481984./(1+1./76.*t.^2).^29+7391536347803839./72057594037927936./(1+1./76.*t.^2).^30+14023284727082855679./147573952589676412928./(1+1./76.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./76.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./76.*t.^2).^37+12155./65536./(1+1./76.*t.^2).^9+46189./262144./(1+1./76.*t.^2).^10+88179./524288./(1+1./76.*t.^2).^11+676039./4194304./(1+1./76.*t.^2).^12+1300075./8388608./(1+1./76.*t.^2).^13+5014575./33554432./(1+1./76.*t.^2).^14+9694845./67108864./(1+1./76.*t.^2).^15+300540195./2147483648./(1+1./76.*t.^2).^16+583401555./4294967296./(1+1./76.*t.^2).^17+2268783825./17179869184./(1+1./76.*t.^2).^18+4418157975./34359738368./(1+1./76.*t.^2).^19+34461632205./274877906944./(1+1./76.*t.^2).^20+67282234305./549755813888./(1+1./76.*t.^2).^21+263012370465./2199023255552./(1+1./76.*t.^2).^22);

        case 77;
            alpha(set_t)=1./2+1./pi.*atan(1./77.*t.*77.^(1./2))+1./pi.*t.*77.^(1./2)./(77+t.^2).*(1+140737488355328./805867616040669./(1+1./77.*t.^2).^25+562949953421312./3285460280781189./(1+1./77.*t.^2).^26+1125899906842624./6692604275665385./(1+1./77.*t.^2).^27+9007199254740992./54496920530418135./(1+1./77.*t.^2).^28+18014398509481984./110873045217057585./(1+1./77.*t.^2).^29+33554432./145422675./(1+1./77.*t.^2).^14+67108864./300540195./(1+1./77.*t.^2).^15+2147483648./9917826435./(1+1./77.*t.^2).^16+4294967296./20419054425./(1+1./77.*t.^2).^17+17179869184./83945001525./(1+1./77.*t.^2).^18+16./35./(1+1./77.*t.^2).^3+128./315./(1+1./77.*t.^2).^4+256./693./(1+1./77.*t.^2).^5+1024./3003./(1+1./77.*t.^2).^6+2048./6435./(1+1./77.*t.^2).^7+32768./109395./(1+1./77.*t.^2).^8+65536./230945./(1+1./77.*t.^2).^9+262144./969969./(1+1./77.*t.^2).^10+524288./2028117./(1+1./77.*t.^2).^11+4194304./16900975./(1+1./77.*t.^2).^12+2./3./(1+1./77.*t.^2)+8./15./(1+1./77.*t.^2).^2+1180591620717411303424./8075853860052271220473./(1+1./77.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./77.*t.^2).^37+34359738368./172308161025./(1+1./77.*t.^2).^19+72057594037927936./450883717216034179./(1+1./77.*t.^2).^30+144115188075855872./916312070471295267./(1+1./77.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./77.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./77.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./77.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./77.*t.^2).^35+274877906944./1412926920405./(1+1./77.*t.^2).^20+549755813888./2893136075115./(1+1./77.*t.^2).^21+2199023255552./11835556670925./(1+1./77.*t.^2).^22+4398046511104./24185702762325./(1+1./77.*t.^2).^23+70368744177664./395033145117975./(1+1./77.*t.^2).^24+8388608./35102025./(1+1./77.*t.^2).^13);

        case 78;
            alpha(set_t)=1./2+1./2.*t./(78+t.^2).^(1./2).*(1+15801325804719./140737488355328./(1+1./78.*t.^2).^25+5./16./(1+1./78.*t.^2).^3+35./128./(1+1./78.*t.^2).^4+63./256./(1+1./78.*t.^2).^5+231./1024./(1+1./78.*t.^2).^6+429./2048./(1+1./78.*t.^2).^7+218266320541953276229./2361183241434822606848./(1+1./78.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./78.*t.^2).^38+61989816618513./562949953421312./(1+1./78.*t.^2).^26+121683714103007./1125899906842624./(1+1./78.*t.^2).^27+956086325095055./9007199254740992./(1+1./78.*t.^2).^28+1879204156221315./18014398509481984./(1+1./78.*t.^2).^29+7391536347803839./72057594037927936./(1+1./78.*t.^2).^30+1300075./8388608./(1+1./78.*t.^2).^13+4418157975./34359738368./(1+1./78.*t.^2).^19+14544636039226909./144115188075855872./(1+1./78.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./78.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./78.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./78.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./78.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./78.*t.^2).^36+34461632205./274877906944./(1+1./78.*t.^2).^20+67282234305./549755813888./(1+1./78.*t.^2).^21+263012370465./2199023255552./(1+1./78.*t.^2).^22+514589420475./4398046511104./(1+1./78.*t.^2).^23+8061900920775./70368744177664./(1+1./78.*t.^2).^24+5014575./33554432./(1+1./78.*t.^2).^14+9694845./67108864./(1+1./78.*t.^2).^15+300540195./2147483648./(1+1./78.*t.^2).^16+583401555./4294967296./(1+1./78.*t.^2).^17+2268783825./17179869184./(1+1./78.*t.^2).^18+1./2./(1+1./78.*t.^2)+3./8./(1+1./78.*t.^2).^2+6435./32768./(1+1./78.*t.^2).^8+12155./65536./(1+1./78.*t.^2).^9+46189./262144./(1+1./78.*t.^2).^10+88179./524288./(1+1./78.*t.^2).^11+676039./4194304./(1+1./78.*t.^2).^12);

        case 79;
            alpha(set_t)=1./2+1./pi.*atan(1./79.*t.*79.^(1./2))+1./pi.*t.*79.^(1./2)./(79+t.^2).*(1+274877906944./1412926920405./(1+1./79.*t.^2).^20+549755813888./2893136075115./(1+1./79.*t.^2).^21+2199023255552./11835556670925./(1+1./79.*t.^2).^22+65536./230945./(1+1./79.*t.^2).^9+262144./969969./(1+1./79.*t.^2).^10+524288./2028117./(1+1./79.*t.^2).^11+4194304./16900975./(1+1./79.*t.^2).^12+8388608./35102025./(1+1./79.*t.^2).^13+2./3./(1+1./79.*t.^2)+9444732965739290427392./66341473743672640538025./(1+1./79.*t.^2).^38+128./315./(1+1./79.*t.^2).^4+256./693./(1+1./79.*t.^2).^5+1024./3003./(1+1./79.*t.^2).^6+2048./6435./(1+1./79.*t.^2).^7+32768./109395./(1+1./79.*t.^2).^8+33554432./145422675./(1+1./79.*t.^2).^14+67108864./300540195./(1+1./79.*t.^2).^15+2147483648./9917826435./(1+1./79.*t.^2).^16+4294967296./20419054425./(1+1./79.*t.^2).^17+17179869184./83945001525./(1+1./79.*t.^2).^18+34359738368./172308161025./(1+1./79.*t.^2).^19+8./15./(1+1./79.*t.^2).^2+16./35./(1+1./79.*t.^2).^3+9007199254740992./54496920530418135./(1+1./79.*t.^2).^28+18014398509481984./110873045217057585./(1+1./79.*t.^2).^29+72057594037927936./450883717216034179./(1+1./79.*t.^2).^30+144115188075855872./916312070471295267./(1+1./79.*t.^2).^31+4398046511104./24185702762325./(1+1./79.*t.^2).^23+70368744177664./395033145117975./(1+1./79.*t.^2).^24+140737488355328./805867616040669./(1+1./79.*t.^2).^25+562949953421312./3285460280781189./(1+1./79.*t.^2).^26+1125899906842624./6692604275665385./(1+1./79.*t.^2).^27+9223372036854775808./59560284580634192355./(1+1./79.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./79.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./79.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./79.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./79.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./79.*t.^2).^37);

        case 80;
            alpha(set_t)=1./2+1./2.*t./(80+t.^2).^(1./2).*(1+88179./524288./(1+1./80.*t.^2).^11+676039./4194304./(1+1./80.*t.^2).^12+1300075./8388608./(1+1./80.*t.^2).^13+5014575./33554432./(1+1./80.*t.^2).^14+9694845./67108864./(1+1./80.*t.^2).^15+300540195./2147483648./(1+1./80.*t.^2).^16+583401555./4294967296./(1+1./80.*t.^2).^17+231./1024./(1+1./80.*t.^2).^6+429./2048./(1+1./80.*t.^2).^7+6435./32768./(1+1./80.*t.^2).^8+12155./65536./(1+1./80.*t.^2).^9+46189./262144./(1+1./80.*t.^2).^10+514589420475./4398046511104./(1+1./80.*t.^2).^23+8061900920775./70368744177664./(1+1./80.*t.^2).^24+15801325804719./140737488355328./(1+1./80.*t.^2).^25+61989816618513./562949953421312./(1+1./80.*t.^2).^26+121683714103007./1125899906842624./(1+1./80.*t.^2).^27+1804857108504066435./18446744073709551616./(1+1./80.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./80.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./80.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./80.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./80.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./80.*t.^2).^38+1./2./(1+1./80.*t.^2)+3./8./(1+1./80.*t.^2).^2+5./16./(1+1./80.*t.^2).^3+35./128./(1+1./80.*t.^2).^4+63./256./(1+1./80.*t.^2).^5+1701063429324939500975./18889465931478580854784./(1+1./80.*t.^2).^39+956086325095055./9007199254740992./(1+1./80.*t.^2).^28+1879204156221315./18014398509481984./(1+1./80.*t.^2).^29+7391536347803839./72057594037927936./(1+1./80.*t.^2).^30+14544636039226909./144115188075855872./(1+1./80.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./80.*t.^2).^32+2268783825./17179869184./(1+1./80.*t.^2).^18+4418157975./34359738368./(1+1./80.*t.^2).^19+34461632205./274877906944./(1+1./80.*t.^2).^20+67282234305./549755813888./(1+1./80.*t.^2).^21+263012370465./2199023255552./(1+1./80.*t.^2).^22);

        case 81;
            alpha(set_t)=1./2+1./pi.*atan(1./81.*t.*81.^(1./2))+1./pi.*t.*81.^(1./2)./(81+t.^2).*(1+256./693./(1+1./81.*t.^2).^5+1024./3003./(1+1./81.*t.^2).^6+2048./6435./(1+1./81.*t.^2).^7+32768./109395./(1+1./81.*t.^2).^8+65536./230945./(1+1./81.*t.^2).^9+262144./969969./(1+1./81.*t.^2).^10+524288./2028117./(1+1./81.*t.^2).^11+4194304./16900975./(1+1./81.*t.^2).^12+8388608./35102025./(1+1./81.*t.^2).^13+33554432./145422675./(1+1./81.*t.^2).^14+67108864./300540195./(1+1./81.*t.^2).^15+562949953421312./3285460280781189./(1+1./81.*t.^2).^26+1125899906842624./6692604275665385./(1+1./81.*t.^2).^27+9007199254740992./54496920530418135./(1+1./81.*t.^2).^28+18014398509481984./110873045217057585./(1+1./81.*t.^2).^29+72057594037927936./450883717216034179./(1+1./81.*t.^2).^30+144115188075855872./916312070471295267./(1+1./81.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./81.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./81.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./81.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./81.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./81.*t.^2).^36+2./3./(1+1./81.*t.^2)+8./15./(1+1./81.*t.^2).^2+16./35./(1+1./81.*t.^2).^3+128./315./(1+1./81.*t.^2).^4+2147483648./9917826435./(1+1./81.*t.^2).^16+4294967296./20419054425./(1+1./81.*t.^2).^17+17179869184./83945001525./(1+1./81.*t.^2).^18+34359738368./172308161025./(1+1./81.*t.^2).^19+274877906944./1412926920405./(1+1./81.*t.^2).^20+2361183241434822606848./16369974040646495717175./(1+1./81.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./81.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./81.*t.^2).^39+549755813888./2893136075115./(1+1./81.*t.^2).^21+2199023255552./11835556670925./(1+1./81.*t.^2).^22+4398046511104./24185702762325./(1+1./81.*t.^2).^23+70368744177664./395033145117975./(1+1./81.*t.^2).^24+140737488355328./805867616040669./(1+1./81.*t.^2).^25);

        case 82;
            alpha(set_t)=1./2+1./2.*t./(82+t.^2).^(1./2).*(1+12155./65536./(1+1./82.*t.^2).^9+46189./262144./(1+1./82.*t.^2).^10+88179./524288./(1+1./82.*t.^2).^11+676039./4194304./(1+1./82.*t.^2).^12+1300075./8388608./(1+1./82.*t.^2).^13+6435./32768./(1+1./82.*t.^2).^8+110628135069209194801./1180591620717411303424./(1+1./82.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./82.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./82.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./82.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./82.*t.^2).^40+7391536347803839./72057594037927936./(1+1./82.*t.^2).^30+14544636039226909./144115188075855872./(1+1./82.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./82.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./82.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./82.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./82.*t.^2).^35+5014575./33554432./(1+1./82.*t.^2).^14+9694845./67108864./(1+1./82.*t.^2).^15+300540195./2147483648./(1+1./82.*t.^2).^16+583401555./4294967296./(1+1./82.*t.^2).^17+2268783825./17179869184./(1+1./82.*t.^2).^18+4418157975./34359738368./(1+1./82.*t.^2).^19+34461632205./274877906944./(1+1./82.*t.^2).^20+67282234305./549755813888./(1+1./82.*t.^2).^21+263012370465./2199023255552./(1+1./82.*t.^2).^22+514589420475./4398046511104./(1+1./82.*t.^2).^23+8061900920775./70368744177664./(1+1./82.*t.^2).^24+15801325804719./140737488355328./(1+1./82.*t.^2).^25+61989816618513./562949953421312./(1+1./82.*t.^2).^26+121683714103007./1125899906842624./(1+1./82.*t.^2).^27+956086325095055./9007199254740992./(1+1./82.*t.^2).^28+1879204156221315./18014398509481984./(1+1./82.*t.^2).^29+1./2./(1+1./82.*t.^2)+3./8./(1+1./82.*t.^2).^2+5./16./(1+1./82.*t.^2).^3+35./128./(1+1./82.*t.^2).^4+63./256./(1+1./82.*t.^2).^5+231./1024./(1+1./82.*t.^2).^6+429./2048./(1+1./82.*t.^2).^7);

        case 83;
            alpha(set_t)=1./2+1./pi.*atan(1./83.*t.*83.^(1./2))+1./pi.*t.*83.^(1./2)./(83+t.^2).*(1+2./3./(1+1./83.*t.^2)+2199023255552./11835556670925./(1+1./83.*t.^2).^22+4398046511104./24185702762325./(1+1./83.*t.^2).^23+70368744177664./395033145117975./(1+1./83.*t.^2).^24+140737488355328./805867616040669./(1+1./83.*t.^2).^25+562949953421312./3285460280781189./(1+1./83.*t.^2).^26+4294967296./20419054425./(1+1./83.*t.^2).^17+17179869184./83945001525./(1+1./83.*t.^2).^18+34359738368./172308161025./(1+1./83.*t.^2).^19+274877906944./1412926920405./(1+1./83.*t.^2).^20+549755813888./2893136075115./(1+1./83.*t.^2).^21+9444732965739290427392./66341473743672640538025./(1+1./83.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./83.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./83.*t.^2).^40+1125899906842624./6692604275665385./(1+1./83.*t.^2).^27+9007199254740992./54496920530418135./(1+1./83.*t.^2).^28+18014398509481984./110873045217057585./(1+1./83.*t.^2).^29+72057594037927936./450883717216034179./(1+1./83.*t.^2).^30+144115188075855872./916312070471295267./(1+1./83.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./83.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./83.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./83.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./83.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./83.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./83.*t.^2).^37+8./15./(1+1./83.*t.^2).^2+16./35./(1+1./83.*t.^2).^3+128./315./(1+1./83.*t.^2).^4+256./693./(1+1./83.*t.^2).^5+1024./3003./(1+1./83.*t.^2).^6+2048./6435./(1+1./83.*t.^2).^7+32768./109395./(1+1./83.*t.^2).^8+65536./230945./(1+1./83.*t.^2).^9+262144./969969./(1+1./83.*t.^2).^10+524288./2028117./(1+1./83.*t.^2).^11+4194304./16900975./(1+1./83.*t.^2).^12+8388608./35102025./(1+1./83.*t.^2).^13+33554432./145422675./(1+1./83.*t.^2).^14+67108864./300540195./(1+1./83.*t.^2).^15+2147483648./9917826435./(1+1./83.*t.^2).^16);

        case 84;
            alpha(set_t)=1./2+1./2.*t./(84+t.^2).^(1./2).*(1+7391536347803839./72057594037927936./(1+1./84.*t.^2).^30+14544636039226909./144115188075855872./(1+1./84.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./84.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./84.*t.^2).^33+14023284727082855679./147573952589676412928./(1+1./84.*t.^2).^35+7113260368810144185./73786976294838206464./(1+1./84.*t.^2).^34+110628135069209194801./1180591620717411303424./(1+1./84.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./84.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./84.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./84.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./84.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./84.*t.^2).^41+2268783825./17179869184./(1+1./84.*t.^2).^18+4418157975./34359738368./(1+1./84.*t.^2).^19+34461632205./274877906944./(1+1./84.*t.^2).^20+67282234305./549755813888./(1+1./84.*t.^2).^21+263012370465./2199023255552./(1+1./84.*t.^2).^22+1./2./(1+1./84.*t.^2)+3./8./(1+1./84.*t.^2).^2+5./16./(1+1./84.*t.^2).^3+35./128./(1+1./84.*t.^2).^4+63./256./(1+1./84.*t.^2).^5+231./1024./(1+1./84.*t.^2).^6+429./2048./(1+1./84.*t.^2).^7+6435./32768./(1+1./84.*t.^2).^8+12155./65536./(1+1./84.*t.^2).^9+46189./262144./(1+1./84.*t.^2).^10+88179./524288./(1+1./84.*t.^2).^11+676039./4194304./(1+1./84.*t.^2).^12+1300075./8388608./(1+1./84.*t.^2).^13+5014575./33554432./(1+1./84.*t.^2).^14+9694845./67108864./(1+1./84.*t.^2).^15+300540195./2147483648./(1+1./84.*t.^2).^16+583401555./4294967296./(1+1./84.*t.^2).^17+514589420475./4398046511104./(1+1./84.*t.^2).^23+8061900920775./70368744177664./(1+1./84.*t.^2).^24+15801325804719./140737488355328./(1+1./84.*t.^2).^25+61989816618513./562949953421312./(1+1./84.*t.^2).^26+121683714103007./1125899906842624./(1+1./84.*t.^2).^27+956086325095055./9007199254740992./(1+1./84.*t.^2).^28+1879204156221315./18014398509481984./(1+1./84.*t.^2).^29);

        case 85;
            alpha(set_t)=1./2+1./pi.*atan(1./85.*t.*85.^(1./2))+1./pi.*t.*85.^(1./2)./(85+t.^2).*(1+2./3./(1+1./85.*t.^2)+8./15./(1+1./85.*t.^2).^2+16./35./(1+1./85.*t.^2).^3+128./315./(1+1./85.*t.^2).^4+256./693./(1+1./85.*t.^2).^5+1024./3003./(1+1./85.*t.^2).^6+2048./6435./(1+1./85.*t.^2).^7+32768./109395./(1+1./85.*t.^2).^8+65536./230945./(1+1./85.*t.^2).^9+262144./969969./(1+1./85.*t.^2).^10+4194304./16900975./(1+1./85.*t.^2).^12+524288./2028117./(1+1./85.*t.^2).^11+8388608./35102025./(1+1./85.*t.^2).^13+33554432./145422675./(1+1./85.*t.^2).^14+67108864./300540195./(1+1./85.*t.^2).^15+2147483648./9917826435./(1+1./85.*t.^2).^16+4294967296./20419054425./(1+1./85.*t.^2).^17+17179869184./83945001525./(1+1./85.*t.^2).^18+34359738368./172308161025./(1+1./85.*t.^2).^19+274877906944./1412926920405./(1+1./85.*t.^2).^20+2199023255552./11835556670925./(1+1./85.*t.^2).^22+549755813888./2893136075115./(1+1./85.*t.^2).^21+70368744177664./395033145117975./(1+1./85.*t.^2).^24+4398046511104./24185702762325./(1+1./85.*t.^2).^23+140737488355328./805867616040669./(1+1./85.*t.^2).^25+562949953421312./3285460280781189./(1+1./85.*t.^2).^26+1125899906842624./6692604275665385./(1+1./85.*t.^2).^27+9007199254740992./54496920530418135./(1+1./85.*t.^2).^28+18014398509481984./110873045217057585./(1+1./85.*t.^2).^29+72057594037927936./450883717216034179./(1+1./85.*t.^2).^30+144115188075855872./916312070471295267./(1+1./85.*t.^2).^31+18446744073709551616./120925426269772451145./(1+1./85.*t.^2).^33+9223372036854775808./59560284580634192355./(1+1./85.*t.^2).^32+73786976294838206464./490814965447899948765./(1+1./85.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./85.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./85.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./85.*t.^2).^37+18889465931478580854784./134384010916670220577025./(1+1./85.*t.^2).^39+9444732965739290427392./66341473743672640538025./(1+1./85.*t.^2).^38+302231454903657293676544./2177020976850057573347805./(1+1./85.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./85.*t.^2).^41);

        case 86;
            alpha(set_t)=1./2+1./2.*t./(86+t.^2).^(1./2).*(1+67282234305./549755813888./(1+1./86.*t.^2).^21+263012370465./2199023255552./(1+1./86.*t.^2).^22+514589420475./4398046511104./(1+1./86.*t.^2).^23+8061900920775./70368744177664./(1+1./86.*t.^2).^24+15801325804719./140737488355328./(1+1./86.*t.^2).^25+61989816618513./562949953421312./(1+1./86.*t.^2).^26+121683714103007./1125899906842624./(1+1./86.*t.^2).^27+956086325095055./9007199254740992./(1+1./86.*t.^2).^28+1879204156221315./18014398509481984./(1+1./86.*t.^2).^29+14544636039226909./144115188075855872./(1+1./86.*t.^2).^31+7391536347803839./72057594037927936./(1+1./86.*t.^2).^30+916312070471295267./9223372036854775808./(1+1./86.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./86.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./86.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./86.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./86.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./86.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./86.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./86.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./86.*t.^2).^40+209863810776486386280915./2417851639229258349412352./(1+1./86.*t.^2).^42+1./2./(1+1./86.*t.^2)+53098072606098965203605./604462909807314587353088./(1+1./86.*t.^2).^41+3./8./(1+1./86.*t.^2).^2+5./16./(1+1./86.*t.^2).^3+35./128./(1+1./86.*t.^2).^4+63./256./(1+1./86.*t.^2).^5+231./1024./(1+1./86.*t.^2).^6+429./2048./(1+1./86.*t.^2).^7+6435./32768./(1+1./86.*t.^2).^8+12155./65536./(1+1./86.*t.^2).^9+46189./262144./(1+1./86.*t.^2).^10+88179./524288./(1+1./86.*t.^2).^11+676039./4194304./(1+1./86.*t.^2).^12+1300075./8388608./(1+1./86.*t.^2).^13+5014575./33554432./(1+1./86.*t.^2).^14+9694845./67108864./(1+1./86.*t.^2).^15+300540195./2147483648./(1+1./86.*t.^2).^16+583401555./4294967296./(1+1./86.*t.^2).^17+2268783825./17179869184./(1+1./86.*t.^2).^18+4418157975./34359738368./(1+1./86.*t.^2).^19+34461632205./274877906944./(1+1./86.*t.^2).^20);

        case 87;
            alpha(set_t)=1./2+1./pi.*atan(1./87.*t.*87.^(1./2))+1./pi.*t.*87.^(1./2)./(87+t.^2).*(1+2417851639229258349412352./17838423916001342833877775./(1+1./87.*t.^2).^42+2361183241434822606848./16369974040646495717175./(1+1./87.*t.^2).^37+2./3./(1+1./87.*t.^2)+8./15./(1+1./87.*t.^2).^2+128./315./(1+1./87.*t.^2).^4+16./35./(1+1./87.*t.^2).^3+256./693./(1+1./87.*t.^2).^5+1024./3003./(1+1./87.*t.^2).^6+2048./6435./(1+1./87.*t.^2).^7+32768./109395./(1+1./87.*t.^2).^8+262144./969969./(1+1./87.*t.^2).^10+65536./230945./(1+1./87.*t.^2).^9+4194304./16900975./(1+1./87.*t.^2).^12+524288./2028117./(1+1./87.*t.^2).^11+8388608./35102025./(1+1./87.*t.^2).^13+33554432./145422675./(1+1./87.*t.^2).^14+67108864./300540195./(1+1./87.*t.^2).^15+2147483648./9917826435./(1+1./87.*t.^2).^16+4294967296./20419054425./(1+1./87.*t.^2).^17+17179869184./83945001525./(1+1./87.*t.^2).^18+34359738368./172308161025./(1+1./87.*t.^2).^19+9444732965739290427392./66341473743672640538025./(1+1./87.*t.^2).^38+274877906944./1412926920405./(1+1./87.*t.^2).^20+549755813888./2893136075115./(1+1./87.*t.^2).^21+2199023255552./11835556670925./(1+1./87.*t.^2).^22+4398046511104./24185702762325./(1+1./87.*t.^2).^23+70368744177664./395033145117975./(1+1./87.*t.^2).^24+140737488355328./805867616040669./(1+1./87.*t.^2).^25+562949953421312./3285460280781189./(1+1./87.*t.^2).^26+18889465931478580854784./134384010916670220577025./(1+1./87.*t.^2).^39+1125899906842624./6692604275665385./(1+1./87.*t.^2).^27+9007199254740992./54496920530418135./(1+1./87.*t.^2).^28+18014398509481984./110873045217057585./(1+1./87.*t.^2).^29+72057594037927936./450883717216034179./(1+1./87.*t.^2).^30+144115188075855872./916312070471295267./(1+1./87.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./87.*t.^2).^32+302231454903657293676544./2177020976850057573347805./(1+1./87.*t.^2).^40+18446744073709551616./120925426269772451145./(1+1./87.*t.^2).^33+604462909807314587353088./4407140026306214111899215./(1+1./87.*t.^2).^41+73786976294838206464./490814965447899948765./(1+1./87.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./87.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./87.*t.^2).^36);

        case 88;
            alpha(set_t)=1./2+1./2.*t./(88+t.^2).^(1./2).*(1+1701063429324939500975./18889465931478580854784./(1+1./88.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./88.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./88.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./88.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./88.*t.^2).^43+300540195./2147483648./(1+1./88.*t.^2).^16+1804857108504066435./18446744073709551616./(1+1./88.*t.^2).^33+1./2./(1+1./88.*t.^2)+583401555./4294967296./(1+1./88.*t.^2).^17+3./8./(1+1./88.*t.^2).^2+2268783825./17179869184./(1+1./88.*t.^2).^18+4418157975./34359738368./(1+1./88.*t.^2).^19+7113260368810144185./73786976294838206464./(1+1./88.*t.^2).^34+34461632205./274877906944./(1+1./88.*t.^2).^20+5./16./(1+1./88.*t.^2).^3+67282234305./549755813888./(1+1./88.*t.^2).^21+35./128./(1+1./88.*t.^2).^4+263012370465./2199023255552./(1+1./88.*t.^2).^22+63./256./(1+1./88.*t.^2).^5+14023284727082855679./147573952589676412928./(1+1./88.*t.^2).^35+231./1024./(1+1./88.*t.^2).^6+514589420475./4398046511104./(1+1./88.*t.^2).^23+429./2048./(1+1./88.*t.^2).^7+8061900920775./70368744177664./(1+1./88.*t.^2).^24+110628135069209194801./1180591620717411303424./(1+1./88.*t.^2).^36+6435./32768./(1+1./88.*t.^2).^8+12155./65536./(1+1./88.*t.^2).^9+46189./262144./(1+1./88.*t.^2).^10+15801325804719./140737488355328./(1+1./88.*t.^2).^25+88179./524288./(1+1./88.*t.^2).^11+61989816618513./562949953421312./(1+1./88.*t.^2).^26+218266320541953276229./2361183241434822606848./(1+1./88.*t.^2).^37+121683714103007./1125899906842624./(1+1./88.*t.^2).^27+676039./4194304./(1+1./88.*t.^2).^12+956086325095055./9007199254740992./(1+1./88.*t.^2).^28+1879204156221315./18014398509481984./(1+1./88.*t.^2).^29+1300075./8388608./(1+1./88.*t.^2).^13+7391536347803839./72057594037927936./(1+1./88.*t.^2).^30+5014575./33554432./(1+1./88.*t.^2).^14+14544636039226909./144115188075855872./(1+1./88.*t.^2).^31+861577581086657669325./9444732965739290427392./(1+1./88.*t.^2).^38+9694845./67108864./(1+1./88.*t.^2).^15+916312070471295267./9223372036854775808./(1+1./88.*t.^2).^32);

        case 89;
            alpha(set_t)=1./2+1./pi.*atan(1./89.*t.*89.^(1./2))+1./pi.*t.*89.^(1./2)./(89+t.^2).*(1+2./3./(1+1./89.*t.^2)+16./35./(1+1./89.*t.^2).^3+8./15./(1+1./89.*t.^2).^2+128./315./(1+1./89.*t.^2).^4+1024./3003./(1+1./89.*t.^2).^6+256./693./(1+1./89.*t.^2).^5+32768./109395./(1+1./89.*t.^2).^8+2048./6435./(1+1./89.*t.^2).^7+65536./230945./(1+1./89.*t.^2).^9+524288./2028117./(1+1./89.*t.^2).^11+262144./969969./(1+1./89.*t.^2).^10+4194304./16900975./(1+1./89.*t.^2).^12+8388608./35102025./(1+1./89.*t.^2).^13+33554432./145422675./(1+1./89.*t.^2).^14+67108864./300540195./(1+1./89.*t.^2).^15+4294967296./20419054425./(1+1./89.*t.^2).^17+2147483648./9917826435./(1+1./89.*t.^2).^16+17179869184./83945001525./(1+1./89.*t.^2).^18+274877906944./1412926920405./(1+1./89.*t.^2).^20+34359738368./172308161025./(1+1./89.*t.^2).^19+549755813888./2893136075115./(1+1./89.*t.^2).^21+2199023255552./11835556670925./(1+1./89.*t.^2).^22+4398046511104./24185702762325./(1+1./89.*t.^2).^23+70368744177664./395033145117975./(1+1./89.*t.^2).^24+562949953421312./3285460280781189./(1+1./89.*t.^2).^26+140737488355328./805867616040669./(1+1./89.*t.^2).^25+1125899906842624./6692604275665385./(1+1./89.*t.^2).^27+72057594037927936./450883717216034179./(1+1./89.*t.^2).^30+18014398509481984./110873045217057585./(1+1./89.*t.^2).^29+9007199254740992./54496920530418135./(1+1./89.*t.^2).^28+144115188075855872./916312070471295267./(1+1./89.*t.^2).^31+18446744073709551616./120925426269772451145./(1+1./89.*t.^2).^33+9223372036854775808./59560284580634192355./(1+1./89.*t.^2).^32+147573952589676412928./995653215622882753209./(1+1./89.*t.^2).^35+73786976294838206464./490814965447899948765./(1+1./89.*t.^2).^34+1180591620717411303424./8075853860052271220473./(1+1./89.*t.^2).^36+9444732965739290427392./66341473743672640538025./(1+1./89.*t.^2).^38+2361183241434822606848./16369974040646495717175./(1+1./89.*t.^2).^37+302231454903657293676544./2177020976850057573347805./(1+1./89.*t.^2).^40+18889465931478580854784./134384010916670220577025./(1+1./89.*t.^2).^39+604462909807314587353088./4407140026306214111899215./(1+1./89.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./89.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./89.*t.^2).^43);

        case 90;
            alpha(set_t)=1./2+1./2.*t./(90+t.^2).^(1./2).*(1+231./1024./(1+1./90.*t.^2).^6+429./2048./(1+1./90.*t.^2).^7+6435./32768./(1+1./90.*t.^2).^8+12155./65536./(1+1./90.*t.^2).^9+46189./262144./(1+1./90.*t.^2).^10+88179./524288./(1+1./90.*t.^2).^11+676039./4194304./(1+1./90.*t.^2).^12+5014575./33554432./(1+1./90.*t.^2).^14+1300075./8388608./(1+1./90.*t.^2).^13+9694845./67108864./(1+1./90.*t.^2).^15+300540195./2147483648./(1+1./90.*t.^2).^16+583401555./4294967296./(1+1./90.*t.^2).^17+2268783825./17179869184./(1+1./90.*t.^2).^18+4418157975./34359738368./(1+1./90.*t.^2).^19+34461632205./274877906944./(1+1./90.*t.^2).^20+67282234305./549755813888./(1+1./90.*t.^2).^21+263012370465./2199023255552./(1+1./90.*t.^2).^22+514589420475./4398046511104./(1+1./90.*t.^2).^23+8061900920775./70368744177664./(1+1./90.*t.^2).^24+15801325804719./140737488355328./(1+1./90.*t.^2).^25+61989816618513./562949953421312./(1+1./90.*t.^2).^26+121683714103007./1125899906842624./(1+1./90.*t.^2).^27+956086325095055./9007199254740992./(1+1./90.*t.^2).^28+1879204156221315./18014398509481984./(1+1./90.*t.^2).^29+7391536347803839./72057594037927936./(1+1./90.*t.^2).^30+14544636039226909./144115188075855872./(1+1./90.*t.^2).^31+7113260368810144185./73786976294838206464./(1+1./90.*t.^2).^34+1804857108504066435./18446744073709551616./(1+1./90.*t.^2).^33+916312070471295267./9223372036854775808./(1+1./90.*t.^2).^32+14023284727082855679./147573952589676412928./(1+1./90.*t.^2).^35+1./2./(1+1./90.*t.^2)+110628135069209194801./1180591620717411303424./(1+1./90.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./90.*t.^2).^37+3./8./(1+1./90.*t.^2).^2+861577581086657669325./9444732965739290427392./(1+1./90.*t.^2).^38+5./16./(1+1./90.*t.^2).^3+1701063429324939500975./18889465931478580854784./(1+1./90.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./90.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./90.*t.^2).^41+35./128./(1+1./90.*t.^2).^4+209863810776486386280915./2417851639229258349412352./(1+1./90.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./90.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./90.*t.^2).^44+63./256./(1+1./90.*t.^2).^5);

        case 91;
            alpha(set_t)=1./2+1./pi.*atan(1./91.*t.*91.^(1./2))+1./pi.*t.*91.^(1./2)./(91+t.^2).*(1+2./3./(1+1./91.*t.^2)+8./15./(1+1./91.*t.^2).^2+128./315./(1+1./91.*t.^2).^4+16./35./(1+1./91.*t.^2).^3+256./693./(1+1./91.*t.^2).^5+1024./3003./(1+1./91.*t.^2).^6+32768./109395./(1+1./91.*t.^2).^8+2048./6435./(1+1./91.*t.^2).^7+65536./230945./(1+1./91.*t.^2).^9+262144./969969./(1+1./91.*t.^2).^10+4194304./16900975./(1+1./91.*t.^2).^12+524288./2028117./(1+1./91.*t.^2).^11+8388608./35102025./(1+1./91.*t.^2).^13+33554432./145422675./(1+1./91.*t.^2).^14+2147483648./9917826435./(1+1./91.*t.^2).^16+67108864./300540195./(1+1./91.*t.^2).^15+34359738368./172308161025./(1+1./91.*t.^2).^19+17179869184./83945001525./(1+1./91.*t.^2).^18+4294967296./20419054425./(1+1./91.*t.^2).^17+274877906944./1412926920405./(1+1./91.*t.^2).^20+2199023255552./11835556670925./(1+1./91.*t.^2).^22+549755813888./2893136075115./(1+1./91.*t.^2).^21+70368744177664./395033145117975./(1+1./91.*t.^2).^24+4398046511104./24185702762325./(1+1./91.*t.^2).^23+140737488355328./805867616040669./(1+1./91.*t.^2).^25+562949953421312./3285460280781189./(1+1./91.*t.^2).^26+1125899906842624./6692604275665385./(1+1./91.*t.^2).^27+18014398509481984./110873045217057585./(1+1./91.*t.^2).^29+9007199254740992./54496920530418135./(1+1./91.*t.^2).^28+144115188075855872./916312070471295267./(1+1./91.*t.^2).^31+72057594037927936./450883717216034179./(1+1./91.*t.^2).^30+18446744073709551616./120925426269772451145./(1+1./91.*t.^2).^33+9223372036854775808./59560284580634192355./(1+1./91.*t.^2).^32+147573952589676412928./995653215622882753209./(1+1./91.*t.^2).^35+73786976294838206464./490814965447899948765./(1+1./91.*t.^2).^34+2361183241434822606848./16369974040646495717175./(1+1./91.*t.^2).^37+1180591620717411303424./8075853860052271220473./(1+1./91.*t.^2).^36+9444732965739290427392./66341473743672640538025./(1+1./91.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./91.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./91.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./91.*t.^2).^41+4835703278458516698824704./36091694899816670384822475./(1+1./91.*t.^2).^43+2417851639229258349412352./17838423916001342833877775./(1+1./91.*t.^2).^42+38685626227668133590597632./292014622371243969477200025./(1+1./91.*t.^2).^44);

        case 92;
            alpha(set_t)=1./2+1./2.*t./(92+t.^2).^(1./2).*(1+110628135069209194801./1180591620717411303424./(1+1./92.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./92.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./92.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./92.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./92.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./92.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./92.*t.^2).^42+1./2./(1+1./92.*t.^2)+414847067813984717066925./4835703278458516698824704./(1+1./92.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./92.*t.^2).^44+3./8./(1+1./92.*t.^2).^2+5./16./(1+1./92.*t.^2).^3+35./128./(1+1./92.*t.^2).^4+63./256./(1+1./92.*t.^2).^5+6489213830472088210604445./77371252455336267181195264./(1+1./92.*t.^2).^45+231./1024./(1+1./92.*t.^2).^6+429./2048./(1+1./92.*t.^2).^7+6435./32768./(1+1./92.*t.^2).^8+12155./65536./(1+1./92.*t.^2).^9+46189./262144./(1+1./92.*t.^2).^10+88179./524288./(1+1./92.*t.^2).^11+676039./4194304./(1+1./92.*t.^2).^12+1300075./8388608./(1+1./92.*t.^2).^13+9694845./67108864./(1+1./92.*t.^2).^15+5014575./33554432./(1+1./92.*t.^2).^14+300540195./2147483648./(1+1./92.*t.^2).^16+583401555./4294967296./(1+1./92.*t.^2).^17+2268783825./17179869184./(1+1./92.*t.^2).^18+4418157975./34359738368./(1+1./92.*t.^2).^19+34461632205./274877906944./(1+1./92.*t.^2).^20+263012370465./2199023255552./(1+1./92.*t.^2).^22+67282234305./549755813888./(1+1./92.*t.^2).^21+514589420475./4398046511104./(1+1./92.*t.^2).^23+15801325804719./140737488355328./(1+1./92.*t.^2).^25+8061900920775./70368744177664./(1+1./92.*t.^2).^24+61989816618513./562949953421312./(1+1./92.*t.^2).^26+121683714103007./1125899906842624./(1+1./92.*t.^2).^27+956086325095055./9007199254740992./(1+1./92.*t.^2).^28+1879204156221315./18014398509481984./(1+1./92.*t.^2).^29+7391536347803839./72057594037927936./(1+1./92.*t.^2).^30+14544636039226909./144115188075855872./(1+1./92.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./92.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./92.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./92.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./92.*t.^2).^35);

        case 93;
            alpha(set_t)=1./2+1./pi.*atan(1./93.*t.*93.^(1./2))+1./pi.*t.*93.^(1./2)./(93+t.^2).*(1+2./3./(1+1./93.*t.^2)+8./15./(1+1./93.*t.^2).^2+16./35./(1+1./93.*t.^2).^3+256./693./(1+1./93.*t.^2).^5+128./315./(1+1./93.*t.^2).^4+1024./3003./(1+1./93.*t.^2).^6+32768./109395./(1+1./93.*t.^2).^8+2048./6435./(1+1./93.*t.^2).^7+65536./230945./(1+1./93.*t.^2).^9+262144./969969./(1+1./93.*t.^2).^10+524288./2028117./(1+1./93.*t.^2).^11+8388608./35102025./(1+1./93.*t.^2).^13+4194304./16900975./(1+1./93.*t.^2).^12+67108864./300540195./(1+1./93.*t.^2).^15+33554432./145422675./(1+1./93.*t.^2).^14+17179869184./83945001525./(1+1./93.*t.^2).^18+4294967296./20419054425./(1+1./93.*t.^2).^17+2147483648./9917826435./(1+1./93.*t.^2).^16+34359738368./172308161025./(1+1./93.*t.^2).^19+274877906944./1412926920405./(1+1./93.*t.^2).^20+549755813888./2893136075115./(1+1./93.*t.^2).^21+4398046511104./24185702762325./(1+1./93.*t.^2).^23+2199023255552./11835556670925./(1+1./93.*t.^2).^22+70368744177664./395033145117975./(1+1./93.*t.^2).^24+9007199254740992./54496920530418135./(1+1./93.*t.^2).^28+1125899906842624./6692604275665385./(1+1./93.*t.^2).^27+562949953421312./3285460280781189./(1+1./93.*t.^2).^26+140737488355328./805867616040669./(1+1./93.*t.^2).^25+18014398509481984./110873045217057585./(1+1./93.*t.^2).^29+72057594037927936./450883717216034179./(1+1./93.*t.^2).^30+144115188075855872./916312070471295267./(1+1./93.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./93.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./93.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./93.*t.^2).^34+1180591620717411303424./8075853860052271220473./(1+1./93.*t.^2).^36+147573952589676412928./995653215622882753209./(1+1./93.*t.^2).^35+2361183241434822606848./16369974040646495717175./(1+1./93.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./93.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./93.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./93.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./93.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./93.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./93.*t.^2).^43+77371252455336267181195264./590518458572960027165004495./(1+1./93.*t.^2).^45+38685626227668133590597632./292014622371243969477200025./(1+1./93.*t.^2).^44);

        case 94;
            alpha(set_t)=1./2+1./2.*t./(94+t.^2).^(1./2).*(1+676039./4194304./(1+1./94.*t.^2).^12+1300075./8388608./(1+1./94.*t.^2).^13+5014575./33554432./(1+1./94.*t.^2).^14+9694845./67108864./(1+1./94.*t.^2).^15+300540195./2147483648./(1+1./94.*t.^2).^16+2268783825./17179869184./(1+1./94.*t.^2).^18+583401555./4294967296./(1+1./94.*t.^2).^17+4418157975./34359738368./(1+1./94.*t.^2).^19+34461632205./274877906944./(1+1./94.*t.^2).^20+67282234305./549755813888./(1+1./94.*t.^2).^21+263012370465./2199023255552./(1+1./94.*t.^2).^22+514589420475./4398046511104./(1+1./94.*t.^2).^23+8061900920775./70368744177664./(1+1./94.*t.^2).^24+15801325804719./140737488355328./(1+1./94.*t.^2).^25+61989816618513./562949953421312./(1+1./94.*t.^2).^26+121683714103007./1125899906842624./(1+1./94.*t.^2).^27+1./2./(1+1./94.*t.^2)+1879204156221315./18014398509481984./(1+1./94.*t.^2).^29+956086325095055./9007199254740992./(1+1./94.*t.^2).^28+3./8./(1+1./94.*t.^2).^2+7391536347803839./72057594037927936./(1+1./94.*t.^2).^30+14544636039226909./144115188075855872./(1+1./94.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./94.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./94.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./94.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./94.*t.^2).^35+5./16./(1+1./94.*t.^2).^3+110628135069209194801./1180591620717411303424./(1+1./94.*t.^2).^36+861577581086657669325./9444732965739290427392./(1+1./94.*t.^2).^38+218266320541953276229./2361183241434822606848./(1+1./94.*t.^2).^37+35./128./(1+1./94.*t.^2).^4+1701063429324939500975./18889465931478580854784./(1+1./94.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./94.*t.^2).^40+63./256./(1+1./94.*t.^2).^5+53098072606098965203605./604462909807314587353088./(1+1./94.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./94.*t.^2).^42+231./1024./(1+1./94.*t.^2).^6+414847067813984717066925./4835703278458516698824704./(1+1./94.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./94.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./94.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./94.*t.^2).^46+429./2048./(1+1./94.*t.^2).^7+6435./32768./(1+1./94.*t.^2).^8+12155./65536./(1+1./94.*t.^2).^9+46189./262144./(1+1./94.*t.^2).^10+88179./524288./(1+1./94.*t.^2).^11);

        case 95;
            alpha(set_t)=1./2+1./pi.*atan(1./95.*t.*95.^(1./2))+1./pi.*t.*95.^(1./2)./(95+t.^2).*(1+18446744073709551616./120925426269772451145./(1+1./95.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./95.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./95.*t.^2).^35+70368744177664./395033145117975./(1+1./95.*t.^2).^24+1180591620717411303424./8075853860052271220473./(1+1./95.*t.^2).^36+140737488355328./805867616040669./(1+1./95.*t.^2).^25+2361183241434822606848./16369974040646495717175./(1+1./95.*t.^2).^37+18889465931478580854784./134384010916670220577025./(1+1./95.*t.^2).^39+9444732965739290427392./66341473743672640538025./(1+1./95.*t.^2).^38+302231454903657293676544./2177020976850057573347805./(1+1./95.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./95.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./95.*t.^2).^42+2./3./(1+1./95.*t.^2)+562949953421312./3285460280781189./(1+1./95.*t.^2).^26+8./15./(1+1./95.*t.^2).^2+4835703278458516698824704./36091694899816670384822475./(1+1./95.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./95.*t.^2).^44+16./35./(1+1./95.*t.^2).^3+1125899906842624./6692604275665385./(1+1./95.*t.^2).^27+256./693./(1+1./95.*t.^2).^5+128./315./(1+1./95.*t.^2).^4+1024./3003./(1+1./95.*t.^2).^6+77371252455336267181195264./590518458572960027165004495./(1+1./95.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./95.*t.^2).^46+2048./6435./(1+1./95.*t.^2).^7+32768./109395./(1+1./95.*t.^2).^8+65536./230945./(1+1./95.*t.^2).^9+262144./969969./(1+1./95.*t.^2).^10+9007199254740992./54496920530418135./(1+1./95.*t.^2).^28+524288./2028117./(1+1./95.*t.^2).^11+18014398509481984./110873045217057585./(1+1./95.*t.^2).^29+4194304./16900975./(1+1./95.*t.^2).^12+8388608./35102025./(1+1./95.*t.^2).^13+33554432./145422675./(1+1./95.*t.^2).^14+67108864./300540195./(1+1./95.*t.^2).^15+72057594037927936./450883717216034179./(1+1./95.*t.^2).^30+2147483648./9917826435./(1+1./95.*t.^2).^16+4294967296./20419054425./(1+1./95.*t.^2).^17+17179869184./83945001525./(1+1./95.*t.^2).^18+144115188075855872./916312070471295267./(1+1./95.*t.^2).^31+34359738368./172308161025./(1+1./95.*t.^2).^19+274877906944./1412926920405./(1+1./95.*t.^2).^20+549755813888./2893136075115./(1+1./95.*t.^2).^21+2199023255552./11835556670925./(1+1./95.*t.^2).^22+9223372036854775808./59560284580634192355./(1+1./95.*t.^2).^32+4398046511104./24185702762325./(1+1./95.*t.^2).^23);

        case 96;
            alpha(set_t)=1./2+1./2.*t./(96+t.^2).^(1./2).*(1+4418157975./34359738368./(1+1./96.*t.^2).^19+34461632205./274877906944./(1+1./96.*t.^2).^20+67282234305./549755813888./(1+1./96.*t.^2).^21+6435./32768./(1+1./96.*t.^2).^8+263012370465./2199023255552./(1+1./96.*t.^2).^22+514589420475./4398046511104./(1+1./96.*t.^2).^23+8061900920775./70368744177664./(1+1./96.*t.^2).^24+15801325804719./140737488355328./(1+1./96.*t.^2).^25+12155./65536./(1+1./96.*t.^2).^9+61989816618513./562949953421312./(1+1./96.*t.^2).^26+121683714103007./1125899906842624./(1+1./96.*t.^2).^27+956086325095055./9007199254740992./(1+1./96.*t.^2).^28+1879204156221315./18014398509481984./(1+1./96.*t.^2).^29+7391536347803839./72057594037927936./(1+1./96.*t.^2).^30+14544636039226909./144115188075855872./(1+1./96.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./96.*t.^2).^32+7113260368810144185./73786976294838206464./(1+1./96.*t.^2).^34+1804857108504066435./18446744073709551616./(1+1./96.*t.^2).^33+14023284727082855679./147573952589676412928./(1+1./96.*t.^2).^35+218266320541953276229./2361183241434822606848./(1+1./96.*t.^2).^37+110628135069209194801./1180591620717411303424./(1+1./96.*t.^2).^36+1701063429324939500975./18889465931478580854784./(1+1./96.*t.^2).^39+46189./262144./(1+1./96.*t.^2).^10+861577581086657669325./9444732965739290427392./(1+1./96.*t.^2).^38+26876802183334044115405./302231454903657293676544./(1+1./96.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./96.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./96.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./96.*t.^2).^43+88179./524288./(1+1./96.*t.^2).^11+3281063172710606398620225./38685626227668133590597632./(1+1./96.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./96.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./96.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./96.*t.^2).^47+676039./4194304./(1+1./96.*t.^2).^12+1300075./8388608./(1+1./96.*t.^2).^13+1./2./(1+1./96.*t.^2)+3./8./(1+1./96.*t.^2).^2+5014575./33554432./(1+1./96.*t.^2).^14+5./16./(1+1./96.*t.^2).^3+35./128./(1+1./96.*t.^2).^4+9694845./67108864./(1+1./96.*t.^2).^15+63./256./(1+1./96.*t.^2).^5+300540195./2147483648./(1+1./96.*t.^2).^16+231./1024./(1+1./96.*t.^2).^6+583401555./4294967296./(1+1./96.*t.^2).^17+2268783825./17179869184./(1+1./96.*t.^2).^18+429./2048./(1+1./96.*t.^2).^7);

        case 97;
            alpha(set_t)=1./2+1./pi.*atan(1./97.*t.*97.^(1./2))+1./pi.*t.*97.^(1./2)./(97+t.^2).*(1+147573952589676412928./995653215622882753209./(1+1./97.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./97.*t.^2).^36+70368744177664./395033145117975./(1+1./97.*t.^2).^24+2./3./(1+1./97.*t.^2)+2361183241434822606848./16369974040646495717175./(1+1./97.*t.^2).^37+8./15./(1+1./97.*t.^2).^2+16./35./(1+1./97.*t.^2).^3+128./315./(1+1./97.*t.^2).^4+9444732965739290427392./66341473743672640538025./(1+1./97.*t.^2).^38+140737488355328./805867616040669./(1+1./97.*t.^2).^25+256./693./(1+1./97.*t.^2).^5+18889465931478580854784./134384010916670220577025./(1+1./97.*t.^2).^39+1024./3003./(1+1./97.*t.^2).^6+562949953421312./3285460280781189./(1+1./97.*t.^2).^26+302231454903657293676544./2177020976850057573347805./(1+1./97.*t.^2).^40+2048./6435./(1+1./97.*t.^2).^7+1125899906842624./6692604275665385./(1+1./97.*t.^2).^27+65536./230945./(1+1./97.*t.^2).^9+32768./109395./(1+1./97.*t.^2).^8+604462909807314587353088./4407140026306214111899215./(1+1./97.*t.^2).^41+9007199254740992./54496920530418135./(1+1./97.*t.^2).^28+2417851639229258349412352./17838423916001342833877775./(1+1./97.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./97.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./97.*t.^2).^44+262144./969969./(1+1./97.*t.^2).^10+4194304./16900975./(1+1./97.*t.^2).^12+524288./2028117./(1+1./97.*t.^2).^11+8388608./35102025./(1+1./97.*t.^2).^13+77371252455336267181195264./590518458572960027165004495./(1+1./97.*t.^2).^45+18014398509481984./110873045217057585./(1+1./97.*t.^2).^29+309485009821345068724781056./2387748549881968805493279045./(1+1./97.*t.^2).^46+72057594037927936./450883717216034179./(1+1./97.*t.^2).^30+618970019642690137449562112./4826300260399724181316202325./(1+1./97.*t.^2).^47+33554432./145422675./(1+1./97.*t.^2).^14+67108864./300540195./(1+1./97.*t.^2).^15+144115188075855872./916312070471295267./(1+1./97.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./97.*t.^2).^32+2147483648./9917826435./(1+1./97.*t.^2).^16+4294967296./20419054425./(1+1./97.*t.^2).^17+17179869184./83945001525./(1+1./97.*t.^2).^18+18446744073709551616./120925426269772451145./(1+1./97.*t.^2).^33+34359738368./172308161025./(1+1./97.*t.^2).^19+274877906944./1412926920405./(1+1./97.*t.^2).^20+73786976294838206464./490814965447899948765./(1+1./97.*t.^2).^34+549755813888./2893136075115./(1+1./97.*t.^2).^21+2199023255552./11835556670925./(1+1./97.*t.^2).^22+4398046511104./24185702762325./(1+1./97.*t.^2).^23);

        case 98;
            alpha(set_t)=1./2+1./2.*t./(98+t.^2).^(1./2).*(1+300540195./2147483648./(1+1./98.*t.^2).^16+583401555./4294967296./(1+1./98.*t.^2).^17+2268783825./17179869184./(1+1./98.*t.^2).^18+4418157975./34359738368./(1+1./98.*t.^2).^19+34461632205./274877906944./(1+1./98.*t.^2).^20+67282234305./549755813888./(1+1./98.*t.^2).^21+263012370465./2199023255552./(1+1./98.*t.^2).^22+8061900920775./70368744177664./(1+1./98.*t.^2).^24+514589420475./4398046511104./(1+1./98.*t.^2).^23+1./2./(1+1./98.*t.^2)+15801325804719./140737488355328./(1+1./98.*t.^2).^25+61989816618513./562949953421312./(1+1./98.*t.^2).^26+3./8./(1+1./98.*t.^2).^2+121683714103007./1125899906842624./(1+1./98.*t.^2).^27+5./16./(1+1./98.*t.^2).^3+1879204156221315./18014398509481984./(1+1./98.*t.^2).^29+956086325095055./9007199254740992./(1+1./98.*t.^2).^28+7391536347803839./72057594037927936./(1+1./98.*t.^2).^30+14544636039226909./144115188075855872./(1+1./98.*t.^2).^31+35./128./(1+1./98.*t.^2).^4+916312070471295267./9223372036854775808./(1+1./98.*t.^2).^32+7113260368810144185./73786976294838206464./(1+1./98.*t.^2).^34+1804857108504066435./18446744073709551616./(1+1./98.*t.^2).^33+63./256./(1+1./98.*t.^2).^5+14023284727082855679./147573952589676412928./(1+1./98.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./98.*t.^2).^36+231./1024./(1+1./98.*t.^2).^6+218266320541953276229./2361183241434822606848./(1+1./98.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./98.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./98.*t.^2).^39+429./2048./(1+1./98.*t.^2).^7+26876802183334044115405./302231454903657293676544./(1+1./98.*t.^2).^40+6435./32768./(1+1./98.*t.^2).^8+53098072606098965203605./604462909807314587353088./(1+1./98.*t.^2).^41+12155./65536./(1+1./98.*t.^2).^9+209863810776486386280915./2417851639229258349412352./(1+1./98.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./98.*t.^2).^43+46189./262144./(1+1./98.*t.^2).^10+6489213830472088210604445./77371252455336267181195264./(1+1./98.*t.^2).^45+3281063172710606398620225./38685626227668133590597632./(1+1./98.*t.^2).^44+25674715590128696833261065./309485009821345068724781056./(1+1./98.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./98.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./98.*t.^2).^48+88179./524288./(1+1./98.*t.^2).^11+676039./4194304./(1+1./98.*t.^2).^12+1300075./8388608./(1+1./98.*t.^2).^13+5014575./33554432./(1+1./98.*t.^2).^14+9694845./67108864./(1+1./98.*t.^2).^15);

        case 99;
            alpha(set_t)=1./2+1./pi.*atan(1./99.*t.*99.^(1./2))+1./pi.*t.*99.^(1./2)./(99+t.^2).*(1+4398046511104./24185702762325./(1+1./99.*t.^2).^23+8388608./35102025./(1+1./99.*t.^2).^13+70368744177664./395033145117975./(1+1./99.*t.^2).^24+140737488355328./805867616040669./(1+1./99.*t.^2).^25+33554432./145422675./(1+1./99.*t.^2).^14+562949953421312./3285460280781189./(1+1./99.*t.^2).^26+1125899906842624./6692604275665385./(1+1./99.*t.^2).^27+9007199254740992./54496920530418135./(1+1./99.*t.^2).^28+72057594037927936./450883717216034179./(1+1./99.*t.^2).^30+18014398509481984./110873045217057585./(1+1./99.*t.^2).^29+67108864./300540195./(1+1./99.*t.^2).^15+144115188075855872./916312070471295267./(1+1./99.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./99.*t.^2).^32+147573952589676412928./995653215622882753209./(1+1./99.*t.^2).^35+73786976294838206464./490814965447899948765./(1+1./99.*t.^2).^34+18446744073709551616./120925426269772451145./(1+1./99.*t.^2).^33+1180591620717411303424./8075853860052271220473./(1+1./99.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./99.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./99.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./99.*t.^2).^39+2147483648./9917826435./(1+1./99.*t.^2).^16+302231454903657293676544./2177020976850057573347805./(1+1./99.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./99.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./99.*t.^2).^42+4294967296./20419054425./(1+1./99.*t.^2).^17+38685626227668133590597632./292014622371243969477200025./(1+1./99.*t.^2).^44+4835703278458516698824704./36091694899816670384822475./(1+1./99.*t.^2).^43+17179869184./83945001525./(1+1./99.*t.^2).^18+77371252455336267181195264./590518458572960027165004495./(1+1./99.*t.^2).^45+618970019642690137449562112./4826300260399724181316202325./(1+1./99.*t.^2).^47+309485009821345068724781056./2387748549881968805493279045./(1+1./99.*t.^2).^46+19807040628566084398385987584./156050375086257748529223875175./(1+1./99.*t.^2).^48+2./3./(1+1./99.*t.^2)+8./15./(1+1./99.*t.^2).^2+34359738368./172308161025./(1+1./99.*t.^2).^19+16./35./(1+1./99.*t.^2).^3+256./693./(1+1./99.*t.^2).^5+128./315./(1+1./99.*t.^2).^4+1024./3003./(1+1./99.*t.^2).^6+32768./109395./(1+1./99.*t.^2).^8+2048./6435./(1+1./99.*t.^2).^7+274877906944./1412926920405./(1+1./99.*t.^2).^20+65536./230945./(1+1./99.*t.^2).^9+262144./969969./(1+1./99.*t.^2).^10+549755813888./2893136075115./(1+1./99.*t.^2).^21+2199023255552./11835556670925./(1+1./99.*t.^2).^22+524288./2028117./(1+1./99.*t.^2).^11+4194304./16900975./(1+1./99.*t.^2).^12);

        case 100;
            alpha(set_t)=1./2+1./2.*t./(100+t.^2).^(1./2).*(1+4418157975./34359738368./(1+1./100.*t.^2).^19+34461632205./274877906944./(1+1./100.*t.^2).^20+67282234305./549755813888./(1+1./100.*t.^2).^21+263012370465./2199023255552./(1+1./100.*t.^2).^22+514589420475./4398046511104./(1+1./100.*t.^2).^23+8061900920775./70368744177664./(1+1./100.*t.^2).^24+15801325804719./140737488355328./(1+1./100.*t.^2).^25+61989816618513./562949953421312./(1+1./100.*t.^2).^26+121683714103007./1125899906842624./(1+1./100.*t.^2).^27+956086325095055./9007199254740992./(1+1./100.*t.^2).^28+1879204156221315./18014398509481984./(1+1./100.*t.^2).^29+7391536347803839./72057594037927936./(1+1./100.*t.^2).^30+14544636039226909./144115188075855872./(1+1./100.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./100.*t.^2).^32+1./2./(1+1./100.*t.^2)+1804857108504066435./18446744073709551616./(1+1./100.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./100.*t.^2).^34+3./8./(1+1./100.*t.^2).^2+14023284727082855679./147573952589676412928./(1+1./100.*t.^2).^35+5./16./(1+1./100.*t.^2).^3+110628135069209194801./1180591620717411303424./(1+1./100.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./100.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./100.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./100.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./100.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./100.*t.^2).^41+35./128./(1+1./100.*t.^2).^4+63./256./(1+1./100.*t.^2).^5+209863810776486386280915./2417851639229258349412352./(1+1./100.*t.^2).^42+429./2048./(1+1./100.*t.^2).^7+231./1024./(1+1./100.*t.^2).^6+6435./32768./(1+1./100.*t.^2).^8+12155./65536./(1+1./100.*t.^2).^9+46189./262144./(1+1./100.*t.^2).^10+414847067813984717066925./4835703278458516698824704./(1+1./100.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./100.*t.^2).^44+88179./524288./(1+1./100.*t.^2).^11+6489213830472088210604445./77371252455336267181195264./(1+1./100.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./100.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./100.*t.^2).^47+676039./4194304./(1+1./100.*t.^2).^12+1608766753466574727105400775./19807040628566084398385987584./(1+1./100.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./100.*t.^2).^49+1300075./8388608./(1+1./100.*t.^2).^13+5014575./33554432./(1+1./100.*t.^2).^14+9694845./67108864./(1+1./100.*t.^2).^15+300540195./2147483648./(1+1./100.*t.^2).^16+583401555./4294967296./(1+1./100.*t.^2).^17+2268783825./17179869184./(1+1./100.*t.^2).^18);

        case 101;
            alpha(set_t)=1./2+1./pi.*atan(1./101.*t.*101.^(1./2))+1./pi.*t.*101.^(1./2)./(101+t.^2).*(1+17179869184./83945001525./(1+1./101.*t.^2).^18+256./693./(1+1./101.*t.^2).^5+34359738368./172308161025./(1+1./101.*t.^2).^19+274877906944./1412926920405./(1+1./101.*t.^2).^20+549755813888./2893136075115./(1+1./101.*t.^2).^21+1024./3003./(1+1./101.*t.^2).^6+4398046511104./24185702762325./(1+1./101.*t.^2).^23+2199023255552./11835556670925./(1+1./101.*t.^2).^22+70368744177664./395033145117975./(1+1./101.*t.^2).^24+140737488355328./805867616040669./(1+1./101.*t.^2).^25+2048./6435./(1+1./101.*t.^2).^7+562949953421312./3285460280781189./(1+1./101.*t.^2).^26+1125899906842624./6692604275665385./(1+1./101.*t.^2).^27+9007199254740992./54496920530418135./(1+1./101.*t.^2).^28+72057594037927936./450883717216034179./(1+1./101.*t.^2).^30+18014398509481984./110873045217057585./(1+1./101.*t.^2).^29+144115188075855872./916312070471295267./(1+1./101.*t.^2).^31+32768./109395./(1+1./101.*t.^2).^8+9223372036854775808./59560284580634192355./(1+1./101.*t.^2).^32+73786976294838206464./490814965447899948765./(1+1./101.*t.^2).^34+18446744073709551616./120925426269772451145./(1+1./101.*t.^2).^33+65536./230945./(1+1./101.*t.^2).^9+262144./969969./(1+1./101.*t.^2).^10+524288./2028117./(1+1./101.*t.^2).^11+147573952589676412928./995653215622882753209./(1+1./101.*t.^2).^35+4194304./16900975./(1+1./101.*t.^2).^12+1180591620717411303424./8075853860052271220473./(1+1./101.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./101.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./101.*t.^2).^38+8388608./35102025./(1+1./101.*t.^2).^13+18889465931478580854784./134384010916670220577025./(1+1./101.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./101.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./101.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./101.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./101.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./101.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./101.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./101.*t.^2).^46+67108864./300540195./(1+1./101.*t.^2).^15+33554432./145422675./(1+1./101.*t.^2).^14+618970019642690137449562112./4826300260399724181316202325./(1+1./101.*t.^2).^47+2147483648./9917826435./(1+1./101.*t.^2).^16+4294967296./20419054425./(1+1./101.*t.^2).^17+19807040628566084398385987584./156050375086257748529223875175./(1+1./101.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./101.*t.^2).^49+2./3./(1+1./101.*t.^2)+8./15./(1+1./101.*t.^2).^2+16./35./(1+1./101.*t.^2).^3+128./315./(1+1./101.*t.^2).^4);

        case 102;
            alpha(set_t)=1./2+1./2.*t./(102+t.^2).^(1./2).*(1+26876802183334044115405./302231454903657293676544./(1+1./102.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./102.*t.^2).^41+7391536347803839./72057594037927936./(1+1./102.*t.^2).^30+14544636039226909./144115188075855872./(1+1./102.*t.^2).^31+209863810776486386280915./2417851639229258349412352./(1+1./102.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./102.*t.^2).^43+916312070471295267./9223372036854775808./(1+1./102.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./102.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./102.*t.^2).^34+1./2./(1+1./102.*t.^2)+3281063172710606398620225./38685626227668133590597632./(1+1./102.*t.^2).^44+3./8./(1+1./102.*t.^2).^2+5./16./(1+1./102.*t.^2).^3+63./256./(1+1./102.*t.^2).^5+35./128./(1+1./102.*t.^2).^4+14023284727082855679./147573952589676412928./(1+1./102.*t.^2).^35+6489213830472088210604445./77371252455336267181195264./(1+1./102.*t.^2).^45+231./1024./(1+1./102.*t.^2).^6+429./2048./(1+1./102.*t.^2).^7+6435./32768./(1+1./102.*t.^2).^8+46189./262144./(1+1./102.*t.^2).^10+12155./65536./(1+1./102.*t.^2).^9+25674715590128696833261065./309485009821345068724781056./(1+1./102.*t.^2).^46+88179./524288./(1+1./102.*t.^2).^11+1300075./8388608./(1+1./102.*t.^2).^13+676039./4194304./(1+1./102.*t.^2).^12+110628135069209194801./1180591620717411303424./(1+1./102.*t.^2).^36+50803160635786570329644235./618970019642690137449562112./(1+1./102.*t.^2).^47+218266320541953276229./2361183241434822606848./(1+1./102.*t.^2).^37+5014575./33554432./(1+1./102.*t.^2).^14+861577581086657669325./9444732965739290427392./(1+1./102.*t.^2).^38+1608766753466574727105400775./19807040628566084398385987584./(1+1./102.*t.^2).^48+300540195./2147483648./(1+1./102.*t.^2).^16+9694845./67108864./(1+1./102.*t.^2).^15+3184701532372607112841303575./39614081257132168796771975168./(1+1./102.*t.^2).^49+583401555./4294967296./(1+1./102.*t.^2).^17+12611418068195524166851562157./158456325028528675187087900672./(1+1./102.*t.^2).^50+4418157975./34359738368./(1+1./102.*t.^2).^19+2268783825./17179869184./(1+1./102.*t.^2).^18+1701063429324939500975./18889465931478580854784./(1+1./102.*t.^2).^39+34461632205./274877906944./(1+1./102.*t.^2).^20+67282234305./549755813888./(1+1./102.*t.^2).^21+263012370465./2199023255552./(1+1./102.*t.^2).^22+514589420475./4398046511104./(1+1./102.*t.^2).^23+15801325804719./140737488355328./(1+1./102.*t.^2).^25+8061900920775./70368744177664./(1+1./102.*t.^2).^24+61989816618513./562949953421312./(1+1./102.*t.^2).^26+121683714103007./1125899906842624./(1+1./102.*t.^2).^27+956086325095055./9007199254740992./(1+1./102.*t.^2).^28+1879204156221315./18014398509481984./(1+1./102.*t.^2).^29);

        case 103;
            alpha(set_t)=1./2+1./pi.*atan(1./103.*t.*103.^(1./2))+1./pi.*t.*103.^(1./2)./(103+t.^2).*(1+72057594037927936./450883717216034179./(1+1./103.*t.^2).^30+144115188075855872./916312070471295267./(1+1./103.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./103.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./103.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./103.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./103.*t.^2).^35+2147483648./9917826435./(1+1./103.*t.^2).^16+4294967296./20419054425./(1+1./103.*t.^2).^17+2361183241434822606848./16369974040646495717175./(1+1./103.*t.^2).^37+1180591620717411303424./8075853860052271220473./(1+1./103.*t.^2).^36+34359738368./172308161025./(1+1./103.*t.^2).^19+17179869184./83945001525./(1+1./103.*t.^2).^18+9444732965739290427392./66341473743672640538025./(1+1./103.*t.^2).^38+274877906944./1412926920405./(1+1./103.*t.^2).^20+18889465931478580854784./134384010916670220577025./(1+1./103.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./103.*t.^2).^40+549755813888./2893136075115./(1+1./103.*t.^2).^21+604462909807314587353088./4407140026306214111899215./(1+1./103.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./103.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./103.*t.^2).^43+2199023255552./11835556670925./(1+1./103.*t.^2).^22+38685626227668133590597632./292014622371243969477200025./(1+1./103.*t.^2).^44+4398046511104./24185702762325./(1+1./103.*t.^2).^23+70368744177664./395033145117975./(1+1./103.*t.^2).^24+77371252455336267181195264./590518458572960027165004495./(1+1./103.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./103.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./103.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./103.*t.^2).^48+2./3./(1+1./103.*t.^2)+39614081257132168796771975168./315285451704888104171289053925./(1+1./103.*t.^2).^49+562949953421312./3285460280781189./(1+1./103.*t.^2).^26+140737488355328./805867616040669./(1+1./103.*t.^2).^25+8./15./(1+1./103.*t.^2).^2+158456325028528675187087900672./1273753224887747940852007777857./(1+1./103.*t.^2).^50+16./35./(1+1./103.*t.^2).^3+1125899906842624./6692604275665385./(1+1./103.*t.^2).^27+128./315./(1+1./103.*t.^2).^4+256./693./(1+1./103.*t.^2).^5+1024./3003./(1+1./103.*t.^2).^6+9007199254740992./54496920530418135./(1+1./103.*t.^2).^28+2048./6435./(1+1./103.*t.^2).^7+32768./109395./(1+1./103.*t.^2).^8+262144./969969./(1+1./103.*t.^2).^10+65536./230945./(1+1./103.*t.^2).^9+524288./2028117./(1+1./103.*t.^2).^11+18014398509481984./110873045217057585./(1+1./103.*t.^2).^29+4194304./16900975./(1+1./103.*t.^2).^12+8388608./35102025./(1+1./103.*t.^2).^13+33554432./145422675./(1+1./103.*t.^2).^14+67108864./300540195./(1+1./103.*t.^2).^15);

        case 104;
            alpha(set_t)=1./2+1./2.*t./(104+t.^2).^(1./2).*(1+110628135069209194801./1180591620717411303424./(1+1./104.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./104.*t.^2).^37+50803160635786570329644235./618970019642690137449562112./(1+1./104.*t.^2).^47+6435./32768./(1+1./104.*t.^2).^8+1608766753466574727105400775./19807040628566084398385987584./(1+1./104.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./104.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./104.*t.^2).^50+12155./65536./(1+1./104.*t.^2).^9+24975553429171528252000152507./316912650057057350374175801344./(1+1./104.*t.^2).^51+861577581086657669325./9444732965739290427392./(1+1./104.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./104.*t.^2).^39+67282234305./549755813888./(1+1./104.*t.^2).^21+263012370465./2199023255552./(1+1./104.*t.^2).^22+46189./262144./(1+1./104.*t.^2).^10+514589420475./4398046511104./(1+1./104.*t.^2).^23+8061900920775./70368744177664./(1+1./104.*t.^2).^24+26876802183334044115405./302231454903657293676544./(1+1./104.*t.^2).^40+88179./524288./(1+1./104.*t.^2).^11+676039./4194304./(1+1./104.*t.^2).^12+15801325804719./140737488355328./(1+1./104.*t.^2).^25+61989816618513./562949953421312./(1+1./104.*t.^2).^26+53098072606098965203605./604462909807314587353088./(1+1./104.*t.^2).^41+1300075./8388608./(1+1./104.*t.^2).^13+121683714103007./1125899906842624./(1+1./104.*t.^2).^27+5014575./33554432./(1+1./104.*t.^2).^14+209863810776486386280915./2417851639229258349412352./(1+1./104.*t.^2).^42+956086325095055./9007199254740992./(1+1./104.*t.^2).^28+9694845./67108864./(1+1./104.*t.^2).^15+1879204156221315./18014398509481984./(1+1./104.*t.^2).^29+300540195./2147483648./(1+1./104.*t.^2).^16+7391536347803839./72057594037927936./(1+1./104.*t.^2).^30+14544636039226909./144115188075855872./(1+1./104.*t.^2).^31+414847067813984717066925./4835703278458516698824704./(1+1./104.*t.^2).^43+583401555./4294967296./(1+1./104.*t.^2).^17+2268783825./17179869184./(1+1./104.*t.^2).^18+916312070471295267./9223372036854775808./(1+1./104.*t.^2).^32+3281063172710606398620225./38685626227668133590597632./(1+1./104.*t.^2).^44+1./2./(1+1./104.*t.^2)+1804857108504066435./18446744073709551616./(1+1./104.*t.^2).^33+6489213830472088210604445./77371252455336267181195264./(1+1./104.*t.^2).^45+7113260368810144185./73786976294838206464./(1+1./104.*t.^2).^34+4418157975./34359738368./(1+1./104.*t.^2).^19+3./8./(1+1./104.*t.^2).^2+5./16./(1+1./104.*t.^2).^3+35./128./(1+1./104.*t.^2).^4+63./256./(1+1./104.*t.^2).^5+25674715590128696833261065./309485009821345068724781056./(1+1./104.*t.^2).^46+231./1024./(1+1./104.*t.^2).^6+429./2048./(1+1./104.*t.^2).^7+34461632205./274877906944./(1+1./104.*t.^2).^20+14023284727082855679./147573952589676412928./(1+1./104.*t.^2).^35);

        case 105;
            alpha(set_t)=1./2+1./pi.*atan(1./105.*t.*105.^(1./2))+1./pi.*t.*105.^(1./2)./(105+t.^2).*(1+18446744073709551616./120925426269772451145./(1+1./105.*t.^2).^33+77371252455336267181195264./590518458572960027165004495./(1+1./105.*t.^2).^45+73786976294838206464./490814965447899948765./(1+1./105.*t.^2).^34+309485009821345068724781056./2387748549881968805493279045./(1+1./105.*t.^2).^46+147573952589676412928./995653215622882753209./(1+1./105.*t.^2).^35+618970019642690137449562112./4826300260399724181316202325./(1+1./105.*t.^2).^47+1180591620717411303424./8075853860052271220473./(1+1./105.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./105.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./105.*t.^2).^38+19807040628566084398385987584./156050375086257748529223875175./(1+1./105.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./105.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./105.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./105.*t.^2).^51+2./3./(1+1./105.*t.^2)+18889465931478580854784./134384010916670220577025./(1+1./105.*t.^2).^39+8./15./(1+1./105.*t.^2).^2+16./35./(1+1./105.*t.^2).^3+128./315./(1+1./105.*t.^2).^4+256./693./(1+1./105.*t.^2).^5+2048./6435./(1+1./105.*t.^2).^7+1024./3003./(1+1./105.*t.^2).^6+302231454903657293676544./2177020976850057573347805./(1+1./105.*t.^2).^40+262144./969969./(1+1./105.*t.^2).^10+65536./230945./(1+1./105.*t.^2).^9+32768./109395./(1+1./105.*t.^2).^8+524288./2028117./(1+1./105.*t.^2).^11+4194304./16900975./(1+1./105.*t.^2).^12+8388608./35102025./(1+1./105.*t.^2).^13+67108864./300540195./(1+1./105.*t.^2).^15+33554432./145422675./(1+1./105.*t.^2).^14+2147483648./9917826435./(1+1./105.*t.^2).^16+604462909807314587353088./4407140026306214111899215./(1+1./105.*t.^2).^41+34359738368./172308161025./(1+1./105.*t.^2).^19+17179869184./83945001525./(1+1./105.*t.^2).^18+4294967296./20419054425./(1+1./105.*t.^2).^17+549755813888./2893136075115./(1+1./105.*t.^2).^21+274877906944./1412926920405./(1+1./105.*t.^2).^20+2199023255552./11835556670925./(1+1./105.*t.^2).^22+70368744177664./395033145117975./(1+1./105.*t.^2).^24+140737488355328./805867616040669./(1+1./105.*t.^2).^25+4398046511104./24185702762325./(1+1./105.*t.^2).^23+2417851639229258349412352./17838423916001342833877775./(1+1./105.*t.^2).^42+562949953421312./3285460280781189./(1+1./105.*t.^2).^26+1125899906842624./6692604275665385./(1+1./105.*t.^2).^27+9007199254740992./54496920530418135./(1+1./105.*t.^2).^28+18014398509481984./110873045217057585./(1+1./105.*t.^2).^29+4835703278458516698824704./36091694899816670384822475./(1+1./105.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./105.*t.^2).^44+72057594037927936./450883717216034179./(1+1./105.*t.^2).^30+144115188075855872./916312070471295267./(1+1./105.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./105.*t.^2).^32);

        case 106;
            alpha(set_t)=1./2+1./2.*t./(106+t.^2).^(1./2).*(1+7113260368810144185./73786976294838206464./(1+1./106.*t.^2).^34+1./2./(1+1./106.*t.^2)+3./8./(1+1./106.*t.^2).^2+14023284727082855679./147573952589676412928./(1+1./106.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./106.*t.^2).^36+6489213830472088210604445./77371252455336267181195264./(1+1./106.*t.^2).^45+5./16./(1+1./106.*t.^2).^3+63./256./(1+1./106.*t.^2).^5+35./128./(1+1./106.*t.^2).^4+429./2048./(1+1./106.*t.^2).^7+231./1024./(1+1./106.*t.^2).^6+6435./32768./(1+1./106.*t.^2).^8+25674715590128696833261065./309485009821345068724781056./(1+1./106.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./106.*t.^2).^47+12155./65536./(1+1./106.*t.^2).^9+218266320541953276229./2361183241434822606848./(1+1./106.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./106.*t.^2).^38+46189./262144./(1+1./106.*t.^2).^10+88179./524288./(1+1./106.*t.^2).^11+1608766753466574727105400775./19807040628566084398385987584./(1+1./106.*t.^2).^48+676039./4194304./(1+1./106.*t.^2).^12+3184701532372607112841303575./39614081257132168796771975168./(1+1./106.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./106.*t.^2).^50+1300075./8388608./(1+1./106.*t.^2).^13+5014575./33554432./(1+1./106.*t.^2).^14+24975553429171528252000152507./316912650057057350374175801344./(1+1./106.*t.^2).^51+1701063429324939500975./18889465931478580854784./(1+1./106.*t.^2).^39+9694845./67108864./(1+1./106.*t.^2).^15+300540195./2147483648./(1+1./106.*t.^2).^16+197883231015743646919693516017./2535301200456458802993406410752./(1+1./106.*t.^2).^52+583401555./4294967296./(1+1./106.*t.^2).^17+2268783825./17179869184./(1+1./106.*t.^2).^18+34461632205./274877906944./(1+1./106.*t.^2).^20+67282234305./549755813888./(1+1./106.*t.^2).^21+4418157975./34359738368./(1+1./106.*t.^2).^19+263012370465./2199023255552./(1+1./106.*t.^2).^22+514589420475./4398046511104./(1+1./106.*t.^2).^23+8061900920775./70368744177664./(1+1./106.*t.^2).^24+26876802183334044115405./302231454903657293676544./(1+1./106.*t.^2).^40+15801325804719./140737488355328./(1+1./106.*t.^2).^25+53098072606098965203605./604462909807314587353088./(1+1./106.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./106.*t.^2).^42+61989816618513./562949953421312./(1+1./106.*t.^2).^26+121683714103007./1125899906842624./(1+1./106.*t.^2).^27+956086325095055./9007199254740992./(1+1./106.*t.^2).^28+1879204156221315./18014398509481984./(1+1./106.*t.^2).^29+7391536347803839./72057594037927936./(1+1./106.*t.^2).^30+14544636039226909./144115188075855872./(1+1./106.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./106.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./106.*t.^2).^33+414847067813984717066925./4835703278458516698824704./(1+1./106.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./106.*t.^2).^44);

        case 107;
            alpha(set_t)=1./2+1./pi.*atan(1./107.*t.*107.^(1./2))+1./pi.*t.*107.^(1./2)./(107+t.^2).*(1+140737488355328./805867616040669./(1+1./107.*t.^2).^25+562949953421312./3285460280781189./(1+1./107.*t.^2).^26+9444732965739290427392./66341473743672640538025./(1+1./107.*t.^2).^38+1125899906842624./6692604275665385./(1+1./107.*t.^2).^27+9007199254740992./54496920530418135./(1+1./107.*t.^2).^28+18014398509481984./110873045217057585./(1+1./107.*t.^2).^29+18889465931478580854784./134384010916670220577025./(1+1./107.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./107.*t.^2).^40+2417851639229258349412352./17838423916001342833877775./(1+1./107.*t.^2).^42+604462909807314587353088./4407140026306214111899215./(1+1./107.*t.^2).^41+4835703278458516698824704./36091694899816670384822475./(1+1./107.*t.^2).^43+32768./109395./(1+1./107.*t.^2).^8+38685626227668133590597632./292014622371243969477200025./(1+1./107.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./107.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./107.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./107.*t.^2).^47+65536./230945./(1+1./107.*t.^2).^9+19807040628566084398385987584./156050375086257748529223875175./(1+1./107.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./107.*t.^2).^49+262144./969969./(1+1./107.*t.^2).^10+524288./2028117./(1+1./107.*t.^2).^11+4194304./16900975./(1+1./107.*t.^2).^12+158456325028528675187087900672./1273753224887747940852007777857./(1+1./107.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./107.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./107.*t.^2).^52+72057594037927936./450883717216034179./(1+1./107.*t.^2).^30+8388608./35102025./(1+1./107.*t.^2).^13+144115188075855872./916312070471295267./(1+1./107.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./107.*t.^2).^32+33554432./145422675./(1+1./107.*t.^2).^14+67108864./300540195./(1+1./107.*t.^2).^15+18446744073709551616./120925426269772451145./(1+1./107.*t.^2).^33+2147483648./9917826435./(1+1./107.*t.^2).^16+4294967296./20419054425./(1+1./107.*t.^2).^17+17179869184./83945001525./(1+1./107.*t.^2).^18+2./3./(1+1./107.*t.^2)+274877906944./1412926920405./(1+1./107.*t.^2).^20+549755813888./2893136075115./(1+1./107.*t.^2).^21+34359738368./172308161025./(1+1./107.*t.^2).^19+8./15./(1+1./107.*t.^2).^2+73786976294838206464./490814965447899948765./(1+1./107.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./107.*t.^2).^35+16./35./(1+1./107.*t.^2).^3+128./315./(1+1./107.*t.^2).^4+1180591620717411303424./8075853860052271220473./(1+1./107.*t.^2).^36+256./693./(1+1./107.*t.^2).^5+2361183241434822606848./16369974040646495717175./(1+1./107.*t.^2).^37+2199023255552./11835556670925./(1+1./107.*t.^2).^22+4398046511104./24185702762325./(1+1./107.*t.^2).^23+1024./3003./(1+1./107.*t.^2).^6+2048./6435./(1+1./107.*t.^2).^7+70368744177664./395033145117975./(1+1./107.*t.^2).^24);

        case 108;
            alpha(set_t)=1./2+1./2.*t./(108+t.^2).^(1./2).*(1+88179./524288./(1+1./108.*t.^2).^11+676039./4194304./(1+1./108.*t.^2).^12+1300075./8388608./(1+1./108.*t.^2).^13+5014575./33554432./(1+1./108.*t.^2).^14+9694845./67108864./(1+1./108.*t.^2).^15+300540195./2147483648./(1+1./108.*t.^2).^16+583401555./4294967296./(1+1./108.*t.^2).^17+2268783825./17179869184./(1+1./108.*t.^2).^18+4418157975./34359738368./(1+1./108.*t.^2).^19+34461632205./274877906944./(1+1./108.*t.^2).^20+67282234305./549755813888./(1+1./108.*t.^2).^21+514589420475./4398046511104./(1+1./108.*t.^2).^23+263012370465./2199023255552./(1+1./108.*t.^2).^22+8061900920775./70368744177664./(1+1./108.*t.^2).^24+15801325804719./140737488355328./(1+1./108.*t.^2).^25+61989816618513./562949953421312./(1+1./108.*t.^2).^26+121683714103007./1125899906842624./(1+1./108.*t.^2).^27+956086325095055./9007199254740992./(1+1./108.*t.^2).^28+1879204156221315./18014398509481984./(1+1./108.*t.^2).^29+916312070471295267./9223372036854775808./(1+1./108.*t.^2).^32+7391536347803839./72057594037927936./(1+1./108.*t.^2).^30+14544636039226909./144115188075855872./(1+1./108.*t.^2).^31+1804857108504066435./18446744073709551616./(1+1./108.*t.^2).^33+1./2./(1+1./108.*t.^2)+14023284727082855679./147573952589676412928./(1+1./108.*t.^2).^35+7113260368810144185./73786976294838206464./(1+1./108.*t.^2).^34+3./8./(1+1./108.*t.^2).^2+5./16./(1+1./108.*t.^2).^3+110628135069209194801./1180591620717411303424./(1+1./108.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./108.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./108.*t.^2).^38+35./128./(1+1./108.*t.^2).^4+1701063429324939500975./18889465931478580854784./(1+1./108.*t.^2).^39+63./256./(1+1./108.*t.^2).^5+26876802183334044115405./302231454903657293676544./(1+1./108.*t.^2).^40+231./1024./(1+1./108.*t.^2).^6+53098072606098965203605./604462909807314587353088./(1+1./108.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./108.*t.^2).^42+3281063172710606398620225./38685626227668133590597632./(1+1./108.*t.^2).^44+414847067813984717066925./4835703278458516698824704./(1+1./108.*t.^2).^43+429./2048./(1+1./108.*t.^2).^7+6435./32768./(1+1./108.*t.^2).^8+6489213830472088210604445./77371252455336267181195264./(1+1./108.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./108.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./108.*t.^2).^47+12155./65536./(1+1./108.*t.^2).^9+3184701532372607112841303575./39614081257132168796771975168./(1+1./108.*t.^2).^49+1608766753466574727105400775./19807040628566084398385987584./(1+1./108.*t.^2).^48+12611418068195524166851562157./158456325028528675187087900672./(1+1./108.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./108.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./108.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./108.*t.^2).^53+46189./262144./(1+1./108.*t.^2).^10);

        case 109;
            alpha(set_t)=1./2+1./pi.*atan(1./109.*t.*109.^(1./2))+1./pi.*t.*109.^(1./2)./(109+t.^2).*(1+39614081257132168796771975168./315285451704888104171289053925./(1+1./109.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./109.*t.^2).^50+2./3./(1+1./109.*t.^2)+140737488355328./805867616040669./(1+1./109.*t.^2).^25+562949953421312./3285460280781189./(1+1./109.*t.^2).^26+16./35./(1+1./109.*t.^2).^3+302231454903657293676544./2177020976850057573347805./(1+1./109.*t.^2).^40+8./15./(1+1./109.*t.^2).^2+1125899906842624./6692604275665385./(1+1./109.*t.^2).^27+128./315./(1+1./109.*t.^2).^4+256./693./(1+1./109.*t.^2).^5+316912650057057350374175801344./2572482003204667409956015708221./(1+1./109.*t.^2).^51+1024./3003./(1+1./109.*t.^2).^6+604462909807314587353088./4407140026306214111899215./(1+1./109.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./109.*t.^2).^42+2048./6435./(1+1./109.*t.^2).^7+32768./109395./(1+1./109.*t.^2).^8+4835703278458516698824704./36091694899816670384822475./(1+1./109.*t.^2).^43+9007199254740992./54496920530418135./(1+1./109.*t.^2).^28+18014398509481984./110873045217057585./(1+1./109.*t.^2).^29+38685626227668133590597632./292014622371243969477200025./(1+1./109.*t.^2).^44+65536./230945./(1+1./109.*t.^2).^9+72057594037927936./450883717216034179./(1+1./109.*t.^2).^30+262144./969969./(1+1./109.*t.^2).^10+524288./2028117./(1+1./109.*t.^2).^11+144115188075855872./916312070471295267./(1+1./109.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./109.*t.^2).^32+77371252455336267181195264./590518458572960027165004495./(1+1./109.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./109.*t.^2).^46+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./109.*t.^2).^52+18446744073709551616./120925426269772451145./(1+1./109.*t.^2).^33+4194304./16900975./(1+1./109.*t.^2).^12+73786976294838206464./490814965447899948765./(1+1./109.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./109.*t.^2).^35+8388608./35102025./(1+1./109.*t.^2).^13+33554432./145422675./(1+1./109.*t.^2).^14+67108864./300540195./(1+1./109.*t.^2).^15+2147483648./9917826435./(1+1./109.*t.^2).^16+4294967296./20419054425./(1+1./109.*t.^2).^17+17179869184./83945001525./(1+1./109.*t.^2).^18+34359738368./172308161025./(1+1./109.*t.^2).^19+274877906944./1412926920405./(1+1./109.*t.^2).^20+549755813888./2893136075115./(1+1./109.*t.^2).^21+2199023255552./11835556670925./(1+1./109.*t.^2).^22+1180591620717411303424./8075853860052271220473./(1+1./109.*t.^2).^36+618970019642690137449562112./4826300260399724181316202325./(1+1./109.*t.^2).^47+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./109.*t.^2).^53+4398046511104./24185702762325./(1+1./109.*t.^2).^23+70368744177664./395033145117975./(1+1./109.*t.^2).^24+2361183241434822606848./16369974040646495717175./(1+1./109.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./109.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./109.*t.^2).^39+19807040628566084398385987584./156050375086257748529223875175./(1+1./109.*t.^2).^48);

        case 110;
            alpha(set_t)=1./2+1./2.*t./(110+t.^2).^(1./2).*(1+3184701532372607112841303575./39614081257132168796771975168./(1+1./110.*t.^2).^49+34461632205./274877906944./(1+1./110.*t.^2).^20+231./1024./(1+1./110.*t.^2).^6+429./2048./(1+1./110.*t.^2).^7+6435./32768./(1+1./110.*t.^2).^8+67282234305./549755813888./(1+1./110.*t.^2).^21+263012370465./2199023255552./(1+1./110.*t.^2).^22+15801325804719./140737488355328./(1+1./110.*t.^2).^25+514589420475./4398046511104./(1+1./110.*t.^2).^23+8061900920775./70368744177664./(1+1./110.*t.^2).^24+61989816618513./562949953421312./(1+1./110.*t.^2).^26+12611418068195524166851562157./158456325028528675187087900672./(1+1./110.*t.^2).^50+12155./65536./(1+1./110.*t.^2).^9+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./110.*t.^2).^54+956086325095055./9007199254740992./(1+1./110.*t.^2).^28+121683714103007./1125899906842624./(1+1./110.*t.^2).^27+24975553429171528252000152507./316912650057057350374175801344./(1+1./110.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./110.*t.^2).^52+1879204156221315./18014398509481984./(1+1./110.*t.^2).^29+7391536347803839./72057594037927936./(1+1./110.*t.^2).^30+46189./262144./(1+1./110.*t.^2).^10+14544636039226909./144115188075855872./(1+1./110.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./110.*t.^2).^32+88179./524288./(1+1./110.*t.^2).^11+676039./4194304./(1+1./110.*t.^2).^12+1804857108504066435./18446744073709551616./(1+1./110.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./110.*t.^2).^34+1300075./8388608./(1+1./110.*t.^2).^13+14023284727082855679./147573952589676412928./(1+1./110.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./110.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./110.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./110.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./110.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./110.*t.^2).^40+5014575./33554432./(1+1./110.*t.^2).^14+9694845./67108864./(1+1./110.*t.^2).^15+53098072606098965203605./604462909807314587353088./(1+1./110.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./110.*t.^2).^42+2268783825./17179869184./(1+1./110.*t.^2).^18+300540195./2147483648./(1+1./110.*t.^2).^16+583401555./4294967296./(1+1./110.*t.^2).^17+1./2./(1+1./110.*t.^2)+414847067813984717066925./4835703278458516698824704./(1+1./110.*t.^2).^43+4418157975./34359738368./(1+1./110.*t.^2).^19+3281063172710606398620225./38685626227668133590597632./(1+1./110.*t.^2).^44+3./8./(1+1./110.*t.^2).^2+5./16./(1+1./110.*t.^2).^3+6489213830472088210604445./77371252455336267181195264./(1+1./110.*t.^2).^45+35./128./(1+1./110.*t.^2).^4+392032816163265715595619229845./5070602400912917605986812821504./(1+1./110.*t.^2).^53+25674715590128696833261065./309485009821345068724781056./(1+1./110.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./110.*t.^2).^47+63./256./(1+1./110.*t.^2).^5+1608766753466574727105400775./19807040628566084398385987584./(1+1./110.*t.^2).^48);

        case 111;
            alpha(set_t)=1./2+1./pi.*atan(1./111.*t.*111.^(1./2))+1./pi.*t.*111.^(1./2)./(111+t.^2).*(1+4194304./16900975./(1+1./111.*t.^2).^12+8388608./35102025./(1+1./111.*t.^2).^13+2./3./(1+1./111.*t.^2)+18446744073709551616./120925426269772451145./(1+1./111.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./111.*t.^2).^34+33554432./145422675./(1+1./111.*t.^2).^14+67108864./300540195./(1+1./111.*t.^2).^15+8./15./(1+1./111.*t.^2).^2+147573952589676412928./995653215622882753209./(1+1./111.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./111.*t.^2).^36+2147483648./9917826435./(1+1./111.*t.^2).^16+140737488355328./805867616040669./(1+1./111.*t.^2).^25+562949953421312./3285460280781189./(1+1./111.*t.^2).^26+34359738368./172308161025./(1+1./111.*t.^2).^19+4294967296./20419054425./(1+1./111.*t.^2).^17+17179869184./83945001525./(1+1./111.*t.^2).^18+2361183241434822606848./16369974040646495717175./(1+1./111.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./111.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./111.*t.^2).^39+16./35./(1+1./111.*t.^2).^3+128./315./(1+1./111.*t.^2).^4+302231454903657293676544./2177020976850057573347805./(1+1./111.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./111.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./111.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./111.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./111.*t.^2).^44+274877906944./1412926920405./(1+1./111.*t.^2).^20+549755813888./2893136075115./(1+1./111.*t.^2).^21+77371252455336267181195264./590518458572960027165004495./(1+1./111.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./111.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./111.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./111.*t.^2).^48+1125899906842624./6692604275665385./(1+1./111.*t.^2).^27+9007199254740992./54496920530418135./(1+1./111.*t.^2).^28+39614081257132168796771975168./315285451704888104171289053925./(1+1./111.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./111.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./111.*t.^2).^51+256./693./(1+1./111.*t.^2).^5+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./111.*t.^2).^52+1024./3003./(1+1./111.*t.^2).^6+2048./6435./(1+1./111.*t.^2).^7+18014398509481984./110873045217057585./(1+1./111.*t.^2).^29+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./111.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./111.*t.^2).^54+2199023255552./11835556670925./(1+1./111.*t.^2).^22+4398046511104./24185702762325./(1+1./111.*t.^2).^23+72057594037927936./450883717216034179./(1+1./111.*t.^2).^30+144115188075855872./916312070471295267./(1+1./111.*t.^2).^31+32768./109395./(1+1./111.*t.^2).^8+65536./230945./(1+1./111.*t.^2).^9+262144./969969./(1+1./111.*t.^2).^10+70368744177664./395033145117975./(1+1./111.*t.^2).^24+9223372036854775808./59560284580634192355./(1+1./111.*t.^2).^32+524288./2028117./(1+1./111.*t.^2).^11);

        case 112;
            alpha(set_t)=1./2+1./2.*t./(112+t.^2).^(1./2).*(1+12611418068195524166851562157./158456325028528675187087900672./(1+1./112.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./112.*t.^2).^51+1300075./8388608./(1+1./112.*t.^2).^13+5014575./33554432./(1+1./112.*t.^2).^14+9694845./67108864./(1+1./112.*t.^2).^15+300540195./2147483648./(1+1./112.*t.^2).^16+583401555./4294967296./(1+1./112.*t.^2).^17+197883231015743646919693516017./2535301200456458802993406410752./(1+1./112.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./112.*t.^2).^53+2268783825./17179869184./(1+1./112.*t.^2).^18+4418157975./34359738368./(1+1./112.*t.^2).^19+34461632205./274877906944./(1+1./112.*t.^2).^20+67282234305./549755813888./(1+1./112.*t.^2).^21+26876802183334044115405./302231454903657293676544./(1+1./112.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./112.*t.^2).^41+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./112.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./112.*t.^2).^55+263012370465./2199023255552./(1+1./112.*t.^2).^22+514589420475./4398046511104./(1+1./112.*t.^2).^23+209863810776486386280915./2417851639229258349412352./(1+1./112.*t.^2).^42+1./2./(1+1./112.*t.^2)+3./8./(1+1./112.*t.^2).^2+8061900920775./70368744177664./(1+1./112.*t.^2).^24+15801325804719./140737488355328./(1+1./112.*t.^2).^25+61989816618513./562949953421312./(1+1./112.*t.^2).^26+5./16./(1+1./112.*t.^2).^3+35./128./(1+1./112.*t.^2).^4+121683714103007./1125899906842624./(1+1./112.*t.^2).^27+956086325095055./9007199254740992./(1+1./112.*t.^2).^28+414847067813984717066925./4835703278458516698824704./(1+1./112.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./112.*t.^2).^44+63./256./(1+1./112.*t.^2).^5+231./1024./(1+1./112.*t.^2).^6+1879204156221315./18014398509481984./(1+1./112.*t.^2).^29+7391536347803839./72057594037927936./(1+1./112.*t.^2).^30+429./2048./(1+1./112.*t.^2).^7+6435./32768./(1+1./112.*t.^2).^8+14544636039226909./144115188075855872./(1+1./112.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./112.*t.^2).^32+6489213830472088210604445./77371252455336267181195264./(1+1./112.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./112.*t.^2).^46+7113260368810144185./73786976294838206464./(1+1./112.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./112.*t.^2).^35+1804857108504066435./18446744073709551616./(1+1./112.*t.^2).^33+50803160635786570329644235./618970019642690137449562112./(1+1./112.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./112.*t.^2).^48+110628135069209194801./1180591620717411303424./(1+1./112.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./112.*t.^2).^37+12155./65536./(1+1./112.*t.^2).^9+861577581086657669325./9444732965739290427392./(1+1./112.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./112.*t.^2).^39+3184701532372607112841303575./39614081257132168796771975168./(1+1./112.*t.^2).^49+46189./262144./(1+1./112.*t.^2).^10+88179./524288./(1+1./112.*t.^2).^11+676039./4194304./(1+1./112.*t.^2).^12);

        case 113;
            alpha(set_t)=1./2+1./pi.*atan(1./113.*t.*113.^(1./2))+1./pi.*t.*113.^(1./2)./(113+t.^2).*(1+4294967296./20419054425./(1+1./113.*t.^2).^17+17179869184./83945001525./(1+1./113.*t.^2).^18+70368744177664./395033145117975./(1+1./113.*t.^2).^24+140737488355328./805867616040669./(1+1./113.*t.^2).^25+256./693./(1+1./113.*t.^2).^5+1024./3003./(1+1./113.*t.^2).^6+9223372036854775808./59560284580634192355./(1+1./113.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./113.*t.^2).^33+562949953421312./3285460280781189./(1+1./113.*t.^2).^26+1125899906842624./6692604275665385./(1+1./113.*t.^2).^27+147573952589676412928./995653215622882753209./(1+1./113.*t.^2).^35+73786976294838206464./490814965447899948765./(1+1./113.*t.^2).^34+2048./6435./(1+1./113.*t.^2).^7+32768./109395./(1+1./113.*t.^2).^8+1180591620717411303424./8075853860052271220473./(1+1./113.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./113.*t.^2).^37+34359738368./172308161025./(1+1./113.*t.^2).^19+274877906944./1412926920405./(1+1./113.*t.^2).^20+9444732965739290427392./66341473743672640538025./(1+1./113.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./113.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./113.*t.^2).^40+65536./230945./(1+1./113.*t.^2).^9+604462909807314587353088./4407140026306214111899215./(1+1./113.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./113.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./113.*t.^2).^43+262144./969969./(1+1./113.*t.^2).^10+524288./2028117./(1+1./113.*t.^2).^11+38685626227668133590597632./292014622371243969477200025./(1+1./113.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./113.*t.^2).^45+9007199254740992./54496920530418135./(1+1./113.*t.^2).^28+18014398509481984./110873045217057585./(1+1./113.*t.^2).^29+72057594037927936./450883717216034179./(1+1./113.*t.^2).^30+309485009821345068724781056./2387748549881968805493279045./(1+1./113.*t.^2).^46+144115188075855872./916312070471295267./(1+1./113.*t.^2).^31+618970019642690137449562112./4826300260399724181316202325./(1+1./113.*t.^2).^47+33554432./145422675./(1+1./113.*t.^2).^14+4194304./16900975./(1+1./113.*t.^2).^12+8388608./35102025./(1+1./113.*t.^2).^13+19807040628566084398385987584./156050375086257748529223875175./(1+1./113.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./113.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./113.*t.^2).^50+67108864./300540195./(1+1./113.*t.^2).^15+2147483648./9917826435./(1+1./113.*t.^2).^16+316912650057057350374175801344./2572482003204667409956015708221./(1+1./113.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./113.*t.^2).^52+549755813888./2893136075115./(1+1./113.*t.^2).^21+2199023255552./11835556670925./(1+1./113.*t.^2).^22+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./113.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./113.*t.^2).^54+2./3./(1+1./113.*t.^2)+8./15./(1+1./113.*t.^2).^2+4398046511104./24185702762325./(1+1./113.*t.^2).^23+16./35./(1+1./113.*t.^2).^3+128./315./(1+1./113.*t.^2).^4+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./113.*t.^2).^55);

        case 114;
            alpha(set_t)=1./2+1./2.*t./(114+t.^2).^(1./2).*(1+110628135069209194801./1180591620717411303424./(1+1./114.*t.^2).^36+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./114.*t.^2).^56+861577581086657669325./9444732965739290427392./(1+1./114.*t.^2).^38+218266320541953276229./2361183241434822606848./(1+1./114.*t.^2).^37+1701063429324939500975./18889465931478580854784./(1+1./114.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./114.*t.^2).^40+1879204156221315./18014398509481984./(1+1./114.*t.^2).^29+1./2./(1+1./114.*t.^2)+53098072606098965203605./604462909807314587353088./(1+1./114.*t.^2).^41+3184701532372607112841303575./39614081257132168796771975168./(1+1./114.*t.^2).^49+3./8./(1+1./114.*t.^2).^2+5./16./(1+1./114.*t.^2).^3+35./128./(1+1./114.*t.^2).^4+231./1024./(1+1./114.*t.^2).^6+63./256./(1+1./114.*t.^2).^5+429./2048./(1+1./114.*t.^2).^7+6435./32768./(1+1./114.*t.^2).^8+12155./65536./(1+1./114.*t.^2).^9+209863810776486386280915./2417851639229258349412352./(1+1./114.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./114.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./114.*t.^2).^44+46189./262144./(1+1./114.*t.^2).^10+88179./524288./(1+1./114.*t.^2).^11+676039./4194304./(1+1./114.*t.^2).^12+5014575./33554432./(1+1./114.*t.^2).^14+1300075./8388608./(1+1./114.*t.^2).^13+12611418068195524166851562157./158456325028528675187087900672./(1+1./114.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./114.*t.^2).^51+9694845./67108864./(1+1./114.*t.^2).^15+300540195./2147483648./(1+1./114.*t.^2).^16+197883231015743646919693516017./2535301200456458802993406410752./(1+1./114.*t.^2).^52+583401555./4294967296./(1+1./114.*t.^2).^17+7391536347803839./72057594037927936./(1+1./114.*t.^2).^30+14544636039226909./144115188075855872./(1+1./114.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./114.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./114.*t.^2).^33+2268783825./17179869184./(1+1./114.*t.^2).^18+4418157975./34359738368./(1+1./114.*t.^2).^19+392032816163265715595619229845./5070602400912917605986812821504./(1+1./114.*t.^2).^53+6489213830472088210604445./77371252455336267181195264./(1+1./114.*t.^2).^45+263012370465./2199023255552./(1+1./114.*t.^2).^22+34461632205./274877906944./(1+1./114.*t.^2).^20+67282234305./549755813888./(1+1./114.*t.^2).^21+7113260368810144185./73786976294838206464./(1+1./114.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./114.*t.^2).^35+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./114.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./114.*t.^2).^55+514589420475./4398046511104./(1+1./114.*t.^2).^23+8061900920775./70368744177664./(1+1./114.*t.^2).^24+25674715590128696833261065./309485009821345068724781056./(1+1./114.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./114.*t.^2).^47+15801325804719./140737488355328./(1+1./114.*t.^2).^25+61989816618513./562949953421312./(1+1./114.*t.^2).^26+1608766753466574727105400775./19807040628566084398385987584./(1+1./114.*t.^2).^48+121683714103007./1125899906842624./(1+1./114.*t.^2).^27+956086325095055./9007199254740992./(1+1./114.*t.^2).^28);

        case 115;
            alpha(set_t)=1./2+1./pi.*atan(1./115.*t.*115.^(1./2))+1./pi.*t.*115.^(1./2)./(115+t.^2).*(1+4398046511104./24185702762325./(1+1./115.*t.^2).^23+70368744177664./395033145117975./(1+1./115.*t.^2).^24+140737488355328./805867616040669./(1+1./115.*t.^2).^25+562949953421312./3285460280781189./(1+1./115.*t.^2).^26+73786976294838206464./490814965447899948765./(1+1./115.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./115.*t.^2).^35+1125899906842624./6692604275665385./(1+1./115.*t.^2).^27+9007199254740992./54496920530418135./(1+1./115.*t.^2).^28+34359738368./172308161025./(1+1./115.*t.^2).^19+274877906944./1412926920405./(1+1./115.*t.^2).^20+1180591620717411303424./8075853860052271220473./(1+1./115.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./115.*t.^2).^37+4835703278458516698824704./36091694899816670384822475./(1+1./115.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./115.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./115.*t.^2).^45+9444732965739290427392./66341473743672640538025./(1+1./115.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./115.*t.^2).^39+19807040628566084398385987584./156050375086257748529223875175./(1+1./115.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./115.*t.^2).^49+618970019642690137449562112./4826300260399724181316202325./(1+1./115.*t.^2).^47+309485009821345068724781056./2387748549881968805493279045./(1+1./115.*t.^2).^46+158456325028528675187087900672./1273753224887747940852007777857./(1+1./115.*t.^2).^50+2./3./(1+1./115.*t.^2)+316912650057057350374175801344./2572482003204667409956015708221./(1+1./115.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./115.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./115.*t.^2).^53+18014398509481984./110873045217057585./(1+1./115.*t.^2).^29+72057594037927936./450883717216034179./(1+1./115.*t.^2).^30+16./35./(1+1./115.*t.^2).^3+128./315./(1+1./115.*t.^2).^4+8./15./(1+1./115.*t.^2).^2+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./115.*t.^2).^56+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./115.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./115.*t.^2).^55+256./693./(1+1./115.*t.^2).^5+1024./3003./(1+1./115.*t.^2).^6+2048./6435./(1+1./115.*t.^2).^7+32768./109395./(1+1./115.*t.^2).^8+65536./230945./(1+1./115.*t.^2).^9+262144./969969./(1+1./115.*t.^2).^10+302231454903657293676544./2177020976850057573347805./(1+1./115.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./115.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./115.*t.^2).^42+524288./2028117./(1+1./115.*t.^2).^11+4194304./16900975./(1+1./115.*t.^2).^12+8388608./35102025./(1+1./115.*t.^2).^13+33554432./145422675./(1+1./115.*t.^2).^14+67108864./300540195./(1+1./115.*t.^2).^15+2147483648./9917826435./(1+1./115.*t.^2).^16+549755813888./2893136075115./(1+1./115.*t.^2).^21+2199023255552./11835556670925./(1+1./115.*t.^2).^22+144115188075855872./916312070471295267./(1+1./115.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./115.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./115.*t.^2).^33+4294967296./20419054425./(1+1./115.*t.^2).^17+17179869184./83945001525./(1+1./115.*t.^2).^18);

        case 116;
            alpha(set_t)=1./2+1./2.*t./(116+t.^2).^(1./2).*(1+61989816618513./562949953421312./(1+1./116.*t.^2).^26+34461632205./274877906944./(1+1./116.*t.^2).^20+3184701532372607112841303575./39614081257132168796771975168./(1+1./116.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./116.*t.^2).^50+1804857108504066435./18446744073709551616./(1+1./116.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./116.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./116.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./116.*t.^2).^36+24975553429171528252000152507./316912650057057350374175801344./(1+1./116.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./116.*t.^2).^52+1./2./(1+1./116.*t.^2)+218266320541953276229./2361183241434822606848./(1+1./116.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./116.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./116.*t.^2).^39+3./8./(1+1./116.*t.^2).^2+5./16./(1+1./116.*t.^2).^3+35./128./(1+1./116.*t.^2).^4+392032816163265715595619229845./5070602400912917605986812821504./(1+1./116.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./116.*t.^2).^54+67282234305./549755813888./(1+1./116.*t.^2).^21+263012370465./2199023255552./(1+1./116.*t.^2).^22+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./116.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./116.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./116.*t.^2).^57+63./256./(1+1./116.*t.^2).^5+231./1024./(1+1./116.*t.^2).^6+429./2048./(1+1./116.*t.^2).^7+6435./32768./(1+1./116.*t.^2).^8+26876802183334044115405./302231454903657293676544./(1+1./116.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./116.*t.^2).^41+121683714103007./1125899906842624./(1+1./116.*t.^2).^27+956086325095055./9007199254740992./(1+1./116.*t.^2).^28+1879204156221315./18014398509481984./(1+1./116.*t.^2).^29+12155./65536./(1+1./116.*t.^2).^9+46189./262144./(1+1./116.*t.^2).^10+209863810776486386280915./2417851639229258349412352./(1+1./116.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./116.*t.^2).^43+514589420475./4398046511104./(1+1./116.*t.^2).^23+8061900920775./70368744177664./(1+1./116.*t.^2).^24+15801325804719./140737488355328./(1+1./116.*t.^2).^25+7391536347803839./72057594037927936./(1+1./116.*t.^2).^30+3281063172710606398620225./38685626227668133590597632./(1+1./116.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./116.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./116.*t.^2).^46+88179./524288./(1+1./116.*t.^2).^11+676039./4194304./(1+1./116.*t.^2).^12+1300075./8388608./(1+1./116.*t.^2).^13+50803160635786570329644235./618970019642690137449562112./(1+1./116.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./116.*t.^2).^48+5014575./33554432./(1+1./116.*t.^2).^14+9694845./67108864./(1+1./116.*t.^2).^15+14544636039226909./144115188075855872./(1+1./116.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./116.*t.^2).^32+300540195./2147483648./(1+1./116.*t.^2).^16+583401555./4294967296./(1+1./116.*t.^2).^17+2268783825./17179869184./(1+1./116.*t.^2).^18+4418157975./34359738368./(1+1./116.*t.^2).^19);

        case 117;
            alpha(set_t)=1./2+1./pi.*atan(1./117.*t.*117.^(1./2))+1./pi.*t.*117.^(1./2)./(117+t.^2).*(1+2./3./(1+1./117.*t.^2)+8./15./(1+1./117.*t.^2).^2+16./35./(1+1./117.*t.^2).^3+256./693./(1+1./117.*t.^2).^5+1024./3003./(1+1./117.*t.^2).^6+128./315./(1+1./117.*t.^2).^4+2048./6435./(1+1./117.*t.^2).^7+32768./109395./(1+1./117.*t.^2).^8+65536./230945./(1+1./117.*t.^2).^9+262144./969969./(1+1./117.*t.^2).^10+8388608./35102025./(1+1./117.*t.^2).^13+33554432./145422675./(1+1./117.*t.^2).^14+67108864./300540195./(1+1./117.*t.^2).^15+524288./2028117./(1+1./117.*t.^2).^11+4194304./16900975./(1+1./117.*t.^2).^12+2147483648./9917826435./(1+1./117.*t.^2).^16+4294967296./20419054425./(1+1./117.*t.^2).^17+73786976294838206464./490814965447899948765./(1+1./117.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./117.*t.^2).^35+17179869184./83945001525./(1+1./117.*t.^2).^18+34359738368./172308161025./(1+1./117.*t.^2).^19+19807040628566084398385987584./156050375086257748529223875175./(1+1./117.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./117.*t.^2).^49+1180591620717411303424./8075853860052271220473./(1+1./117.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./117.*t.^2).^37+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./117.*t.^2).^54+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./117.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./117.*t.^2).^53+549755813888./2893136075115./(1+1./117.*t.^2).^21+2199023255552./11835556670925./(1+1./117.*t.^2).^22+274877906944./1412926920405./(1+1./117.*t.^2).^20+158456325028528675187087900672./1273753224887747940852007777857./(1+1./117.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./117.*t.^2).^51+9444732965739290427392./66341473743672640538025./(1+1./117.*t.^2).^38+4398046511104./24185702762325./(1+1./117.*t.^2).^23+70368744177664./395033145117975./(1+1./117.*t.^2).^24+140737488355328./805867616040669./(1+1./117.*t.^2).^25+38685626227668133590597632./292014622371243969477200025./(1+1./117.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./117.*t.^2).^45+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./117.*t.^2).^55+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./117.*t.^2).^56+562949953421312./3285460280781189./(1+1./117.*t.^2).^26+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./117.*t.^2).^57+1125899906842624./6692604275665385./(1+1./117.*t.^2).^27+9007199254740992./54496920530418135./(1+1./117.*t.^2).^28+18014398509481984./110873045217057585./(1+1./117.*t.^2).^29+302231454903657293676544./2177020976850057573347805./(1+1./117.*t.^2).^40+72057594037927936./450883717216034179./(1+1./117.*t.^2).^30+309485009821345068724781056./2387748549881968805493279045./(1+1./117.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./117.*t.^2).^47+9223372036854775808./59560284580634192355./(1+1./117.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./117.*t.^2).^33+144115188075855872./916312070471295267./(1+1./117.*t.^2).^31+18889465931478580854784./134384010916670220577025./(1+1./117.*t.^2).^39+604462909807314587353088./4407140026306214111899215./(1+1./117.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./117.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./117.*t.^2).^43);

        case 118;
            alpha(set_t)=1./2+1./2.*t./(118+t.^2).^(1./2).*(1+34461632205./274877906944./(1+1./118.*t.^2).^20+67282234305./549755813888./(1+1./118.*t.^2).^21+14023284727082855679./147573952589676412928./(1+1./118.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./118.*t.^2).^36+88179./524288./(1+1./118.*t.^2).^11+676039./4194304./(1+1./118.*t.^2).^12+263012370465./2199023255552./(1+1./118.*t.^2).^22+514589420475./4398046511104./(1+1./118.*t.^2).^23+218266320541953276229./2361183241434822606848./(1+1./118.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./118.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./118.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./118.*t.^2).^40+8061900920775./70368744177664./(1+1./118.*t.^2).^24+15801325804719./140737488355328./(1+1./118.*t.^2).^25+53098072606098965203605./604462909807314587353088./(1+1./118.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./118.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./118.*t.^2).^43+61989816618513./562949953421312./(1+1./118.*t.^2).^26+121683714103007./1125899906842624./(1+1./118.*t.^2).^27+1300075./8388608./(1+1./118.*t.^2).^13+5014575./33554432./(1+1./118.*t.^2).^14+63./256./(1+1./118.*t.^2).^5+231./1024./(1+1./118.*t.^2).^6+429./2048./(1+1./118.*t.^2).^7+3281063172710606398620225./38685626227668133590597632./(1+1./118.*t.^2).^44+956086325095055./9007199254740992./(1+1./118.*t.^2).^28+1879204156221315./18014398509481984./(1+1./118.*t.^2).^29+7391536347803839./72057594037927936./(1+1./118.*t.^2).^30+14544636039226909./144115188075855872./(1+1./118.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./118.*t.^2).^32+6489213830472088210604445./77371252455336267181195264./(1+1./118.*t.^2).^45+6435./32768./(1+1./118.*t.^2).^8+300540195./2147483648./(1+1./118.*t.^2).^16+583401555./4294967296./(1+1./118.*t.^2).^17+9694845./67108864./(1+1./118.*t.^2).^15+25674715590128696833261065./309485009821345068724781056./(1+1./118.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./118.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./118.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./118.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./118.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./118.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./118.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./118.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./118.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./118.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./118.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./118.*t.^2).^57+12155./65536./(1+1./118.*t.^2).^9+46189./262144./(1+1./118.*t.^2).^10+2268783825./17179869184./(1+1./118.*t.^2).^18+4418157975./34359738368./(1+1./118.*t.^2).^19+1804857108504066435./18446744073709551616./(1+1./118.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./118.*t.^2).^34+1./2./(1+1./118.*t.^2)+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./118.*t.^2).^58+3./8./(1+1./118.*t.^2).^2+5./16./(1+1./118.*t.^2).^3+35./128./(1+1./118.*t.^2).^4);

        case 119;
            alpha(set_t)=1./2+1./pi.*atan(1./119.*t.*119.^(1./2))+1./pi.*t.*119.^(1./2)./(119+t.^2).*(1+2417851639229258349412352./17838423916001342833877775./(1+1./119.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./119.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./119.*t.^2).^44+144115188075855872./916312070471295267./(1+1./119.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./119.*t.^2).^32+2361183241434822606848./16369974040646495717175./(1+1./119.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./119.*t.^2).^38+18446744073709551616./120925426269772451145./(1+1./119.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./119.*t.^2).^34+77371252455336267181195264./590518458572960027165004495./(1+1./119.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./119.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./119.*t.^2).^47+2./3./(1+1./119.*t.^2)+8./15./(1+1./119.*t.^2).^2+19807040628566084398385987584./156050375086257748529223875175./(1+1./119.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./119.*t.^2).^49+16./35./(1+1./119.*t.^2).^3+18889465931478580854784./134384010916670220577025./(1+1./119.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./119.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./119.*t.^2).^41+2048./6435./(1+1./119.*t.^2).^7+32768./109395./(1+1./119.*t.^2).^8+128./315./(1+1./119.*t.^2).^4+256./693./(1+1./119.*t.^2).^5+1024./3003./(1+1./119.*t.^2).^6+158456325028528675187087900672./1273753224887747940852007777857./(1+1./119.*t.^2).^50+147573952589676412928./995653215622882753209./(1+1./119.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./119.*t.^2).^36+65536./230945./(1+1./119.*t.^2).^9+262144./969969./(1+1./119.*t.^2).^10+524288./2028117./(1+1./119.*t.^2).^11+4194304./16900975./(1+1./119.*t.^2).^12+8388608./35102025./(1+1./119.*t.^2).^13+2147483648./9917826435./(1+1./119.*t.^2).^16+4294967296./20419054425./(1+1./119.*t.^2).^17+33554432./145422675./(1+1./119.*t.^2).^14+67108864./300540195./(1+1./119.*t.^2).^15+17179869184./83945001525./(1+1./119.*t.^2).^18+34359738368./172308161025./(1+1./119.*t.^2).^19+274877906944./1412926920405./(1+1./119.*t.^2).^20+549755813888./2893136075115./(1+1./119.*t.^2).^21+2199023255552./11835556670925./(1+1./119.*t.^2).^22+316912650057057350374175801344./2572482003204667409956015708221./(1+1./119.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./119.*t.^2).^52+70368744177664./395033145117975./(1+1./119.*t.^2).^24+140737488355328./805867616040669./(1+1./119.*t.^2).^25+4398046511104./24185702762325./(1+1./119.*t.^2).^23+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./119.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./119.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./119.*t.^2).^55+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./119.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./119.*t.^2).^57+562949953421312./3285460280781189./(1+1./119.*t.^2).^26+1125899906842624./6692604275665385./(1+1./119.*t.^2).^27+9007199254740992./54496920530418135./(1+1./119.*t.^2).^28+18014398509481984./110873045217057585./(1+1./119.*t.^2).^29+72057594037927936./450883717216034179./(1+1./119.*t.^2).^30+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./119.*t.^2).^58);

        case 120;
            alpha(set_t)=1./2+1./2.*t./(120+t.^2).^(1./2).*(1+5./16./(1+1./120.*t.^2).^3+35./128./(1+1./120.*t.^2).^4+63./256./(1+1./120.*t.^2).^5+300540195./2147483648./(1+1./120.*t.^2).^16+583401555./4294967296./(1+1./120.*t.^2).^17+2268783825./17179869184./(1+1./120.*t.^2).^18+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./120.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./120.*t.^2).^58+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./120.*t.^2).^59+14023284727082855679./147573952589676412928./(1+1./120.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./120.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./120.*t.^2).^37+4418157975./34359738368./(1+1./120.*t.^2).^19+34461632205./274877906944./(1+1./120.*t.^2).^20+861577581086657669325./9444732965739290427392./(1+1./120.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./120.*t.^2).^39+6435./32768./(1+1./120.*t.^2).^8+12155./65536./(1+1./120.*t.^2).^9+231./1024./(1+1./120.*t.^2).^6+429./2048./(1+1./120.*t.^2).^7+12611418068195524166851562157./158456325028528675187087900672./(1+1./120.*t.^2).^50+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./120.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./120.*t.^2).^55+7391536347803839./72057594037927936./(1+1./120.*t.^2).^30+14544636039226909./144115188075855872./(1+1./120.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./120.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./120.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./120.*t.^2).^34+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./120.*t.^2).^56+46189./262144./(1+1./120.*t.^2).^10+88179./524288./(1+1./120.*t.^2).^11+676039./4194304./(1+1./120.*t.^2).^12+1300075./8388608./(1+1./120.*t.^2).^13+67282234305./549755813888./(1+1./120.*t.^2).^21+263012370465./2199023255552./(1+1./120.*t.^2).^22+514589420475./4398046511104./(1+1./120.*t.^2).^23+26876802183334044115405./302231454903657293676544./(1+1./120.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./120.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./120.*t.^2).^42+24975553429171528252000152507./316912650057057350374175801344./(1+1./120.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./120.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./120.*t.^2).^53+8061900920775./70368744177664./(1+1./120.*t.^2).^24+15801325804719./140737488355328./(1+1./120.*t.^2).^25+414847067813984717066925./4835703278458516698824704./(1+1./120.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./120.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./120.*t.^2).^45+1./2./(1+1./120.*t.^2)+3./8./(1+1./120.*t.^2).^2+61989816618513./562949953421312./(1+1./120.*t.^2).^26+121683714103007./1125899906842624./(1+1./120.*t.^2).^27+25674715590128696833261065./309485009821345068724781056./(1+1./120.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./120.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./120.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./120.*t.^2).^49+956086325095055./9007199254740992./(1+1./120.*t.^2).^28+1879204156221315./18014398509481984./(1+1./120.*t.^2).^29+5014575./33554432./(1+1./120.*t.^2).^14+9694845./67108864./(1+1./120.*t.^2).^15);

        case 121;
            alpha(set_t)=1./2+1./pi.*atan(1./121.*t.*121.^(1./2))+1./pi.*t.*121.^(1./2)./(121+t.^2).*(1+1125899906842624./6692604275665385./(1+1./121.*t.^2).^27+9007199254740992./54496920530418135./(1+1./121.*t.^2).^28+18014398509481984./110873045217057585./(1+1./121.*t.^2).^29+8388608./35102025./(1+1./121.*t.^2).^13+33554432./145422675./(1+1./121.*t.^2).^14+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./121.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./121.*t.^2).^55+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./121.*t.^2).^56+67108864./300540195./(1+1./121.*t.^2).^15+2147483648./9917826435./(1+1./121.*t.^2).^16+4294967296./20419054425./(1+1./121.*t.^2).^17+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./121.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./121.*t.^2).^58+1180591620717411303424./8075853860052271220473./(1+1./121.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./121.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./121.*t.^2).^38+72057594037927936./450883717216034179./(1+1./121.*t.^2).^30+144115188075855872./916312070471295267./(1+1./121.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./121.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./121.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./121.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./121.*t.^2).^35+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./121.*t.^2).^59+309485009821345068724781056./2387748549881968805493279045./(1+1./121.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./121.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./121.*t.^2).^48+70368744177664./395033145117975./(1+1./121.*t.^2).^24+140737488355328./805867616040669./(1+1./121.*t.^2).^25+18889465931478580854784./134384010916670220577025./(1+1./121.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./121.*t.^2).^40+17179869184./83945001525./(1+1./121.*t.^2).^18+34359738368./172308161025./(1+1./121.*t.^2).^19+274877906944./1412926920405./(1+1./121.*t.^2).^20+2./3./(1+1./121.*t.^2)+8./15./(1+1./121.*t.^2).^2+39614081257132168796771975168./315285451704888104171289053925./(1+1./121.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./121.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./121.*t.^2).^51+562949953421312./3285460280781189./(1+1./121.*t.^2).^26+16./35./(1+1./121.*t.^2).^3+128./315./(1+1./121.*t.^2).^4+2048./6435./(1+1./121.*t.^2).^7+32768./109395./(1+1./121.*t.^2).^8+65536./230945./(1+1./121.*t.^2).^9+604462909807314587353088./4407140026306214111899215./(1+1./121.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./121.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./121.*t.^2).^43+549755813888./2893136075115./(1+1./121.*t.^2).^21+2199023255552./11835556670925./(1+1./121.*t.^2).^22+4398046511104./24185702762325./(1+1./121.*t.^2).^23+262144./969969./(1+1./121.*t.^2).^10+524288./2028117./(1+1./121.*t.^2).^11+4194304./16900975./(1+1./121.*t.^2).^12+256./693./(1+1./121.*t.^2).^5+1024./3003./(1+1./121.*t.^2).^6+38685626227668133590597632./292014622371243969477200025./(1+1./121.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./121.*t.^2).^45+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./121.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./121.*t.^2).^53);

        case 122;
            alpha(set_t)=1./2+1./2.*t./(122+t.^2).^(1./2).*(1+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./122.*t.^2).^58+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./122.*t.^2).^59+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./122.*t.^2).^60+263012370465./2199023255552./(1+1./122.*t.^2).^22+514589420475./4398046511104./(1+1./122.*t.^2).^23+8061900920775./70368744177664./(1+1./122.*t.^2).^24+61989816618513./562949953421312./(1+1./122.*t.^2).^26+121683714103007./1125899906842624./(1+1./122.*t.^2).^27+956086325095055./9007199254740992./(1+1./122.*t.^2).^28+1879204156221315./18014398509481984./(1+1./122.*t.^2).^29+12611418068195524166851562157./158456325028528675187087900672./(1+1./122.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./122.*t.^2).^51+7391536347803839./72057594037927936./(1+1./122.*t.^2).^30+14544636039226909./144115188075855872./(1+1./122.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./122.*t.^2).^32+676039./4194304./(1+1./122.*t.^2).^12+1300075./8388608./(1+1./122.*t.^2).^13+25674715590128696833261065./309485009821345068724781056./(1+1./122.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./122.*t.^2).^47+1./2./(1+1./122.*t.^2)+197883231015743646919693516017./2535301200456458802993406410752./(1+1./122.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./122.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./122.*t.^2).^54+3./8./(1+1./122.*t.^2).^2+5./16./(1+1./122.*t.^2).^3+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./122.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./122.*t.^2).^56+5014575./33554432./(1+1./122.*t.^2).^14+9694845./67108864./(1+1./122.*t.^2).^15+300540195./2147483648./(1+1./122.*t.^2).^16+110628135069209194801./1180591620717411303424./(1+1./122.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./122.*t.^2).^37+583401555./4294967296./(1+1./122.*t.^2).^17+2268783825./17179869184./(1+1./122.*t.^2).^18+4418157975./34359738368./(1+1./122.*t.^2).^19+26876802183334044115405./302231454903657293676544./(1+1./122.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./122.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./122.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./122.*t.^2).^43+34461632205./274877906944./(1+1./122.*t.^2).^20+67282234305./549755813888./(1+1./122.*t.^2).^21+35./128./(1+1./122.*t.^2).^4+63./256./(1+1./122.*t.^2).^5+15801325804719./140737488355328./(1+1./122.*t.^2).^25+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./122.*t.^2).^57+861577581086657669325./9444732965739290427392./(1+1./122.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./122.*t.^2).^39+3281063172710606398620225./38685626227668133590597632./(1+1./122.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./122.*t.^2).^45+1608766753466574727105400775./19807040628566084398385987584./(1+1./122.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./122.*t.^2).^49+231./1024./(1+1./122.*t.^2).^6+429./2048./(1+1./122.*t.^2).^7+6435./32768./(1+1./122.*t.^2).^8+1804857108504066435./18446744073709551616./(1+1./122.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./122.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./122.*t.^2).^35+12155./65536./(1+1./122.*t.^2).^9+46189./262144./(1+1./122.*t.^2).^10+88179./524288./(1+1./122.*t.^2).^11);

        case 123;
            alpha(set_t)=1./2+1./pi.*atan(1./123.*t.*123.^(1./2))+1./pi.*t.*123.^(1./2)./(123+t.^2).*(1+8388608./35102025./(1+1./123.*t.^2).^13+33554432./145422675./(1+1./123.*t.^2).^14+67108864./300540195./(1+1./123.*t.^2).^15+2147483648./9917826435./(1+1./123.*t.^2).^16+1125899906842624./6692604275665385./(1+1./123.*t.^2).^27+9007199254740992./54496920530418135./(1+1./123.*t.^2).^28+18014398509481984./110873045217057585./(1+1./123.*t.^2).^29+4294967296./20419054425./(1+1./123.*t.^2).^17+17179869184./83945001525./(1+1./123.*t.^2).^18+34359738368./172308161025./(1+1./123.*t.^2).^19+302231454903657293676544./2177020976850057573347805./(1+1./123.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./123.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./123.*t.^2).^42+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./123.*t.^2).^55+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./123.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./123.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./123.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./123.*t.^2).^59+262144./969969./(1+1./123.*t.^2).^10+524288./2028117./(1+1./123.*t.^2).^11+4194304./16900975./(1+1./123.*t.^2).^12+274877906944./1412926920405./(1+1./123.*t.^2).^20+549755813888./2893136075115./(1+1./123.*t.^2).^21+2199023255552./11835556670925./(1+1./123.*t.^2).^22+128./315./(1+1./123.*t.^2).^4+256./693./(1+1./123.*t.^2).^5+1024./3003./(1+1./123.*t.^2).^6+72057594037927936./450883717216034179./(1+1./123.*t.^2).^30+144115188075855872./916312070471295267./(1+1./123.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./123.*t.^2).^32+4835703278458516698824704./36091694899816670384822475./(1+1./123.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./123.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./123.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./123.*t.^2).^46+4398046511104./24185702762325./(1+1./123.*t.^2).^23+70368744177664./395033145117975./(1+1./123.*t.^2).^24+140737488355328./805867616040669./(1+1./123.*t.^2).^25+562949953421312./3285460280781189./(1+1./123.*t.^2).^26+1180591620717411303424./8075853860052271220473./(1+1./123.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./123.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./123.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./123.*t.^2).^39+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./123.*t.^2).^60+2./3./(1+1./123.*t.^2)+8./15./(1+1./123.*t.^2).^2+16./35./(1+1./123.*t.^2).^3+618970019642690137449562112./4826300260399724181316202325./(1+1./123.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./123.*t.^2).^48+2048./6435./(1+1./123.*t.^2).^7+32768./109395./(1+1./123.*t.^2).^8+65536./230945./(1+1./123.*t.^2).^9+39614081257132168796771975168./315285451704888104171289053925./(1+1./123.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./123.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./123.*t.^2).^51+18446744073709551616./120925426269772451145./(1+1./123.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./123.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./123.*t.^2).^35+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./123.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./123.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./123.*t.^2).^54);

        case 124;
            alpha(set_t)=1./2+1./2.*t./(124+t.^2).^(1./2).*(1+14544636039226909./144115188075855872./(1+1./124.*t.^2).^31+1804857108504066435./18446744073709551616./(1+1./124.*t.^2).^33+583401555./4294967296./(1+1./124.*t.^2).^17+2268783825./17179869184./(1+1./124.*t.^2).^18+4418157975./34359738368./(1+1./124.*t.^2).^19+1./2./(1+1./124.*t.^2)+3./8./(1+1./124.*t.^2).^2+7113260368810144185./73786976294838206464./(1+1./124.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./124.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./124.*t.^2).^36+34461632205./274877906944./(1+1./124.*t.^2).^20+67282234305./549755813888./(1+1./124.*t.^2).^21+263012370465./2199023255552./(1+1./124.*t.^2).^22+514589420475./4398046511104./(1+1./124.*t.^2).^23+5./16./(1+1./124.*t.^2).^3+35./128./(1+1./124.*t.^2).^4+63./256./(1+1./124.*t.^2).^5+231./1024./(1+1./124.*t.^2).^6+429./2048./(1+1./124.*t.^2).^7+6435./32768./(1+1./124.*t.^2).^8+916312070471295267./9223372036854775808./(1+1./124.*t.^2).^32+218266320541953276229./2361183241434822606848./(1+1./124.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./124.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./124.*t.^2).^39+88179./524288./(1+1./124.*t.^2).^11+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./124.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./124.*t.^2).^61+26876802183334044115405./302231454903657293676544./(1+1./124.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./124.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./124.*t.^2).^42+12155./65536./(1+1./124.*t.^2).^9+46189./262144./(1+1./124.*t.^2).^10+414847067813984717066925./4835703278458516698824704./(1+1./124.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./124.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./124.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./124.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./124.*t.^2).^47+676039./4194304./(1+1./124.*t.^2).^12+1300075./8388608./(1+1./124.*t.^2).^13+1608766753466574727105400775./19807040628566084398385987584./(1+1./124.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./124.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./124.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./124.*t.^2).^51+8061900920775./70368744177664./(1+1./124.*t.^2).^24+15801325804719./140737488355328./(1+1./124.*t.^2).^25+61989816618513./562949953421312./(1+1./124.*t.^2).^26+5014575./33554432./(1+1./124.*t.^2).^14+9694845./67108864./(1+1./124.*t.^2).^15+300540195./2147483648./(1+1./124.*t.^2).^16+197883231015743646919693516017./2535301200456458802993406410752./(1+1./124.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./124.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./124.*t.^2).^54+121683714103007./1125899906842624./(1+1./124.*t.^2).^27+956086325095055./9007199254740992./(1+1./124.*t.^2).^28+1879204156221315./18014398509481984./(1+1./124.*t.^2).^29+7391536347803839./72057594037927936./(1+1./124.*t.^2).^30+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./124.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./124.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./124.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./124.*t.^2).^58+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./124.*t.^2).^59);

        case 125;
            alpha(set_t)=1./2+1./pi.*atan(1./125.*t.*125.^(1./2))+1./pi.*t.*125.^(1./2)./(125+t.^2).*(1+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./125.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./125.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./125.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./125.*t.^2).^59+128./315./(1+1./125.*t.^2).^4+256./693./(1+1./125.*t.^2).^5+158456325028528675187087900672./1273753224887747940852007777857./(1+1./125.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./125.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./125.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./125.*t.^2).^53+302231454903657293676544./2177020976850057573347805./(1+1./125.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./125.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./125.*t.^2).^42+1024./3003./(1+1./125.*t.^2).^6+2048./6435./(1+1./125.*t.^2).^7+32768./109395./(1+1./125.*t.^2).^8+309485009821345068724781056./2387748549881968805493279045./(1+1./125.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./125.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./125.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./125.*t.^2).^49+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./125.*t.^2).^54+2361183241434822606848./16369974040646495717175./(1+1./125.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./125.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./125.*t.^2).^39+4835703278458516698824704./36091694899816670384822475./(1+1./125.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./125.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./125.*t.^2).^45+144115188075855872./916312070471295267./(1+1./125.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./125.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./125.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./125.*t.^2).^34+1180591620717411303424./8075853860052271220473./(1+1./125.*t.^2).^36+147573952589676412928./995653215622882753209./(1+1./125.*t.^2).^35+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./125.*t.^2).^60+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./125.*t.^2).^61+65536./230945./(1+1./125.*t.^2).^9+262144./969969./(1+1./125.*t.^2).^10+524288./2028117./(1+1./125.*t.^2).^11+4194304./16900975./(1+1./125.*t.^2).^12+274877906944./1412926920405./(1+1./125.*t.^2).^20+549755813888./2893136075115./(1+1./125.*t.^2).^21+2199023255552./11835556670925./(1+1./125.*t.^2).^22+4398046511104./24185702762325./(1+1./125.*t.^2).^23+70368744177664./395033145117975./(1+1./125.*t.^2).^24+140737488355328./805867616040669./(1+1./125.*t.^2).^25+562949953421312./3285460280781189./(1+1./125.*t.^2).^26+1125899906842624./6692604275665385./(1+1./125.*t.^2).^27+9007199254740992./54496920530418135./(1+1./125.*t.^2).^28+18014398509481984./110873045217057585./(1+1./125.*t.^2).^29+72057594037927936./450883717216034179./(1+1./125.*t.^2).^30+4294967296./20419054425./(1+1./125.*t.^2).^17+17179869184./83945001525./(1+1./125.*t.^2).^18+34359738368./172308161025./(1+1./125.*t.^2).^19+8388608./35102025./(1+1./125.*t.^2).^13+33554432./145422675./(1+1./125.*t.^2).^14+67108864./300540195./(1+1./125.*t.^2).^15+2147483648./9917826435./(1+1./125.*t.^2).^16+2./3./(1+1./125.*t.^2)+8./15./(1+1./125.*t.^2).^2+16./35./(1+1./125.*t.^2).^3+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./125.*t.^2).^55);

        case 126;
            alpha(set_t)=1./2+1./2.*t./(126+t.^2).^(1./2).*(1+1./2./(1+1./126.*t.^2)+24975553429171528252000152507./316912650057057350374175801344./(1+1./126.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./126.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./126.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./126.*t.^2).^54+6435./32768./(1+1./126.*t.^2).^8+12155./65536./(1+1./126.*t.^2).^9+46189./262144./(1+1./126.*t.^2).^10+88179./524288./(1+1./126.*t.^2).^11+676039./4194304./(1+1./126.*t.^2).^12+1300075./8388608./(1+1./126.*t.^2).^13+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./126.*t.^2).^62+5014575./33554432./(1+1./126.*t.^2).^14+9694845./67108864./(1+1./126.*t.^2).^15+300540195./2147483648./(1+1./126.*t.^2).^16+583401555./4294967296./(1+1./126.*t.^2).^17+2268783825./17179869184./(1+1./126.*t.^2).^18+4418157975./34359738368./(1+1./126.*t.^2).^19+34461632205./274877906944./(1+1./126.*t.^2).^20+50803160635786570329644235./618970019642690137449562112./(1+1./126.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./126.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./126.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./126.*t.^2).^50+3./8./(1+1./126.*t.^2).^2+5./16./(1+1./126.*t.^2).^3+35./128./(1+1./126.*t.^2).^4+67282234305./549755813888./(1+1./126.*t.^2).^21+263012370465./2199023255552./(1+1./126.*t.^2).^22+514589420475./4398046511104./(1+1./126.*t.^2).^23+8061900920775./70368744177664./(1+1./126.*t.^2).^24+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./126.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./126.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./126.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./126.*t.^2).^58+110628135069209194801./1180591620717411303424./(1+1./126.*t.^2).^36+14544636039226909./144115188075855872./(1+1./126.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./126.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./126.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./126.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./126.*t.^2).^35+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./126.*t.^2).^59+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./126.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./126.*t.^2).^61+209863810776486386280915./2417851639229258349412352./(1+1./126.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./126.*t.^2).^43+218266320541953276229./2361183241434822606848./(1+1./126.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./126.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./126.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./126.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./126.*t.^2).^41+15801325804719./140737488355328./(1+1./126.*t.^2).^25+61989816618513./562949953421312./(1+1./126.*t.^2).^26+121683714103007./1125899906842624./(1+1./126.*t.^2).^27+3281063172710606398620225./38685626227668133590597632./(1+1./126.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./126.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./126.*t.^2).^46+63./256./(1+1./126.*t.^2).^5+231./1024./(1+1./126.*t.^2).^6+429./2048./(1+1./126.*t.^2).^7+956086325095055./9007199254740992./(1+1./126.*t.^2).^28+1879204156221315./18014398509481984./(1+1./126.*t.^2).^29+7391536347803839./72057594037927936./(1+1./126.*t.^2).^30);

        case 127;
            alpha(set_t)=1./2+1./pi.*atan(1./127.*t.*127.^(1./2))+1./pi.*t.*127.^(1./2)./(127+t.^2).*(1+524288./2028117./(1+1./127.*t.^2).^11+4194304./16900975./(1+1./127.*t.^2).^12+8388608./35102025./(1+1./127.*t.^2).^13+33554432./145422675./(1+1./127.*t.^2).^14+549755813888./2893136075115./(1+1./127.*t.^2).^21+2199023255552./11835556670925./(1+1./127.*t.^2).^22+4398046511104./24185702762325./(1+1./127.*t.^2).^23+70368744177664./395033145117975./(1+1./127.*t.^2).^24+274877906944./1412926920405./(1+1./127.*t.^2).^20+140737488355328./805867616040669./(1+1./127.*t.^2).^25+562949953421312./3285460280781189./(1+1./127.*t.^2).^26+1125899906842624./6692604275665385./(1+1./127.*t.^2).^27+9007199254740992./54496920530418135./(1+1./127.*t.^2).^28+18014398509481984./110873045217057585./(1+1./127.*t.^2).^29+67108864./300540195./(1+1./127.*t.^2).^15+2048./6435./(1+1./127.*t.^2).^7+32768./109395./(1+1./127.*t.^2).^8+65536./230945./(1+1./127.*t.^2).^9+262144./969969./(1+1./127.*t.^2).^10+2./3./(1+1./127.*t.^2)+8./15./(1+1./127.*t.^2).^2+16./35./(1+1./127.*t.^2).^3+128./315./(1+1./127.*t.^2).^4+256./693./(1+1./127.*t.^2).^5+2147483648./9917826435./(1+1./127.*t.^2).^16+4294967296./20419054425./(1+1./127.*t.^2).^17+17179869184./83945001525./(1+1./127.*t.^2).^18+34359738368./172308161025./(1+1./127.*t.^2).^19+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./127.*t.^2).^55+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./127.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./127.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./127.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./127.*t.^2).^59+72057594037927936./450883717216034179./(1+1./127.*t.^2).^30+144115188075855872./916312070471295267./(1+1./127.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./127.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./127.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./127.*t.^2).^34+1180591620717411303424./8075853860052271220473./(1+1./127.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./127.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./127.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./127.*t.^2).^39+147573952589676412928./995653215622882753209./(1+1./127.*t.^2).^35+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./127.*t.^2).^60+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./127.*t.^2).^61+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./127.*t.^2).^62+302231454903657293676544./2177020976850057573347805./(1+1./127.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./127.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./127.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./127.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./127.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./127.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./127.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./127.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./127.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./127.*t.^2).^49+1024./3003./(1+1./127.*t.^2).^6+158456325028528675187087900672./1273753224887747940852007777857./(1+1./127.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./127.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./127.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./127.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./127.*t.^2).^54);

        case 128;
            alpha(set_t)=1./2+1./2.*t./(128+t.^2).^(1./2).*(1+14544636039226909./144115188075855872./(1+1./128.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./128.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./128.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./128.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./128.*t.^2).^35+26876802183334044115405./302231454903657293676544./(1+1./128.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./128.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./128.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./128.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./128.*t.^2).^44+514589420475./4398046511104./(1+1./128.*t.^2).^23+6489213830472088210604445./77371252455336267181195264./(1+1./128.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./128.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./128.*t.^2).^47+231./1024./(1+1./128.*t.^2).^6+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./128.*t.^2).^59+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./128.*t.^2).^60+1608766753466574727105400775./19807040628566084398385987584./(1+1./128.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./128.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./128.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./128.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./128.*t.^2).^52+5./16./(1+1./128.*t.^2).^3+35./128./(1+1./128.*t.^2).^4+63./256./(1+1./128.*t.^2).^5+121683714103007./1125899906842624./(1+1./128.*t.^2).^27+956086325095055./9007199254740992./(1+1./128.*t.^2).^28+1879204156221315./18014398509481984./(1+1./128.*t.^2).^29+7391536347803839./72057594037927936./(1+1./128.*t.^2).^30+8061900920775./70368744177664./(1+1./128.*t.^2).^24+15801325804719./140737488355328./(1+1./128.*t.^2).^25+61989816618513./562949953421312./(1+1./128.*t.^2).^26+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./128.*t.^2).^58+9694845./67108864./(1+1./128.*t.^2).^15+300540195./2147483648./(1+1./128.*t.^2).^16+583401555./4294967296./(1+1./128.*t.^2).^17+46189./262144./(1+1./128.*t.^2).^10+88179./524288./(1+1./128.*t.^2).^11+676039./4194304./(1+1./128.*t.^2).^12+1300075./8388608./(1+1./128.*t.^2).^13+5014575./33554432./(1+1./128.*t.^2).^14+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./128.*t.^2).^61+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./128.*t.^2).^62+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./128.*t.^2).^63+110628135069209194801./1180591620717411303424./(1+1./128.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./128.*t.^2).^37+1701063429324939500975./18889465931478580854784./(1+1./128.*t.^2).^39+861577581086657669325./9444732965739290427392./(1+1./128.*t.^2).^38+2268783825./17179869184./(1+1./128.*t.^2).^18+4418157975./34359738368./(1+1./128.*t.^2).^19+34461632205./274877906944./(1+1./128.*t.^2).^20+67282234305./549755813888./(1+1./128.*t.^2).^21+263012370465./2199023255552./(1+1./128.*t.^2).^22+392032816163265715595619229845./5070602400912917605986812821504./(1+1./128.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./128.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./128.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./128.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./128.*t.^2).^57+429./2048./(1+1./128.*t.^2).^7+6435./32768./(1+1./128.*t.^2).^8+12155./65536./(1+1./128.*t.^2).^9+1./2./(1+1./128.*t.^2)+3./8./(1+1./128.*t.^2).^2);

        case 129;
            alpha(set_t)=1./2+1./pi.*atan(1./129.*t.*129.^(1./2))+1./pi.*t.*129.^(1./2)./(129+t.^2).*(1+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./129.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./129.*t.^2).^63+256./693./(1+1./129.*t.^2).^5+1024./3003./(1+1./129.*t.^2).^6+2048./6435./(1+1./129.*t.^2).^7+32768./109395./(1+1./129.*t.^2).^8+65536./230945./(1+1./129.*t.^2).^9+38685626227668133590597632./292014622371243969477200025./(1+1./129.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./129.*t.^2).^45+140737488355328./805867616040669./(1+1./129.*t.^2).^25+562949953421312./3285460280781189./(1+1./129.*t.^2).^26+1125899906842624./6692604275665385./(1+1./129.*t.^2).^27+9007199254740992./54496920530418135./(1+1./129.*t.^2).^28+18014398509481984./110873045217057585./(1+1./129.*t.^2).^29+2./3./(1+1./129.*t.^2)+8./15./(1+1./129.*t.^2).^2+16./35./(1+1./129.*t.^2).^3+128./315./(1+1./129.*t.^2).^4+274877906944./1412926920405./(1+1./129.*t.^2).^20+549755813888./2893136075115./(1+1./129.*t.^2).^21+2199023255552./11835556670925./(1+1./129.*t.^2).^22+4398046511104./24185702762325./(1+1./129.*t.^2).^23+70368744177664./395033145117975./(1+1./129.*t.^2).^24+9223372036854775808./59560284580634192355./(1+1./129.*t.^2).^32+73786976294838206464./490814965447899948765./(1+1./129.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./129.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./129.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./129.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./129.*t.^2).^38+67108864./300540195./(1+1./129.*t.^2).^15+2147483648./9917826435./(1+1./129.*t.^2).^16+4294967296./20419054425./(1+1./129.*t.^2).^17+17179869184./83945001525./(1+1./129.*t.^2).^18+34359738368./172308161025./(1+1./129.*t.^2).^19+262144./969969./(1+1./129.*t.^2).^10+524288./2028117./(1+1./129.*t.^2).^11+4194304./16900975./(1+1./129.*t.^2).^12+8388608./35102025./(1+1./129.*t.^2).^13+33554432./145422675./(1+1./129.*t.^2).^14+18446744073709551616./120925426269772451145./(1+1./129.*t.^2).^33+72057594037927936./450883717216034179./(1+1./129.*t.^2).^30+144115188075855872./916312070471295267./(1+1./129.*t.^2).^31+309485009821345068724781056./2387748549881968805493279045./(1+1./129.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./129.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./129.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./129.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./129.*t.^2).^50+18889465931478580854784./134384010916670220577025./(1+1./129.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./129.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./129.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./129.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./129.*t.^2).^43+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./129.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./129.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./129.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./129.*t.^2).^59+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./129.*t.^2).^60+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./129.*t.^2).^61+316912650057057350374175801344./2572482003204667409956015708221./(1+1./129.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./129.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./129.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./129.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./129.*t.^2).^55);

        case 130;
            alpha(set_t)=1./2+1./2.*t./(130+t.^2).^(1./2).*(1+1./2./(1+1./130.*t.^2)+3./8./(1+1./130.*t.^2).^2+5./16./(1+1./130.*t.^2).^3+35./128./(1+1./130.*t.^2).^4+63./256./(1+1./130.*t.^2).^5+231./1024./(1+1./130.*t.^2).^6+429./2048./(1+1./130.*t.^2).^7+6435./32768./(1+1./130.*t.^2).^8+12155./65536./(1+1./130.*t.^2).^9+46189./262144./(1+1./130.*t.^2).^10+88179./524288./(1+1./130.*t.^2).^11+676039./4194304./(1+1./130.*t.^2).^12+1300075./8388608./(1+1./130.*t.^2).^13+5014575./33554432./(1+1./130.*t.^2).^14+9694845./67108864./(1+1./130.*t.^2).^15+300540195./2147483648./(1+1./130.*t.^2).^16+2268783825./17179869184./(1+1./130.*t.^2).^18+583401555./4294967296./(1+1./130.*t.^2).^17+4418157975./34359738368./(1+1./130.*t.^2).^19+34461632205./274877906944./(1+1./130.*t.^2).^20+67282234305./549755813888./(1+1./130.*t.^2).^21+263012370465./2199023255552./(1+1./130.*t.^2).^22+514589420475./4398046511104./(1+1./130.*t.^2).^23+8061900920775./70368744177664./(1+1./130.*t.^2).^24+61989816618513./562949953421312./(1+1./130.*t.^2).^26+15801325804719./140737488355328./(1+1./130.*t.^2).^25+121683714103007./1125899906842624./(1+1./130.*t.^2).^27+956086325095055./9007199254740992./(1+1./130.*t.^2).^28+7391536347803839./72057594037927936./(1+1./130.*t.^2).^30+1879204156221315./18014398509481984./(1+1./130.*t.^2).^29+14544636039226909./144115188075855872./(1+1./130.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./130.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./130.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./130.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./130.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./130.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./130.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./130.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./130.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./130.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./130.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./130.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./130.*t.^2).^43+6489213830472088210604445./77371252455336267181195264./(1+1./130.*t.^2).^45+3281063172710606398620225./38685626227668133590597632./(1+1./130.*t.^2).^44+25674715590128696833261065./309485009821345068724781056./(1+1./130.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./130.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./130.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./130.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./130.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./130.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./130.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./130.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./130.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./130.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./130.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./130.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./130.*t.^2).^58+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./130.*t.^2).^59+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./130.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./130.*t.^2).^61+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./130.*t.^2).^62+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./130.*t.^2).^63+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./130.*t.^2).^64);

        case 131;
            alpha(set_t)=1./2+1./pi.*atan(1./131.*t.*131.^(1./2))+1./pi.*t.*131.^(1./2)./(131+t.^2).*(1+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./131.*t.^2).^61+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./131.*t.^2).^60+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./131.*t.^2).^63+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./131.*t.^2).^62+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./131.*t.^2).^64+2./3./(1+1./131.*t.^2)+8./15./(1+1./131.*t.^2).^2+16./35./(1+1./131.*t.^2).^3+128./315./(1+1./131.*t.^2).^4+256./693./(1+1./131.*t.^2).^5+1024./3003./(1+1./131.*t.^2).^6+2048./6435./(1+1./131.*t.^2).^7+32768./109395./(1+1./131.*t.^2).^8+65536./230945./(1+1./131.*t.^2).^9+262144./969969./(1+1./131.*t.^2).^10+524288./2028117./(1+1./131.*t.^2).^11+4194304./16900975./(1+1./131.*t.^2).^12+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./131.*t.^2).^56+33554432./145422675./(1+1./131.*t.^2).^14+8388608./35102025./(1+1./131.*t.^2).^13+67108864./300540195./(1+1./131.*t.^2).^15+2147483648./9917826435./(1+1./131.*t.^2).^16+4294967296./20419054425./(1+1./131.*t.^2).^17+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./131.*t.^2).^57+17179869184./83945001525./(1+1./131.*t.^2).^18+34359738368./172308161025./(1+1./131.*t.^2).^19+274877906944./1412926920405./(1+1./131.*t.^2).^20+549755813888./2893136075115./(1+1./131.*t.^2).^21+4398046511104./24185702762325./(1+1./131.*t.^2).^23+2199023255552./11835556670925./(1+1./131.*t.^2).^22+70368744177664./395033145117975./(1+1./131.*t.^2).^24+140737488355328./805867616040669./(1+1./131.*t.^2).^25+562949953421312./3285460280781189./(1+1./131.*t.^2).^26+1125899906842624./6692604275665385./(1+1./131.*t.^2).^27+9007199254740992./54496920530418135./(1+1./131.*t.^2).^28+18014398509481984./110873045217057585./(1+1./131.*t.^2).^29+72057594037927936./450883717216034179./(1+1./131.*t.^2).^30+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./131.*t.^2).^58+144115188075855872./916312070471295267./(1+1./131.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./131.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./131.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./131.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./131.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./131.*t.^2).^36+9444732965739290427392./66341473743672640538025./(1+1./131.*t.^2).^38+2361183241434822606848./16369974040646495717175./(1+1./131.*t.^2).^37+18889465931478580854784./134384010916670220577025./(1+1./131.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./131.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./131.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./131.*t.^2).^42+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./131.*t.^2).^59+4835703278458516698824704./36091694899816670384822475./(1+1./131.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./131.*t.^2).^44+309485009821345068724781056./2387748549881968805493279045./(1+1./131.*t.^2).^46+77371252455336267181195264./590518458572960027165004495./(1+1./131.*t.^2).^45+618970019642690137449562112./4826300260399724181316202325./(1+1./131.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./131.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./131.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./131.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./131.*t.^2).^51+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./131.*t.^2).^53+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./131.*t.^2).^52+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./131.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./131.*t.^2).^55);

        case 132;
            alpha(set_t)=1./2+1./2.*t./(132+t.^2).^(1./2).*(1+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./132.*t.^2).^65+34461632205./274877906944./(1+1./132.*t.^2).^20+67282234305./549755813888./(1+1./132.*t.^2).^21+392032816163265715595619229845./5070602400912917605986812821504./(1+1./132.*t.^2).^53+263012370465./2199023255552./(1+1./132.*t.^2).^22+514589420475./4398046511104./(1+1./132.*t.^2).^23+8061900920775./70368744177664./(1+1./132.*t.^2).^24+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./132.*t.^2).^54+15801325804719./140737488355328./(1+1./132.*t.^2).^25+61989816618513./562949953421312./(1+1./132.*t.^2).^26+121683714103007./1125899906842624./(1+1./132.*t.^2).^27+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./132.*t.^2).^55+956086325095055./9007199254740992./(1+1./132.*t.^2).^28+1879204156221315./18014398509481984./(1+1./132.*t.^2).^29+9694845./67108864./(1+1./132.*t.^2).^15+7391536347803839./72057594037927936./(1+1./132.*t.^2).^30+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./132.*t.^2).^56+14544636039226909./144115188075855872./(1+1./132.*t.^2).^31+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./132.*t.^2).^57+916312070471295267./9223372036854775808./(1+1./132.*t.^2).^32+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./132.*t.^2).^58+1804857108504066435./18446744073709551616./(1+1./132.*t.^2).^33+300540195./2147483648./(1+1./132.*t.^2).^16+7113260368810144185./73786976294838206464./(1+1./132.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./132.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./132.*t.^2).^36+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./132.*t.^2).^59+218266320541953276229./2361183241434822606848./(1+1./132.*t.^2).^37+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./132.*t.^2).^60+861577581086657669325./9444732965739290427392./(1+1./132.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./132.*t.^2).^39+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./132.*t.^2).^61+1./2./(1+1./132.*t.^2)+26876802183334044115405./302231454903657293676544./(1+1./132.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./132.*t.^2).^41+3./8./(1+1./132.*t.^2).^2+209863810776486386280915./2417851639229258349412352./(1+1./132.*t.^2).^42+5./16./(1+1./132.*t.^2).^3+35./128./(1+1./132.*t.^2).^4+583401555./4294967296./(1+1./132.*t.^2).^17+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./132.*t.^2).^62+414847067813984717066925./4835703278458516698824704./(1+1./132.*t.^2).^43+63./256./(1+1./132.*t.^2).^5+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./132.*t.^2).^63+3281063172710606398620225./38685626227668133590597632./(1+1./132.*t.^2).^44+231./1024./(1+1./132.*t.^2).^6+429./2048./(1+1./132.*t.^2).^7+6489213830472088210604445./77371252455336267181195264./(1+1./132.*t.^2).^45+6435./32768./(1+1./132.*t.^2).^8+25674715590128696833261065./309485009821345068724781056./(1+1./132.*t.^2).^46+12155./65536./(1+1./132.*t.^2).^9+50803160635786570329644235./618970019642690137449562112./(1+1./132.*t.^2).^47+46189./262144./(1+1./132.*t.^2).^10+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./132.*t.^2).^64+1608766753466574727105400775./19807040628566084398385987584./(1+1./132.*t.^2).^48+2268783825./17179869184./(1+1./132.*t.^2).^18+88179./524288./(1+1./132.*t.^2).^11+3184701532372607112841303575./39614081257132168796771975168./(1+1./132.*t.^2).^49+676039./4194304./(1+1./132.*t.^2).^12+4418157975./34359738368./(1+1./132.*t.^2).^19+12611418068195524166851562157./158456325028528675187087900672./(1+1./132.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./132.*t.^2).^51+1300075./8388608./(1+1./132.*t.^2).^13+197883231015743646919693516017./2535301200456458802993406410752./(1+1./132.*t.^2).^52+5014575./33554432./(1+1./132.*t.^2).^14);

        case 133;
            alpha(set_t)=1./2+1./pi.*atan(1./133.*t.*133.^(1./2))+1./pi.*t.*133.^(1./2)./(133+t.^2).*(1+8./15./(1+1./133.*t.^2).^2+2./3./(1+1./133.*t.^2)+128./315./(1+1./133.*t.^2).^4+16./35./(1+1./133.*t.^2).^3+2048./6435./(1+1./133.*t.^2).^7+1024./3003./(1+1./133.*t.^2).^6+256./693./(1+1./133.*t.^2).^5+65536./230945./(1+1./133.*t.^2).^9+32768./109395./(1+1./133.*t.^2).^8+524288./2028117./(1+1./133.*t.^2).^11+262144./969969./(1+1./133.*t.^2).^10+4194304./16900975./(1+1./133.*t.^2).^12+33554432./145422675./(1+1./133.*t.^2).^14+8388608./35102025./(1+1./133.*t.^2).^13+67108864./300540195./(1+1./133.*t.^2).^15+2147483648./9917826435./(1+1./133.*t.^2).^16+4294967296./20419054425./(1+1./133.*t.^2).^17+17179869184./83945001525./(1+1./133.*t.^2).^18+274877906944./1412926920405./(1+1./133.*t.^2).^20+34359738368./172308161025./(1+1./133.*t.^2).^19+549755813888./2893136075115./(1+1./133.*t.^2).^21+2199023255552./11835556670925./(1+1./133.*t.^2).^22+4398046511104./24185702762325./(1+1./133.*t.^2).^23+70368744177664./395033145117975./(1+1./133.*t.^2).^24+140737488355328./805867616040669./(1+1./133.*t.^2).^25+562949953421312./3285460280781189./(1+1./133.*t.^2).^26+1125899906842624./6692604275665385./(1+1./133.*t.^2).^27+9007199254740992./54496920530418135./(1+1./133.*t.^2).^28+18014398509481984./110873045217057585./(1+1./133.*t.^2).^29+72057594037927936./450883717216034179./(1+1./133.*t.^2).^30+144115188075855872./916312070471295267./(1+1./133.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./133.*t.^2).^32+73786976294838206464./490814965447899948765./(1+1./133.*t.^2).^34+18446744073709551616./120925426269772451145./(1+1./133.*t.^2).^33+1180591620717411303424./8075853860052271220473./(1+1./133.*t.^2).^36+147573952589676412928./995653215622882753209./(1+1./133.*t.^2).^35+9444732965739290427392./66341473743672640538025./(1+1./133.*t.^2).^38+2361183241434822606848./16369974040646495717175./(1+1./133.*t.^2).^37+18889465931478580854784./134384010916670220577025./(1+1./133.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./133.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./133.*t.^2).^41+4835703278458516698824704./36091694899816670384822475./(1+1./133.*t.^2).^43+2417851639229258349412352./17838423916001342833877775./(1+1./133.*t.^2).^42+38685626227668133590597632./292014622371243969477200025./(1+1./133.*t.^2).^44+309485009821345068724781056./2387748549881968805493279045./(1+1./133.*t.^2).^46+77371252455336267181195264./590518458572960027165004495./(1+1./133.*t.^2).^45+19807040628566084398385987584./156050375086257748529223875175./(1+1./133.*t.^2).^48+618970019642690137449562112./4826300260399724181316202325./(1+1./133.*t.^2).^47+158456325028528675187087900672./1273753224887747940852007777857./(1+1./133.*t.^2).^50+39614081257132168796771975168./315285451704888104171289053925./(1+1./133.*t.^2).^49+316912650057057350374175801344./2572482003204667409956015708221./(1+1./133.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./133.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./133.*t.^2).^53+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./133.*t.^2).^55+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./133.*t.^2).^54+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./133.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./133.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./133.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./133.*t.^2).^59+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./133.*t.^2).^61+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./133.*t.^2).^60+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./133.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./133.*t.^2).^63+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./133.*t.^2).^64+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./133.*t.^2).^65);

        case 134;
            alpha(set_t)=1./2+1./2.*t./(134+t.^2).^(1./2).*(1+15801325804719./140737488355328./(1+1./134.*t.^2).^25+61989816618513./562949953421312./(1+1./134.*t.^2).^26+12155./65536./(1+1./134.*t.^2).^9+121683714103007./1125899906842624./(1+1./134.*t.^2).^27+956086325095055./9007199254740992./(1+1./134.*t.^2).^28+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./134.*t.^2).^62+7391536347803839./72057594037927936./(1+1./134.*t.^2).^30+1879204156221315./18014398509481984./(1+1./134.*t.^2).^29+14544636039226909./144115188075855872./(1+1./134.*t.^2).^31+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./134.*t.^2).^63+46189./262144./(1+1./134.*t.^2).^10+88179./524288./(1+1./134.*t.^2).^11+1804857108504066435./18446744073709551616./(1+1./134.*t.^2).^33+916312070471295267./9223372036854775808./(1+1./134.*t.^2).^32+676039./4194304./(1+1./134.*t.^2).^12+7113260368810144185./73786976294838206464./(1+1./134.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./134.*t.^2).^35+1300075./8388608./(1+1./134.*t.^2).^13+110628135069209194801./1180591620717411303424./(1+1./134.*t.^2).^36+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./134.*t.^2).^64+5014575./33554432./(1+1./134.*t.^2).^14+218266320541953276229./2361183241434822606848./(1+1./134.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./134.*t.^2).^38+9694845./67108864./(1+1./134.*t.^2).^15+1701063429324939500975./18889465931478580854784./(1+1./134.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./134.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./134.*t.^2).^41+300540195./2147483648./(1+1./134.*t.^2).^16+209863810776486386280915./2417851639229258349412352./(1+1./134.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./134.*t.^2).^43+583401555./4294967296./(1+1./134.*t.^2).^17+1./2./(1+1./134.*t.^2)+3281063172710606398620225./38685626227668133590597632./(1+1./134.*t.^2).^44+2268783825./17179869184./(1+1./134.*t.^2).^18+6489213830472088210604445./77371252455336267181195264./(1+1./134.*t.^2).^45+3./8./(1+1./134.*t.^2).^2+25674715590128696833261065./309485009821345068724781056./(1+1./134.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./134.*t.^2).^47+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./134.*t.^2).^65+1608766753466574727105400775./19807040628566084398385987584./(1+1./134.*t.^2).^48+5./16./(1+1./134.*t.^2).^3+3184701532372607112841303575./39614081257132168796771975168./(1+1./134.*t.^2).^49+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./134.*t.^2).^66+12611418068195524166851562157./158456325028528675187087900672./(1+1./134.*t.^2).^50+4418157975./34359738368./(1+1./134.*t.^2).^19+24975553429171528252000152507./316912650057057350374175801344./(1+1./134.*t.^2).^51+35./128./(1+1./134.*t.^2).^4+34461632205./274877906944./(1+1./134.*t.^2).^20+197883231015743646919693516017./2535301200456458802993406410752./(1+1./134.*t.^2).^52+67282234305./549755813888./(1+1./134.*t.^2).^21+392032816163265715595619229845./5070602400912917605986812821504./(1+1./134.*t.^2).^53+63./256./(1+1./134.*t.^2).^5+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./134.*t.^2).^54+263012370465./2199023255552./(1+1./134.*t.^2).^22+231./1024./(1+1./134.*t.^2).^6+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./134.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./134.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./134.*t.^2).^57+514589420475./4398046511104./(1+1./134.*t.^2).^23+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./134.*t.^2).^58+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./134.*t.^2).^59+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./134.*t.^2).^60+429./2048./(1+1./134.*t.^2).^7+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./134.*t.^2).^61+8061900920775./70368744177664./(1+1./134.*t.^2).^24+6435./32768./(1+1./134.*t.^2).^8);

        case 135;
            alpha(set_t)=1./2+1./pi.*atan(1./135.*t.*135.^(1./2))+1./pi.*t.*135.^(1./2)./(135+t.^2).*(1+39614081257132168796771975168./315285451704888104171289053925./(1+1./135.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./135.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./135.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./135.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./135.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./135.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./135.*t.^2).^55+2417851639229258349412352./17838423916001342833877775./(1+1./135.*t.^2).^42+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./135.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./135.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./135.*t.^2).^58+4835703278458516698824704./36091694899816670384822475./(1+1./135.*t.^2).^43+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./135.*t.^2).^59+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./135.*t.^2).^60+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./135.*t.^2).^61+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./135.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./135.*t.^2).^63+38685626227668133590597632./292014622371243969477200025./(1+1./135.*t.^2).^44+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./135.*t.^2).^64+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./135.*t.^2).^65+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./135.*t.^2).^66+77371252455336267181195264./590518458572960027165004495./(1+1./135.*t.^2).^45+16./35./(1+1./135.*t.^2).^3+8./15./(1+1./135.*t.^2).^2+2./3./(1+1./135.*t.^2)+128./315./(1+1./135.*t.^2).^4+309485009821345068724781056./2387748549881968805493279045./(1+1./135.*t.^2).^46+1024./3003./(1+1./135.*t.^2).^6+256./693./(1+1./135.*t.^2).^5+2048./6435./(1+1./135.*t.^2).^7+65536./230945./(1+1./135.*t.^2).^9+32768./109395./(1+1./135.*t.^2).^8+262144./969969./(1+1./135.*t.^2).^10+4194304./16900975./(1+1./135.*t.^2).^12+524288./2028117./(1+1./135.*t.^2).^11+8388608./35102025./(1+1./135.*t.^2).^13+33554432./145422675./(1+1./135.*t.^2).^14+2147483648./9917826435./(1+1./135.*t.^2).^16+67108864./300540195./(1+1./135.*t.^2).^15+4294967296./20419054425./(1+1./135.*t.^2).^17+34359738368./172308161025./(1+1./135.*t.^2).^19+17179869184./83945001525./(1+1./135.*t.^2).^18+549755813888./2893136075115./(1+1./135.*t.^2).^21+274877906944./1412926920405./(1+1./135.*t.^2).^20+4398046511104./24185702762325./(1+1./135.*t.^2).^23+2199023255552./11835556670925./(1+1./135.*t.^2).^22+70368744177664./395033145117975./(1+1./135.*t.^2).^24+140737488355328./805867616040669./(1+1./135.*t.^2).^25+1125899906842624./6692604275665385./(1+1./135.*t.^2).^27+562949953421312./3285460280781189./(1+1./135.*t.^2).^26+18014398509481984./110873045217057585./(1+1./135.*t.^2).^29+9007199254740992./54496920530418135./(1+1./135.*t.^2).^28+72057594037927936./450883717216034179./(1+1./135.*t.^2).^30+144115188075855872./916312070471295267./(1+1./135.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./135.*t.^2).^32+147573952589676412928./995653215622882753209./(1+1./135.*t.^2).^35+73786976294838206464./490814965447899948765./(1+1./135.*t.^2).^34+18446744073709551616./120925426269772451145./(1+1./135.*t.^2).^33+1180591620717411303424./8075853860052271220473./(1+1./135.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./135.*t.^2).^37+618970019642690137449562112./4826300260399724181316202325./(1+1./135.*t.^2).^47+9444732965739290427392./66341473743672640538025./(1+1./135.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./135.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./135.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./135.*t.^2).^41+19807040628566084398385987584./156050375086257748529223875175./(1+1./135.*t.^2).^48);

        case 136;
            alpha(set_t)=1./2+1./2.*t./(136+t.^2).^(1./2).*(1+861577581086657669325./9444732965739290427392./(1+1./136.*t.^2).^38+12611418068195524166851562157./158456325028528675187087900672./(1+1./136.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./136.*t.^2).^51+61989816618513./562949953421312./(1+1./136.*t.^2).^26+197883231015743646919693516017./2535301200456458802993406410752./(1+1./136.*t.^2).^52+3./8./(1+1./136.*t.^2).^2+392032816163265715595619229845./5070602400912917605986812821504./(1+1./136.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./136.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./136.*t.^2).^55+5./16./(1+1./136.*t.^2).^3+1701063429324939500975./18889465931478580854784./(1+1./136.*t.^2).^39+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./136.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./136.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./136.*t.^2).^58+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./136.*t.^2).^60+121683714103007./1125899906842624./(1+1./136.*t.^2).^27+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./136.*t.^2).^59+35./128./(1+1./136.*t.^2).^4+63./256./(1+1./136.*t.^2).^5+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./136.*t.^2).^61+231./1024./(1+1./136.*t.^2).^6+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./136.*t.^2).^62+956086325095055./9007199254740992./(1+1./136.*t.^2).^28+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./136.*t.^2).^64+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./136.*t.^2).^63+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./136.*t.^2).^67+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./136.*t.^2).^66+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./136.*t.^2).^65+26876802183334044115405./302231454903657293676544./(1+1./136.*t.^2).^40+429./2048./(1+1./136.*t.^2).^7+1879204156221315./18014398509481984./(1+1./136.*t.^2).^29+53098072606098965203605./604462909807314587353088./(1+1./136.*t.^2).^41+6435./32768./(1+1./136.*t.^2).^8+7391536347803839./72057594037927936./(1+1./136.*t.^2).^30+12155./65536./(1+1./136.*t.^2).^9+14544636039226909./144115188075855872./(1+1./136.*t.^2).^31+209863810776486386280915./2417851639229258349412352./(1+1./136.*t.^2).^42+46189./262144./(1+1./136.*t.^2).^10+88179./524288./(1+1./136.*t.^2).^11+414847067813984717066925./4835703278458516698824704./(1+1./136.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./136.*t.^2).^44+676039./4194304./(1+1./136.*t.^2).^12+1804857108504066435./18446744073709551616./(1+1./136.*t.^2).^33+916312070471295267./9223372036854775808./(1+1./136.*t.^2).^32+1300075./8388608./(1+1./136.*t.^2).^13+6489213830472088210604445./77371252455336267181195264./(1+1./136.*t.^2).^45+7113260368810144185./73786976294838206464./(1+1./136.*t.^2).^34+5014575./33554432./(1+1./136.*t.^2).^14+9694845./67108864./(1+1./136.*t.^2).^15+300540195./2147483648./(1+1./136.*t.^2).^16+25674715590128696833261065./309485009821345068724781056./(1+1./136.*t.^2).^46+14023284727082855679./147573952589676412928./(1+1./136.*t.^2).^35+50803160635786570329644235./618970019642690137449562112./(1+1./136.*t.^2).^47+583401555./4294967296./(1+1./136.*t.^2).^17+2268783825./17179869184./(1+1./136.*t.^2).^18+4418157975./34359738368./(1+1./136.*t.^2).^19+110628135069209194801./1180591620717411303424./(1+1./136.*t.^2).^36+34461632205./274877906944./(1+1./136.*t.^2).^20+67282234305./549755813888./(1+1./136.*t.^2).^21+263012370465./2199023255552./(1+1./136.*t.^2).^22+514589420475./4398046511104./(1+1./136.*t.^2).^23+218266320541953276229./2361183241434822606848./(1+1./136.*t.^2).^37+1608766753466574727105400775./19807040628566084398385987584./(1+1./136.*t.^2).^48+8061900920775./70368744177664./(1+1./136.*t.^2).^24+3184701532372607112841303575./39614081257132168796771975168./(1+1./136.*t.^2).^49+15801325804719./140737488355328./(1+1./136.*t.^2).^25+1./2./(1+1./136.*t.^2));

        case 137;
            alpha(set_t)=1./2+1./pi.*atan(1./137.*t.*137.^(1./2))+1./pi.*t.*137.^(1./2)./(137+t.^2).*(1+38685626227668133590597632./292014622371243969477200025./(1+1./137.*t.^2).^44+8388608./35102025./(1+1./137.*t.^2).^13+33554432./145422675./(1+1./137.*t.^2).^14+128./315./(1+1./137.*t.^2).^4+77371252455336267181195264./590518458572960027165004495./(1+1./137.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./137.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./137.*t.^2).^47+39614081257132168796771975168./315285451704888104171289053925./(1+1./137.*t.^2).^49+72057594037927936./450883717216034179./(1+1./137.*t.^2).^30+19807040628566084398385987584./156050375086257748529223875175./(1+1./137.*t.^2).^48+256./693./(1+1./137.*t.^2).^5+67108864./300540195./(1+1./137.*t.^2).^15+2147483648./9917826435./(1+1./137.*t.^2).^16+158456325028528675187087900672./1273753224887747940852007777857./(1+1./137.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./137.*t.^2).^51+144115188075855872./916312070471295267./(1+1./137.*t.^2).^31+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./137.*t.^2).^52+4294967296./20419054425./(1+1./137.*t.^2).^17+1024./3003./(1+1./137.*t.^2).^6+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./137.*t.^2).^53+17179869184./83945001525./(1+1./137.*t.^2).^18+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./137.*t.^2).^54+34359738368./172308161025./(1+1./137.*t.^2).^19+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./137.*t.^2).^55+9223372036854775808./59560284580634192355./(1+1./137.*t.^2).^32+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./137.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./137.*t.^2).^57+274877906944./1412926920405./(1+1./137.*t.^2).^20+2048./6435./(1+1./137.*t.^2).^7+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./137.*t.^2).^58+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./137.*t.^2).^60+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./137.*t.^2).^59+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./137.*t.^2).^61+73786976294838206464./490814965447899948765./(1+1./137.*t.^2).^34+18446744073709551616./120925426269772451145./(1+1./137.*t.^2).^33+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./137.*t.^2).^62+32768./109395./(1+1./137.*t.^2).^8+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./137.*t.^2).^64+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./137.*t.^2).^63+549755813888./2893136075115./(1+1./137.*t.^2).^21+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./137.*t.^2).^65+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./137.*t.^2).^66+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./137.*t.^2).^67+147573952589676412928./995653215622882753209./(1+1./137.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./137.*t.^2).^36+2199023255552./11835556670925./(1+1./137.*t.^2).^22+65536./230945./(1+1./137.*t.^2).^9+2361183241434822606848./16369974040646495717175./(1+1./137.*t.^2).^37+4398046511104./24185702762325./(1+1./137.*t.^2).^23+70368744177664./395033145117975./(1+1./137.*t.^2).^24+262144./969969./(1+1./137.*t.^2).^10+9444732965739290427392./66341473743672640538025./(1+1./137.*t.^2).^38+140737488355328./805867616040669./(1+1./137.*t.^2).^25+562949953421312./3285460280781189./(1+1./137.*t.^2).^26+18889465931478580854784./134384010916670220577025./(1+1./137.*t.^2).^39+1125899906842624./6692604275665385./(1+1./137.*t.^2).^27+604462909807314587353088./4407140026306214111899215./(1+1./137.*t.^2).^41+302231454903657293676544./2177020976850057573347805./(1+1./137.*t.^2).^40+524288./2028117./(1+1./137.*t.^2).^11+9007199254740992./54496920530418135./(1+1./137.*t.^2).^28+2417851639229258349412352./17838423916001342833877775./(1+1./137.*t.^2).^42+4194304./16900975./(1+1./137.*t.^2).^12+18014398509481984./110873045217057585./(1+1./137.*t.^2).^29+2./3./(1+1./137.*t.^2)+8./15./(1+1./137.*t.^2).^2+16./35./(1+1./137.*t.^2).^3+4835703278458516698824704./36091694899816670384822475./(1+1./137.*t.^2).^43);

        case 138;
            alpha(set_t)=1./2+1./2.*t./(138+t.^2).^(1./2).*(1+1./2./(1+1./138.*t.^2)+3./8./(1+1./138.*t.^2).^2+6489213830472088210604445./77371252455336267181195264./(1+1./138.*t.^2).^45+5./16./(1+1./138.*t.^2).^3+35./128./(1+1./138.*t.^2).^4+861577581086657669325./9444732965739290427392./(1+1./138.*t.^2).^38+63./256./(1+1./138.*t.^2).^5+231./1024./(1+1./138.*t.^2).^6+25674715590128696833261065./309485009821345068724781056./(1+1./138.*t.^2).^46+1701063429324939500975./18889465931478580854784./(1+1./138.*t.^2).^39+429./2048./(1+1./138.*t.^2).^7+50803160635786570329644235./618970019642690137449562112./(1+1./138.*t.^2).^47+6435./32768./(1+1./138.*t.^2).^8+12155./65536./(1+1./138.*t.^2).^9+1608766753466574727105400775./19807040628566084398385987584./(1+1./138.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./138.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./138.*t.^2).^50+88179./524288./(1+1./138.*t.^2).^11+46189./262144./(1+1./138.*t.^2).^10+676039./4194304./(1+1./138.*t.^2).^12+24975553429171528252000152507./316912650057057350374175801344./(1+1./138.*t.^2).^51+1300075./8388608./(1+1./138.*t.^2).^13+5014575./33554432./(1+1./138.*t.^2).^14+9694845./67108864./(1+1./138.*t.^2).^15+197883231015743646919693516017./2535301200456458802993406410752./(1+1./138.*t.^2).^52+26876802183334044115405./302231454903657293676544./(1+1./138.*t.^2).^40+392032816163265715595619229845./5070602400912917605986812821504./(1+1./138.*t.^2).^53+300540195./2147483648./(1+1./138.*t.^2).^16+583401555./4294967296./(1+1./138.*t.^2).^17+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./138.*t.^2).^54+2268783825./17179869184./(1+1./138.*t.^2).^18+53098072606098965203605./604462909807314587353088./(1+1./138.*t.^2).^41+4418157975./34359738368./(1+1./138.*t.^2).^19+67282234305./549755813888./(1+1./138.*t.^2).^21+34461632205./274877906944./(1+1./138.*t.^2).^20+263012370465./2199023255552./(1+1./138.*t.^2).^22+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./138.*t.^2).^55+514589420475./4398046511104./(1+1./138.*t.^2).^23+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./138.*t.^2).^56+8061900920775./70368744177664./(1+1./138.*t.^2).^24+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./138.*t.^2).^57+15801325804719./140737488355328./(1+1./138.*t.^2).^25+209863810776486386280915./2417851639229258349412352./(1+1./138.*t.^2).^42+61989816618513./562949953421312./(1+1./138.*t.^2).^26+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./138.*t.^2).^58+956086325095055./9007199254740992./(1+1./138.*t.^2).^28+121683714103007./1125899906842624./(1+1./138.*t.^2).^27+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./138.*t.^2).^59+7391536347803839./72057594037927936./(1+1./138.*t.^2).^30+1879204156221315./18014398509481984./(1+1./138.*t.^2).^29+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./138.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./138.*t.^2).^61+14544636039226909./144115188075855872./(1+1./138.*t.^2).^31+414847067813984717066925./4835703278458516698824704./(1+1./138.*t.^2).^43+916312070471295267./9223372036854775808./(1+1./138.*t.^2).^32+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./138.*t.^2).^62+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./138.*t.^2).^63+1804857108504066435./18446744073709551616./(1+1./138.*t.^2).^33+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./138.*t.^2).^65+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./138.*t.^2).^64+7113260368810144185./73786976294838206464./(1+1./138.*t.^2).^34+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./138.*t.^2).^66+14023284727082855679./147573952589676412928./(1+1./138.*t.^2).^35+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./138.*t.^2).^67+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./138.*t.^2).^68+110628135069209194801./1180591620717411303424./(1+1./138.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./138.*t.^2).^37+3281063172710606398620225./38685626227668133590597632./(1+1./138.*t.^2).^44);

        case 139;
            alpha(set_t)=1./2+1./pi.*atan(1./139.*t.*139.^(1./2))+1./pi.*t.*139.^(1./2)./(139+t.^2).*(1+2147483648./9917826435./(1+1./139.*t.^2).^16+4294967296./20419054425./(1+1./139.*t.^2).^17+17179869184./83945001525./(1+1./139.*t.^2).^18+34359738368./172308161025./(1+1./139.*t.^2).^19+16./35./(1+1./139.*t.^2).^3+549755813888./2893136075115./(1+1./139.*t.^2).^21+274877906944./1412926920405./(1+1./139.*t.^2).^20+2199023255552./11835556670925./(1+1./139.*t.^2).^22+128./315./(1+1./139.*t.^2).^4+4398046511104./24185702762325./(1+1./139.*t.^2).^23+70368744177664./395033145117975./(1+1./139.*t.^2).^24+140737488355328./805867616040669./(1+1./139.*t.^2).^25+1125899906842624./6692604275665385./(1+1./139.*t.^2).^27+562949953421312./3285460280781189./(1+1./139.*t.^2).^26+9007199254740992./54496920530418135./(1+1./139.*t.^2).^28+144115188075855872./916312070471295267./(1+1./139.*t.^2).^31+72057594037927936./450883717216034179./(1+1./139.*t.^2).^30+18014398509481984./110873045217057585./(1+1./139.*t.^2).^29+9223372036854775808./59560284580634192355./(1+1./139.*t.^2).^32+73786976294838206464./490814965447899948765./(1+1./139.*t.^2).^34+18446744073709551616./120925426269772451145./(1+1./139.*t.^2).^33+147573952589676412928./995653215622882753209./(1+1./139.*t.^2).^35+2361183241434822606848./16369974040646495717175./(1+1./139.*t.^2).^37+256./693./(1+1./139.*t.^2).^5+1180591620717411303424./8075853860052271220473./(1+1./139.*t.^2).^36+9444732965739290427392./66341473743672640538025./(1+1./139.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./139.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./139.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./139.*t.^2).^41+4835703278458516698824704./36091694899816670384822475./(1+1./139.*t.^2).^43+2417851639229258349412352./17838423916001342833877775./(1+1./139.*t.^2).^42+38685626227668133590597632./292014622371243969477200025./(1+1./139.*t.^2).^44+1024./3003./(1+1./139.*t.^2).^6+77371252455336267181195264./590518458572960027165004495./(1+1./139.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./139.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./139.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./139.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./139.*t.^2).^49+2048./6435./(1+1./139.*t.^2).^7+158456325028528675187087900672./1273753224887747940852007777857./(1+1./139.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./139.*t.^2).^51+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./139.*t.^2).^53+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./139.*t.^2).^52+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./139.*t.^2).^54+32768./109395./(1+1./139.*t.^2).^8+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./139.*t.^2).^56+65536./230945./(1+1./139.*t.^2).^9+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./139.*t.^2).^55+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./139.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./139.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./139.*t.^2).^59+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./139.*t.^2).^61+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./139.*t.^2).^60+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./139.*t.^2).^63+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./139.*t.^2).^62+262144./969969./(1+1./139.*t.^2).^10+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./139.*t.^2).^64+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./139.*t.^2).^65+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./139.*t.^2).^66+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./139.*t.^2).^67+524288./2028117./(1+1./139.*t.^2).^11+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./139.*t.^2).^68+4194304./16900975./(1+1./139.*t.^2).^12+8388608./35102025./(1+1./139.*t.^2).^13+33554432./145422675./(1+1./139.*t.^2).^14+8./15./(1+1./139.*t.^2).^2+67108864./300540195./(1+1./139.*t.^2).^15+2./3./(1+1./139.*t.^2));

        case 140;
            alpha(set_t)=1./2+1./2.*t./(140+t.^2).^(1./2).*(1+2268783825./17179869184./(1+1./140.*t.^2).^18+4418157975./34359738368./(1+1./140.*t.^2).^19+3281063172710606398620225./38685626227668133590597632./(1+1./140.*t.^2).^44+3./8./(1+1./140.*t.^2).^2+6489213830472088210604445./77371252455336267181195264./(1+1./140.*t.^2).^45+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./140.*t.^2).^66+34461632205./274877906944./(1+1./140.*t.^2).^20+5./16./(1+1./140.*t.^2).^3+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./140.*t.^2).^67+25674715590128696833261065./309485009821345068724781056./(1+1./140.*t.^2).^46+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./140.*t.^2).^68+50803160635786570329644235./618970019642690137449562112./(1+1./140.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./140.*t.^2).^48+35./128./(1+1./140.*t.^2).^4+3184701532372607112841303575./39614081257132168796771975168./(1+1./140.*t.^2).^49+67282234305./549755813888./(1+1./140.*t.^2).^21+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./140.*t.^2).^69+1701063429324939500975./18889465931478580854784./(1+1./140.*t.^2).^39+12611418068195524166851562157./158456325028528675187087900672./(1+1./140.*t.^2).^50+63./256./(1+1./140.*t.^2).^5+24975553429171528252000152507./316912650057057350374175801344./(1+1./140.*t.^2).^51+263012370465./2199023255552./(1+1./140.*t.^2).^22+26876802183334044115405./302231454903657293676544./(1+1./140.*t.^2).^40+197883231015743646919693516017./2535301200456458802993406410752./(1+1./140.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./140.*t.^2).^53+514589420475./4398046511104./(1+1./140.*t.^2).^23+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./140.*t.^2).^54+231./1024./(1+1./140.*t.^2).^6+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./140.*t.^2).^55+429./2048./(1+1./140.*t.^2).^7+8061900920775./70368744177664./(1+1./140.*t.^2).^24+6435./32768./(1+1./140.*t.^2).^8+15801325804719./140737488355328./(1+1./140.*t.^2).^25+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./140.*t.^2).^56+61989816618513./562949953421312./(1+1./140.*t.^2).^26+53098072606098965203605./604462909807314587353088./(1+1./140.*t.^2).^41+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./140.*t.^2).^57+121683714103007./1125899906842624./(1+1./140.*t.^2).^27+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./140.*t.^2).^58+1879204156221315./18014398509481984./(1+1./140.*t.^2).^29+12155./65536./(1+1./140.*t.^2).^9+956086325095055./9007199254740992./(1+1./140.*t.^2).^28+46189./262144./(1+1./140.*t.^2).^10+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./140.*t.^2).^59+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./140.*t.^2).^60+88179./524288./(1+1./140.*t.^2).^11+209863810776486386280915./2417851639229258349412352./(1+1./140.*t.^2).^42+676039./4194304./(1+1./140.*t.^2).^12+7391536347803839./72057594037927936./(1+1./140.*t.^2).^30+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./140.*t.^2).^61+1300075./8388608./(1+1./140.*t.^2).^13+14544636039226909./144115188075855872./(1+1./140.*t.^2).^31+5014575./33554432./(1+1./140.*t.^2).^14+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./140.*t.^2).^62+916312070471295267./9223372036854775808./(1+1./140.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./140.*t.^2).^33+9694845./67108864./(1+1./140.*t.^2).^15+7113260368810144185./73786976294838206464./(1+1./140.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./140.*t.^2).^35+300540195./2147483648./(1+1./140.*t.^2).^16+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./140.*t.^2).^63+110628135069209194801./1180591620717411303424./(1+1./140.*t.^2).^36+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./140.*t.^2).^64+218266320541953276229./2361183241434822606848./(1+1./140.*t.^2).^37+583401555./4294967296./(1+1./140.*t.^2).^17+414847067813984717066925./4835703278458516698824704./(1+1./140.*t.^2).^43+861577581086657669325./9444732965739290427392./(1+1./140.*t.^2).^38+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./140.*t.^2).^65+1./2./(1+1./140.*t.^2));

        case 141;
            alpha(set_t)=1./2+1./pi.*atan(1./141.*t.*141.^(1./2))+1./pi.*t.*141.^(1./2)./(141+t.^2).*(1+302231454903657293676544./2177020976850057573347805./(1+1./141.*t.^2).^40+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./141.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./141.*t.^2).^63+604462909807314587353088./4407140026306214111899215./(1+1./141.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./141.*t.^2).^42+562949953421312./3285460280781189./(1+1./141.*t.^2).^26+1125899906842624./6692604275665385./(1+1./141.*t.^2).^27+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./141.*t.^2).^64+4835703278458516698824704./36091694899816670384822475./(1+1./141.*t.^2).^43+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./141.*t.^2).^65+9007199254740992./54496920530418135./(1+1./141.*t.^2).^28+18014398509481984./110873045217057585./(1+1./141.*t.^2).^29+38685626227668133590597632./292014622371243969477200025./(1+1./141.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./141.*t.^2).^45+72057594037927936./450883717216034179./(1+1./141.*t.^2).^30+309485009821345068724781056./2387748549881968805493279045./(1+1./141.*t.^2).^46+19807040628566084398385987584./156050375086257748529223875175./(1+1./141.*t.^2).^48+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./141.*t.^2).^66+618970019642690137449562112./4826300260399724181316202325./(1+1./141.*t.^2).^47+39614081257132168796771975168./315285451704888104171289053925./(1+1./141.*t.^2).^49+144115188075855872./916312070471295267./(1+1./141.*t.^2).^31+158456325028528675187087900672./1273753224887747940852007777857./(1+1./141.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./141.*t.^2).^51+9223372036854775808./59560284580634192355./(1+1./141.*t.^2).^32+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./141.*t.^2).^67+18446744073709551616./120925426269772451145./(1+1./141.*t.^2).^33+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./141.*t.^2).^52+73786976294838206464./490814965447899948765./(1+1./141.*t.^2).^34+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./141.*t.^2).^68+147573952589676412928./995653215622882753209./(1+1./141.*t.^2).^35+2./3./(1+1./141.*t.^2)+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./141.*t.^2).^53+8./15./(1+1./141.*t.^2).^2+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./141.*t.^2).^55+16./35./(1+1./141.*t.^2).^3+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./141.*t.^2).^54+1180591620717411303424./8075853860052271220473./(1+1./141.*t.^2).^36+128./315./(1+1./141.*t.^2).^4+256./693./(1+1./141.*t.^2).^5+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./141.*t.^2).^56+1024./3003./(1+1./141.*t.^2).^6+32768./109395./(1+1./141.*t.^2).^8+2048./6435./(1+1./141.*t.^2).^7+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./141.*t.^2).^57+65536./230945./(1+1./141.*t.^2).^9+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./141.*t.^2).^58+262144./969969./(1+1./141.*t.^2).^10+524288./2028117./(1+1./141.*t.^2).^11+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./141.*t.^2).^59+4194304./16900975./(1+1./141.*t.^2).^12+2361183241434822606848./16369974040646495717175./(1+1./141.*t.^2).^37+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./141.*t.^2).^69+33554432./145422675./(1+1./141.*t.^2).^14+8388608./35102025./(1+1./141.*t.^2).^13+67108864./300540195./(1+1./141.*t.^2).^15+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./141.*t.^2).^60+9444732965739290427392./66341473743672640538025./(1+1./141.*t.^2).^38+2147483648./9917826435./(1+1./141.*t.^2).^16+4294967296./20419054425./(1+1./141.*t.^2).^17+17179869184./83945001525./(1+1./141.*t.^2).^18+274877906944./1412926920405./(1+1./141.*t.^2).^20+34359738368./172308161025./(1+1./141.*t.^2).^19+549755813888./2893136075115./(1+1./141.*t.^2).^21+2199023255552./11835556670925./(1+1./141.*t.^2).^22+18889465931478580854784./134384010916670220577025./(1+1./141.*t.^2).^39+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./141.*t.^2).^61+4398046511104./24185702762325./(1+1./141.*t.^2).^23+70368744177664./395033145117975./(1+1./141.*t.^2).^24+140737488355328./805867616040669./(1+1./141.*t.^2).^25);

        case 142;
            alpha(set_t)=1./2+1./2.*t./(142+t.^2).^(1./2).*(1+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./142.*t.^2).^56+8061900920775./70368744177664./(1+1./142.*t.^2).^24+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./142.*t.^2).^57+2268783825./17179869184./(1+1./142.*t.^2).^18+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./142.*t.^2).^58+861577581086657669325./9444732965739290427392./(1+1./142.*t.^2).^38+4418157975./34359738368./(1+1./142.*t.^2).^19+61989816618513./562949953421312./(1+1./142.*t.^2).^26+15801325804719./140737488355328./(1+1./142.*t.^2).^25+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./142.*t.^2).^59+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./142.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./142.*t.^2).^61+121683714103007./1125899906842624./(1+1./142.*t.^2).^27+1701063429324939500975./18889465931478580854784./(1+1./142.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./142.*t.^2).^40+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./142.*t.^2).^62+53098072606098965203605./604462909807314587353088./(1+1./142.*t.^2).^41+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./142.*t.^2).^63+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./142.*t.^2).^64+209863810776486386280915./2417851639229258349412352./(1+1./142.*t.^2).^42+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./142.*t.^2).^65+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./142.*t.^2).^66+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./142.*t.^2).^67+956086325095055./9007199254740992./(1+1./142.*t.^2).^28+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./142.*t.^2).^69+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./142.*t.^2).^68+414847067813984717066925./4835703278458516698824704./(1+1./142.*t.^2).^43+1879204156221315./18014398509481984./(1+1./142.*t.^2).^29+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./142.*t.^2).^70+7391536347803839./72057594037927936./(1+1./142.*t.^2).^30+3281063172710606398620225./38685626227668133590597632./(1+1./142.*t.^2).^44+34461632205./274877906944./(1+1./142.*t.^2).^20+14544636039226909./144115188075855872./(1+1./142.*t.^2).^31+6489213830472088210604445./77371252455336267181195264./(1+1./142.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./142.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./142.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./142.*t.^2).^48+916312070471295267./9223372036854775808./(1+1./142.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./142.*t.^2).^33+1./2./(1+1./142.*t.^2)+3184701532372607112841303575./39614081257132168796771975168./(1+1./142.*t.^2).^49+3./8./(1+1./142.*t.^2).^2+67282234305./549755813888./(1+1./142.*t.^2).^21+5./16./(1+1./142.*t.^2).^3+12611418068195524166851562157./158456325028528675187087900672./(1+1./142.*t.^2).^50+263012370465./2199023255552./(1+1./142.*t.^2).^22+35./128./(1+1./142.*t.^2).^4+24975553429171528252000152507./316912650057057350374175801344./(1+1./142.*t.^2).^51+231./1024./(1+1./142.*t.^2).^6+63./256./(1+1./142.*t.^2).^5+7113260368810144185./73786976294838206464./(1+1./142.*t.^2).^34+197883231015743646919693516017./2535301200456458802993406410752./(1+1./142.*t.^2).^52+514589420475./4398046511104./(1+1./142.*t.^2).^23+6435./32768./(1+1./142.*t.^2).^8+429./2048./(1+1./142.*t.^2).^7+14023284727082855679./147573952589676412928./(1+1./142.*t.^2).^35+12155./65536./(1+1./142.*t.^2).^9+110628135069209194801./1180591620717411303424./(1+1./142.*t.^2).^36+392032816163265715595619229845./5070602400912917605986812821504./(1+1./142.*t.^2).^53+88179./524288./(1+1./142.*t.^2).^11+46189./262144./(1+1./142.*t.^2).^10+676039./4194304./(1+1./142.*t.^2).^12+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./142.*t.^2).^54+5014575./33554432./(1+1./142.*t.^2).^14+1300075./8388608./(1+1./142.*t.^2).^13+300540195./2147483648./(1+1./142.*t.^2).^16+9694845./67108864./(1+1./142.*t.^2).^15+218266320541953276229./2361183241434822606848./(1+1./142.*t.^2).^37+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./142.*t.^2).^55+583401555./4294967296./(1+1./142.*t.^2).^17);

        case 143;
            alpha(set_t)=1./2+1./pi.*atan(1./143.*t.*143.^(1./2))+1./pi.*t.*143.^(1./2)./(143+t.^2).*(1+2199023255552./11835556670925./(1+1./143.*t.^2).^22+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./143.*t.^2).^68+1180591620717411303424./8075853860052271220473./(1+1./143.*t.^2).^36+147573952589676412928./995653215622882753209./(1+1./143.*t.^2).^35+73786976294838206464./490814965447899948765./(1+1./143.*t.^2).^34+2361183241434822606848./16369974040646495717175./(1+1./143.*t.^2).^37+4398046511104./24185702762325./(1+1./143.*t.^2).^23+9444732965739290427392./66341473743672640538025./(1+1./143.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./143.*t.^2).^39+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./143.*t.^2).^69+302231454903657293676544./2177020976850057573347805./(1+1./143.*t.^2).^40+70368744177664./395033145117975./(1+1./143.*t.^2).^24+2./3./(1+1./143.*t.^2)+604462909807314587353088./4407140026306214111899215./(1+1./143.*t.^2).^41+140737488355328./805867616040669./(1+1./143.*t.^2).^25+8./15./(1+1./143.*t.^2).^2+38685626227668133590597632./292014622371243969477200025./(1+1./143.*t.^2).^44+4835703278458516698824704./36091694899816670384822475./(1+1./143.*t.^2).^43+2417851639229258349412352./17838423916001342833877775./(1+1./143.*t.^2).^42+77371252455336267181195264./590518458572960027165004495./(1+1./143.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./143.*t.^2).^46+16./35./(1+1./143.*t.^2).^3+618970019642690137449562112./4826300260399724181316202325./(1+1./143.*t.^2).^47+128./315./(1+1./143.*t.^2).^4+562949953421312./3285460280781189./(1+1./143.*t.^2).^26+256./693./(1+1./143.*t.^2).^5+19807040628566084398385987584./156050375086257748529223875175./(1+1./143.*t.^2).^48+1125899906842624./6692604275665385./(1+1./143.*t.^2).^27+9007199254740992./54496920530418135./(1+1./143.*t.^2).^28+39614081257132168796771975168./315285451704888104171289053925./(1+1./143.*t.^2).^49+18014398509481984./110873045217057585./(1+1./143.*t.^2).^29+316912650057057350374175801344./2572482003204667409956015708221./(1+1./143.*t.^2).^51+158456325028528675187087900672./1273753224887747940852007777857./(1+1./143.*t.^2).^50+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./143.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./143.*t.^2).^53+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./143.*t.^2).^70+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./143.*t.^2).^54+1024./3003./(1+1./143.*t.^2).^6+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./143.*t.^2).^56+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./143.*t.^2).^55+2048./6435./(1+1./143.*t.^2).^7+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./143.*t.^2).^57+65536./230945./(1+1./143.*t.^2).^9+32768./109395./(1+1./143.*t.^2).^8+262144./969969./(1+1./143.*t.^2).^10+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./143.*t.^2).^58+524288./2028117./(1+1./143.*t.^2).^11+72057594037927936./450883717216034179./(1+1./143.*t.^2).^30+144115188075855872./916312070471295267./(1+1./143.*t.^2).^31+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./143.*t.^2).^59+9223372036854775808./59560284580634192355./(1+1./143.*t.^2).^32+8388608./35102025./(1+1./143.*t.^2).^13+4194304./16900975./(1+1./143.*t.^2).^12+33554432./145422675./(1+1./143.*t.^2).^14+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./143.*t.^2).^60+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./143.*t.^2).^61+67108864./300540195./(1+1./143.*t.^2).^15+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./143.*t.^2).^62+2147483648./9917826435./(1+1./143.*t.^2).^16+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./143.*t.^2).^64+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./143.*t.^2).^63+4294967296./20419054425./(1+1./143.*t.^2).^17+18446744073709551616./120925426269772451145./(1+1./143.*t.^2).^33+17179869184./83945001525./(1+1./143.*t.^2).^18+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./143.*t.^2).^65+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./143.*t.^2).^66+34359738368./172308161025./(1+1./143.*t.^2).^19+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./143.*t.^2).^67+274877906944./1412926920405./(1+1./143.*t.^2).^20+549755813888./2893136075115./(1+1./143.*t.^2).^21);

        case 144;
            alpha(set_t)=1./2+1./2.*t./(144+t.^2).^(1./2).*(1+1804857108504066435./18446744073709551616./(1+1./144.*t.^2).^33+46189./262144./(1+1./144.*t.^2).^10+88179./524288./(1+1./144.*t.^2).^11+7113260368810144185./73786976294838206464./(1+1./144.*t.^2).^34+3184701532372607112841303575./39614081257132168796771975168./(1+1./144.*t.^2).^49+14023284727082855679./147573952589676412928./(1+1./144.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./144.*t.^2).^36+12611418068195524166851562157./158456325028528675187087900672./(1+1./144.*t.^2).^50+676039./4194304./(1+1./144.*t.^2).^12+1300075./8388608./(1+1./144.*t.^2).^13+24975553429171528252000152507./316912650057057350374175801344./(1+1./144.*t.^2).^51+5014575./33554432./(1+1./144.*t.^2).^14+7391536347803839./72057594037927936./(1+1./144.*t.^2).^30+218266320541953276229./2361183241434822606848./(1+1./144.*t.^2).^37+197883231015743646919693516017./2535301200456458802993406410752./(1+1./144.*t.^2).^52+861577581086657669325./9444732965739290427392./(1+1./144.*t.^2).^38+514589420475./4398046511104./(1+1./144.*t.^2).^23+392032816163265715595619229845./5070602400912917605986812821504./(1+1./144.*t.^2).^53+9694845./67108864./(1+1./144.*t.^2).^15+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./144.*t.^2).^54+300540195./2147483648./(1+1./144.*t.^2).^16+1701063429324939500975./18889465931478580854784./(1+1./144.*t.^2).^39+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./144.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./144.*t.^2).^56+8061900920775./70368744177664./(1+1./144.*t.^2).^24+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./144.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./144.*t.^2).^58+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./144.*t.^2).^59+15801325804719./140737488355328./(1+1./144.*t.^2).^25+26876802183334044115405./302231454903657293676544./(1+1./144.*t.^2).^40+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./144.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./144.*t.^2).^61+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./144.*t.^2).^62+61989816618513./562949953421312./(1+1./144.*t.^2).^26+53098072606098965203605./604462909807314587353088./(1+1./144.*t.^2).^41+414847067813984717066925./4835703278458516698824704./(1+1./144.*t.^2).^43+583401555./4294967296./(1+1./144.*t.^2).^17+209863810776486386280915./2417851639229258349412352./(1+1./144.*t.^2).^42+3281063172710606398620225./38685626227668133590597632./(1+1./144.*t.^2).^44+121683714103007./1125899906842624./(1+1./144.*t.^2).^27+2268783825./17179869184./(1+1./144.*t.^2).^18+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./144.*t.^2).^64+4418157975./34359738368./(1+1./144.*t.^2).^19+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./144.*t.^2).^63+956086325095055./9007199254740992./(1+1./144.*t.^2).^28+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./144.*t.^2).^65+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./144.*t.^2).^66+1879204156221315./18014398509481984./(1+1./144.*t.^2).^29+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./144.*t.^2).^67+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./144.*t.^2).^68+34461632205./274877906944./(1+1./144.*t.^2).^20+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./144.*t.^2).^69+6489213830472088210604445./77371252455336267181195264./(1+1./144.*t.^2).^45+14544636039226909./144115188075855872./(1+1./144.*t.^2).^31+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./144.*t.^2).^70+1./2./(1+1./144.*t.^2)+3./8./(1+1./144.*t.^2).^2+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./144.*t.^2).^71+25674715590128696833261065./309485009821345068724781056./(1+1./144.*t.^2).^46+5./16./(1+1./144.*t.^2).^3+35./128./(1+1./144.*t.^2).^4+63./256./(1+1./144.*t.^2).^5+50803160635786570329644235./618970019642690137449562112./(1+1./144.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./144.*t.^2).^48+231./1024./(1+1./144.*t.^2).^6+429./2048./(1+1./144.*t.^2).^7+6435./32768./(1+1./144.*t.^2).^8+916312070471295267./9223372036854775808./(1+1./144.*t.^2).^32+67282234305./549755813888./(1+1./144.*t.^2).^21+263012370465./2199023255552./(1+1./144.*t.^2).^22+12155./65536./(1+1./144.*t.^2).^9);

        case 145;
            alpha(set_t)=1./2+1./pi.*atan(1./145.*t.*145.^(1./2))+1./pi.*t.*145.^(1./2)./(145+t.^2).*(1+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./145.*t.^2).^54+34359738368./172308161025./(1+1./145.*t.^2).^19+1024./3003./(1+1./145.*t.^2).^6+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./145.*t.^2).^55+274877906944./1412926920405./(1+1./145.*t.^2).^20+2048./6435./(1+1./145.*t.^2).^7+32768./109395./(1+1./145.*t.^2).^8+549755813888./2893136075115./(1+1./145.*t.^2).^21+2199023255552./11835556670925./(1+1./145.*t.^2).^22+4398046511104./24185702762325./(1+1./145.*t.^2).^23+70368744177664./395033145117975./(1+1./145.*t.^2).^24+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./145.*t.^2).^67+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./145.*t.^2).^56+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./145.*t.^2).^68+140737488355328./805867616040669./(1+1./145.*t.^2).^25+1125899906842624./6692604275665385./(1+1./145.*t.^2).^27+562949953421312./3285460280781189./(1+1./145.*t.^2).^26+65536./230945./(1+1./145.*t.^2).^9+262144./969969./(1+1./145.*t.^2).^10+9007199254740992./54496920530418135./(1+1./145.*t.^2).^28+18014398509481984./110873045217057585./(1+1./145.*t.^2).^29+72057594037927936./450883717216034179./(1+1./145.*t.^2).^30+524288./2028117./(1+1./145.*t.^2).^11+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./145.*t.^2).^69+144115188075855872./916312070471295267./(1+1./145.*t.^2).^31+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./145.*t.^2).^57+18446744073709551616./120925426269772451145./(1+1./145.*t.^2).^33+9223372036854775808./59560284580634192355./(1+1./145.*t.^2).^32+73786976294838206464./490814965447899948765./(1+1./145.*t.^2).^34+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./145.*t.^2).^59+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./145.*t.^2).^58+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./145.*t.^2).^70+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./145.*t.^2).^71+4194304./16900975./(1+1./145.*t.^2).^12+8388608./35102025./(1+1./145.*t.^2).^13+147573952589676412928./995653215622882753209./(1+1./145.*t.^2).^35+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./145.*t.^2).^60+33554432./145422675./(1+1./145.*t.^2).^14+1180591620717411303424./8075853860052271220473./(1+1./145.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./145.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./145.*t.^2).^38+604462909807314587353088./4407140026306214111899215./(1+1./145.*t.^2).^41+302231454903657293676544./2177020976850057573347805./(1+1./145.*t.^2).^40+18889465931478580854784./134384010916670220577025./(1+1./145.*t.^2).^39+2417851639229258349412352./17838423916001342833877775./(1+1./145.*t.^2).^42+67108864./300540195./(1+1./145.*t.^2).^15+2./3./(1+1./145.*t.^2)+4835703278458516698824704./36091694899816670384822475./(1+1./145.*t.^2).^43+8./15./(1+1./145.*t.^2).^2+38685626227668133590597632./292014622371243969477200025./(1+1./145.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./145.*t.^2).^45+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./145.*t.^2).^61+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./145.*t.^2).^62+2147483648./9917826435./(1+1./145.*t.^2).^16+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./145.*t.^2).^63+4294967296./20419054425./(1+1./145.*t.^2).^17+618970019642690137449562112./4826300260399724181316202325./(1+1./145.*t.^2).^47+309485009821345068724781056./2387748549881968805493279045./(1+1./145.*t.^2).^46+19807040628566084398385987584./156050375086257748529223875175./(1+1./145.*t.^2).^48+128./315./(1+1./145.*t.^2).^4+16./35./(1+1./145.*t.^2).^3+39614081257132168796771975168./315285451704888104171289053925./(1+1./145.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./145.*t.^2).^50+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./145.*t.^2).^64+17179869184./83945001525./(1+1./145.*t.^2).^18+316912650057057350374175801344./2572482003204667409956015708221./(1+1./145.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./145.*t.^2).^52+256./693./(1+1./145.*t.^2).^5+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./145.*t.^2).^65+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./145.*t.^2).^53+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./145.*t.^2).^66);

        case 146;
            alpha(set_t)=1./2+1./2.*t./(146+t.^2).^(1./2).*(1+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./146.*t.^2).^72+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./146.*t.^2).^62+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./146.*t.^2).^63+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./146.*t.^2).^64+5./16./(1+1./146.*t.^2).^3+35./128./(1+1./146.*t.^2).^4+1./2./(1+1./146.*t.^2)+3./8./(1+1./146.*t.^2).^2+429./2048./(1+1./146.*t.^2).^7+231./1024./(1+1./146.*t.^2).^6+63./256./(1+1./146.*t.^2).^5+12155./65536./(1+1./146.*t.^2).^9+46189./262144./(1+1./146.*t.^2).^10+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./146.*t.^2).^65+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./146.*t.^2).^66+6435./32768./(1+1./146.*t.^2).^8+88179./524288./(1+1./146.*t.^2).^11+676039./4194304./(1+1./146.*t.^2).^12+1300075./8388608./(1+1./146.*t.^2).^13+5014575./33554432./(1+1./146.*t.^2).^14+300540195./2147483648./(1+1./146.*t.^2).^16+9694845./67108864./(1+1./146.*t.^2).^15+583401555./4294967296./(1+1./146.*t.^2).^17+4418157975./34359738368./(1+1./146.*t.^2).^19+2268783825./17179869184./(1+1./146.*t.^2).^18+34461632205./274877906944./(1+1./146.*t.^2).^20+67282234305./549755813888./(1+1./146.*t.^2).^21+514589420475./4398046511104./(1+1./146.*t.^2).^23+263012370465./2199023255552./(1+1./146.*t.^2).^22+8061900920775./70368744177664./(1+1./146.*t.^2).^24+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./146.*t.^2).^67+61989816618513./562949953421312./(1+1./146.*t.^2).^26+15801325804719./140737488355328./(1+1./146.*t.^2).^25+1879204156221315./18014398509481984./(1+1./146.*t.^2).^29+121683714103007./1125899906842624./(1+1./146.*t.^2).^27+956086325095055./9007199254740992./(1+1./146.*t.^2).^28+7391536347803839./72057594037927936./(1+1./146.*t.^2).^30+14544636039226909./144115188075855872./(1+1./146.*t.^2).^31+1804857108504066435./18446744073709551616./(1+1./146.*t.^2).^33+916312070471295267./9223372036854775808./(1+1./146.*t.^2).^32+14023284727082855679./147573952589676412928./(1+1./146.*t.^2).^35+7113260368810144185./73786976294838206464./(1+1./146.*t.^2).^34+110628135069209194801./1180591620717411303424./(1+1./146.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./146.*t.^2).^37+1701063429324939500975./18889465931478580854784./(1+1./146.*t.^2).^39+861577581086657669325./9444732965739290427392./(1+1./146.*t.^2).^38+53098072606098965203605./604462909807314587353088./(1+1./146.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./146.*t.^2).^42+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./146.*t.^2).^68+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./146.*t.^2).^69+26876802183334044115405./302231454903657293676544./(1+1./146.*t.^2).^40+414847067813984717066925./4835703278458516698824704./(1+1./146.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./146.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./146.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./146.*t.^2).^46+1608766753466574727105400775./19807040628566084398385987584./(1+1./146.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./146.*t.^2).^49+50803160635786570329644235./618970019642690137449562112./(1+1./146.*t.^2).^47+12611418068195524166851562157./158456325028528675187087900672./(1+1./146.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./146.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./146.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./146.*t.^2).^53+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./146.*t.^2).^70+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./146.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./146.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./146.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./146.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./146.*t.^2).^58+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./146.*t.^2).^59+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./146.*t.^2).^61+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./146.*t.^2).^60+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./146.*t.^2).^71);

        case 147;
            alpha(set_t)=1./2+1./pi.*atan(1./147.*t.*147.^(1./2))+1./pi.*t.*147.^(1./2)./(147+t.^2).*(1+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./147.*t.^2).^56+32768./109395./(1+1./147.*t.^2).^8+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./147.*t.^2).^57+77371252455336267181195264./590518458572960027165004495./(1+1./147.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./147.*t.^2).^46+65536./230945./(1+1./147.*t.^2).^9+262144./969969./(1+1./147.*t.^2).^10+524288./2028117./(1+1./147.*t.^2).^11+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./147.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./147.*t.^2).^59+4194304./16900975./(1+1./147.*t.^2).^12+8388608./35102025./(1+1./147.*t.^2).^13+33554432./145422675./(1+1./147.*t.^2).^14+67108864./300540195./(1+1./147.*t.^2).^15+618970019642690137449562112./4826300260399724181316202325./(1+1./147.*t.^2).^47+2147483648./9917826435./(1+1./147.*t.^2).^16+4294967296./20419054425./(1+1./147.*t.^2).^17+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./147.*t.^2).^60+17179869184./83945001525./(1+1./147.*t.^2).^18+34359738368./172308161025./(1+1./147.*t.^2).^19+274877906944./1412926920405./(1+1./147.*t.^2).^20+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./147.*t.^2).^61+19807040628566084398385987584./156050375086257748529223875175./(1+1./147.*t.^2).^48+549755813888./2893136075115./(1+1./147.*t.^2).^21+2199023255552./11835556670925./(1+1./147.*t.^2).^22+4398046511104./24185702762325./(1+1./147.*t.^2).^23+39614081257132168796771975168./315285451704888104171289053925./(1+1./147.*t.^2).^49+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./147.*t.^2).^62+16./35./(1+1./147.*t.^2).^3+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./147.*t.^2).^63+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./147.*t.^2).^64+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./147.*t.^2).^65+70368744177664./395033145117975./(1+1./147.*t.^2).^24+140737488355328./805867616040669./(1+1./147.*t.^2).^25+158456325028528675187087900672./1273753224887747940852007777857./(1+1./147.*t.^2).^50+562949953421312./3285460280781189./(1+1./147.*t.^2).^26+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./147.*t.^2).^66+316912650057057350374175801344./2572482003204667409956015708221./(1+1./147.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./147.*t.^2).^52+1125899906842624./6692604275665385./(1+1./147.*t.^2).^27+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./147.*t.^2).^67+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./147.*t.^2).^68+9007199254740992./54496920530418135./(1+1./147.*t.^2).^28+18014398509481984./110873045217057585./(1+1./147.*t.^2).^29+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./147.*t.^2).^69+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./147.*t.^2).^70+9223372036854775808./59560284580634192355./(1+1./147.*t.^2).^32+144115188075855872./916312070471295267./(1+1./147.*t.^2).^31+72057594037927936./450883717216034179./(1+1./147.*t.^2).^30+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./147.*t.^2).^71+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./147.*t.^2).^72+18446744073709551616./120925426269772451145./(1+1./147.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./147.*t.^2).^34+128./315./(1+1./147.*t.^2).^4+256./693./(1+1./147.*t.^2).^5+147573952589676412928./995653215622882753209./(1+1./147.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./147.*t.^2).^36+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./147.*t.^2).^53+1024./3003./(1+1./147.*t.^2).^6+2361183241434822606848./16369974040646495717175./(1+1./147.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./147.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./147.*t.^2).^39+2048./6435./(1+1./147.*t.^2).^7+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./147.*t.^2).^54+302231454903657293676544./2177020976850057573347805./(1+1./147.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./147.*t.^2).^41+2./3./(1+1./147.*t.^2)+2417851639229258349412352./17838423916001342833877775./(1+1./147.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./147.*t.^2).^43+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./147.*t.^2).^55+8./15./(1+1./147.*t.^2).^2+38685626227668133590597632./292014622371243969477200025./(1+1./147.*t.^2).^44);

        case 148;
            alpha(set_t)=1./2+1./2.*t./(148+t.^2).^(1./2).*(1+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./148.*t.^2).^68+67282234305./549755813888./(1+1./148.*t.^2).^21+263012370465./2199023255552./(1+1./148.*t.^2).^22+1804857108504066435./18446744073709551616./(1+1./148.*t.^2).^33+514589420475./4398046511104./(1+1./148.*t.^2).^23+7113260368810144185./73786976294838206464./(1+1./148.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./148.*t.^2).^35+8061900920775./70368744177664./(1+1./148.*t.^2).^24+110628135069209194801./1180591620717411303424./(1+1./148.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./148.*t.^2).^37+15801325804719./140737488355328./(1+1./148.*t.^2).^25+861577581086657669325./9444732965739290427392./(1+1./148.*t.^2).^38+1./2./(1+1./148.*t.^2)+1701063429324939500975./18889465931478580854784./(1+1./148.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./148.*t.^2).^40+3./8./(1+1./148.*t.^2).^2+53098072606098965203605./604462909807314587353088./(1+1./148.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./148.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./148.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./148.*t.^2).^44+5./16./(1+1./148.*t.^2).^3+6489213830472088210604445./77371252455336267181195264./(1+1./148.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./148.*t.^2).^46+61989816618513./562949953421312./(1+1./148.*t.^2).^26+35./128./(1+1./148.*t.^2).^4+63./256./(1+1./148.*t.^2).^5+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./148.*t.^2).^69+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./148.*t.^2).^70+956086325095055./9007199254740992./(1+1./148.*t.^2).^28+121683714103007./1125899906842624./(1+1./148.*t.^2).^27+50803160635786570329644235./618970019642690137449562112./(1+1./148.*t.^2).^47+231./1024./(1+1./148.*t.^2).^6+429./2048./(1+1./148.*t.^2).^7+1608766753466574727105400775./19807040628566084398385987584./(1+1./148.*t.^2).^48+6435./32768./(1+1./148.*t.^2).^8+12611418068195524166851562157./158456325028528675187087900672./(1+1./148.*t.^2).^50+3184701532372607112841303575./39614081257132168796771975168./(1+1./148.*t.^2).^49+24975553429171528252000152507./316912650057057350374175801344./(1+1./148.*t.^2).^51+1879204156221315./18014398509481984./(1+1./148.*t.^2).^29+7391536347803839./72057594037927936./(1+1./148.*t.^2).^30+12155./65536./(1+1./148.*t.^2).^9+197883231015743646919693516017./2535301200456458802993406410752./(1+1./148.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./148.*t.^2).^53+46189./262144./(1+1./148.*t.^2).^10+88179./524288./(1+1./148.*t.^2).^11+676039./4194304./(1+1./148.*t.^2).^12+1300075./8388608./(1+1./148.*t.^2).^13+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./148.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./148.*t.^2).^55+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./148.*t.^2).^58+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./148.*t.^2).^57+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./148.*t.^2).^56+300540195./2147483648./(1+1./148.*t.^2).^16+5014575./33554432./(1+1./148.*t.^2).^14+9694845./67108864./(1+1./148.*t.^2).^15+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./148.*t.^2).^59+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./148.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./148.*t.^2).^61+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./148.*t.^2).^71+14544636039226909./144115188075855872./(1+1./148.*t.^2).^31+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./148.*t.^2).^62+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./148.*t.^2).^63+583401555./4294967296./(1+1./148.*t.^2).^17+2268783825./17179869184./(1+1./148.*t.^2).^18+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./148.*t.^2).^64+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./148.*t.^2).^65+4418157975./34359738368./(1+1./148.*t.^2).^19+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./148.*t.^2).^72+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./148.*t.^2).^73+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./148.*t.^2).^66+34461632205./274877906944./(1+1./148.*t.^2).^20+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./148.*t.^2).^67+916312070471295267./9223372036854775808./(1+1./148.*t.^2).^32);

        case 149;
            alpha(set_t)=1./2+1./pi.*atan(1./149.*t.*149.^(1./2))+1./pi.*t.*149.^(1./2)./(149+t.^2).*(1+604462909807314587353088./4407140026306214111899215./(1+1./149.*t.^2).^41+19807040628566084398385987584./156050375086257748529223875175./(1+1./149.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./149.*t.^2).^49+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./149.*t.^2).^73+158456325028528675187087900672./1273753224887747940852007777857./(1+1./149.*t.^2).^50+2417851639229258349412352./17838423916001342833877775./(1+1./149.*t.^2).^42+316912650057057350374175801344./2572482003204667409956015708221./(1+1./149.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./149.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./149.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./149.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./149.*t.^2).^55+4835703278458516698824704./36091694899816670384822475./(1+1./149.*t.^2).^43+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./149.*t.^2).^56+2./3./(1+1./149.*t.^2)+8./15./(1+1./149.*t.^2).^2+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./149.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./149.*t.^2).^58+16./35./(1+1./149.*t.^2).^3+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./149.*t.^2).^59+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./149.*t.^2).^60+256./693./(1+1./149.*t.^2).^5+1024./3003./(1+1./149.*t.^2).^6+128./315./(1+1./149.*t.^2).^4+65536./230945./(1+1./149.*t.^2).^9+2048./6435./(1+1./149.*t.^2).^7+32768./109395./(1+1./149.*t.^2).^8+262144./969969./(1+1./149.*t.^2).^10+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./149.*t.^2).^61+524288./2028117./(1+1./149.*t.^2).^11+4194304./16900975./(1+1./149.*t.^2).^12+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./149.*t.^2).^62+8388608./35102025./(1+1./149.*t.^2).^13+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./149.*t.^2).^63+38685626227668133590597632./292014622371243969477200025./(1+1./149.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./149.*t.^2).^45+33554432./145422675./(1+1./149.*t.^2).^14+67108864./300540195./(1+1./149.*t.^2).^15+2147483648./9917826435./(1+1./149.*t.^2).^16+4294967296./20419054425./(1+1./149.*t.^2).^17+17179869184./83945001525./(1+1./149.*t.^2).^18+309485009821345068724781056./2387748549881968805493279045./(1+1./149.*t.^2).^46+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./149.*t.^2).^64+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./149.*t.^2).^65+34359738368./172308161025./(1+1./149.*t.^2).^19+274877906944./1412926920405./(1+1./149.*t.^2).^20+4398046511104./24185702762325./(1+1./149.*t.^2).^23+549755813888./2893136075115./(1+1./149.*t.^2).^21+2199023255552./11835556670925./(1+1./149.*t.^2).^22+562949953421312./3285460280781189./(1+1./149.*t.^2).^26+140737488355328./805867616040669./(1+1./149.*t.^2).^25+70368744177664./395033145117975./(1+1./149.*t.^2).^24+9007199254740992./54496920530418135./(1+1./149.*t.^2).^28+1125899906842624./6692604275665385./(1+1./149.*t.^2).^27+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./149.*t.^2).^66+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./149.*t.^2).^67+18014398509481984./110873045217057585./(1+1./149.*t.^2).^29+9223372036854775808./59560284580634192355./(1+1./149.*t.^2).^32+72057594037927936./450883717216034179./(1+1./149.*t.^2).^30+144115188075855872./916312070471295267./(1+1./149.*t.^2).^31+18446744073709551616./120925426269772451145./(1+1./149.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./149.*t.^2).^34+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./149.*t.^2).^68+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./149.*t.^2).^69+147573952589676412928./995653215622882753209./(1+1./149.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./149.*t.^2).^36+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./149.*t.^2).^70+2361183241434822606848./16369974040646495717175./(1+1./149.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./149.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./149.*t.^2).^39+618970019642690137449562112./4826300260399724181316202325./(1+1./149.*t.^2).^47+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./149.*t.^2).^71+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./149.*t.^2).^72+302231454903657293676544./2177020976850057573347805./(1+1./149.*t.^2).^40);

        case 150;
            alpha(set_t)=1./2+1./2.*t./(150+t.^2).^(1./2).*(1+1879204156221315./18014398509481984./(1+1./150.*t.^2).^29+88179./524288./(1+1./150.*t.^2).^11+8061900920775./70368744177664./(1+1./150.*t.^2).^24+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./150.*t.^2).^62+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./150.*t.^2).^63+7391536347803839./72057594037927936./(1+1./150.*t.^2).^30+14544636039226909./144115188075855872./(1+1./150.*t.^2).^31+676039./4194304./(1+1./150.*t.^2).^12+1300075./8388608./(1+1./150.*t.^2).^13+53098072606098965203605./604462909807314587353088./(1+1./150.*t.^2).^41+916312070471295267./9223372036854775808./(1+1./150.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./150.*t.^2).^33+15801325804719./140737488355328./(1+1./150.*t.^2).^25+5014575./33554432./(1+1./150.*t.^2).^14+7113260368810144185./73786976294838206464./(1+1./150.*t.^2).^34+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./150.*t.^2).^64+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./150.*t.^2).^65+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./150.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./150.*t.^2).^58+9694845./67108864./(1+1./150.*t.^2).^15+14023284727082855679./147573952589676412928./(1+1./150.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./150.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./150.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./150.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./150.*t.^2).^39+61989816618513./562949953421312./(1+1./150.*t.^2).^26+121683714103007./1125899906842624./(1+1./150.*t.^2).^27+26876802183334044115405./302231454903657293676544./(1+1./150.*t.^2).^40+300540195./2147483648./(1+1./150.*t.^2).^16+583401555./4294967296./(1+1./150.*t.^2).^17+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./150.*t.^2).^66+2268783825./17179869184./(1+1./150.*t.^2).^18+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./150.*t.^2).^59+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./150.*t.^2).^67+1./2./(1+1./150.*t.^2)+3./8./(1+1./150.*t.^2).^2+209863810776486386280915./2417851639229258349412352./(1+1./150.*t.^2).^42+4418157975./34359738368./(1+1./150.*t.^2).^19+414847067813984717066925./4835703278458516698824704./(1+1./150.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./150.*t.^2).^44+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./150.*t.^2).^68+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./150.*t.^2).^69+5./16./(1+1./150.*t.^2).^3+34461632205./274877906944./(1+1./150.*t.^2).^20+67282234305./549755813888./(1+1./150.*t.^2).^21+35./128./(1+1./150.*t.^2).^4+6489213830472088210604445./77371252455336267181195264./(1+1./150.*t.^2).^45+956086325095055./9007199254740992./(1+1./150.*t.^2).^28+25674715590128696833261065./309485009821345068724781056./(1+1./150.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./150.*t.^2).^47+12611418068195524166851562157./158456325028528675187087900672./(1+1./150.*t.^2).^50+3184701532372607112841303575./39614081257132168796771975168./(1+1./150.*t.^2).^49+1608766753466574727105400775./19807040628566084398385987584./(1+1./150.*t.^2).^48+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./150.*t.^2).^70+263012370465./2199023255552./(1+1./150.*t.^2).^22+514589420475./4398046511104./(1+1./150.*t.^2).^23+63./256./(1+1./150.*t.^2).^5+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./150.*t.^2).^71+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./150.*t.^2).^72+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./150.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./150.*t.^2).^61+24975553429171528252000152507./316912650057057350374175801344./(1+1./150.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./150.*t.^2).^52+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./150.*t.^2).^73+231./1024./(1+1./150.*t.^2).^6+392032816163265715595619229845./5070602400912917605986812821504./(1+1./150.*t.^2).^53+429./2048./(1+1./150.*t.^2).^7+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./150.*t.^2).^54+6435./32768./(1+1./150.*t.^2).^8+12155./65536./(1+1./150.*t.^2).^9+46189./262144./(1+1./150.*t.^2).^10+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./150.*t.^2).^74+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./150.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./150.*t.^2).^56);

        case 151;
            alpha(set_t)=1./2+1./pi.*atan(1./151.*t.*151.^(1./2))+1./pi.*t.*151.^(1./2)./(151+t.^2).*(1+147573952589676412928./995653215622882753209./(1+1./151.*t.^2).^35+70368744177664./395033145117975./(1+1./151.*t.^2).^24+140737488355328./805867616040669./(1+1./151.*t.^2).^25+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./151.*t.^2).^65+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./151.*t.^2).^66+1180591620717411303424./8075853860052271220473./(1+1./151.*t.^2).^36+65536./230945./(1+1./151.*t.^2).^9+2048./6435./(1+1./151.*t.^2).^7+32768./109395./(1+1./151.*t.^2).^8+2361183241434822606848./16369974040646495717175./(1+1./151.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./151.*t.^2).^38+562949953421312./3285460280781189./(1+1./151.*t.^2).^26+2./3./(1+1./151.*t.^2)+8./15./(1+1./151.*t.^2).^2+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./151.*t.^2).^67+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./151.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./151.*t.^2).^57+1125899906842624./6692604275665385./(1+1./151.*t.^2).^27+9007199254740992./54496920530418135./(1+1./151.*t.^2).^28+18889465931478580854784./134384010916670220577025./(1+1./151.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./151.*t.^2).^40+16./35./(1+1./151.*t.^2).^3+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./151.*t.^2).^68+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./151.*t.^2).^69+604462909807314587353088./4407140026306214111899215./(1+1./151.*t.^2).^41+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./151.*t.^2).^71+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./151.*t.^2).^70+4194304./16900975./(1+1./151.*t.^2).^12+262144./969969./(1+1./151.*t.^2).^10+524288./2028117./(1+1./151.*t.^2).^11+8388608./35102025./(1+1./151.*t.^2).^13+33554432./145422675./(1+1./151.*t.^2).^14+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./151.*t.^2).^58+2417851639229258349412352./17838423916001342833877775./(1+1./151.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./151.*t.^2).^43+18014398509481984./110873045217057585./(1+1./151.*t.^2).^29+72057594037927936./450883717216034179./(1+1./151.*t.^2).^30+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./151.*t.^2).^72+67108864./300540195./(1+1./151.*t.^2).^15+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./151.*t.^2).^59+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./151.*t.^2).^60+38685626227668133590597632./292014622371243969477200025./(1+1./151.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./151.*t.^2).^45+144115188075855872./916312070471295267./(1+1./151.*t.^2).^31+2147483648./9917826435./(1+1./151.*t.^2).^16+4294967296./20419054425./(1+1./151.*t.^2).^17+17179869184./83945001525./(1+1./151.*t.^2).^18+34359738368./172308161025./(1+1./151.*t.^2).^19+128./315./(1+1./151.*t.^2).^4+256./693./(1+1./151.*t.^2).^5+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./151.*t.^2).^61+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./151.*t.^2).^62+309485009821345068724781056./2387748549881968805493279045./(1+1./151.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./151.*t.^2).^47+274877906944./1412926920405./(1+1./151.*t.^2).^20+549755813888./2893136075115./(1+1./151.*t.^2).^21+2199023255552./11835556670925./(1+1./151.*t.^2).^22+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./151.*t.^2).^73+19807040628566084398385987584./156050375086257748529223875175./(1+1./151.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./151.*t.^2).^49+9223372036854775808./59560284580634192355./(1+1./151.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./151.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./151.*t.^2).^34+158456325028528675187087900672./1273753224887747940852007777857./(1+1./151.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./151.*t.^2).^51+4398046511104./24185702762325./(1+1./151.*t.^2).^23+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./151.*t.^2).^52+1024./3003./(1+1./151.*t.^2).^6+44601490397061246283071436545296723011960832./435122201890822451713996007215336509094300875./(1+1./151.*t.^2).^74+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./151.*t.^2).^53+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./151.*t.^2).^63+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./151.*t.^2).^64+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./151.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./151.*t.^2).^55);

        case 152;
            alpha(set_t)=1./2+1./2.*t./(152+t.^2).^(1./2).*(1+3184701532372607112841303575./39614081257132168796771975168./(1+1./152.*t.^2).^49+583401555./4294967296./(1+1./152.*t.^2).^17+14023284727082855679./147573952589676412928./(1+1./152.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./152.*t.^2).^36+2268783825./17179869184./(1+1./152.*t.^2).^18+4418157975./34359738368./(1+1./152.*t.^2).^19+3./8./(1+1./152.*t.^2).^2+5./16./(1+1./152.*t.^2).^3+12611418068195524166851562157./158456325028528675187087900672./(1+1./152.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./152.*t.^2).^51+218266320541953276229./2361183241434822606848./(1+1./152.*t.^2).^37+34461632205./274877906944./(1+1./152.*t.^2).^20+67282234305./549755813888./(1+1./152.*t.^2).^21+197883231015743646919693516017./2535301200456458802993406410752./(1+1./152.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./152.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./152.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./152.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./152.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./152.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./152.*t.^2).^58+263012370465./2199023255552./(1+1./152.*t.^2).^22+861577581086657669325./9444732965739290427392./(1+1./152.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./152.*t.^2).^39+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./152.*t.^2).^59+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./152.*t.^2).^60+26876802183334044115405./302231454903657293676544./(1+1./152.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./152.*t.^2).^41+514589420475./4398046511104./(1+1./152.*t.^2).^23+8061900920775./70368744177664./(1+1./152.*t.^2).^24+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./152.*t.^2).^61+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./152.*t.^2).^62+15801325804719./140737488355328./(1+1./152.*t.^2).^25+209863810776486386280915./2417851639229258349412352./(1+1./152.*t.^2).^42+61989816618513./562949953421312./(1+1./152.*t.^2).^26+414847067813984717066925./4835703278458516698824704./(1+1./152.*t.^2).^43+121683714103007./1125899906842624./(1+1./152.*t.^2).^27+956086325095055./9007199254740992./(1+1./152.*t.^2).^28+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./152.*t.^2).^63+1879204156221315./18014398509481984./(1+1./152.*t.^2).^29+35./128./(1+1./152.*t.^2).^4+63./256./(1+1./152.*t.^2).^5+231./1024./(1+1./152.*t.^2).^6+429./2048./(1+1./152.*t.^2).^7+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./152.*t.^2).^64+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./152.*t.^2).^65+3281063172710606398620225./38685626227668133590597632./(1+1./152.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./152.*t.^2).^45+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./152.*t.^2).^66+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./152.*t.^2).^67+6435./32768./(1+1./152.*t.^2).^8+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./152.*t.^2).^68+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./152.*t.^2).^69+12155./65536./(1+1./152.*t.^2).^9+7391536347803839./72057594037927936./(1+1./152.*t.^2).^30+14544636039226909./144115188075855872./(1+1./152.*t.^2).^31+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./152.*t.^2).^70+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./152.*t.^2).^71+46189./262144./(1+1./152.*t.^2).^10+88179./524288./(1+1./152.*t.^2).^11+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./152.*t.^2).^72+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./152.*t.^2).^73+676039./4194304./(1+1./152.*t.^2).^12+916312070471295267./9223372036854775808./(1+1./152.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./152.*t.^2).^33+25674715590128696833261065./309485009821345068724781056./(1+1./152.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./152.*t.^2).^47+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./152.*t.^2).^74+5801629358544299356186613429537820121257345./89202980794122492566142873090593446023921664./(1+1./152.*t.^2).^75+1./2./(1+1./152.*t.^2)+7113260368810144185./73786976294838206464./(1+1./152.*t.^2).^34+1608766753466574727105400775./19807040628566084398385987584./(1+1./152.*t.^2).^48+1300075./8388608./(1+1./152.*t.^2).^13+5014575./33554432./(1+1./152.*t.^2).^14+9694845./67108864./(1+1./152.*t.^2).^15+300540195./2147483648./(1+1./152.*t.^2).^16);

        case 153;
            alpha(set_t)=1./2+1./pi.*atan(1./153.*t.*153.^(1./2))+1./pi.*t.*153.^(1./2)./(153+t.^2).*(1+44601490397061246283071436545296723011960832./435122201890822451713996007215336509094300875./(1+1./153.*t.^2).^74+309485009821345068724781056./2387748549881968805493279045./(1+1./153.*t.^2).^46+9223372036854775808./59560284580634192355./(1+1./153.*t.^2).^32+618970019642690137449562112./4826300260399724181316202325./(1+1./153.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./153.*t.^2).^48+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./153.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./153.*t.^2).^55+89202980794122492566142873090593446023921664./876046033140189202784178627860210838309859095./(1+1./153.*t.^2).^75+2./3./(1+1./153.*t.^2)+8./15./(1+1./153.*t.^2).^2+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./153.*t.^2).^56+16./35./(1+1./153.*t.^2).^3+128./315./(1+1./153.*t.^2).^4+2048./6435./(1+1./153.*t.^2).^7+256./693./(1+1./153.*t.^2).^5+1024./3003./(1+1./153.*t.^2).^6+2417851639229258349412352./17838423916001342833877775./(1+1./153.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./153.*t.^2).^43+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./153.*t.^2).^59+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./153.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./153.*t.^2).^58+65536./230945./(1+1./153.*t.^2).^9+32768./109395./(1+1./153.*t.^2).^8+262144./969969./(1+1./153.*t.^2).^10+524288./2028117./(1+1./153.*t.^2).^11+18446744073709551616./120925426269772451145./(1+1./153.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./153.*t.^2).^34+4194304./16900975./(1+1./153.*t.^2).^12+8388608./35102025./(1+1./153.*t.^2).^13+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./153.*t.^2).^60+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./153.*t.^2).^61+38685626227668133590597632./292014622371243969477200025./(1+1./153.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./153.*t.^2).^45+33554432./145422675./(1+1./153.*t.^2).^14+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./153.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./153.*t.^2).^63+39614081257132168796771975168./315285451704888104171289053925./(1+1./153.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./153.*t.^2).^50+67108864./300540195./(1+1./153.*t.^2).^15+2147483648./9917826435./(1+1./153.*t.^2).^16+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./153.*t.^2).^64+316912650057057350374175801344./2572482003204667409956015708221./(1+1./153.*t.^2).^51+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./153.*t.^2).^65+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./153.*t.^2).^66+4294967296./20419054425./(1+1./153.*t.^2).^17+17179869184./83945001525./(1+1./153.*t.^2).^18+147573952589676412928./995653215622882753209./(1+1./153.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./153.*t.^2).^36+549755813888./2893136075115./(1+1./153.*t.^2).^21+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./153.*t.^2).^67+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./153.*t.^2).^68+34359738368./172308161025./(1+1./153.*t.^2).^19+274877906944./1412926920405./(1+1./153.*t.^2).^20+2361183241434822606848./16369974040646495717175./(1+1./153.*t.^2).^37+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./153.*t.^2).^69+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./153.*t.^2).^70+4398046511104./24185702762325./(1+1./153.*t.^2).^23+2199023255552./11835556670925./(1+1./153.*t.^2).^22+70368744177664./395033145117975./(1+1./153.*t.^2).^24+140737488355328./805867616040669./(1+1./153.*t.^2).^25+9444732965739290427392./66341473743672640538025./(1+1./153.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./153.*t.^2).^39+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./153.*t.^2).^71+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./153.*t.^2).^72+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./153.*t.^2).^73+562949953421312./3285460280781189./(1+1./153.*t.^2).^26+1125899906842624./6692604275665385./(1+1./153.*t.^2).^27+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./153.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./153.*t.^2).^53+9007199254740992./54496920530418135./(1+1./153.*t.^2).^28+18014398509481984./110873045217057585./(1+1./153.*t.^2).^29+302231454903657293676544./2177020976850057573347805./(1+1./153.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./153.*t.^2).^41+72057594037927936./450883717216034179./(1+1./153.*t.^2).^30+144115188075855872./916312070471295267./(1+1./153.*t.^2).^31);

        case 154;
            alpha(set_t)=1./2+1./2.*t./(154+t.^2).^(1./2).*(1+50803160635786570329644235./618970019642690137449562112./(1+1./154.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./154.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./154.*t.^2).^49+861577581086657669325./9444732965739290427392./(1+1./154.*t.^2).^38+956086325095055./9007199254740992./(1+1./154.*t.^2).^28+1879204156221315./18014398509481984./(1+1./154.*t.^2).^29+26876802183334044115405./302231454903657293676544./(1+1./154.*t.^2).^40+1701063429324939500975./18889465931478580854784./(1+1./154.*t.^2).^39+1./2./(1+1./154.*t.^2)+3./8./(1+1./154.*t.^2).^2+392032816163265715595619229845./5070602400912917605986812821504./(1+1./154.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./154.*t.^2).^54+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./154.*t.^2).^63+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./154.*t.^2).^64+5./16./(1+1./154.*t.^2).^3+35./128./(1+1./154.*t.^2).^4+7391536347803839./72057594037927936./(1+1./154.*t.^2).^30+14544636039226909./144115188075855872./(1+1./154.*t.^2).^31+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./154.*t.^2).^65+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./154.*t.^2).^66+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./154.*t.^2).^55+231./1024./(1+1./154.*t.^2).^6+429./2048./(1+1./154.*t.^2).^7+63./256./(1+1./154.*t.^2).^5+53098072606098965203605./604462909807314587353088./(1+1./154.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./154.*t.^2).^42+6435./32768./(1+1./154.*t.^2).^8+12155./65536./(1+1./154.*t.^2).^9+46189./262144./(1+1./154.*t.^2).^10+414847067813984717066925./4835703278458516698824704./(1+1./154.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./154.*t.^2).^44+88179./524288./(1+1./154.*t.^2).^11+676039./4194304./(1+1./154.*t.^2).^12+1300075./8388608./(1+1./154.*t.^2).^13+67282234305./549755813888./(1+1./154.*t.^2).^21+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./154.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./154.*t.^2).^57+12611418068195524166851562157./158456325028528675187087900672./(1+1./154.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./154.*t.^2).^51+5014575./33554432./(1+1./154.*t.^2).^14+9694845./67108864./(1+1./154.*t.^2).^15+300540195./2147483648./(1+1./154.*t.^2).^16+583401555./4294967296./(1+1./154.*t.^2).^17+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./154.*t.^2).^67+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./154.*t.^2).^58+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./154.*t.^2).^59+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./154.*t.^2).^72+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./154.*t.^2).^73+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./154.*t.^2).^74+916312070471295267./9223372036854775808./(1+1./154.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./154.*t.^2).^33+218266320541953276229./2361183241434822606848./(1+1./154.*t.^2).^37+2268783825./17179869184./(1+1./154.*t.^2).^18+7113260368810144185./73786976294838206464./(1+1./154.*t.^2).^34+197883231015743646919693516017./2535301200456458802993406410752./(1+1./154.*t.^2).^52+4418157975./34359738368./(1+1./154.*t.^2).^19+34461632205./274877906944./(1+1./154.*t.^2).^20+6489213830472088210604445./77371252455336267181195264./(1+1./154.*t.^2).^45+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./154.*t.^2).^68+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./154.*t.^2).^69+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./154.*t.^2).^60+263012370465./2199023255552./(1+1./154.*t.^2).^22+514589420475./4398046511104./(1+1./154.*t.^2).^23+5801629358544299356186613429537820121257345./89202980794122492566142873090593446023921664./(1+1./154.*t.^2).^75+46107685954746800146535717255800570437361005./713623846352979940529142984724747568191373312./(1+1./154.*t.^2).^76+8061900920775./70368744177664./(1+1./154.*t.^2).^24+15801325804719./140737488355328./(1+1./154.*t.^2).^25+61989816618513./562949953421312./(1+1./154.*t.^2).^26+121683714103007./1125899906842624./(1+1./154.*t.^2).^27+14023284727082855679./147573952589676412928./(1+1./154.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./154.*t.^2).^36+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./154.*t.^2).^70+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./154.*t.^2).^71+25674715590128696833261065./309485009821345068724781056./(1+1./154.*t.^2).^46+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./154.*t.^2).^61+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./154.*t.^2).^62);

        case 155;
            alpha(set_t)=1./2+1./pi.*atan(1./155.*t.*155.^(1./2))+1./pi.*t.*155.^(1./2)./(155+t.^2).*(1+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./155.*t.^2).^64+4835703278458516698824704./36091694899816670384822475./(1+1./155.*t.^2).^43+2417851639229258349412352./17838423916001342833877775./(1+1./155.*t.^2).^42+33554432./145422675./(1+1./155.*t.^2).^14+67108864./300540195./(1+1./155.*t.^2).^15+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./155.*t.^2).^65+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./155.*t.^2).^66+16./35./(1+1./155.*t.^2).^3+38685626227668133590597632./292014622371243969477200025./(1+1./155.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./155.*t.^2).^45+2./3./(1+1./155.*t.^2)+8./15./(1+1./155.*t.^2).^2+70368744177664./395033145117975./(1+1./155.*t.^2).^24+140737488355328./805867616040669./(1+1./155.*t.^2).^25+2147483648./9917826435./(1+1./155.*t.^2).^16+4294967296./20419054425./(1+1./155.*t.^2).^17+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./155.*t.^2).^73+44601490397061246283071436545296723011960832./435122201890822451713996007215336509094300875./(1+1./155.*t.^2).^74+17179869184./83945001525./(1+1./155.*t.^2).^18+34359738368./172308161025./(1+1./155.*t.^2).^19+309485009821345068724781056./2387748549881968805493279045./(1+1./155.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./155.*t.^2).^47+562949953421312./3285460280781189./(1+1./155.*t.^2).^26+1125899906842624./6692604275665385./(1+1./155.*t.^2).^27+9007199254740992./54496920530418135./(1+1./155.*t.^2).^28+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./155.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./155.*t.^2).^55+89202980794122492566142873090593446023921664./876046033140189202784178627860210838309859095./(1+1./155.*t.^2).^75+713623846352979940529142984724747568191373312./7054475951076260422419964740137487276916233765./(1+1./155.*t.^2).^76+18014398509481984./110873045217057585./(1+1./155.*t.^2).^29+72057594037927936./450883717216034179./(1+1./155.*t.^2).^30+144115188075855872./916312070471295267./(1+1./155.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./155.*t.^2).^32+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./155.*t.^2).^67+128./315./(1+1./155.*t.^2).^4+256./693./(1+1./155.*t.^2).^5+18446744073709551616./120925426269772451145./(1+1./155.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./155.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./155.*t.^2).^35+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./155.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./155.*t.^2).^57+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./155.*t.^2).^68+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./155.*t.^2).^69+1180591620717411303424./8075853860052271220473./(1+1./155.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./155.*t.^2).^37+1024./3003./(1+1./155.*t.^2).^6+2048./6435./(1+1./155.*t.^2).^7+19807040628566084398385987584./156050375086257748529223875175./(1+1./155.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./155.*t.^2).^49+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./155.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./155.*t.^2).^59+32768./109395./(1+1./155.*t.^2).^8+65536./230945./(1+1./155.*t.^2).^9+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./155.*t.^2).^60+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./155.*t.^2).^61+274877906944./1412926920405./(1+1./155.*t.^2).^20+549755813888./2893136075115./(1+1./155.*t.^2).^21+158456325028528675187087900672./1273753224887747940852007777857./(1+1./155.*t.^2).^50+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./155.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./155.*t.^2).^63+9444732965739290427392./66341473743672640538025./(1+1./155.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./155.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./155.*t.^2).^40+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./155.*t.^2).^70+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./155.*t.^2).^71+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./155.*t.^2).^72+262144./969969./(1+1./155.*t.^2).^10+524288./2028117./(1+1./155.*t.^2).^11+2199023255552./11835556670925./(1+1./155.*t.^2).^22+4398046511104./24185702762325./(1+1./155.*t.^2).^23+316912650057057350374175801344./2572482003204667409956015708221./(1+1./155.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./155.*t.^2).^52+604462909807314587353088./4407140026306214111899215./(1+1./155.*t.^2).^41+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./155.*t.^2).^53+4194304./16900975./(1+1./155.*t.^2).^12+8388608./35102025./(1+1./155.*t.^2).^13);

        case 156;
            alpha(set_t)=1./2+1./2.*t./(156+t.^2).^(1./2).*(1+7391536347803839./72057594037927936./(1+1./156.*t.^2).^30+14544636039226909./144115188075855872./(1+1./156.*t.^2).^31+218266320541953276229./2361183241434822606848./(1+1./156.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./156.*t.^2).^38+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./156.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./156.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./156.*t.^2).^58+63./256./(1+1./156.*t.^2).^5+231./1024./(1+1./156.*t.^2).^6+429./2048./(1+1./156.*t.^2).^7+414847067813984717066925./4835703278458516698824704./(1+1./156.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./156.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./156.*t.^2).^45+916312070471295267./9223372036854775808./(1+1./156.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./156.*t.^2).^33+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./156.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./156.*t.^2).^61+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./156.*t.^2).^59+6435./32768./(1+1./156.*t.^2).^8+12155./65536./(1+1./156.*t.^2).^9+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./156.*t.^2).^62+46189./262144./(1+1./156.*t.^2).^10+88179./524288./(1+1./156.*t.^2).^11+676039./4194304./(1+1./156.*t.^2).^12+1300075./8388608./(1+1./156.*t.^2).^13+25674715590128696833261065./309485009821345068724781056./(1+1./156.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./156.*t.^2).^47+5014575./33554432./(1+1./156.*t.^2).^14+9694845./67108864./(1+1./156.*t.^2).^15+300540195./2147483648./(1+1./156.*t.^2).^16+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./156.*t.^2).^63+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./156.*t.^2).^64+1701063429324939500975./18889465931478580854784./(1+1./156.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./156.*t.^2).^40+583401555./4294967296./(1+1./156.*t.^2).^17+2268783825./17179869184./(1+1./156.*t.^2).^18+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./156.*t.^2).^65+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./156.*t.^2).^66+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./156.*t.^2).^67+4418157975./34359738368./(1+1./156.*t.^2).^19+34461632205./274877906944./(1+1./156.*t.^2).^20+67282234305./549755813888./(1+1./156.*t.^2).^21+263012370465./2199023255552./(1+1./156.*t.^2).^22+514589420475./4398046511104./(1+1./156.*t.^2).^23+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./156.*t.^2).^68+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./156.*t.^2).^69+53098072606098965203605./604462909807314587353088./(1+1./156.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./156.*t.^2).^42+1./2./(1+1./156.*t.^2)+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./156.*t.^2).^70+1608766753466574727105400775./19807040628566084398385987584./(1+1./156.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./156.*t.^2).^49+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./156.*t.^2).^73+12611418068195524166851562157./158456325028528675187087900672./(1+1./156.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./156.*t.^2).^51+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./156.*t.^2).^71+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./156.*t.^2).^72+8061900920775./70368744177664./(1+1./156.*t.^2).^24+15801325804719./140737488355328./(1+1./156.*t.^2).^25+61989816618513./562949953421312./(1+1./156.*t.^2).^26+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./156.*t.^2).^74+5801629358544299356186613429537820121257345./89202980794122492566142873090593446023921664./(1+1./156.*t.^2).^75+46107685954746800146535717255800570437361005./713623846352979940529142984724747568191373312./(1+1./156.*t.^2).^76+91616570793198187304155386235551782817093945./1427247692705959881058285969449495136382746624./(1+1./156.*t.^2).^77+956086325095055./9007199254740992./(1+1./156.*t.^2).^28+1879204156221315./18014398509481984./(1+1./156.*t.^2).^29+121683714103007./1125899906842624./(1+1./156.*t.^2).^27+197883231015743646919693516017./2535301200456458802993406410752./(1+1./156.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./156.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./156.*t.^2).^54+7113260368810144185./73786976294838206464./(1+1./156.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./156.*t.^2).^35+3./8./(1+1./156.*t.^2).^2+5./16./(1+1./156.*t.^2).^3+35./128./(1+1./156.*t.^2).^4+110628135069209194801./1180591620717411303424./(1+1./156.*t.^2).^36+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./156.*t.^2).^55);

        case 157;
            alpha(set_t)=1./2+1./pi.*atan(1./157.*t.*157.^(1./2))+1./pi.*t.*157.^(1./2)./(157+t.^2).*(1+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./157.*t.^2).^67+128./315./(1+1./157.*t.^2).^4+256./693./(1+1./157.*t.^2).^5+316912650057057350374175801344./2572482003204667409956015708221./(1+1./157.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./157.*t.^2).^52+1024./3003./(1+1./157.*t.^2).^6+2048./6435./(1+1./157.*t.^2).^7+32768./109395./(1+1./157.*t.^2).^8+65536./230945./(1+1./157.*t.^2).^9+262144./969969./(1+1./157.*t.^2).^10+2417851639229258349412352./17838423916001342833877775./(1+1./157.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./157.*t.^2).^43+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./157.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./157.*t.^2).^54+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./157.*t.^2).^68+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./157.*t.^2).^69+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./157.*t.^2).^70+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./157.*t.^2).^71+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./157.*t.^2).^72+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./157.*t.^2).^55+524288./2028117./(1+1./157.*t.^2).^11+4194304./16900975./(1+1./157.*t.^2).^12+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./157.*t.^2).^73+44601490397061246283071436545296723011960832./435122201890822451713996007215336509094300875./(1+1./157.*t.^2).^74+89202980794122492566142873090593446023921664./876046033140189202784178627860210838309859095./(1+1./157.*t.^2).^75+713623846352979940529142984724747568191373312./7054475951076260422419964740137487276916233765./(1+1./157.*t.^2).^76+8388608./35102025./(1+1./157.*t.^2).^13+33554432./145422675./(1+1./157.*t.^2).^14+67108864./300540195./(1+1./157.*t.^2).^15+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./157.*t.^2).^56+18446744073709551616./120925426269772451145./(1+1./157.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./157.*t.^2).^34+2147483648./9917826435./(1+1./157.*t.^2).^16+4294967296./20419054425./(1+1./157.*t.^2).^17+1427247692705959881058285969449495136382746624./14200568472945719032144084866510526336649561475./(1+1./157.*t.^2).^77+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./157.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./157.*t.^2).^58+17179869184./83945001525./(1+1./157.*t.^2).^18+34359738368./172308161025./(1+1./157.*t.^2).^19+274877906944./1412926920405./(1+1./157.*t.^2).^20+38685626227668133590597632./292014622371243969477200025./(1+1./157.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./157.*t.^2).^45+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./157.*t.^2).^59+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./157.*t.^2).^60+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./157.*t.^2).^61+549755813888./2893136075115./(1+1./157.*t.^2).^21+2199023255552./11835556670925./(1+1./157.*t.^2).^22+147573952589676412928./995653215622882753209./(1+1./157.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./157.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./157.*t.^2).^37+309485009821345068724781056./2387748549881968805493279045./(1+1./157.*t.^2).^46+9444732965739290427392./66341473743672640538025./(1+1./157.*t.^2).^38+70368744177664./395033145117975./(1+1./157.*t.^2).^24+2./3./(1+1./157.*t.^2)+8./15./(1+1./157.*t.^2).^2+16./35./(1+1./157.*t.^2).^3+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./157.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./157.*t.^2).^63+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./157.*t.^2).^64+4398046511104./24185702762325./(1+1./157.*t.^2).^23+618970019642690137449562112./4826300260399724181316202325./(1+1./157.*t.^2).^47+302231454903657293676544./2177020976850057573347805./(1+1./157.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./157.*t.^2).^41+18889465931478580854784./134384010916670220577025./(1+1./157.*t.^2).^39+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./157.*t.^2).^65+140737488355328./805867616040669./(1+1./157.*t.^2).^25+562949953421312./3285460280781189./(1+1./157.*t.^2).^26+1125899906842624./6692604275665385./(1+1./157.*t.^2).^27+9007199254740992./54496920530418135./(1+1./157.*t.^2).^28+18014398509481984./110873045217057585./(1+1./157.*t.^2).^29+72057594037927936./450883717216034179./(1+1./157.*t.^2).^30+144115188075855872./916312070471295267./(1+1./157.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./157.*t.^2).^32+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./157.*t.^2).^66+19807040628566084398385987584./156050375086257748529223875175./(1+1./157.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./157.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./157.*t.^2).^50);

        case 158;
            alpha(set_t)=1./2+1./2.*t./(158+t.^2).^(1./2).*(1+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./158.*t.^2).^55+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./158.*t.^2).^72+429./2048./(1+1./158.*t.^2).^7+6435./32768./(1+1./158.*t.^2).^8+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./158.*t.^2).^67+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./158.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./158.*t.^2).^57+209863810776486386280915./2417851639229258349412352./(1+1./158.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./158.*t.^2).^43+50803160635786570329644235./618970019642690137449562112./(1+1./158.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./158.*t.^2).^48+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./158.*t.^2).^69+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./158.*t.^2).^58+12155./65536./(1+1./158.*t.^2).^9+46189./262144./(1+1./158.*t.^2).^10+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./158.*t.^2).^73+88179./524288./(1+1./158.*t.^2).^11+676039./4194304./(1+1./158.*t.^2).^12+1300075./8388608./(1+1./158.*t.^2).^13+5014575./33554432./(1+1./158.*t.^2).^14+300540195./2147483648./(1+1./158.*t.^2).^16+53098072606098965203605./604462909807314587353088./(1+1./158.*t.^2).^41+583401555./4294967296./(1+1./158.*t.^2).^17+2268783825./17179869184./(1+1./158.*t.^2).^18+67282234305./549755813888./(1+1./158.*t.^2).^21+263012370465./2199023255552./(1+1./158.*t.^2).^22+4418157975./34359738368./(1+1./158.*t.^2).^19+34461632205./274877906944./(1+1./158.*t.^2).^20+514589420475./4398046511104./(1+1./158.*t.^2).^23+8061900920775./70368744177664./(1+1./158.*t.^2).^24+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./158.*t.^2).^59+3281063172710606398620225./38685626227668133590597632./(1+1./158.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./158.*t.^2).^45+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./158.*t.^2).^60+15801325804719./140737488355328./(1+1./158.*t.^2).^25+61989816618513./562949953421312./(1+1./158.*t.^2).^26+3./8./(1+1./158.*t.^2).^2+1879204156221315./18014398509481984./(1+1./158.*t.^2).^29+121683714103007./1125899906842624./(1+1./158.*t.^2).^27+956086325095055./9007199254740992./(1+1./158.*t.^2).^28+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./158.*t.^2).^70+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./158.*t.^2).^71+7391536347803839./72057594037927936./(1+1./158.*t.^2).^30+14544636039226909./144115188075855872./(1+1./158.*t.^2).^31+1804857108504066435./18446744073709551616./(1+1./158.*t.^2).^33+916312070471295267./9223372036854775808./(1+1./158.*t.^2).^32+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./158.*t.^2).^74+5801629358544299356186613429537820121257345./89202980794122492566142873090593446023921664./(1+1./158.*t.^2).^75+46107685954746800146535717255800570437361005./713623846352979940529142984724747568191373312./(1+1./158.*t.^2).^76+9694845./67108864./(1+1./158.*t.^2).^15+7113260368810144185./73786976294838206464./(1+1./158.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./158.*t.^2).^35+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./158.*t.^2).^61+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./158.*t.^2).^62+25674715590128696833261065./309485009821345068724781056./(1+1./158.*t.^2).^46+91616570793198187304155386235551782817093945./1427247692705959881058285969449495136382746624./(1+1./158.*t.^2).^77+364117140331941513644720124782321188119219525./5708990770823839524233143877797980545530986496./(1+1./158.*t.^2).^78+1./2./(1+1./158.*t.^2)+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./158.*t.^2).^63+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./158.*t.^2).^64+3184701532372607112841303575./39614081257132168796771975168./(1+1./158.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./158.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./158.*t.^2).^51+110628135069209194801./1180591620717411303424./(1+1./158.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./158.*t.^2).^37+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./158.*t.^2).^65+861577581086657669325./9444732965739290427392./(1+1./158.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./158.*t.^2).^39+5./16./(1+1./158.*t.^2).^3+35./128./(1+1./158.*t.^2).^4+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./158.*t.^2).^66+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./158.*t.^2).^68+63./256./(1+1./158.*t.^2).^5+231./1024./(1+1./158.*t.^2).^6+26876802183334044115405./302231454903657293676544./(1+1./158.*t.^2).^40+197883231015743646919693516017./2535301200456458802993406410752./(1+1./158.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./158.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./158.*t.^2).^54);

        case 159;
            alpha(set_t)=1./2+1./pi.*atan(1./159.*t.*159.^(1./2))+1./pi.*t.*159.^(1./2)./(159+t.^2).*(1+524288./2028117./(1+1./159.*t.^2).^11+4194304./16900975./(1+1./159.*t.^2).^12+562949953421312./3285460280781189./(1+1./159.*t.^2).^26+1125899906842624./6692604275665385./(1+1./159.*t.^2).^27+9007199254740992./54496920530418135./(1+1./159.*t.^2).^28+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./159.*t.^2).^68+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./159.*t.^2).^69+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./159.*t.^2).^70+302231454903657293676544./2177020976850057573347805./(1+1./159.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./159.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./159.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./159.*t.^2).^43+8388608./35102025./(1+1./159.*t.^2).^13+33554432./145422675./(1+1./159.*t.^2).^14+67108864./300540195./(1+1./159.*t.^2).^15+73786976294838206464./490814965447899948765./(1+1./159.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./159.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./159.*t.^2).^36+2147483648./9917826435./(1+1./159.*t.^2).^16+4294967296./20419054425./(1+1./159.*t.^2).^17+17179869184./83945001525./(1+1./159.*t.^2).^18+618970019642690137449562112./4826300260399724181316202325./(1+1./159.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./159.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./159.*t.^2).^49+1024./3003./(1+1./159.*t.^2).^6+2048./6435./(1+1./159.*t.^2).^7+32768./109395./(1+1./159.*t.^2).^8+89202980794122492566142873090593446023921664./876046033140189202784178627860210838309859095./(1+1./159.*t.^2).^75+713623846352979940529142984724747568191373312./7054475951076260422419964740137487276916233765./(1+1./159.*t.^2).^76+158456325028528675187087900672./1273753224887747940852007777857./(1+1./159.*t.^2).^50+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./159.*t.^2).^65+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./159.*t.^2).^66+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./159.*t.^2).^67+18014398509481984./110873045217057585./(1+1./159.*t.^2).^29+72057594037927936./450883717216034179./(1+1./159.*t.^2).^30+144115188075855872./916312070471295267./(1+1./159.*t.^2).^31+38685626227668133590597632./292014622371243969477200025./(1+1./159.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./159.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./159.*t.^2).^46+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./159.*t.^2).^71+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./159.*t.^2).^72+316912650057057350374175801344./2572482003204667409956015708221./(1+1./159.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./159.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./159.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./159.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./159.*t.^2).^55+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./159.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./159.*t.^2).^57+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./159.*t.^2).^73+44601490397061246283071436545296723011960832./435122201890822451713996007215336509094300875./(1+1./159.*t.^2).^74+34359738368./172308161025./(1+1./159.*t.^2).^19+274877906944./1412926920405./(1+1./159.*t.^2).^20+9223372036854775808./59560284580634192355./(1+1./159.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./159.*t.^2).^33+2361183241434822606848./16369974040646495717175./(1+1./159.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./159.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./159.*t.^2).^39+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./159.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./159.*t.^2).^59+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./159.*t.^2).^60+549755813888./2893136075115./(1+1./159.*t.^2).^21+2199023255552./11835556670925./(1+1./159.*t.^2).^22+4398046511104./24185702762325./(1+1./159.*t.^2).^23+65536./230945./(1+1./159.*t.^2).^9+262144./969969./(1+1./159.*t.^2).^10+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./159.*t.^2).^61+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./159.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./159.*t.^2).^63+1427247692705959881058285969449495136382746624./14200568472945719032144084866510526336649561475./(1+1./159.*t.^2).^77+5708990770823839524233143877797980545530986496./57166391032114817642221059590824426534717465425./(1+1./159.*t.^2).^78+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./159.*t.^2).^64+2./3./(1+1./159.*t.^2)+8./15./(1+1./159.*t.^2).^2+70368744177664./395033145117975./(1+1./159.*t.^2).^24+140737488355328./805867616040669./(1+1./159.*t.^2).^25+16./35./(1+1./159.*t.^2).^3+128./315./(1+1./159.*t.^2).^4+256./693./(1+1./159.*t.^2).^5);

        case 160;
            alpha(set_t)=1./2+1./2.*t./(160+t.^2).^(1./2).*(1+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./160.*t.^2).^73+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./160.*t.^2).^74+5801629358544299356186613429537820121257345./89202980794122492566142873090593446023921664./(1+1./160.*t.^2).^75+3184701532372607112841303575./39614081257132168796771975168./(1+1./160.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./160.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./160.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./160.*t.^2).^52+9694845./67108864./(1+1./160.*t.^2).^15+300540195./2147483648./(1+1./160.*t.^2).^16+583401555./4294967296./(1+1./160.*t.^2).^17+364117140331941513644720124782321188119219525./5708990770823839524233143877797980545530986496./(1+1./160.*t.^2).^78+723625202938162248635709615073726918160980575./11417981541647679048466287755595961091061972992./(1+1./160.*t.^2).^79+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./160.*t.^2).^64+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./160.*t.^2).^65+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./160.*t.^2).^66+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./160.*t.^2).^67+3281063172710606398620225./38685626227668133590597632./(1+1./160.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./160.*t.^2).^45+25674715590128696833261065./309485009821345068724781056./(1+1./160.*t.^2).^46+218266320541953276229./2361183241434822606848./(1+1./160.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./160.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./160.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./160.*t.^2).^40+4418157975./34359738368./(1+1./160.*t.^2).^19+34461632205./274877906944./(1+1./160.*t.^2).^20+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./160.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./160.*t.^2).^57+7113260368810144185./73786976294838206464./(1+1./160.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./160.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./160.*t.^2).^36+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./160.*t.^2).^68+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./160.*t.^2).^69+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./160.*t.^2).^70+50803160635786570329644235./618970019642690137449562112./(1+1./160.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./160.*t.^2).^48+231./1024./(1+1./160.*t.^2).^6+429./2048./(1+1./160.*t.^2).^7+67282234305./549755813888./(1+1./160.*t.^2).^21+263012370465./2199023255552./(1+1./160.*t.^2).^22+514589420475./4398046511104./(1+1./160.*t.^2).^23+8061900920775./70368744177664./(1+1./160.*t.^2).^24+15801325804719./140737488355328./(1+1./160.*t.^2).^25+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./160.*t.^2).^58+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./160.*t.^2).^59+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./160.*t.^2).^60+392032816163265715595619229845./5070602400912917605986812821504./(1+1./160.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./160.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./160.*t.^2).^55+53098072606098965203605./604462909807314587353088./(1+1./160.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./160.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./160.*t.^2).^43+6435./32768./(1+1./160.*t.^2).^8+12155./65536./(1+1./160.*t.^2).^9+46189./262144./(1+1./160.*t.^2).^10+1./2./(1+1./160.*t.^2)+3./8./(1+1./160.*t.^2).^2+88179./524288./(1+1./160.*t.^2).^11+676039./4194304./(1+1./160.*t.^2).^12+61989816618513./562949953421312./(1+1./160.*t.^2).^26+121683714103007./1125899906842624./(1+1./160.*t.^2).^27+956086325095055./9007199254740992./(1+1./160.*t.^2).^28+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./160.*t.^2).^71+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./160.*t.^2).^72+2268783825./17179869184./(1+1./160.*t.^2).^18+1879204156221315./18014398509481984./(1+1./160.*t.^2).^29+7391536347803839./72057594037927936./(1+1./160.*t.^2).^30+46107685954746800146535717255800570437361005./713623846352979940529142984724747568191373312./(1+1./160.*t.^2).^76+91616570793198187304155386235551782817093945./1427247692705959881058285969449495136382746624./(1+1./160.*t.^2).^77+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./160.*t.^2).^61+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./160.*t.^2).^62+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./160.*t.^2).^63+5./16./(1+1./160.*t.^2).^3+35./128./(1+1./160.*t.^2).^4+63./256./(1+1./160.*t.^2).^5+1300075./8388608./(1+1./160.*t.^2).^13+5014575./33554432./(1+1./160.*t.^2).^14+14544636039226909./144115188075855872./(1+1./160.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./160.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./160.*t.^2).^33);

        case 161;
            alpha(set_t)=1./2+1./pi.*atan(1./161.*t.*161.^(1./2))+1./pi.*t.*161.^(1./2)./(161+t.^2).*(1+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./161.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./161.*t.^2).^55+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./161.*t.^2).^56+18014398509481984./110873045217057585./(1+1./161.*t.^2).^29+72057594037927936./450883717216034179./(1+1./161.*t.^2).^30+144115188075855872./916312070471295267./(1+1./161.*t.^2).^31+9007199254740992./54496920530418135./(1+1./161.*t.^2).^28+4835703278458516698824704./36091694899816670384822475./(1+1./161.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./161.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./161.*t.^2).^45+9223372036854775808./59560284580634192355./(1+1./161.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./161.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./161.*t.^2).^34+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./161.*t.^2).^65+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./161.*t.^2).^66+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./161.*t.^2).^67+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./161.*t.^2).^68+262144./969969./(1+1./161.*t.^2).^10+524288./2028117./(1+1./161.*t.^2).^11+4194304./16900975./(1+1./161.*t.^2).^12+5708990770823839524233143877797980545530986496./57166391032114817642221059590824426534717465425./(1+1./161.*t.^2).^78+11417981541647679048466287755595961091061972992./115056407267167797533077828796722579987595911425./(1+1./161.*t.^2).^79+309485009821345068724781056./2387748549881968805493279045./(1+1./161.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./161.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./161.*t.^2).^48+2417851639229258349412352./17838423916001342833877775./(1+1./161.*t.^2).^42+2./3./(1+1./161.*t.^2)+8./15./(1+1./161.*t.^2).^2+8388608./35102025./(1+1./161.*t.^2).^13+33554432./145422675./(1+1./161.*t.^2).^14+67108864./300540195./(1+1./161.*t.^2).^15+2147483648./9917826435./(1+1./161.*t.^2).^16+4294967296./20419054425./(1+1./161.*t.^2).^17+17179869184./83945001525./(1+1./161.*t.^2).^18+16./35./(1+1./161.*t.^2).^3+128./315./(1+1./161.*t.^2).^4+256./693./(1+1./161.*t.^2).^5+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./161.*t.^2).^69+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./161.*t.^2).^70+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./161.*t.^2).^71+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./161.*t.^2).^61+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./161.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./161.*t.^2).^63+34359738368./172308161025./(1+1./161.*t.^2).^19+274877906944./1412926920405./(1+1./161.*t.^2).^20+549755813888./2893136075115./(1+1./161.*t.^2).^21+39614081257132168796771975168./315285451704888104171289053925./(1+1./161.*t.^2).^49+158456325028528675187087900672./1273753224887747940852007777857./(1+1./161.*t.^2).^50+316912650057057350374175801344./2572482003204667409956015708221./(1+1./161.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./161.*t.^2).^52+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./161.*t.^2).^64+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./161.*t.^2).^72+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./161.*t.^2).^73+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./161.*t.^2).^53+44601490397061246283071436545296723011960832./435122201890822451713996007215336509094300875./(1+1./161.*t.^2).^74+89202980794122492566142873090593446023921664./876046033140189202784178627860210838309859095./(1+1./161.*t.^2).^75+147573952589676412928./995653215622882753209./(1+1./161.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./161.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./161.*t.^2).^37+2199023255552./11835556670925./(1+1./161.*t.^2).^22+4398046511104./24185702762325./(1+1./161.*t.^2).^23+70368744177664./395033145117975./(1+1./161.*t.^2).^24+713623846352979940529142984724747568191373312./7054475951076260422419964740137487276916233765./(1+1./161.*t.^2).^76+1427247692705959881058285969449495136382746624./14200568472945719032144084866510526336649561475./(1+1./161.*t.^2).^77+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./161.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./161.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./161.*t.^2).^59+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./161.*t.^2).^60+1024./3003./(1+1./161.*t.^2).^6+2048./6435./(1+1./161.*t.^2).^7+32768./109395./(1+1./161.*t.^2).^8+65536./230945./(1+1./161.*t.^2).^9+9444732965739290427392./66341473743672640538025./(1+1./161.*t.^2).^38+18889465931478580854784./134384010916670220577025./(1+1./161.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./161.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./161.*t.^2).^41+140737488355328./805867616040669./(1+1./161.*t.^2).^25+562949953421312./3285460280781189./(1+1./161.*t.^2).^26+1125899906842624./6692604275665385./(1+1./161.*t.^2).^27);

        case 162;
            alpha(set_t)=1./2+1./2.*t./(162+t.^2).^(1./2).*(1+14544636039226909./144115188075855872./(1+1./162.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./162.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./162.*t.^2).^33+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./162.*t.^2).^67+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./162.*t.^2).^68+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./162.*t.^2).^69+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./162.*t.^2).^70+1./2./(1+1./162.*t.^2)+3./8./(1+1./162.*t.^2).^2+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./162.*t.^2).^63+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./162.*t.^2).^71+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./162.*t.^2).^72+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./162.*t.^2).^73+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./162.*t.^2).^74+7113260368810144185./73786976294838206464./(1+1./162.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./162.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./162.*t.^2).^36+5./16./(1+1./162.*t.^2).^3+35./128./(1+1./162.*t.^2).^4+63./256./(1+1./162.*t.^2).^5+231./1024./(1+1./162.*t.^2).^6+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./162.*t.^2).^64+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./162.*t.^2).^65+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./162.*t.^2).^66+67282234305./549755813888./(1+1./162.*t.^2).^21+263012370465./2199023255552./(1+1./162.*t.^2).^22+514589420475./4398046511104./(1+1./162.*t.^2).^23+8061900920775./70368744177664./(1+1./162.*t.^2).^24+218266320541953276229./2361183241434822606848./(1+1./162.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./162.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./162.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./162.*t.^2).^40+15801325804719./140737488355328./(1+1./162.*t.^2).^25+61989816618513./562949953421312./(1+1./162.*t.^2).^26+121683714103007./1125899906842624./(1+1./162.*t.^2).^27+429./2048./(1+1./162.*t.^2).^7+6435./32768./(1+1./162.*t.^2).^8+12155./65536./(1+1./162.*t.^2).^9+53098072606098965203605./604462909807314587353088./(1+1./162.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./162.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./162.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./162.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./162.*t.^2).^45+46189./262144./(1+1./162.*t.^2).^10+88179./524288./(1+1./162.*t.^2).^11+25674715590128696833261065./309485009821345068724781056./(1+1./162.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./162.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./162.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./162.*t.^2).^49+5801629358544299356186613429537820121257345./89202980794122492566142873090593446023921664./(1+1./162.*t.^2).^75+46107685954746800146535717255800570437361005./713623846352979940529142984724747568191373312./(1+1./162.*t.^2).^76+91616570793198187304155386235551782817093945./1427247692705959881058285969449495136382746624./(1+1./162.*t.^2).^77+364117140331941513644720124782321188119219525./5708990770823839524233143877797980545530986496./(1+1./162.*t.^2).^78+723625202938162248635709615073726918160980575./11417981541647679048466287755595961091061972992./(1+1./162.*t.^2).^79+23011281453433559506615565759344515997519182285./365375409332725729550921208179070754913983135744./(1+1./162.*t.^2).^80+956086325095055./9007199254740992./(1+1./162.*t.^2).^28+1879204156221315./18014398509481984./(1+1./162.*t.^2).^29+7391536347803839./72057594037927936./(1+1./162.*t.^2).^30+676039./4194304./(1+1./162.*t.^2).^12+1300075./8388608./(1+1./162.*t.^2).^13+5014575./33554432./(1+1./162.*t.^2).^14+9694845./67108864./(1+1./162.*t.^2).^15+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./162.*t.^2).^59+12611418068195524166851562157./158456325028528675187087900672./(1+1./162.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./162.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./162.*t.^2).^52+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./162.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./162.*t.^2).^61+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./162.*t.^2).^62+300540195./2147483648./(1+1./162.*t.^2).^16+583401555./4294967296./(1+1./162.*t.^2).^17+2268783825./17179869184./(1+1./162.*t.^2).^18+4418157975./34359738368./(1+1./162.*t.^2).^19+34461632205./274877906944./(1+1./162.*t.^2).^20+392032816163265715595619229845./5070602400912917605986812821504./(1+1./162.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./162.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./162.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./162.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./162.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./162.*t.^2).^58);

        case 163;
            alpha(set_t)=1./2+1./pi.*atan(1./163.*t.*163.^(1./2))+1./pi.*t.*163.^(1./2)./(163+t.^2).*(1+316912650057057350374175801344./2572482003204667409956015708221./(1+1./163.*t.^2).^51+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./163.*t.^2).^52+158456325028528675187087900672./1273753224887747940852007777857./(1+1./163.*t.^2).^50+70368744177664./395033145117975./(1+1./163.*t.^2).^24+140737488355328./805867616040669./(1+1./163.*t.^2).^25+562949953421312./3285460280781189./(1+1./163.*t.^2).^26+38685626227668133590597632./292014622371243969477200025./(1+1./163.*t.^2).^44+77371252455336267181195264./590518458572960027165004495./(1+1./163.*t.^2).^45+309485009821345068724781056./2387748549881968805493279045./(1+1./163.*t.^2).^46+618970019642690137449562112./4826300260399724181316202325./(1+1./163.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./163.*t.^2).^48+18889465931478580854784./134384010916670220577025./(1+1./163.*t.^2).^39+302231454903657293676544./2177020976850057573347805./(1+1./163.*t.^2).^40+604462909807314587353088./4407140026306214111899215./(1+1./163.*t.^2).^41+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./163.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./163.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./163.*t.^2).^55+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./163.*t.^2).^56+144115188075855872./916312070471295267./(1+1./163.*t.^2).^31+9223372036854775808./59560284580634192355./(1+1./163.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./163.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./163.*t.^2).^34+4194304./16900975./(1+1./163.*t.^2).^12+8388608./35102025./(1+1./163.*t.^2).^13+33554432./145422675./(1+1./163.*t.^2).^14+67108864./300540195./(1+1./163.*t.^2).^15+1125899906842624./6692604275665385./(1+1./163.*t.^2).^27+9007199254740992./54496920530418135./(1+1./163.*t.^2).^28+18014398509481984./110873045217057585./(1+1./163.*t.^2).^29+72057594037927936./450883717216034179./(1+1./163.*t.^2).^30+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./163.*t.^2).^63+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./163.*t.^2).^64+2147483648./9917826435./(1+1./163.*t.^2).^16+4294967296./20419054425./(1+1./163.*t.^2).^17+17179869184./83945001525./(1+1./163.*t.^2).^18+34359738368./172308161025./(1+1./163.*t.^2).^19+2./3./(1+1./163.*t.^2)+8./15./(1+1./163.*t.^2).^2+16./35./(1+1./163.*t.^2).^3+128./315./(1+1./163.*t.^2).^4+256./693./(1+1./163.*t.^2).^5+1024./3003./(1+1./163.*t.^2).^6+2048./6435./(1+1./163.*t.^2).^7+32768./109395./(1+1./163.*t.^2).^8+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./163.*t.^2).^73+44601490397061246283071436545296723011960832./435122201890822451713996007215336509094300875./(1+1./163.*t.^2).^74+89202980794122492566142873090593446023921664./876046033140189202784178627860210838309859095./(1+1./163.*t.^2).^75+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./163.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./163.*t.^2).^58+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./163.*t.^2).^59+713623846352979940529142984724747568191373312./7054475951076260422419964740137487276916233765./(1+1./163.*t.^2).^76+39614081257132168796771975168./315285451704888104171289053925./(1+1./163.*t.^2).^49+147573952589676412928./995653215622882753209./(1+1./163.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./163.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./163.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./163.*t.^2).^38+1427247692705959881058285969449495136382746624./14200568472945719032144084866510526336649561475./(1+1./163.*t.^2).^77+5708990770823839524233143877797980545530986496./57166391032114817642221059590824426534717465425./(1+1./163.*t.^2).^78+11417981541647679048466287755595961091061972992./115056407267167797533077828796722579987595911425./(1+1./163.*t.^2).^79+365375409332725729550921208179070754913983135744./3704816314002803080565106087254467075600588347885./(1+1./163.*t.^2).^80+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./163.*t.^2).^65+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./163.*t.^2).^66+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./163.*t.^2).^67+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./163.*t.^2).^68+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./163.*t.^2).^60+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./163.*t.^2).^61+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./163.*t.^2).^62+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./163.*t.^2).^69+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./163.*t.^2).^70+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./163.*t.^2).^71+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./163.*t.^2).^72+274877906944./1412926920405./(1+1./163.*t.^2).^20+549755813888./2893136075115./(1+1./163.*t.^2).^21+2199023255552./11835556670925./(1+1./163.*t.^2).^22+4398046511104./24185702762325./(1+1./163.*t.^2).^23+2417851639229258349412352./17838423916001342833877775./(1+1./163.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./163.*t.^2).^43+65536./230945./(1+1./163.*t.^2).^9+262144./969969./(1+1./163.*t.^2).^10+524288./2028117./(1+1./163.*t.^2).^11);

        case 164;
            alpha(set_t)=1./2+1./2.*t./(164+t.^2).^(1./2).*(1+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./164.*t.^2).^65+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./164.*t.^2).^66+514589420475./4398046511104./(1+1./164.*t.^2).^23+8061900920775./70368744177664./(1+1./164.*t.^2).^24+15801325804719./140737488355328./(1+1./164.*t.^2).^25+61989816618513./562949953421312./(1+1./164.*t.^2).^26+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./164.*t.^2).^68+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./164.*t.^2).^69+35./128./(1+1./164.*t.^2).^4+63./256./(1+1./164.*t.^2).^5+231./1024./(1+1./164.*t.^2).^6+429./2048./(1+1./164.*t.^2).^7+6435./32768./(1+1./164.*t.^2).^8+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./164.*t.^2).^73+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./164.*t.^2).^74+5801629358544299356186613429537820121257345./89202980794122492566142873090593446023921664./(1+1./164.*t.^2).^75+23011281453433559506615565759344515997519182285./365375409332725729550921208179070754913983135744./(1+1./164.*t.^2).^80+45738473012380284945248223299437865130871461085./730750818665451459101842416358141509827966271488./(1+1./164.*t.^2).^81+4418157975./34359738368./(1+1./164.*t.^2).^19+34461632205./274877906944./(1+1./164.*t.^2).^20+67282234305./549755813888./(1+1./164.*t.^2).^21+263012370465./2199023255552./(1+1./164.*t.^2).^22+12155./65536./(1+1./164.*t.^2).^9+46189./262144./(1+1./164.*t.^2).^10+24975553429171528252000152507./316912650057057350374175801344./(1+1./164.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./164.*t.^2).^52+46107685954746800146535717255800570437361005./713623846352979940529142984724747568191373312./(1+1./164.*t.^2).^76+91616570793198187304155386235551782817093945./1427247692705959881058285969449495136382746624./(1+1./164.*t.^2).^77+364117140331941513644720124782321188119219525./5708990770823839524233143877797980545530986496./(1+1./164.*t.^2).^78+723625202938162248635709615073726918160980575./11417981541647679048466287755595961091061972992./(1+1./164.*t.^2).^79+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./164.*t.^2).^60+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./164.*t.^2).^61+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./164.*t.^2).^67+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./164.*t.^2).^62+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./164.*t.^2).^63+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./164.*t.^2).^64+121683714103007./1125899906842624./(1+1./164.*t.^2).^27+956086325095055./9007199254740992./(1+1./164.*t.^2).^28+1879204156221315./18014398509481984./(1+1./164.*t.^2).^29+7391536347803839./72057594037927936./(1+1./164.*t.^2).^30+50803160635786570329644235./618970019642690137449562112./(1+1./164.*t.^2).^47+1608766753466574727105400775./19807040628566084398385987584./(1+1./164.*t.^2).^48+3184701532372607112841303575./39614081257132168796771975168./(1+1./164.*t.^2).^49+12611418068195524166851562157./158456325028528675187087900672./(1+1./164.*t.^2).^50+1./2./(1+1./164.*t.^2)+3./8./(1+1./164.*t.^2).^2+5./16./(1+1./164.*t.^2).^3+53098072606098965203605./604462909807314587353088./(1+1./164.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./164.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./164.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./164.*t.^2).^44+6489213830472088210604445./77371252455336267181195264./(1+1./164.*t.^2).^45+14544636039226909./144115188075855872./(1+1./164.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./164.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./164.*t.^2).^33+861577581086657669325./9444732965739290427392./(1+1./164.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./164.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./164.*t.^2).^40+7113260368810144185./73786976294838206464./(1+1./164.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./164.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./164.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./164.*t.^2).^37+392032816163265715595619229845./5070602400912917605986812821504./(1+1./164.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./164.*t.^2).^54+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./164.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./164.*t.^2).^56+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./164.*t.^2).^57+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./164.*t.^2).^58+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./164.*t.^2).^59+88179./524288./(1+1./164.*t.^2).^11+676039./4194304./(1+1./164.*t.^2).^12+1300075./8388608./(1+1./164.*t.^2).^13+5014575./33554432./(1+1./164.*t.^2).^14+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./164.*t.^2).^70+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./164.*t.^2).^71+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./164.*t.^2).^72+9694845./67108864./(1+1./164.*t.^2).^15+300540195./2147483648./(1+1./164.*t.^2).^16+583401555./4294967296./(1+1./164.*t.^2).^17+2268783825./17179869184./(1+1./164.*t.^2).^18+25674715590128696833261065./309485009821345068724781056./(1+1./164.*t.^2).^46);

        case 165;
            alpha(set_t)=1./2+1./pi.*atan(1./165.*t.*165.^(1./2))+1./pi.*t.*165.^(1./2)./(165+t.^2).*(1+1024./3003./(1+1./165.*t.^2).^6+2048./6435./(1+1./165.*t.^2).^7+32768./109395./(1+1./165.*t.^2).^8+262144./969969./(1+1./165.*t.^2).^10+65536./230945./(1+1./165.*t.^2).^9+524288./2028117./(1+1./165.*t.^2).^11+4194304./16900975./(1+1./165.*t.^2).^12+8388608./35102025./(1+1./165.*t.^2).^13+33554432./145422675./(1+1./165.*t.^2).^14+67108864./300540195./(1+1./165.*t.^2).^15+2147483648./9917826435./(1+1./165.*t.^2).^16+4294967296./20419054425./(1+1./165.*t.^2).^17+17179869184./83945001525./(1+1./165.*t.^2).^18+34359738368./172308161025./(1+1./165.*t.^2).^19+274877906944./1412926920405./(1+1./165.*t.^2).^20+549755813888./2893136075115./(1+1./165.*t.^2).^21+2199023255552./11835556670925./(1+1./165.*t.^2).^22+4398046511104./24185702762325./(1+1./165.*t.^2).^23+70368744177664./395033145117975./(1+1./165.*t.^2).^24+140737488355328./805867616040669./(1+1./165.*t.^2).^25+562949953421312./3285460280781189./(1+1./165.*t.^2).^26+1125899906842624./6692604275665385./(1+1./165.*t.^2).^27+18014398509481984./110873045217057585./(1+1./165.*t.^2).^29+9007199254740992./54496920530418135./(1+1./165.*t.^2).^28+144115188075855872./916312070471295267./(1+1./165.*t.^2).^31+72057594037927936./450883717216034179./(1+1./165.*t.^2).^30+9223372036854775808./59560284580634192355./(1+1./165.*t.^2).^32+18446744073709551616./120925426269772451145./(1+1./165.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./165.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./165.*t.^2).^35+1180591620717411303424./8075853860052271220473./(1+1./165.*t.^2).^36+2361183241434822606848./16369974040646495717175./(1+1./165.*t.^2).^37+9444732965739290427392./66341473743672640538025./(1+1./165.*t.^2).^38+302231454903657293676544./2177020976850057573347805./(1+1./165.*t.^2).^40+18889465931478580854784./134384010916670220577025./(1+1./165.*t.^2).^39+604462909807314587353088./4407140026306214111899215./(1+1./165.*t.^2).^41+2417851639229258349412352./17838423916001342833877775./(1+1./165.*t.^2).^42+4835703278458516698824704./36091694899816670384822475./(1+1./165.*t.^2).^43+38685626227668133590597632./292014622371243969477200025./(1+1./165.*t.^2).^44+309485009821345068724781056./2387748549881968805493279045./(1+1./165.*t.^2).^46+77371252455336267181195264./590518458572960027165004495./(1+1./165.*t.^2).^45+618970019642690137449562112./4826300260399724181316202325./(1+1./165.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./165.*t.^2).^48+39614081257132168796771975168./315285451704888104171289053925./(1+1./165.*t.^2).^49+316912650057057350374175801344./2572482003204667409956015708221./(1+1./165.*t.^2).^51+158456325028528675187087900672./1273753224887747940852007777857./(1+1./165.*t.^2).^50+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./165.*t.^2).^52+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./165.*t.^2).^53+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./165.*t.^2).^54+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./165.*t.^2).^55+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./165.*t.^2).^56+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./165.*t.^2).^57+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./165.*t.^2).^58+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./165.*t.^2).^60+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./165.*t.^2).^59+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./165.*t.^2).^61+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./165.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./165.*t.^2).^63+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./165.*t.^2).^65+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./165.*t.^2).^64+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./165.*t.^2).^66+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./165.*t.^2).^67+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./165.*t.^2).^68+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./165.*t.^2).^70+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./165.*t.^2).^69+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./165.*t.^2).^71+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./165.*t.^2).^72+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./165.*t.^2).^73+8./15./(1+1./165.*t.^2).^2+2./3./(1+1./165.*t.^2)+44601490397061246283071436545296723011960832./435122201890822451713996007215336509094300875./(1+1./165.*t.^2).^74+89202980794122492566142873090593446023921664./876046033140189202784178627860210838309859095./(1+1./165.*t.^2).^75+713623846352979940529142984724747568191373312./7054475951076260422419964740137487276916233765./(1+1./165.*t.^2).^76+16./35./(1+1./165.*t.^2).^3+1427247692705959881058285969449495136382746624./14200568472945719032144084866510526336649561475./(1+1./165.*t.^2).^77+5708990770823839524233143877797980545530986496./57166391032114817642221059590824426534717465425./(1+1./165.*t.^2).^78+11417981541647679048466287755595961091061972992./115056407267167797533077828796722579987595911425./(1+1./165.*t.^2).^79+365375409332725729550921208179070754913983135744./3704816314002803080565106087254467075600588347885./(1+1./165.*t.^2).^80+730750818665451459101842416358141509827966271488./7455371101017986446075460397808372016332048156855./(1+1./165.*t.^2).^81+128./315./(1+1./165.*t.^2).^4+256./693./(1+1./165.*t.^2).^5);

        case 166;
            alpha(set_t)=1./2+1./2.*t./(166+t.^2).^(1./2).*(1+25674715590128696833261065./309485009821345068724781056./(1+1./166.*t.^2).^46+6489213830472088210604445./77371252455336267181195264./(1+1./166.*t.^2).^45+50803160635786570329644235./618970019642690137449562112./(1+1./166.*t.^2).^47+3184701532372607112841303575./39614081257132168796771975168./(1+1./166.*t.^2).^49+1608766753466574727105400775./19807040628566084398385987584./(1+1./166.*t.^2).^48+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./166.*t.^2).^55+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./166.*t.^2).^56+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./166.*t.^2).^58+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./166.*t.^2).^57+1./2./(1+1./166.*t.^2)+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./166.*t.^2).^59+3./8./(1+1./166.*t.^2).^2+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./166.*t.^2).^60+5./16./(1+1./166.*t.^2).^3+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./166.*t.^2).^61+35./128./(1+1./166.*t.^2).^4+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./166.*t.^2).^62+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./166.*t.^2).^63+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./166.*t.^2).^64+63./256./(1+1./166.*t.^2).^5+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./166.*t.^2).^65+231./1024./(1+1./166.*t.^2).^6+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./166.*t.^2).^66+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./166.*t.^2).^67+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./166.*t.^2).^69+6435./32768./(1+1./166.*t.^2).^8+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./166.*t.^2).^68+429./2048./(1+1./166.*t.^2).^7+46189./262144./(1+1./166.*t.^2).^10+12155./65536./(1+1./166.*t.^2).^9+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./166.*t.^2).^70+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./166.*t.^2).^71+88179./524288./(1+1./166.*t.^2).^11+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./166.*t.^2).^72+676039./4194304./(1+1./166.*t.^2).^12+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./166.*t.^2).^73+1300075./8388608./(1+1./166.*t.^2).^13+5801629358544299356186613429537820121257345./89202980794122492566142873090593446023921664./(1+1./166.*t.^2).^75+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./166.*t.^2).^74+9694845./67108864./(1+1./166.*t.^2).^15+46107685954746800146535717255800570437361005./713623846352979940529142984724747568191373312./(1+1./166.*t.^2).^76+5014575./33554432./(1+1./166.*t.^2).^14+364117140331941513644720124782321188119219525./5708990770823839524233143877797980545530986496./(1+1./166.*t.^2).^78+91616570793198187304155386235551782817093945./1427247692705959881058285969449495136382746624./(1+1./166.*t.^2).^77+300540195./2147483648./(1+1./166.*t.^2).^16+583401555./4294967296./(1+1./166.*t.^2).^17+723625202938162248635709615073726918160980575./11417981541647679048466287755595961091061972992./(1+1./166.*t.^2).^79+23011281453433559506615565759344515997519182285./365375409332725729550921208179070754913983135744./(1+1./166.*t.^2).^80+2268783825./17179869184./(1+1./166.*t.^2).^18+181838319537024059660377082873374927227610930655./2923003274661805836407369665432566039311865085952./(1+1./166.*t.^2).^82+4418157975./34359738368./(1+1./166.*t.^2).^19+34461632205./274877906944./(1+1./166.*t.^2).^20+67282234305./549755813888./(1+1./166.*t.^2).^21+263012370465./2199023255552./(1+1./166.*t.^2).^22+514589420475./4398046511104./(1+1./166.*t.^2).^23+8061900920775./70368744177664./(1+1./166.*t.^2).^24+15801325804719./140737488355328./(1+1./166.*t.^2).^25+61989816618513./562949953421312./(1+1./166.*t.^2).^26+121683714103007./1125899906842624./(1+1./166.*t.^2).^27+956086325095055./9007199254740992./(1+1./166.*t.^2).^28+1879204156221315./18014398509481984./(1+1./166.*t.^2).^29+14544636039226909./144115188075855872./(1+1./166.*t.^2).^31+7391536347803839./72057594037927936./(1+1./166.*t.^2).^30+916312070471295267./9223372036854775808./(1+1./166.*t.^2).^32+45738473012380284945248223299437865130871461085./730750818665451459101842416358141509827966271488./(1+1./166.*t.^2).^81+1804857108504066435./18446744073709551616./(1+1./166.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./166.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./166.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./166.*t.^2).^36+218266320541953276229./2361183241434822606848./(1+1./166.*t.^2).^37+861577581086657669325./9444732965739290427392./(1+1./166.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./166.*t.^2).^39+26876802183334044115405./302231454903657293676544./(1+1./166.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./166.*t.^2).^41+209863810776486386280915./2417851639229258349412352./(1+1./166.*t.^2).^42+414847067813984717066925./4835703278458516698824704./(1+1./166.*t.^2).^43+3281063172710606398620225./38685626227668133590597632./(1+1./166.*t.^2).^44+12611418068195524166851562157./158456325028528675187087900672./(1+1./166.*t.^2).^50+24975553429171528252000152507./316912650057057350374175801344./(1+1./166.*t.^2).^51+197883231015743646919693516017./2535301200456458802993406410752./(1+1./166.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./166.*t.^2).^53+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./166.*t.^2).^54);

        case 167;
            alpha(set_t)=1./2+1./pi.*atan(1./167.*t.*167.^(1./2))+1./pi.*t.*167.^(1./2)./(167+t.^2).*(1+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./167.*t.^2).^67+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./167.*t.^2).^68+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./167.*t.^2).^59+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./167.*t.^2).^69+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./167.*t.^2).^70+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./167.*t.^2).^71+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./167.*t.^2).^72+89202980794122492566142873090593446023921664./876046033140189202784178627860210838309859095./(1+1./167.*t.^2).^75+44601490397061246283071436545296723011960832./435122201890822451713996007215336509094300875./(1+1./167.*t.^2).^74+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./167.*t.^2).^73+713623846352979940529142984724747568191373312./7054475951076260422419964740137487276916233765./(1+1./167.*t.^2).^76+1427247692705959881058285969449495136382746624./14200568472945719032144084866510526336649561475./(1+1./167.*t.^2).^77+5708990770823839524233143877797980545530986496./57166391032114817642221059590824426534717465425./(1+1./167.*t.^2).^78+11417981541647679048466287755595961091061972992./115056407267167797533077828796722579987595911425./(1+1./167.*t.^2).^79+77371252455336267181195264./590518458572960027165004495./(1+1./167.*t.^2).^45+365375409332725729550921208179070754913983135744./3704816314002803080565106087254467075600588347885./(1+1./167.*t.^2).^80+72057594037927936./450883717216034179./(1+1./167.*t.^2).^30+730750818665451459101842416358141509827966271488./7455371101017986446075460397808372016332048156855./(1+1./167.*t.^2).^81+2923003274661805836407369665432566039311865085952./30003322723608969843962218674106862992555803558075./(1+1./167.*t.^2).^82+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./167.*t.^2).^60+309485009821345068724781056./2387748549881968805493279045./(1+1./167.*t.^2).^46+144115188075855872./916312070471295267./(1+1./167.*t.^2).^31+618970019642690137449562112./4826300260399724181316202325./(1+1./167.*t.^2).^47+19807040628566084398385987584./156050375086257748529223875175./(1+1./167.*t.^2).^48+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./167.*t.^2).^61+9223372036854775808./59560284580634192355./(1+1./167.*t.^2).^32+39614081257132168796771975168./315285451704888104171289053925./(1+1./167.*t.^2).^49+18446744073709551616./120925426269772451145./(1+1./167.*t.^2).^33+73786976294838206464./490814965447899948765./(1+1./167.*t.^2).^34+147573952589676412928./995653215622882753209./(1+1./167.*t.^2).^35+158456325028528675187087900672./1273753224887747940852007777857./(1+1./167.*t.^2).^50+1180591620717411303424./8075853860052271220473./(1+1./167.*t.^2).^36+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./167.*t.^2).^62+2./3./(1+1./167.*t.^2)+8./15./(1+1./167.*t.^2).^2+16./35./(1+1./167.*t.^2).^3+316912650057057350374175801344./2572482003204667409956015708221./(1+1./167.*t.^2).^51+2361183241434822606848./16369974040646495717175./(1+1./167.*t.^2).^37+128./315./(1+1./167.*t.^2).^4+9444732965739290427392./66341473743672640538025./(1+1./167.*t.^2).^38+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./167.*t.^2).^63+256./693./(1+1./167.*t.^2).^5+18889465931478580854784./134384010916670220577025./(1+1./167.*t.^2).^39+1024./3003./(1+1./167.*t.^2).^6+2048./6435./(1+1./167.*t.^2).^7+302231454903657293676544./2177020976850057573347805./(1+1./167.*t.^2).^40+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./167.*t.^2).^52+32768./109395./(1+1./167.*t.^2).^8+65536./230945./(1+1./167.*t.^2).^9+262144./969969./(1+1./167.*t.^2).^10+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./167.*t.^2).^53+604462909807314587353088./4407140026306214111899215./(1+1./167.*t.^2).^41+524288./2028117./(1+1./167.*t.^2).^11+4194304./16900975./(1+1./167.*t.^2).^12+8388608./35102025./(1+1./167.*t.^2).^13+33554432./145422675./(1+1./167.*t.^2).^14+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./167.*t.^2).^64+2417851639229258349412352./17838423916001342833877775./(1+1./167.*t.^2).^42+67108864./300540195./(1+1./167.*t.^2).^15+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./167.*t.^2).^55+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./167.*t.^2).^54+2147483648./9917826435./(1+1./167.*t.^2).^16+4294967296./20419054425./(1+1./167.*t.^2).^17+17179869184./83945001525./(1+1./167.*t.^2).^18+34359738368./172308161025./(1+1./167.*t.^2).^19+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./167.*t.^2).^65+4835703278458516698824704./36091694899816670384822475./(1+1./167.*t.^2).^43+274877906944./1412926920405./(1+1./167.*t.^2).^20+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./167.*t.^2).^56+549755813888./2893136075115./(1+1./167.*t.^2).^21+2199023255552./11835556670925./(1+1./167.*t.^2).^22+4398046511104./24185702762325./(1+1./167.*t.^2).^23+38685626227668133590597632./292014622371243969477200025./(1+1./167.*t.^2).^44+70368744177664./395033145117975./(1+1./167.*t.^2).^24+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./167.*t.^2).^57+140737488355328./805867616040669./(1+1./167.*t.^2).^25+562949953421312./3285460280781189./(1+1./167.*t.^2).^26+1125899906842624./6692604275665385./(1+1./167.*t.^2).^27+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./167.*t.^2).^66+9007199254740992./54496920530418135./(1+1./167.*t.^2).^28+18014398509481984./110873045217057585./(1+1./167.*t.^2).^29+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./167.*t.^2).^58);

        case 168;
            alpha(set_t)=1./2+1./2.*t./(168+t.^2).^(1./2).*(1+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./168.*t.^2).^71+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./168.*t.^2).^72+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./168.*t.^2).^59+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./168.*t.^2).^73+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./168.*t.^2).^74+861577581086657669325./9444732965739290427392./(1+1./168.*t.^2).^38+5801629358544299356186613429537820121257345./89202980794122492566142873090593446023921664./(1+1./168.*t.^2).^75+12611418068195524166851562157./158456325028528675187087900672./(1+1./168.*t.^2).^50+46107685954746800146535717255800570437361005./713623846352979940529142984724747568191373312./(1+1./168.*t.^2).^76+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./168.*t.^2).^60+91616570793198187304155386235551782817093945./1427247692705959881058285969449495136382746624./(1+1./168.*t.^2).^77+1701063429324939500975./18889465931478580854784./(1+1./168.*t.^2).^39+364117140331941513644720124782321188119219525./5708990770823839524233143877797980545530986496./(1+1./168.*t.^2).^78+24975553429171528252000152507./316912650057057350374175801344./(1+1./168.*t.^2).^51+723625202938162248635709615073726918160980575./11417981541647679048466287755595961091061972992./(1+1./168.*t.^2).^79+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./168.*t.^2).^61+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./168.*t.^2).^62+26876802183334044115405./302231454903657293676544./(1+1./168.*t.^2).^40+23011281453433559506615565759344515997519182285./365375409332725729550921208179070754913983135744./(1+1./168.*t.^2).^80+45738473012380284945248223299437865130871461085./730750818665451459101842416358141509827966271488./(1+1./168.*t.^2).^81+53098072606098965203605./604462909807314587353088./(1+1./168.*t.^2).^41+181838319537024059660377082873374927227610930655./2923003274661805836407369665432566039311865085952./(1+1./168.*t.^2).^82+361485815947096022216412273182010397500672332025./5846006549323611672814739330865132078623730171904./(1+1./168.*t.^2).^83+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./168.*t.^2).^63+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./168.*t.^2).^64+197883231015743646919693516017./2535301200456458802993406410752./(1+1./168.*t.^2).^52+392032816163265715595619229845./5070602400912917605986812821504./(1+1./168.*t.^2).^53+209863810776486386280915./2417851639229258349412352./(1+1./168.*t.^2).^42+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./168.*t.^2).^54+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./168.*t.^2).^65+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./168.*t.^2).^66+414847067813984717066925./4835703278458516698824704./(1+1./168.*t.^2).^43+3./8./(1+1./168.*t.^2).^2+1./2./(1+1./168.*t.^2)+5./16./(1+1./168.*t.^2).^3+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./168.*t.^2).^55+3281063172710606398620225./38685626227668133590597632./(1+1./168.*t.^2).^44+35./128./(1+1./168.*t.^2).^4+63./256./(1+1./168.*t.^2).^5+231./1024./(1+1./168.*t.^2).^6+429./2048./(1+1./168.*t.^2).^7+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./168.*t.^2).^67+6435./32768./(1+1./168.*t.^2).^8+46189./262144./(1+1./168.*t.^2).^10+6489213830472088210604445./77371252455336267181195264./(1+1./168.*t.^2).^45+12155./65536./(1+1./168.*t.^2).^9+88179./524288./(1+1./168.*t.^2).^11+676039./4194304./(1+1./168.*t.^2).^12+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./168.*t.^2).^68+1300075./8388608./(1+1./168.*t.^2).^13+9694845./67108864./(1+1./168.*t.^2).^15+5014575./33554432./(1+1./168.*t.^2).^14+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./168.*t.^2).^56+300540195./2147483648./(1+1./168.*t.^2).^16+583401555./4294967296./(1+1./168.*t.^2).^17+2268783825./17179869184./(1+1./168.*t.^2).^18+4418157975./34359738368./(1+1./168.*t.^2).^19+25674715590128696833261065./309485009821345068724781056./(1+1./168.*t.^2).^46+50803160635786570329644235./618970019642690137449562112./(1+1./168.*t.^2).^47+34461632205./274877906944./(1+1./168.*t.^2).^20+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./168.*t.^2).^69+67282234305./549755813888./(1+1./168.*t.^2).^21+263012370465./2199023255552./(1+1./168.*t.^2).^22+514589420475./4398046511104./(1+1./168.*t.^2).^23+8061900920775./70368744177664./(1+1./168.*t.^2).^24+15801325804719./140737488355328./(1+1./168.*t.^2).^25+121683714103007./1125899906842624./(1+1./168.*t.^2).^27+1608766753466574727105400775./19807040628566084398385987584./(1+1./168.*t.^2).^48+61989816618513./562949953421312./(1+1./168.*t.^2).^26+956086325095055./9007199254740992./(1+1./168.*t.^2).^28+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./168.*t.^2).^57+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./168.*t.^2).^70+1879204156221315./18014398509481984./(1+1./168.*t.^2).^29+7391536347803839./72057594037927936./(1+1./168.*t.^2).^30+14544636039226909./144115188075855872./(1+1./168.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./168.*t.^2).^32+1804857108504066435./18446744073709551616./(1+1./168.*t.^2).^33+7113260368810144185./73786976294838206464./(1+1./168.*t.^2).^34+14023284727082855679./147573952589676412928./(1+1./168.*t.^2).^35+110628135069209194801./1180591620717411303424./(1+1./168.*t.^2).^36+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./168.*t.^2).^58+218266320541953276229./2361183241434822606848./(1+1./168.*t.^2).^37+3184701532372607112841303575./39614081257132168796771975168./(1+1./168.*t.^2).^49);

        case 169;
            alpha(set_t)=1./2+1./pi.*atan(1./169.*t.*169.^(1./2))+1./pi.*t.*169.^(1./2)./(169+t.^2).*(1+2./3./(1+1./169.*t.^2)+8./15./(1+1./169.*t.^2).^2+16./35./(1+1./169.*t.^2).^3+128./315./(1+1./169.*t.^2).^4+256./693./(1+1./169.*t.^2).^5+1024./3003./(1+1./169.*t.^2).^6+32768./109395./(1+1./169.*t.^2).^8+2048./6435./(1+1./169.*t.^2).^7+65536./230945./(1+1./169.*t.^2).^9+262144./969969./(1+1./169.*t.^2).^10+524288./2028117./(1+1./169.*t.^2).^11+4194304./16900975./(1+1./169.*t.^2).^12+8388608./35102025./(1+1./169.*t.^2).^13+33554432./145422675./(1+1./169.*t.^2).^14+2147483648./9917826435./(1+1./169.*t.^2).^16+67108864./300540195./(1+1./169.*t.^2).^15+17179869184./83945001525./(1+1./169.*t.^2).^18+4294967296./20419054425./(1+1./169.*t.^2).^17+34359738368./172308161025./(1+1./169.*t.^2).^19+274877906944./1412926920405./(1+1./169.*t.^2).^20+549755813888./2893136075115./(1+1./169.*t.^2).^21+70368744177664./395033145117975./(1+1./169.*t.^2).^24+4398046511104./24185702762325./(1+1./169.*t.^2).^23+2199023255552./11835556670925./(1+1./169.*t.^2).^22+140737488355328./805867616040669./(1+1./169.*t.^2).^25+562949953421312./3285460280781189./(1+1./169.*t.^2).^26+1125899906842624./6692604275665385./(1+1./169.*t.^2).^27+9007199254740992./54496920530418135./(1+1./169.*t.^2).^28+18014398509481984./110873045217057585./(1+1./169.*t.^2).^29+144115188075855872./916312070471295267./(1+1./169.*t.^2).^31+72057594037927936./450883717216034179./(1+1./169.*t.^2).^30+147573952589676412928./995653215622882753209./(1+1./169.*t.^2).^35+73786976294838206464./490814965447899948765./(1+1./169.*t.^2).^34+18446744073709551616./120925426269772451145./(1+1./169.*t.^2).^33+9223372036854775808./59560284580634192355./(1+1./169.*t.^2).^32+1180591620717411303424./8075853860052271220473./(1+1./169.*t.^2).^36+9444732965739290427392./66341473743672640538025./(1+1./169.*t.^2).^38+2361183241434822606848./16369974040646495717175./(1+1./169.*t.^2).^37+18889465931478580854784./134384010916670220577025./(1+1./169.*t.^2).^39+604462909807314587353088./4407140026306214111899215./(1+1./169.*t.^2).^41+302231454903657293676544./2177020976850057573347805./(1+1./169.*t.^2).^40+2417851639229258349412352./17838423916001342833877775./(1+1./169.*t.^2).^42+38685626227668133590597632./292014622371243969477200025./(1+1./169.*t.^2).^44+4835703278458516698824704./36091694899816670384822475./(1+1./169.*t.^2).^43+77371252455336267181195264./590518458572960027165004495./(1+1./169.*t.^2).^45+618970019642690137449562112./4826300260399724181316202325./(1+1./169.*t.^2).^47+309485009821345068724781056./2387748549881968805493279045./(1+1./169.*t.^2).^46+19807040628566084398385987584./156050375086257748529223875175./(1+1./169.*t.^2).^48+316912650057057350374175801344./2572482003204667409956015708221./(1+1./169.*t.^2).^51+158456325028528675187087900672./1273753224887747940852007777857./(1+1./169.*t.^2).^50+39614081257132168796771975168./315285451704888104171289053925./(1+1./169.*t.^2).^49+20282409603651670423947251286016./169343656848598816333026188062305./(1+1./169.*t.^2).^54+5070602400912917605986812821504./41947511329469431568731257593415./(1+1./169.*t.^2).^53+2535301200456458802993406410752./20777739256653082926567819181785./(1+1./169.*t.^2).^52+40564819207303340847894502572032./341766289276263065690289215907561./(1+1./169.*t.^2).^55+5192296858534827628530496329220096./44907665085988204046452430922486765./(1+1./169.*t.^2).^58+1298074214633706907132624082305024./11130959722168016387582226467966805./(1+1./169.*t.^2).^57+649037107316853453566312041152512./5517084384031103774714668771079199./(1+1./169.*t.^2).^56+83076749736557242056487941267521536./730650248105247627191941867630380711./(1+1./169.*t.^2).^60+10384593717069655257060992658440192./90576477037840614941149818301286865./(1+1./169.*t.^2).^59+166153499473114484112975882535043072./1473278369130253412206702454402243073./(1+1./169.*t.^2).^61+664613997892457936451903530140172288./5940638585202634726639929251621947875./(1+1./169.*t.^2).^62+1329227995784915872903807060280344576./11975573020964041433067793888190275875./(1+1./169.*t.^2).^63+170141183460469231731687303715884105728./1544848919704361344865745411576545587875./(1+1./169.*t.^2).^64+1361129467683753853853498429727072845824./12548206400004236695382695713705432996525./(1+1./169.*t.^2).^66+340282366920938463463374607431768211456./3113464745865712864267886906408114954025./(1+1./169.*t.^2).^65+2722258935367507707706996859454145691648./25283699462695103789203939124630350067625./(1+1./169.*t.^2).^67+21778071482940061661655974875633165533184./203756872140542895242408215298491644662625./(1+1./169.*t.^2).^68+43556142965880123323311949751266331066368./410466742428050180270938288789715052291375./(1+1./169.*t.^2).^69+348449143727040986586495598010130648530944./3330479246509768927486088014835824788471225./(1+1./169.*t.^2).^71+174224571863520493293247799005065324265472./1653594590924430726234351391981423496373825./(1+1./169.*t.^2).^70+5575186299632655785383929568162090376495104./53657721193768499387275862461243843814258625./(1+1./169.*t.^2).^72+11150372599265311570767859136324180752990208./108050479664163964519582901120586918365698875./(1+1./169.*t.^2).^73+44601490397061246283071436545296723011960832./435122201890822451713996007215336509094300875./(1+1./169.*t.^2).^74+89202980794122492566142873090593446023921664./876046033140189202784178627860210838309859095./(1+1./169.*t.^2).^75+713623846352979940529142984724747568191373312./7054475951076260422419964740137487276916233765./(1+1./169.*t.^2).^76+1427247692705959881058285969449495136382746624./14200568472945719032144084866510526336649561475./(1+1./169.*t.^2).^77+5708990770823839524233143877797980545530986496./57166391032114817642221059590824426534717465425./(1+1./169.*t.^2).^78+11417981541647679048466287755595961091061972992./115056407267167797533077828796722579987595911425./(1+1./169.*t.^2).^79+365375409332725729550921208179070754913983135744./3704816314002803080565106087254467075600588347885./(1+1./169.*t.^2).^80+2923003274661805836407369665432566039311865085952./30003322723608969843962218674106862992555803558075./(1+1./169.*t.^2).^82+730750818665451459101842416358141509827966271488./7455371101017986446075460397808372016332048156855./(1+1./169.*t.^2).^81+5846006549323611672814739330865132078623730171904./60368131263165035710140849621395736382612279448175./(1+1./169.*t.^2).^83);

        case 170;
            alpha(set_t)=1./2+1./2.*t./(170+t.^2).^(1./2).*(1+23766906456990174536396083255023778275./340282366920938463463374607431768211456./(1+1./170.*t.^2).^65+723625202938162248635709615073726918160980575./11417981541647679048466287755595961091061972992./(1+1./170.*t.^2).^79+23011281453433559506615565759344515997519182285./365375409332725729550921208179070754913983135744./(1+1./170.*t.^2).^80+1553611530721090058101157688645./20282409603651670423947251286016./(1+1./170.*t.^2).^54+110628135069209194801./1180591620717411303424./(1+1./170.*t.^2).^36+45738473012380284945248223299437865130871461085./730750818665451459101842416358141509827966271488./(1+1./170.*t.^2).^81+361485815947096022216412273182010397500672332025./5846006549323611672814739330865132078623730171904./(1+1./170.*t.^2).^83+181838319537024059660377082873374927227610930655./2923003274661805836407369665432566039311865085952./(1+1./170.*t.^2).^82+2874672917293573129054326172447416018219632354675./46768052394588893382517914646921056628989841375232./(1+1./170.*t.^2).^84+94347416541385238311148088072973180425./1361129467683753853853498429727072845824./(1+1./170.*t.^2).^66+3078975579065433024236839782951./40564819207303340847894502572032./(1+1./170.*t.^2).^55+218266320541953276229./2361183241434822606848./(1+1./170.*t.^2).^37+48823755610894723670041316558223./649037107316853453566312041152512./(1+1./170.*t.^2).^56+861577581086657669325./9444732965739290427392./(1+1./170.*t.^2).^38+1701063429324939500975./18889465931478580854784./(1+1./170.*t.^2).^39+187286662686630398438547697219484074575./2722258935367507707706996859454145691648./(1+1./170.*t.^2).^67+1487276438982064928776702301448844121625./21778071482940061661655974875633165533184./(1+1./170.*t.^2).^68+96790954105808838152888925808407./1298074214633706907132624082305024./(1+1./170.*t.^2).^57+26876802183334044115405./302231454903657293676544./(1+1./170.*t.^2).^40+53098072606098965203605./604462909807314587353088./(1+1./170.*t.^2).^41+2952998146964389786121858192731762966125./43556142965880123323311949751266331066368./(1+1./170.*t.^2).^69+383826197316138496123525050619545./5192296858534827628530496329220096./(1+1./170.*t.^2).^58+11727621212230005150598236822563287208325./174224571863520493293247799005065324265472./(1+1./170.*t.^2).^70+209863810776486386280915./2417851639229258349412352./(1+1./170.*t.^2).^42+23290064660907475017385230872977795723575./348449143727040986586495598010130648530944./(1+1./170.*t.^2).^71+761146865864206848244956456313335./10384593717069655257060992658440192./(1+1./170.*t.^2).^59+370053249612196547498454223870647198719025./5575186299632655785383929568162090376495104./(1+1./170.*t.^2).^72+1./2./(1+1./170.*t.^2)+414847067813984717066925./4835703278458516698824704./(1+1./170.*t.^2).^43+735037276626965745031176198099230737181625./11150372599265311570767859136324180752990208./(1+1./170.*t.^2).^73+3281063172710606398620225./38685626227668133590597632./(1+1./170.*t.^2).^44+3./8./(1+1./170.*t.^2).^2+6038431802522707662743321220085791./83076749736557242056487941267521536./(1+1./170.*t.^2).^60+5./16./(1+1./170.*t.^2).^3+35./128./(1+1./170.*t.^2).^4+63./256./(1+1./170.*t.^2).^5+231./1024./(1+1./170.*t.^2).^6+11977872919758157822818719141481651./166153499473114484112975882535043072./(1+1./170.*t.^2).^61+6489213830472088210604445./77371252455336267181195264./(1+1./170.*t.^2).^45+429./2048./(1+1./170.*t.^2).^7+2920283234166593635664402732988835631505375./44601490397061246283071436545296723011960832./(1+1./170.*t.^2).^74+25674715590128696833261065./309485009821345068724781056./(1+1./170.*t.^2).^46+6435./32768./(1+1./170.*t.^2).^8+47525108681621077813119434012975583./664613997892457936451903530140172288./(1+1./170.*t.^2).^62+12155./65536./(1+1./170.*t.^2).^9+50803160635786570329644235./618970019642690137449562112./(1+1./170.*t.^2).^47+46189./262144./(1+1./170.*t.^2).^10+5801629358544299356186613429537820121257345./89202980794122492566142873090593446023921664./(1+1./170.*t.^2).^75+88179./524288./(1+1./170.*t.^2).^11+676039./4194304./(1+1./170.*t.^2).^12+1300075./8388608./(1+1./170.*t.^2).^13+1608766753466574727105400775./19807040628566084398385987584./(1+1./170.*t.^2).^48+9694845./67108864./(1+1./170.*t.^2).^15+5014575./33554432./(1+1./170.*t.^2).^14+300540195./2147483648./(1+1./170.*t.^2).^16+583401555./4294967296./(1+1./170.*t.^2).^17+3184701532372607112841303575./39614081257132168796771975168./(1+1./170.*t.^2).^49+2268783825./17179869184./(1+1./170.*t.^2).^18+94295850558771979787935384946380125./1329227995784915872903807060280344576./(1+1./170.*t.^2).^63+4418157975./34359738368./(1+1./170.*t.^2).^19+67282234305./549755813888./(1+1./170.*t.^2).^21+34461632205./274877906944./(1+1./170.*t.^2).^20+263012370465./2199023255552./(1+1./170.*t.^2).^22+12611418068195524166851562157./158456325028528675187087900672./(1+1./170.*t.^2).^50+514589420475./4398046511104./(1+1./170.*t.^2).^23+61989816618513./562949953421312./(1+1./170.*t.^2).^26+15801325804719./140737488355328./(1+1./170.*t.^2).^25+8061900920775./70368744177664./(1+1./170.*t.^2).^24+956086325095055./9007199254740992./(1+1./170.*t.^2).^28+121683714103007./1125899906842624./(1+1./170.*t.^2).^27+46107685954746800146535717255800570437361005./713623846352979940529142984724747568191373312./(1+1./170.*t.^2).^76+24975553429171528252000152507./316912650057057350374175801344./(1+1./170.*t.^2).^51+11975573020964041433067793888190275875./170141183460469231731687303715884105728./(1+1./170.*t.^2).^64+1879204156221315./18014398509481984./(1+1./170.*t.^2).^29+197883231015743646919693516017./2535301200456458802993406410752./(1+1./170.*t.^2).^52+7391536347803839./72057594037927936./(1+1./170.*t.^2).^30+14544636039226909./144115188075855872./(1+1./170.*t.^2).^31+916312070471295267./9223372036854775808./(1+1./170.*t.^2).^32+91616570793198187304155386235551782817093945./1427247692705959881058285969449495136382746624./(1+1./170.*t.^2).^77+392032816163265715595619229845./5070602400912917605986812821504./(1+1./170.*t.^2).^53+1804857108504066435./18446744073709551616./(1+1./170.*t.^2).^33+364117140331941513644720124782321188119219525./5708990770823839524233143877797980545530986496./(1+1./170.*t.^2).^78+14023284727082855679./147573952589676412928./(1+1./170.*t.^2).^35+7113260368810144185./73786976294838206464./(1+1./170.*t.^2).^34);

        otherwise

            % A general solution is used for nu < 320 and the normal distribution is
            % used when the general solution fails

            if nu <= 170;
            %if nu <= 320;
                % The general solution runs extremely slow! 
                % To speed up processing, install fortran 90 compiler and use the mex file
                % 
                % The option for the general solution is disabled becasue
                % it is so slow.
                % 
                % To use the general solution: 
                % Change the comments in the above if statement
                % uncomment the if nu <= 320;
                % and comment out the if nu <= 170;
                % 
                % The general solution uses the generalized hypergeometric function
                % The general solution was derived with Maple
                try
                    [pfq]=real(genHyper([1, 0.5.*(nu+1)],[0.5.*nu+1],nu./(nu+t.^2), 0, 0, 10));
                    denom=sqrt(pi);
                    alpha(set_t)=0.5+sign(t)./2./denom.*( sqrt(pi)-gamma(0.5.*(nu+1)).*sqrt(t.^2./(nu+t.^2)).*pfq./(gamma(0.5.*nu+1).*(1+t.^2./nu).^(nu./2)));
                catch
                    % If the hypergeometric function crashes then the
                    % normal distribution is the backup.
                    %
                    % For very large nu the t-ditribution approaches the normal
                    % distribution with standard deviation of 1.
                    mu=0; % mu is the mean
                    sigma=1;    % unit Normal distribution

                    alpha(set_t)=0.5+0.5.*erf(0.5.*sqrt(2).*(t-mu)./sigma);
                end

            else
                % For very large nu the hypergeometric function crashes.
                % The t-ditribution approaches the normal
                % distribution with standard deviation of 1.
                mu=0; % mu is the mean
                sigma=1;    % unit Normal distribution

                alpha(set_t)=0.5+0.5.*erf(0.5.*sqrt(2).*(t-mu)./sigma);
            end

    end
end
